"use strict";

/* Copyright © 2014-2017 APDM, Inc.
 *
 * bilateral_one_norm_bar_vis.js
 */
define(['jquery', 'd3', 'utils', 'app/apdm_vis'], function ($, d3, utils, ApdmVis) {
  'use strict';
  /**
   * Set ApdmVis as base class.
   */

  BilateralOneNormBarVis.prototype = new ApdmVis();
  /**
   * Constructor for BilateralOneNormBarVis.
   * @param {d3.selection} container The d3 container to build
   * visualization in.
   * @param {String} baseSvg File name of the SVG to use for
   * visualization.
   * @param {String} testName The name of the test.
   * @param {String} conditionName The condition of the test.
   * @param {String} metricName The name of the metric.
   * @param {String} metrics The set of metrics for the current test.
   */

  function BilateralOneNormBarVis(container, baseSvg, testName, conditionName, metricName, metrics) {
    ApdmVis.call(this, container, baseSvg, testName, conditionName, metricName, metrics);
    this.scale = d3.scaleLinear(); // Is the layout of the plot horizontal or vertical?
    // Default is vertical.

    this.isHorizontal = false;
  }
  /**
   * Update the plot.
   */


  BilateralOneNormBarVis.prototype.update = function () {
    var self = this;
    $(self.container.node()).load(self.baseSvg, baseLoaded); // Determine which direction to move the rectangles.

    var rectThickness = 'height';
    var posAxis = 'y';

    if (this.isHorizontal) {
      rectThickness = 'width';
      posAxis = 'x';
    }

    function baseLoaded() {
      self.container.select('svg').attr({
        'width': 770,
        'height': 260
      });
      var visData = self.getBilateralData(); // Set the max range text.

      self.container.select('#metric-vis-max-range').text(self.scale.domain()[1]); // TODO: Determine if we should left or right norm.

      var height;
      var norms = visData.norms;

      if (norms !== null && norms.fifthToNinteyFifth && norms.fifthToNinteyFifth !== null) {
        var minMax = norms.fifthToNinteyFifth;
        height = Math.abs(self.scale(minMax[0]) - self.scale(minMax[1]));
        self.container.select('#bilateral-vis-norm-range').attr({
          'rx': 4,
          'ry': 4
        });
        var rectPos = self.scale(minMax[1]);

        if (self.isHorizontal) {
          rectPos = rectPos - height;
        }

        self.container.select('#bilateral-vis-norm-range').attr(rectThickness, height).attr(posAxis, rectPos);
      } else {
        self.container.select('#bilateral-vis-norm-range').remove();
      }

      doLeftOrRight('left', visData.left);
      doLeftOrRight('right', visData.right);
      self.setBilateralTextValues(visData);
      self.doBilateralTextAlignment();
    }

    function removeRange(foot) {
      var div = self.container;
      div.select('#bilateral-vis-' + foot + '-subject-range-top').remove();
      div.select('#bilateral-vis-' + foot + '-subject-range-bottom').remove();
      div.select('#bilateral-vis-' + foot + '-subject-range-line').remove();
    }

    function doLeftOrRight(foot, dataValues) {
      var div = self.container;

      if (dataValues.values === null || dataValues.values.length < 2 || isNaN(dataValues.mean)) {
        removeRange(foot);
      } else {
        var minMax = d3.extent(dataValues.values);
        div.select('#bilateral-vis-' + foot + '-subject-range-top').attr('c' + posAxis, self.scale(minMax[1]));
        div.select('#bilateral-vis-' + foot + '-subject-range-bottom').attr('c' + posAxis, self.scale(minMax[0]));
        div.select('#bilateral-vis-' + foot + '-subject-range-line').attr(posAxis + '1', self.scale(minMax[1])).attr(posAxis + '2', self.scale(minMax[0])); // If range it to high, add small arrow

        if (minMax[1] > self.scale.domain()[1]) {
          self.addOverrangeArrow(foot, '/mobility_clinic/res/overrange_arrow_sm.svg', 'overrange-arrow-sm');
        }
      }

      if (dataValues.mean === null || isNaN(dataValues.mean)) {
        div.select('#bilateral-vis-' + foot + '-subject').remove();
        return;
      } // A little added to the height, to make sure it is below the line.


      var height = Math.abs(self.scale(0) - self.scale(dataValues.mean)) + 3;
      var rectPos = self.scale(dataValues.mean);

      if (self.isHorizontal) {
        rectPos = rectPos - height;
      }

      div.select('#bilateral-vis-' + foot + '-subject').attr(posAxis, rectPos).attr(rectThickness, height).attr({
        'rx': 4,
        'ry': 4
      }); // If the data value is above the domain, then put in the up arrow.

      if (dataValues.mean > self.scale.domain()[1]) {
        self.addOverrangeArrow(foot, '/mobility_clinic/res/overrange_arrow.svg', 'overrange-arrow');
      }
    }
  };
  /**
   * Add an overrange arrow to center of the bar.
   * @param {String} foot The foot being handled.
   * @param {String} arrowFile Path to the arrow SVG file.
   * @param {String} arrowId The id of the arrow's g element.
   */


  BilateralOneNormBarVis.prototype.addOverrangeArrow = function (foot, arrowFile, arrowId) {
    var self = this;
    d3.xml(arrowFile, function (err, arrow) {
      if (err) {
        utils.error(err);
        return;
      }

      var outerSvg = self.container.select('svg');
      var arrowSvg = arrow.getElementsByTagName('svg')[0];
      arrowSvg.setAttribute('id', arrowId + '-' + foot);
      outerSvg.node().appendChild(arrowSvg); // Get the position of the rectangle

      var x = outerSvg.select('#bilateral-vis-' + foot + '-subject').node().getBBox().x; // Get the transform on the bar container.

      var containerTrans = d3.transform(self.container.select('#bilateral-bar-graph-container').attr('transform'));
      x = x + containerTrans.translate[0];
      var arrowG = outerSvg.select('#' + arrowId + '-' + foot).select('#' + arrowId);
      x = x + arrowG.node().getBBox().width * 0.5 + 1;
      arrowG.attr('transform', 'translate(' + x + ',0)');
    });
  };
  /**
   * Get or set the range of the plot.
   * @param {Array} plotRange The range of the plot as two element array.
   * @return The range of the plot if called with no arguments,
   * otherwise this instance.
   */


  BilateralOneNormBarVis.prototype.range = function (plotRange) {
    if (!arguments.length) {
      return this.scale.range();
    }

    this.scale.range(plotRange);
    return this;
  };
  /**
   * Get or set the domain of the plot.
   * @param {Array} plotDomain The domain of the plot.
   * @return The domain of the plot if called with no arguments,
   * otherwise this instance.
   */


  BilateralOneNormBarVis.prototype.domain = function (plotDomain) {
    if (!arguments.length) {
      return this.scale.domain();
    }

    this.scale.domain(plotDomain);
    return this;
  };
  /**
   * Get or set the flag that determines if the layout of the bars are
   * horizontal instead of vertical.
   * @param {Boolean} isHorizontal Flag that determines if layout of bars
   * are horizontal.
   * @return {BilateralOneNormBarVis} Current instance if called with value,
   * otherwise the current flag value.
   */


  BilateralOneNormBarVis.prototype.horizontal = function (isHorizontal) {
    if (!arguments.length) {
      return this.isHorizontal;
    }

    this.isHorizontal = isHorizontal;
    return this;
  };

  return BilateralOneNormBarVis;
});
//# sourceMappingURL=bilateral_one_norm_bar_vis.js.map
