"use strict";

/* Copyright (c) 2016-2019 APDM, Inc.
 *
 * configuration.js
 *
 * Load and setup the hardware configuration page.
 */
define(['jquery', 'd3', 'model_protocol', 'bootstrap-dialog', 'server_handler', 'error_handler', 'progress_dialog', 'app/placement_labels_EN', 'app/user_options_handler'], function ($, d3, ModelProtocol, BootstrapDialog, ServerHandler, ErrorHandler, ProgressDialog, labelNames, UserOptionsHandler) {
  var WIRELESS_CHANNELS = [10, 20, 30, 40, 50, 60, 70, 80];
  var ANCHOR_ARRAY = [1, 2, 3, 4, 5, 6, 7, 8, 'Calibration'];
  var CUSTOM_PLACEMENTS = ['CUSTOM_ONE', 'CUSTOM_TWO', 'CUSTOM_THREE']; // Possible status from configuration.

  var HW_CHECK_COMPLETED = 'HW_CHECK_COMPLETED';
  var HW_CHECK_COMPLETED_END = 'HW_CHECK_COMPLETED_END';
  var HW_CHECK_FAILED = 'HW_CHECK_FAILED';
  var HW_CONFIGURATION_COMPLETED = 'HW_CONFIGURATION_COMPLETED';
  var HW_CONFIGURATION_FAILED = 'HW_CONFIGURATION_FAILED';
  var SENSOR_SD_CLEAR_REQUIRED = 'SENSOR_SD_CLEAR_REQUIRED';
  var FIRMWARE_UPDATE_REQUIRED = 'FIRMWARE_UPDATE_REQUIRED';
  var FIRMWARE_UPDATE_FAILED = 'FIRMWARE_UPDATE_FAILED';
  var FIRMWARE_CHECK_COMPLETED = 'FIRMWARE_CHECK_COMPLETED';
  var WIRELESS_SYNC = 'WIRELESS_SYNC';
  var MESH_SYNC = 'MESH_SYNC';
  var BASE_PATH = '/mobility_clinic/';
  var currentState; // Configuration states to send to server

  var CANCEL_CONFIGURATION = 'CANCEL_CONFIGURATION';
  var CLEAR_SD_CARD = 'CLEAR_SD_CARD';
  var UPDATE_FIRMWARE = 'UPDATE_FIRMWARE';
  var server;

  var _scannedHw;

  var _available = [];

  var _allowLogging;

  var doAnchors;
  var isSixSensor;
  var currentConfig = {
    placements: {},
    anchorDistances: [],
    wirelessChannel: 80,
    recordMode: WIRELESS_SYNC
  };
  var savedConfigs = {};
  var progressDialog;
  var pollInterval;
  /**
   * Load the HTML for the configuration page.
   * @param {Boolean} configAnchors If true, do configuration for
   * anchor placement as well as body monitors.
   * @param {Boolean} sixSensor If true, than this is a six sensor
   * system. If false, is seventeen.
   */

  function openConfigurationPage(configAnchors, sixSensors) {
    progressDialog = new ProgressDialog({
      title: 'Check for existing configurations'
    });
    progressDialog.show();
    doAnchors = configAnchors ? configAnchors : false;
    isSixSensor = sixSensors ? sixSensors : false;
    _allowLogging = UserOptionsHandler.getInstance().getAllowLoggingConfiguration();
    var pageHtml = 'html/configuration_no_anchors.html';

    if (doAnchors) {
      pageHtml = 'html/configuration_with_anchors.html';
    }

    server = ServerHandler.getInstance();
    $('#switchable').load(BASE_PATH + pageHtml, pageLoaded);

    function pageLoaded() {
      var sensorMan = 'sensor_placement_man_17.svg';

      if (isSixSensor) {
        sensorMan = 'sensor_placement_man_6.svg';
      }

      $('#sensor-body-placement').load('../../mobility_clinic/res/' + sensorMan, imageLoaded);
    }
  }
  /**
   * Initialize the view once the image for the sensor man has
   * loaded.
   */


  function imageLoaded() {
    initializeView();
    loadSavedConfigurations();
  }
  /**
   * Load the saved configurations.
   */


  function loadSavedConfigurations() {
    server.post({
      type: ModelProtocol.load_sensor_configuration
    }, 'json', function (d) {
      savedConfigs = d.sensorConfiguration;
      progressDialog.setTitle('Checking Firmware');
      startHardwareCheck();
    }, function (d) {
      ErrorHandler.showError(d.error().statusText);
      updateHtml();
    });
  }
  /**
   * Get the connected hardware.
   */


  function startHardwareCheck() {
    currentState = '';
    pollInterval = setInterval(function () {
      pollStatus(updateHardwareCheckStatus);
    }, 250);
    server.post({
      type: ModelProtocol.start_hardware_check
    }, null, null, function (d) {
      progressDialog.close();
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Get the connected hardware after it's been scanned.
   * @param {Array} messages Messages returned from the hardware check.
   */


  function getAvailableHardware(messages) {
    server.post({
      type: ModelProtocol.get_available_hardware
    }, 'json', function (d) {
      _scannedHw = d;
      _available = Object.keys(d.sensors);
      progressDialog.close();

      if (messages.length) {
        configurationComplete(messages);
      }

      updateHtml();
    }, function (d) {
      progressDialog.close();
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Handler called after HTML loaded.
   */


  function initializeView() {
    circleHandlers();
    setupWirelessChannelSelect();
    setupRecordingModeSelect();

    if (doAnchors) {
      buildAnchorDistanceMatrix();
    } // Drop handlers on anchor placements and custom label placements.


    var container = d3.select('#configuration-container');
    container.selectAll('.anchor-placement-zone, .sensor-placement-zone').on('dragenter', function () {
      var parentId = d3.select(this).attr('data-parentcontainerid');

      if (parentId && parentId !== '') {
        d3.select('#' + parentId).selectAll('circle').style('stroke', 'red');
      }
    }).on('dragleave', function () {
      var parentId = d3.select(this).attr('data-parentcontainerid');

      if (parentId && parentId !== '') {
        d3.select('#' + parentId).selectAll('circle').style('stroke', '#009fbf');
      }
    }).on('dragover', function () {
      window.event.preventDefault();
    }).on('drop', function () {
      dragDropBodyLocation(window.event, d3.select(this));
    });
    container.selectAll('.anchor-placement-input').on('input', function () {
      validateTextField(this);
    });
    setupButtonHandlers();
    addInputHandlers(container);
  }
  /**
   * And the hover and drop handlers to the circles on the man.
   */


  function circleHandlers() {
    var container = d3.select('#configuration-container');
    var circles = container.selectAll('.sensor-circles');
    circles.on('mouseover', function () {
      fadeIn(this);
    }).on('mouseout', function () {
      fadeOut(this);
    }).on('dragenter', function () {
      fadeIn(this);
    }).on('dragleave', function () {
      fadeOut(this);
    }).on('dragover', function () {
      window.event.preventDefault();
    }).on('drop', function () {
      dragDropBodyLocation(window.event, d3.select(this));
    });

    function fadeIn(element) {
      var circleG = d3.select(element);
      circleG.transition().delay(100).style('opacity', 1);
      d3.select('#' + circleG.attr('data-sensorlabel') + '-text').transition().delay(100).style('opacity', 1);
    }

    function fadeOut(element) {
      var sensor = d3.select(element).attr('data-sensorlabel');

      if (!hasAssignment(currentConfig, sensor)) {
        d3.select(element).transition().delay(50).style('opacity', 0);
      }

      d3.select('#' + sensor + '-text').transition().delay(50).style('opacity', 0);
    }
  }
  /**
   * Update the page with available hardware and configuration data.
   * Display a dialog for user if retrieved hardware does not match
   * saved configuration.
   */


  function updateHtml() {
    if (savedConfigs && Object.keys(savedConfigs).length) {
      updateFromSavedConfiguration(savedConfigs);
    }

    var warningMsgs = []; // Check for an access point. Only display messaged if there are
    // no access points and user is not configured for logging.

    if (!_scannedHw.aps.length && savedConfigs && savedConfigs.recordMode !== MESH_SYNC) {
      warningMsgs.push('No access points have been detected. ' + 'Please connect any access points.');

      if (UserOptionsHandler.getInstance().getAllowLoggingConfiguration()) {
        warningMsgs.push('If you are configuring for "Logging", ' + 'you may ignore this message.');
      }
    } // Check detected sensors against saved configuration.


    var configured = [];

    if (savedConfigs && savedConfigs.placements) {
      Object.keys(savedConfigs.placements).forEach(function (d) {
        if (savedConfigs.placements[d] !== '') {
          configured.push(savedConfigs.placements[d]);
        }
      });
    }

    var missing = configured.filter(function (d) {
      return Object.keys(_scannedHw.sensors).indexOf(d) < 0;
    });

    if (missing.length) {
      warningMsgs.push('The following sensors are present in the ' + 'saved configuration but are not currently docked:');
      warningMsgs.push(missing.join(', '));
      warningMsgs.push('These sensors will be grayed out in the ' + '"Assigned Sensors" list.');
      warningMsgs.push('If you wish to reconfigure, please make sure ' + 'all sensors are docked, or remove these sensors ' + 'from the configuration by clicking the \'x\' next to the entry.');
    } // Display dialog if necessary


    if (warningMsgs.length) {
      var msg = warningMsgs.reduce(function (prev, curr) {
        return prev + '<p>' + curr + '</p>';
      }, '') + '<p>Re-scan hardware when done.</p>';
      BootstrapDialog.show({
        id: 'configuration-detected-hardware-dialog',
        title: 'Detected hardware does not match saved configuration',
        message: msg,
        buttons: [{
          id: 'configuration-detected-hardware-close-btn',
          icon: 'iconic iconic-x',
          label: 'Close',
          cssClass: 'btn-link',
          action: function action(dialog) {
            dialog.close();
          }
        }]
      });
    }

    buildAvailableSensorList();
    findMissingAssigned(missing);
  }
  /**
   * Update the configuration displayed with values from a saved
   * configuration.
   * @param {Object} savedConfig The configuration to update the
   * current configuration to.
   */


  function updateFromSavedConfiguration(savedConfig) {
    $('#wireless-channel-select').selectpicker('val', savedConfig.wirelessChannel);
    $('#recording-mode-select').selectpicker('val', savedConfig.recordMode);

    if (doAnchors) {
      updateAnchorDistancesFromConfig(savedConfig);
    }

    Object.keys(savedConfig.placements).forEach(function (d) {
      updateConfiguration(savedConfig.placements[d], d);
    });
    var customLabelInputs = d3.selectAll('.custom-label-input');
    Object.keys(savedConfig.customLabels).forEach(function (d) {
      customLabelInputs.each(function () {
        if (d3.select(this).attr('data-sensorlabel') === d && hasAssignment(savedConfig, d)) {
          this.value = savedConfig.customLabels[d];
        }
      });
    });
  }
  /**
   * Setup the dropdown for wireless channel select.
   */


  function setupWirelessChannelSelect() {
    var select = d3.select('#wireless-channel-select');
    select.selectAll('option').data(WIRELESS_CHANNELS).enter().append('option').attr('value', function (d) {
      return d;
    }).text(function (d) {
      return d;
    });
    $(select.node()).selectpicker({
      title: 'Wireless Channel',
      width: 'auto'
    });
    $(select.node()).val(WIRELESS_CHANNELS[7]);
    $(select.node()).selectpicker('refresh');
  }
  /**
   * Setup the dropdown for the recording mode select.
   */


  function setupRecordingModeSelect() {
    if (!_allowLogging) {
      return;
    }

    $('#recording-mode-select-container').css('display', 'block');
    $('#recording-mode-select').selectpicker({
      width: 'auto'
    });
  }
  /**
   * Setup the button handlers.
   */


  function setupButtonHandlers() {
    d3.select('#sensor-placement-save-button').on('click', saveSensorConfiguration);
    d3.select('#apply-configuration-button').on('click', applyConfiguration);

    if (doAnchors) {
      d3.select('#validate-distances-button').on('click', validateAnchorConfiguration);
    }

    d3.select('#rescan-hardware-button').on('click', function () {
      progressDialog = new ProgressDialog({
        title: 'Checking Firmware'
      });
      progressDialog.show();
      $('.sensor-placement-input').val(null);
      startHardwareCheck();
    });
  }
  /**
   * Add handlers to the inputs and the "clear" buttons.
   * @param {d3.selection} container The container with the inputs.
   */


  function addInputHandlers(container) {
    container.selectAll('.anchor-placement-zone,.sensor-placement-zone').each(function () {
      var input = $(this).find('input');
      var clear = $(this).find('span');
      input.on('change keydown paste input', function () {
        validateTextField(this);
      });
      clear.on('click', function () {
        $(this).parent().find('input').val('');
        validateTextField(this);
      });
    });
  }
  /**
   * Build the list of available and assigned sensors.
   */


  function buildAvailableSensorList() {
    // Filter out any assigned sensors from the available list.
    var currentlyAssigned = Object.keys(currentConfig.placements).map(function (d) {
      return currentConfig.placements[d];
    });
    _available = _available.filter(function (d) {
      return currentlyAssigned.indexOf(d) < 0;
    });
    var list = d3.select('#available-list').selectAll('.sensor-list-container').data(_available, function (d) {
      return d;
    });
    list.enter().append('div').attr('class', 'sensor-list-container').append('span').attr('class', 'sensor-entry').attr('draggable', true).attr('id', function (d) {
      return d;
    }).text(function (d) {
      return d + ': ' + _scannedHw.sensors[d];
    }).on('dragstart', function (d) {
      window.event.dataTransfer.setData('text', d);
    });
    list.exit().remove(); // Build the assigned list.

    var assigned = [];
    Object.keys(currentConfig.placements).forEach(function (d) {
      if (hasAssignment(currentConfig, d)) {
        assigned.push({
          placement: d,
          sensor: currentConfig.placements[d]
        });
      }
    });
    var join = d3.select('#assigned-list').selectAll('.sensor-list-container').data(assigned, function (d) {
      return d.placement + d.sensor;
    });
    join.enter().append('div').attr('class', 'sensor-list-container').append('span').attr('class', 'sensor-entry sensor-assigned-entry').attr('draggable', true).attr('id', function (d) {
      return d.sensor;
    }).on('dragstart', function (d) {
      window.event.dataTransfer.setData('text', d.sensor);
    }).text(function (d) {
      return d.sensor + ': ' + labelNames[d.placement];
    }).append('span').attr('class', 'clear-assigned iconic iconic-circle-x').on('click', function () {
      var config = d3.select(this).datum();
      updateConfiguration(config.sensor);
    });
    join.exit().remove();
  }
  /**
   * Find sensors that are in the configurations, but are
   * missing from the detected hardware.
   * @param {Array} missing The list of missing sensors.
   */


  function findMissingAssigned(missing) {
    d3.select('#assigned-list').selectAll('.sensor-assigned-entry').each(function (d) {
      d3.select(this).classed('sensor-missing-entry', missing.indexOf(d.sensor) > -1);
    });
  }
  /**
   * Handle the drop event on a body location label.
   * @param {MouseEvent} evt The event.
   * @param {d3.selection} inputDiv The container for the placement
   * label.
   */


  function dragDropBodyLocation(evt, inputDiv) {
    var sensor = evt.dataTransfer.getData('text');
    var loc = inputDiv.attr('data-sensorlabel');
    updateConfiguration(sensor, loc);
    evt.preventDefault();
  }
  /**
   * See if the text entered is a valid sensor and update
   * available sensors and configuration appropriately.
   * @param {DOM Element} inputEle The input element being edited.
   */


  function validateTextField(inputEle) {
    var parent = $(inputEle).parent();
    var label = parent.attr('data-sensorlabel');
    var sensor = inputEle.value;
    parent.removeClass('has-success').addClass('has-error');

    if (Object.keys(_scannedHw.sensors).indexOf(sensor) < 0) {
      parent.addClass('has-error');
      clearPlacement(label);
      buildAvailableSensorList();
    } else {
      parent.addClass('has-success');
      updateConfiguration(sensor, label);
    }

    if (!sensor || sensor === '') {
      parent.removeClass('has-success has-error');
      parent.find('span').css('display', 'none');
    }
  }
  /**
   * Update the distance matrix with the values in the given
   * configuration.  @param {Object} config The configuration
   * object.
   */


  function updateAnchorDistancesFromConfig(config) {
    if (!config.anchorDistances) {
      return;
    }

    var allDistances = d3.selectAll('.anchor-distance-input');
    config.anchorDistances.forEach(function (pairDistance) {
      allDistances.each(function (d) {
        // Use '+' to force conversion to number
        if (d3.min(d) === d3.min(pairDistance.pair) && d3.max(d) === d3.max(pairDistance.pair)) {
          this.value = pairDistance.distance;
        }
      });
    });
  }
  /**
   * Add the monitor to the available list if it
   * exists in the attached hardware.
   * @param {String} sensorId The id of the sensor.
   */


  function maybeMakeAvailable(sensorId) {
    if (Object.keys(_scannedHw.sensors).indexOf(sensorId) > -1) {
      _available.push(sensorId);
    }
  }
  /**
   * Update the configuration object.
   * @param {String} sensorId The id of the sensor.
   * @param {String} location The location of the sensor. Passing a null
   * value will clear the current configuration for the sensor.
   */


  function updateConfiguration(sensorId, location) {
    // Clear the old location for the given sensor.
    clearSensorPlacement(sensorId);

    if (location) {
      // Clear the sensor at currently at the location.
      if (currentConfig.placements[location] && currentConfig.placements[location] !== null) {
        clearSensorPlacement(currentConfig.placements[location]);
      }

      setSensorPlacement(sensorId, location);
    }

    buildAvailableSensorList();
  }
  /**
   * Clear the given placement from the current configuration.
   * @param {String} placement The placement location to clear.
   */


  function clearPlacement(placement) {
    var sensor = currentConfig.placements[placement];
    delete currentConfig.placements[placement];
    maybeMakeAvailable(sensor);
  }
  /**
   * Clear the sensor from the configuration object and
   * from the UI.
   * @param {String} sensor The sensor to clear.
   */


  function clearSensorPlacement(sensor) {
    var placements = Object.keys(currentConfig.placements).filter(function (d) {
      return currentConfig.placements[d] === sensor;
    });
    placements.forEach(function (d) {
      delete currentConfig.placements[d];
      var zone = $('#' + d + '-sensor-placement,' + '#' + d + 'anchor-placement').removeClass('has-success has-error');
      zone.find('input').val(null);
      zone.find('span').css('display', 'none');
      d3.select('#' + d + '-circles').transition().delay(50).style('opacity', 0);
    });
    maybeMakeAvailable(sensor);
  }

  function setSensorPlacement(sensor, location) {
    currentConfig.placements[location] = sensor;
    var zone = $('#' + location + '-sensor-placement,' + '#' + location + 'anchor-placement').removeClass('has-error').addClass('has-success');
    zone.find('input').val(sensor);
    zone.find('span').css('display', 'inline-block');
    d3.select('#' + location + '-circles').transition().delay(50).style('opacity', 1);
    _available = _available.filter(function (d) {
      return d !== sensor;
    });
  }
  /**
   * Build the matrix for entering the distances between anchors.
   */


  function buildAnchorDistanceMatrix() {
    var body = d3.select('#anchor-distance-matrix').append('table').attr('class', 'table table-bordered');
    body.append('caption').text('Anchor Distances');
    body = body.append('tbody');
    body.selectAll('tr').data(ANCHOR_ARRAY).enter().append('tr').each(function (d) {
      buildAnchorDistanceMatrixRow(d3.select(this), d);
    });
  }
  /**
   * Build the rows in the distance matrix.
   * @param {d3.selection} tr The row element.
   * @param {Number} rowNum The number of the row.
   */


  function buildAnchorDistanceMatrixRow(tr, rowValue) {
    if (rowValue === 1) {
      tr.append('th');
    } else if (rowValue === 'Calibration') {
      tr.append('th').text(rowValue);
      rowValue = 9;
    } else {
      tr.append('th').text(rowValue);
    }

    for (var i = 1; rowValue > i; i++) {
      tr.append('td').append('input').attr('class', 'form-control anchor-distance-input').datum([tr.data()[0], i]);
    }

    if (rowValue !== ANCHOR_ARRAY.length) {
      tr.append('th').text(rowValue);
    }
  }
  /**
   * Save the sensor placement configuration.
   */


  function saveSensorConfiguration() {
    var placementInputs = d3.selectAll('.sensor-placement-input');

    if (!validateInputSensorLabels(placementInputs)) {
      return;
    }

    if (!validateCustomLabels()) {
      return;
    }

    if (doAnchors && !validateAnchorConfiguration()) {
      return;
    }

    server.post({
      type: ModelProtocol.save_sensor_configuration,
      configurations: JSON.stringify(currentConfig)
    }, null, null, function (d) {
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Check sensor names and display dialog if not valid.
   * @return {Boolean} True if all sensor lables are valid,
   * otherwise false.
   */


  function validateInputSensorLabels() {
    // Use the "has-error" class to find the invalid ones.
    var inputs = d3.selectAll('.anchor-placement-input, .sensor-placement-input');
    var badLabels = [];
    inputs.each(function () {
      if (d3.select(this.parentNode).classed('has-error')) {
        badLabels.push({
          location: d3.select(this).attr('placeholder'),
          label: this.value
        });
      }
    });

    if (!badLabels.length) {
      return true;
    }

    BootstrapDialog.show({
      title: 'Invalid Sensor Label',
      message: getBadLabelDialogMessage(badLabels),
      buttons: [{
        icon: 'iconic iconic-x',
        label: 'Close',
        cssClass: 'btn-link',
        action: function action(dialog) {
          dialog.close();
        }
      }]
    });
    return false;
  }
  /**
   * Check that if a custom sensor location has been chosen that
   * there is a label entered for that location.
   * @return {Boolean} False if labels are missing for a sensor.
   */


  function validateCustomLabels() {
    var needLabels = false;
    CUSTOM_PLACEMENTS.forEach(function (d) {
      var input = $('#' + d + "-placement-input");

      if (input && input.val() && input.val() !== '') {
        var label = d3.select('#' + d + '-label-input').node();
        $(label).parent().removeClass('has-error');

        if (!label.value || label.value === '') {
          $(label).parent().addClass('has-error');
          needLabels = true;
        }
      }
    });

    if (needLabels === true) {
      BootstrapDialog.show({
        title: 'Label Required',
        message: 'All custom sensor assignments must have a label.',
        buttons: [{
          icon: 'iconic iconic-x',
          label: 'Close',
          cssClass: 'btn-link',
          action: function action(dialog) {
            dialog.close();
          }
        }]
      });
    }

    return !needLabels;
  }
  /**
   * Do the local validation of the anchor and distance pairs, and
   * then pass it off to the server for validation by an analysis
   * routine there.
   * @return {Boolean} True if anchor pairs and distances valid.
   */


  function validateAnchorConfiguration() {
    var validDistances = validateDistances();

    if (!validDistances.length) {
      return false;
    }

    currentConfig.anchorDistances = validDistances; // Validate the anchor height input

    var heightInput = $('#anchor-height-input');
    heightInput.removeClass('has-error');
    var anchorHeight = heightInput.val();

    if (isNaN(anchorHeight)) {
      heightInput.addClass('has-error');
      new ErrorHandler({
        title: 'Bad Height Value',
        message: 'Anchor height value'
      }).show();
      return false;
    }

    currentConfig.anchorHeight = anchorHeight;
    return true;
  }
  /**
   * Validate the entries in the distance matrix. First pass will
   * make sure distances are present for configured anchors. Data
   * then sent to server to get validation from external algorthim.
   * @return {Array} Array of objects with anchor pair and distance.
   * If needed distances are missing, then returns empty array.
   */


  function validateDistances() {
    // Get inputs, clear any error states.
    var allInputs = d3.selectAll('.anchor-distance-input').each(function () {
      d3.select(this.parentNode).classed('has-error', false);
    });
    var invalidInputs = [];
    var distances = [];
    allInputs.each(function (d) {
      var pairDistance = parseFloat(this.value, 10);

      if (isNaN(pairDistance)) {
        invalidInputs.push(d3.select(this));
        d3.select(this.parentNode).classed('has-error', true);
      } else {
        distances.push({
          pair: d,
          distance: pairDistance
        });
      }
    });
    var msg = null;

    if (invalidInputs.length) {
      msg = '<p>The following pairs have invalid distances:</p>';
      invalidInputs.forEach(function (d) {
        msg = msg + '<p>' + d.data() + '</p>';
      });
      new ErrorHandler({
        title: 'Bad Distance Values',
        message: msg
      }).show();
      return [];
    }

    return distances;
  }
  /**
   * Build the message for the bad label dialog.
   * @param {Array} badLabels The array of bad sensor labels and their
   * location.
   * @return The message to display in dialog.
   */


  function getBadLabelDialogMessage(badLabels) {
    var msg = '<h4>The following sensor labels are invalid:</h4>';
    badLabels.forEach(function (d) {
      msg = msg + '<p>' + d.label + ' - ' + d.location + '</p>';
    });
    return msg;
  }
  /**
   * Apply to current to the configuration. Do the validation before
   * sending the config off to the server.
   */


  function applyConfiguration() {
    // Do local validation before attempting configuration.
    var placementInputs = d3.selectAll('.sensor-placement-input');

    if (!validateInputSensorLabels(placementInputs) || doAnchors && !validateAnchorConfiguration()) {
      return;
    }

    if (!validateCustomLabels()) {
      return;
    } // Get the wireless channel


    currentConfig.wirelessChannel = $('#wireless-channel-select').val(); // Get recording mode

    if (_allowLogging) {
      currentConfig.recordMode = $('#recording-mode-select').val();
    } else {
      currentConfig.recordMode = WIRELESS_SYNC;
    } // Get any custom labels that are set.


    currentConfig.customLabels = {};
    var customLabelInputs = d3.selectAll('.custom-label-input');
    customLabelInputs.each(function () {
      currentConfig.customLabels[d3.select(this).attr('data-sensorlabel')] = this.value;
    });
    saveSensorConfiguration(); // Start the polling, then make the server call to start the
    // configuration process.

    startConfigurationStatusPolling();
    server.post({
      type: ModelProtocol.configure_hardware,
      configuration: JSON.stringify(currentConfig)
    }, null, null, function (d) {
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Open dialog to ask user if SD cards should be erased.
   */


  function clearSDDialog() {
    BootstrapDialog.show({
      title: 'Erase sensor SD cards',
      closable: false,
      message: '<p>One or more of the SD cards on the sensors ' + 'is over 90% full.</p>' + '<p>Do you want to erase the data on the SD cards ' + 'and continue configuration?</p>',
      buttons: [{
        icon: 'iconic iconic-check',
        label: 'Continue',
        cssClass: 'btn-success',
        action: function action(dialog) {
          postConfigUIState(CLEAR_SD_CARD);
          dialog.close();
        }
      }, {
        icon: 'iconic iconic-x',
        label: 'Cancel',
        cssClass: 'btn-link',
        action: function action(dialog) {
          postConfigUIState(CANCEL_CONFIGURATION);
          dialog.close();
        }
      }]
    });
  }
  /**
   * Display dialog to have user confirm update to
   * the hardware firmware.
   */


  function updateFirmwareDialog() {
    BootstrapDialog.show({
      title: 'Firmware update required',
      closable: false,
      message: '<p>The hardware firmware is out of date.</p>' + '<p>Please plug all Opals into the dock and make sure the ' + 'Access Point is connected, then click "Continue"</p>',
      buttons: [{
        icon: 'iconic iconic-x',
        label: 'Cancel',
        cssClass: 'btn-link',
        action: function action(dialog) {
          postConfigUIState(CANCEL_CONFIGURATION);
          dialog.close();
        }
      }, {
        icon: 'iconic iconic-check',
        label: 'Continue',
        cssClass: 'btn-success',
        action: function action(dialog) {
          postConfigUIState(UPDATE_FIRMWARE);
          dialog.close();
        }
      }]
    });
  }
  /**
   * Start the polling for the configuration status.
   */


  function startConfigurationStatusPolling() {
    // Open the progress dialog.
    progressDialog = new ProgressDialog({
      title: 'Checking Firmware'
    });
    currentState = '';
    progressDialog.show();
    pollInterval = setInterval(function () {
      pollStatus(updateConfigurationStatus);
    }, 250);
  }
  /**
   * Poll the server for status.
   * @param {Function} cb The function to call when update returned.
   */


  function pollStatus(cb) {
    server.post({
      type: ModelProtocol.get_configuration_status
    }, 'json', function (d) {
      cb(d);
    }, function (d) {
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Post the configuration state to the server.
   * @param {String} state The state to post.
   */


  function postConfigUIState(state) {
    server.post({
      type: ModelProtocol.set_configuration_ui_state,
      configUIState: state
    }, null, null, function (d) {
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Check the status of the hardware/firmware check.
   * @param {Object} configStatus Status returned from the server.
   */


  function updateHardwareCheckStatus(configStatus) {
    if (!configStatus.currentState || configStatus.currentState === currentState) {
      return;
    }

    currentState = configStatus.currentState;

    if (currentState === FIRMWARE_UPDATE_REQUIRED) {
      progressDialog.setTitle('Updating device firmware');
      updateFirmwareDialog();
    } else if (currentState === FIRMWARE_UPDATE_FAILED) {
      progressDialog.close();
      clearInterval(pollInterval);
      configurationComplete(configStatus.messages);
    } else if (currentState === FIRMWARE_CHECK_COMPLETED) {
      progressDialog.setTitle('Getting connected hardware');
    } else if (currentState === SENSOR_SD_CLEAR_REQUIRED) {
      progressDialog.setTitle('Clearing Sensor SD Cards');
      clearSDDialog();
    } else if (currentState === HW_CHECK_FAILED) {
      progressDialog.close();
      clearInterval(pollInterval);
      configurationComplete(configStatus.messages);
    } else if (currentState === HW_CHECK_COMPLETED_END) {
      clearInterval(pollInterval);
      getAvailableHardware(configStatus.messages);
    }
  }
  /**
   * Handle updates to the configuration status.
   * @param {Object} configStatus Object containing the current
   * configuration state.
   */


  function updateConfigurationStatus(configStatus) {
    if (!configStatus.currentState || configStatus.currentState === currentState) {
      return;
    }

    currentState = configStatus.currentState;

    if (currentState === HW_CHECK_COMPLETED) {
      progressDialog.setTitle('Configuring Hardware');
    } else if (currentState === HW_CONFIGURATION_COMPLETED) {
      clearInterval(pollInterval);
      progressDialog.close();
      configurationComplete(configStatus.messages);
    } else if (currentState === FIRMWARE_UPDATE_REQUIRED) {
      progressDialog.setTitle('Updating device firmware');
      updateFirmwareDialog();
    } else if (currentState === FIRMWARE_CHECK_COMPLETED) {
      progressDialog.setTitle('Getting connected hardware');
    } else if (currentState === HW_CHECK_FAILED || currentState === HW_CONFIGURATION_FAILED || currentState === FIRMWARE_UPDATE_FAILED) {
      progressDialog.close();
      clearInterval(pollInterval);
      configurationComplete(configStatus.messages);
    } else if (currentState === SENSOR_SD_CLEAR_REQUIRED) {
      progressDialog.setTitle('Clearing Sensor SD Cards');
      clearSDDialog();
    }
  }
  /**
   * Open the dialog letting the user now that the
   * configuration is complete.
   * @param {Array} messages Messages received from the
   * configuration process to display to the user.
   */


  function configurationComplete(messages) {
    var msg = '<p>All hardware is configured.</p>';
    var title = 'Configuration Complete'; // TODO: Simple implementation - condsider revisiting this

    if (messages.length) {
      if (messages.filter(function (d) {
        return d.type === 'WARNING';
      }).length) {
        title = 'Warnings returned';
      } else {
        title = 'Errors Detected';
      }

      msg = '';
      messages.forEach(function (d) {
        msg = msg + '<p>' + d.type + ': ' + d.message + '</p>';
      });
    }

    BootstrapDialog.show({
      id: 'configuration-complete-dialog',
      title: title,
      message: msg,
      buttons: [{
        id: 'configuration-complete-dialog-close-btn',
        icon: 'iconic iconic-x',
        label: 'Close',
        cssClass: 'btn-link',
        action: function action(dialog) {
          dialog.close();
        }
      }]
    });
  }
  /**
   * Check and see if the placement in the config object has
   * a non-empty string.
   * @param {Object} config The configuration object.
   * @param {String} loc The placement location label.
   * @return {Boolean} True if the placement has a non-empty string
   * assigned to it.
   */


  function hasAssignment(config, loc) {
    return config.placements[loc] && config.placements[loc] !== '';
  }

  return {
    open: openConfigurationPage
  };
});
//# sourceMappingURL=configuration.js.map
