"use strict";

/* Copyright (c) 2015-2019 APDM, Inc.
 *
 * exchange_handler.js
 */
define(['jquery', 'server_handler', 'error_handler', 'model_protocol', 'utils', 'crossroads', 'bootstrap-dialog', 'model_dispatcher', 'progress_dialog', 'res_req_map'], function ($, ServerHandler, ErrorHandler, ModelProtocol, utils, crossroads, BootstrapDialog, ModelDispatcher, ProgressDialog, ResReqMap) {
  var instance;
  var _messageRoute = null;
  var hasServer = false;
  var _isLoggedIn = false;
  var isOnline = false;
  var _user = '';
  var _serverAddr = '';
  var _localDataExists = false;
  var metricWarningWasShown = false;
  var _webLogin = false;
  var _doingUserTimeout = false;
  var _loginDialogOpen = false;
  var _siteName = 'N/A';
  var _pswdGuidance = '';
  var _pswdRegExList = [];
  var _userTimeout = 0;
  var _loginDialog = null;
  var _progDialog = null;
  var _loggingInUser = null;
  var _singleUserUI = false;
  var _associatedWithStudy = true;
  var _updateHost = null; // Messages we'll handle

  var SUCCESS = 'SUCCESS';
  var SCHEMA_MISMATCH = 'SCHEMA_MISMATCH';
  var FAILURE = 'FAILURE';
  var SUCCESS_FIRST_CONNECTION = 'SUCCESS_FIRST_CONNECTION';
  var FAILED_FIRST_CONNECTION = 'FAILED_FIRST_CONNECTION';
  var LOGOUT_SUCCESS = 'LOGOUT_SUCCESS';
  var LOGOUT_FAILURE = 'LOGOUT_FAILURE';
  var OFFLINE_LOGIN_FAILURE = 'OFFLINE_LOGIN_FAILURE';
  var PASSWORD_CHANGE_SUCCESS = 'PASSWORD_CHANGE_SUCCESS';
  var PASSWORD_CHANGE_FAILURE = 'PASSWORD_CHANGE_FAILURE';
  var BAD_PASSWORD = 'BAD_PASSWORD';
  var BAD_PASSWORD_FIRST_CONNECTION = 'BAD_PASSWORD_FIRST_CONNECTION';
  var UNKNOWN_USERNAME = 'UNKNOWN_USERNAME';
  var UNKNOWN_USERNAME_FIRST_CONNECTION = 'UNKNOWN_USERNAME_FIRST_CONNECTION';

  function init() {
    /**
     * Register for MOBILITY_EXCHANGE messages.
     */
    _messageRoute = crossroads.addRoute('message/MOBILITY_EXCHANGE/{key}', handleMessage);
    getExchangeStatus();
    /**
     * Get the status of the connection with the Mobility Exchange server.
     */

    function getExchangeStatus() {
      ServerHandler.getInstance().post({
        type: ModelProtocol.get_exchange_status
      }, 'json', function (d) {
        hasServer = d.isAssociated;
        _user = d.user;
        _isLoggedIn = d.isLoggedIn;
        isOnline = d.isOnline;
        _localDataExists = d.dataAlreadyCollected;
        _singleUserUI = d.isMTBIHomeUser;
        _updateHost = d.updateHost;

        if (_updateHost && _updateHost.indexOf('mobilityexchange') > -1) {
          // If the update host indicates a mobility exchange instance,
          // set it as the default MX server URL.
          _serverAddr = 'https://' + _updateHost;
        } // User not associated with any studies, but is logged
        // in as a valid user.


        if (_isLoggedIn && !_localDataExists && !d.isMXAdmin) {
          _associatedWithStudy = false;
          showNotAssociatedWithStudyDialog();
          return;
        }

        _associatedWithStudy = true;

        if (_isLoggedIn) {
          $('#exchange-loggedin-user-name').text(utils.getTruncatedDisplayString(_user.userName, 14));
          $('#exchange-user-info-name').text(_user.userName);
        }

        if (_isLoggedIn && _user.changePasswordStatus && _user.changePasswordStatus !== 'NO') {
          passwordExpired();
        }

        if (d.passwordGuidance && d.passwordGuidance !== '') {
          _pswdGuidance = d.passwordGuidance;
        } else {
          _pswdGuidance = '';
        }

        if (d.passwordRegExList && d.passwordRegExList !== '') {
          try {
            _pswdRegExList = JSON.parse(d.passwordRegExList);
          } catch (ex) {
            ErrorHandler.showError('Unable to load password rules.');
          }
        } else {
          _pswdRegExList = [];
        }

        if (d.userTimeout && d.userTimeout !== '') {
          _userTimeout = parseInt(d.userTimeout, 10);
        } else {
          _userTimeout = 0;
        }

        crossroads.parse('exchangeStatusLoaded');
      }, function (d) {
        ErrorHandler.showError(d.error().statusText);
      });
    }
    /**
     * Make sure there are entries in all the field and if so,
     * make the call to server to connect to exchange server.
     * @param {BootstrapDialog} dialog The the server connection dialog.
     */


    function validateConnectionFields(dialog) {
      // Validate the fields - really just check to see if there
      // is something in the fields.
      _loggingInUser = $('#exchange-user-name').val();
      var password = $('#exchange-password').val();
      _serverAddr = $('#exchange-server-address').val();
      var isValid = true; // TODO: There has got to be a nicer way to do this.

      if (!utils.checkValueInBox('exchange-user-name', 'Username is required')) {
        isValid = false;
      }

      if (!utils.checkValueInBox('exchange-password', 'Password is required')) {
        isValid = false;
      }

      if (!utils.checkValueInBox('exchange-server-address', 'Server is required')) {
        isValid = false;
      }

      if (!isValid) {
        return;
      }

      ServerHandler.getInstance().post({
        type: ModelProtocol.new_exchange_connection,
        username: _loggingInUser,
        password: password,
        serverAddress: _serverAddr
      }, null, function () {
        // Call worked, if there is an error, then it will
        // come from a dispatched message.
        dialog.close();
      }, function (d) {
        // We did connect to server
        dialog.close();
        ErrorHandler.showError(d.error().statusText);
      });
      dialog.close();
    }
    /**
     * Validate the fields in the login and make the call to the server
     * to login to the exchange server.
     */


    function validateLogin() {
      // Validate the fields - really just check to see if there
      // is something in the fields.
      _loggingInUser = $('#exchange-user-name').val();
      var password = $('#exchange-password').val();
      var isValid = true; // TODO: There has got to be a nicer way to do this.

      if (!utils.checkValueInBox('exchange-user-name', 'Username is required')) {
        isValid = false;
      }

      if (!utils.checkValueInBox('exchange-password', 'Password is required')) {
        isValid = false;
      }

      if (!isValid) {
        return;
      } // Open a progress spinner.


      _progDialog = new ProgressDialog({
        title: 'Logging on to Mobility Exchange'
      });

      _progDialog.show();

      ServerHandler.getInstance().post({
        type: ModelProtocol.exchange_login,
        username: _loggingInUser,
        password: password,
        serverAddress: _serverAddr
      }, null, function () {
        // Call worked, if there is an error, then it will
        // come from a javascript call from the server.
        _progDialog.close();
      }, function (d) {
        // We didn't connect to server
        _progDialog.close();

        ErrorHandler.showError(d.error().statusText);
      });

      _loginDialog.close();

      _loginDialogOpen = false;
    }
    /**
     * Display a dialog with the current logged in user's information.
     */


    function showUserInfo() {
      BootstrapDialog.show({
        id: 'exchange-user-info-dialog',
        title: 'User Information',
        message: $('<div></div>').load('/mobility_clinic/html/exchange_user_info.html'),
        buttons: [{
          id: 'exchange-user-info-close',
          icon: 'iconic iconic-x',
          label: 'Close',
          cssClass: 'btn-link',
          action: function action(dlg) {
            dlg.close();
          }
        }, {
          id: 'exchange-user-info-change-password',
          icon: 'iconic iconic-key',
          label: 'Change Password',
          cssClass: 'btn-default',
          action: changeUserPassword
        }, {
          id: 'exchange-user-info-logout-btn',
          icon: 'iconic iconic-account-logout',
          label: 'Logout',
          cssClass: 'btn-success',
          action: instance.logout
        }],
        onshown: populateUserFields
      });

      function populateUserFields() {
        $('#exchange-user-name').text(_user.userName);
        $('#exchange-user-first-name').text(_user.firstName);
        $('#exchange-user-last-name').text(_user.lastName);
        $('#exchange-user-assigned-site').text(instance.getSiteName());
        $('#exchange-user-last-modified').text(!!_user.lastModified ? utils.getShortFormattedDate(_user.lastModified) : "");
        $('#exchange-user-last-password-change').text(!!_user.passwordLastModified ? utils.getShortFormattedDate(_user.passwordLastModified) : "");
      }
    }
    /**
     * Show dialog to user explaining why password must be reset.
     */


    function passwordExpired() {
      var msg = '<p>Your password has expired and must be reset ' + 'before login.';
      ErrorHandler.showError(msg, function () {
        changeUserPassword(true);
      });
    }
    /**
     * Open dialog for the changing password for the currently logged
     * in user.
     * @param {Boolean} force If true, user will not be able to exit
     * from the dialog unless password is actually changed.
     */


    function changeUserPassword(force) {
      force = force ? force : false;
      var btns = [{
        id: 'exchange-user-password-change-ok',
        icon: 'iconic iconic-check',
        label: 'OK',
        cssClass: 'btn-success',
        action: validateNewPassword
      }];

      if (force !== true) {
        btns.unshift({
          id: 'exchange-user-password-change-close',
          icon: 'iconic iconic-x',
          label: 'Cancel',
          cssClass: 'btn-link',
          action: function action(dlg) {
            dlg.close();
          }
        });
      }

      BootstrapDialog.show({
        id: 'exchange-user-password-change-dialog',
        title: 'Change Password',
        closable: !force,
        message: $('<div></div>').load('/mobility_clinic/html/exchange_user_change_password.html'),
        buttons: btns,
        onshown: populateGuidance
      });

      function populateGuidance() {
        if (_pswdGuidance !== '') {
          $('#exchange-change-password-container').find('#exchange-password-guidance').html(_pswdGuidance);
        }
      }
    }
    /**
     * Validate the password fields, and if everything looks ok, make the servlet
     * call to change it.
     * @param {BootstrapDialog} dialog The password change dialog.
     */


    function validateNewPassword(dialog) {
      var currentInput = $('#exchange-current-password');
      var newInput = $('#exchange-new-password');
      var confirmInput = $('#exchange-new-password-confirm');
      var valid = true; // Remove existing error classes

      $('#exchange-change-password-container').find('.has-error').removeClass('has-error');
      $('#password-regex-errors').empty();

      if (!currentInput.val() || currentInput.val().length === 0) {
        currentInput.parent().addClass('has-error');
        currentInput.attr('placeholder', 'Must enter current password');
        valid = false;
      }

      if (!newInput.val() || newInput.val().length === 0) {
        newInput.parent().addClass('has-error');
        newInput.attr('placeholder', 'Must enter new password');
        valid = false;
      }

      if (valid === true) {
        if (currentInput.val() === newInput.val()) {
          newInput.val('');
          newInput.parent().addClass('has-error');
          newInput.attr('placeholder', 'New password must be different than current');
          valid = false;
        }
      }

      if (valid === true) {
        if (!confirmInput.val() || confirmInput.val().length === 0) {
          confirmInput.parent().addClass('has-error');
          confirmInput.attr('placeholder', 'Must enter new password');
          valid = false;
        } else if (valid === true && newInput.val() !== confirmInput.val()) {
          newInput.val('');
          newInput.parent().addClass('has-error');
          newInput.attr('placeholder', 'Passwords must match');
          confirmInput.val('');
          confirmInput.parent().addClass('has-error');
          confirmInput.attr('placeholder', 'Passwords must match');
          valid = false;
        }
      }

      if (valid === true) {
        var username = _user.userName;
        var atIndex = username.indexOf("@");

        if (atIndex >= 0) {
          username = username.substring(0, atIndex);
        }

        if (newInput.val().indexOf(username) >= 0) {
          newInput.val('');
          newInput.parent().addClass('has-error');
          newInput.attr('placeholder', 'Invalid password');
          confirmInput.val('');
          confirmInput.parent().addClass('has-error');
          confirmInput.attr('placeholder', 'Invalid password');
          $('#password-regex-errors').append('<p>Password cannot contain your username</p>');
          valid = false;
        }
      }

      if (valid === true) {
        valid = validatePasswordRegEx(newInput.val());
      }

      if (valid === true) {
        dialog.close();
        ServerHandler.getInstance().post({
          type: ModelProtocol.exchange_change_password,
          currentPassword: currentInput.val(),
          newPassword: newInput.val()
        }, null, null, function (d) {
          ErrorHandler.showError(d.error().statusText);
        });
      }
    }
    /**
     * Validate the new password based on the regular expression
     * given by the server.
     * @return {Boolean} True if password passed regular expression.
     */


    function validatePasswordRegEx() {
      var valid = true;
      var newInput = $('#exchange-new-password');
      var confirmInput = $('#exchange-new-password-confirm');
      var newPswd = newInput.val();

      _pswdRegExList.forEach(function (d) {
        var regEx = new RegExp(d.regEx);

        if (!regEx.test(newPswd)) {
          valid = false;
          newInput.val('');
          newInput.parent().addClass('has-error');
          newInput.attr('placeholder', 'Invalid password');
          confirmInput.val('');
          confirmInput.parent().addClass('has-error');
          confirmInput.attr('placeholder', 'Invalid password');
          $('#password-regex-errors').append('<p>' + d.message + '</p>');
        }
      });

      return valid;
    }
    /**
     * Display a connected, but not associated with any subject
     * groups dialog.
     */


    function showNotAssociatedWithStudyDialog() {
      BootstrapDialog.show({
        id: 'exchange-user-not-associated',
        title: 'Not Associated With Any Subject Groups',
        message: $('<div></div>').load('/mobility_clinic/html/exchange_user_not_associated.html'),
        buttons: [{
          id: 'exchange-user-info-close',
          icon: 'iconic iconic-x',
          label: 'Close',
          cssClass: 'btn-link',
          action: function action(dlg) {
            dlg.close();
          }
        }],
        onhide: function onhide() {
          ServerHandler.getInstance().post({
            type: ModelProtocol.exchange_logout
          }, null, null, function (d) {
            ErrorHandler.showError(d.error().statusText);
          });
        }
      });
    }
    /**
     * Handle messages received from the servlet.
     * @param {String} key Key into the ResReqMap
     */


    function handleMessage(key) {
      var message = ResReqMap.getInstance().getResponseData(key); // Split the string on /, only check first part

      var messages = message.split('/');

      if (messages[0] === SCHEMA_MISMATCH) {
        instance.schemaMismatchDuringNewConnectionFailure();
      } else if (messages[0] === FAILURE) {
        instance.loginFailure();
      } else if (messages[0] === UNKNOWN_USERNAME) {
        instance.unknownUsernameFailure();
      } else if (messages[0] === BAD_PASSWORD) {
        instance.badPasswordFailure();
      } else if (messages[0] === OFFLINE_LOGIN_FAILURE) {
        instance.offlineLoginFailure();
      } else if (messages[0] === UNKNOWN_USERNAME_FIRST_CONNECTION) {
        instance.unknownUsernameDuringNewConnectionFailure();
      } else if (messages[0] === BAD_PASSWORD_FIRST_CONNECTION) {
        instance.badPasswordDuringNewConnectionFailure();
      } else if (messages[0] === SUCCESS) {
        instance.loginSuccess();
      } else if (messages[0] === FAILED_FIRST_CONNECTION) {
        instance.newConnectionError();
      } else if (messages[0] === SUCCESS_FIRST_CONNECTION) {
        instance.newConnectionComplete();
      } else if (messages[0] === LOGOUT_SUCCESS) {
        instance.logoutSuccess();
      } else if (messages[0] === LOGOUT_FAILURE) {
        instance.logoutFailure();
      } else if (messages[0] === PASSWORD_CHANGE_SUCCESS) {
        instance.passwordChangeSuccess();
      } else if (messages[0] === PASSWORD_CHANGE_FAILURE) {
        instance.passwordChangeFailure(messages[1]);
      }
    }

    return {
      /**
       * Set whether or not instance is associated with a server.
       */
      associatedWithServer: function associatedWithServer() {
        return hasServer;
      },
      loggedIn: function loggedIn() {
        return _isLoggedIn;
      },
      online: function online() {
        return isOnline;
      },
      updateHost: function updateHost() {
        return _updateHost;
      },

      /**
       * Display the dialog for a new exchange connection.
       */
      setupNewServerConnection: function setupNewServerConnection() {
        // Make sure we don't already have a server.
        if (hasServer) {
          return;
        }

        if (_localDataExists) {
          BootstrapDialog.show({
            title: 'Cannot Connect to Mobility Exchange Server',
            message: '<ul style="list-style-type:circle">' + '<li>You can only connect to a Mobility Exchange' + 'server from a clean workspace with no existing ' + 'data.</li>' + '<li>If you have existing data that you wish to ' + 'upload to the Mobility Exchange server, export ' + 'the data you wish to keep (XML & HDF formats), ' + 'connect to Mobility Exchange from a clean workspace, ' + 'and re-import your data into the new workspace</li>' + '<li>To create a clean workspace, click on File->' + '\"Switch Workspace\" in the menu bar</li></ul>',
            buttons: [{
              icon: 'iconic iconic-x',
              label: 'Cancel',
              cssClass: 'btn-link',
              action: function action(dialog) {
                dialog.close();
              }
            }]
          });
          return;
        }

        var btns = [{
          id: 'exchange-new-connection-connect-btn',
          icon: 'iconic iconic-account-login',
          label: 'Connect',
          cssClass: 'btn-success',
          action: function action(dialog) {
            validateConnectionFields(dialog);
          }
        }]; // Don't allow user to cancel out of exchange
        // connection dialog if update host points to a mobility
        // exchange instance

        if (_updateHost && _updateHost.indexOf('mobilityexchange') <= -1) {
          btns.unshift({
            id: 'exchange-new-connection-close-btn',
            icon: 'iconic iconic-x',
            label: 'Cancel',
            cssClass: 'btn-link',
            action: function action(dialog) {
              dialog.close();
            }
          });
        }

        BootstrapDialog.show({
          id: 'exchange-new-connection-dialog',
          title: 'Connect to Mobility Exchange',
          buttons: btns,
          message: function message(dialog) {
            var msg = $('<div></div>');
            msg.load('/mobility_clinic/html/exchange_new_connection.html', function () {
              msg.on('keypress', function (evt) {
                if (utils.isEnterKey(evt)) {
                  validateConnectionFields(dialog);
                }
              });
            });
            return msg;
          },
          onshown: function onshown() {
            $('#exchange-user-name').val(_loggingInUser);
            $('#exchange-server-address').val(_serverAddr); // Make the server field uneditable if the update host
            // points to a mobility exchange instance

            if (_updateHost && _updateHost.indexOf('mobilityexchange') > -1) {
              $('#exchange-server-address').attr('readonly', true);
            }
          }
        });
      },

      /**
       * Logout from the exchange server.
       * @param {BootstrapDialog} dialog Dialog to close if the
       * user selects logout.
       */
      logout: function logout(userInfoDialog) {
        var title = 'Logout from Mobility Exchange Server';
        var msg = '<p>Are you sure you want to logout user "' + _user.userName + '"?</p>' + '<p>Once logged out, you will be unable to view ' + 'or edit data until you have ' + 'logged back in under a registered user name.</p>';

        if (_webLogin === true) {
          title = 'Logout from Mobility Web';
          msg = '<p>Are you sure you want to logout user "' + _user.userName + '"?</p>';
        }

        utils.log('logging out');
        BootstrapDialog.show({
          title: title,
          message: msg,
          buttons: [{
            id: 'exchange-logout-dialog-cancel-btn',
            icon: 'iconic iconic-x',
            label: 'Cancel',
            cssClass: 'btn-link',
            action: function action(dialog) {
              dialog.close();
            }
          }, {
            id: 'exchange-logout-dialog-logout-btn',
            icon: 'iconic iconic-account-logout',
            label: 'Logout',
            cssClass: 'btn-success',
            action: function action(dialog) {
              metricWarningWasShown = false;
              ServerHandler.getInstance().post({
                type: ModelProtocol.exchange_logout
              }, null, null, function (d) {
                ErrorHandler.showError(d.error().statusText);
              });
              dialog.close();

              if (userInfoDialog) {
                userInfoDialog.close();
              }
            }
          }]
        });
      },

      /**
       * Login to the exchange server.
       */
      login: function login() {
        if (_loginDialogOpen === true) {
          return;
        }

        _loginDialog = new BootstrapDialog({
          id: 'exchange-login-dialog',
          title: 'Login to Mobility Exchange Server',
          closable: false,
          buttons: [{
            icon: 'iconic iconic-account-login',
            label: 'Login',
            id: 'exchange-login-btn',
            cssClass: 'btn-success',
            action: validateLogin
          }],
          onshown: function onshown() {
            $('#exchange-user-name').val(_loggingInUser);
          },
          message: function message() {
            var msg = $('<div></div>');
            msg.load('/mobility_clinic/html/exchange_login.html', function () {
              msg.on('keypress', function (evt) {
                if (utils.isEnterKey(evt)) {
                  validateLogin();
                }
              });
            });
            return msg;
          }
        }).open();
        _loginDialogOpen = true;
      },

      /**
       * Display conformation that new connection has succeded.
       */
      newConnectionComplete: function newConnectionComplete() {
        var msg = '<p>"' + _loggingInUser + '" is connected to the Mobility Exchange server at:</p>' + '<p>' + _serverAddr + '</p>' + '<p>Mobility Lab will refresh with data from the server.</p>';
        BootstrapDialog.show({
          title: 'Connected to exchange server',
          message: msg,
          buttons: [{
            id: 'exchange-new-connection-complete-continue-btn',
            icon: 'iconic iconic-caret-right',
            label: 'Continue',
            cssClass: 'btn-success',
            action: function action(dialog) {
              dialog.close();
            }
          }],
          onhide: function onhide() {
            // Send out a notification to refresh the view.
            ServerHandler.getInstance().post({
              type: ModelProtocol.reset_initial_view
            }, null, null, function (d) {
              ErrorHandler.showError(d.error().statusText);
            });
          }
        });
      },

      /**
       * Display an error message if the new connection to the server fails.
       */
      newConnectionError: function newConnectionError() {
        getExchangeStatus();
        var msg = '<p>There was a problem connecting to the ' + 'Mobility Exchange server.</p>' + '<p>Make sure:</p><ul>' + '<li>You have an established internet connection.</li>' + '<li>The Server field is filled out correctly.</li></ul>';
        ErrorHandler.showError(msg, instance.setupNewServerConnection);
      },

      /**
       * Called with message if a new connection fails due to
       * schema mismatch
       */
      schemaMismatchDuringNewConnectionFailure: function schemaMismatchDuringNewConnectionFailure() {
        var msg = '<p>There was a problem connecting to ' + 'the Mobility Exchange server.</p>' + '<p>Make sure your software is fully up to date.</p>';
        ErrorHandler.showError(msg, instance.setupNewServerConnection);
      },

      /**
       * Called with message if a new connection fails due to
       * bad password. Currently, this case is only applied during
       * the initial MX connection to make the initial process easier.
       */
      badPasswordFailure: function badPasswordFailure() {
        var msg = '<p>There was a problem connecting to ' + 'the Mobility Exchange server.</p>' + '<p>The password is invalid.</p>';
        ErrorHandler.showError(msg, instance.login);
      },

      /**
       * Called with message if a new connection fails due to
       * an unknown username.
       */
      unknownUsernameFailure: function unknownUsernameFailure() {
        var msg = '<p>There was a problem connecting to ' + 'the Mobility Exchange server.</p>' + '<p>The username is unknown.</p>';
        ErrorHandler.showError(msg, instance.login);
      },

      /**
       * Called with message if a new connection fails due to
       * bad password in the context of a first connection to MX
       */
      badPasswordDuringNewConnectionFailure: function badPasswordDuringNewConnectionFailure() {
        var msg = '<p>There was a problem connecting to ' + 'the Mobility Exchange server.</p>' + '<p>The password is invalid.</p>';
        ErrorHandler.showError(msg, instance.setupNewServerConnection);
      },

      /**
       * Called with message if a new connection fails due to
       * an unknown username, in the context of a new connection to MX
       */
      unknownUsernameDuringNewConnectionFailure: function unknownUsernameDuringNewConnectionFailure() {
        var msg = '<p>There was a problem connecting to ' + 'the Mobility Exchange server.</p>' + '<p>The username is unknown.</p>';
        ErrorHandler.showError(msg, instance.setupNewServerConnection);
      },

      /**
       * Called when "login successful" message is received.
       */
      loginSuccess: function loginSuccess() {
        if (_loginDialog) {
          _loginDialog.close();
        }

        _loginDialogOpen = false;

        if (ServerHandler.getInstance().hasJsPostHandler()) {
          // Refresh to load data.
          window.location.reload();
        } else {
          getExchangeStatus();
        }

        crossroads.parse('loggedIntoExchange');
      },

      /**
       * Called when "login failed" message received.
       */
      loginFailure: function loginFailure() {
        var msg = '<p>There was a problem connecting to ' + 'the Mobility Exchange server.</p>' + '<p>Make sure your user name and password are correct.</p>';
        ErrorHandler.showError(msg, instance.login);
      },

      /**
       * Called when attempting to login while offline, but not as the last user
       */
      offlineLoginFailure: function offlineLoginFailure() {
        var msg = '<p>There was a problem connecting to ' + 'the Mobility Exchange server.</p>' + '<p>You can work while offline, but you must log in ' + 'as the previously logged in user.</p>';
        ErrorHandler.showError(msg, instance.login);
      },

      /**
       * Called when SCHEMA_MISMATCH message received from server.
       */
      schemaMismatchFailure: function schemaMismatchFailure() {
        var msg = '<p>There was a problem connecting to ' + 'the Mobility Exchange server.</p>' + '<p>Make sure your software is fully up to date.</p>';
        ErrorHandler.showError(msg, instance.login);
      },

      /**
       * Called when LOGOUT_SUCCESS message received from server.
       * Currently a no-op. Logout triggers a view refresh.
       */
      logoutSuccess: function logoutSuccess() {},

      /**
       * Called when LOGOUT_FAILURE message received from server.
       */
      logoutFailure: function logoutFailure() {
        getExchangeStatus();
        var msg = 'There was a problem logging out of ' + 'the Mobility Exchange server.';
        ErrorHandler.showError(msg);
      },

      /**
       * Show the user information dialog.
       */
      userInfo: function userInfo() {
        showUserInfo();
      },

      /**
       * Called when PASSWORD_CHANGE_SUCCESS message received
       * from server.
       */
      passwordChangeSuccess: function passwordChangeSuccess() {
        BootstrapDialog.show({
          id: 'password-updated-dialog',
          title: 'Password Updated',
          message: 'Password has been successfully updated.',
          buttons: [{
            id: 'password-updated-dialog-close-btn',
            icon: 'iconic iconic-x',
            cssClass: 'btn-link',
            label: 'Close',
            action: function action(dlg) {
              dlg.close();
            }
          }]
        });
      },

      /**
       * Called from within RCP app if password change failed.
       * @param {String} msg Additional message to display to
       * the user.
       * @param {Boolean} force If true, reopen the change
       * password dialog again.
       */
      passwordChangeFailure: function passwordChangeFailure(msg, force) {
        msg = '<p>Password change failed.</p><p>' + msg + '</p>';
        BootstrapDialog.show({
          title: 'Password Change Failed',
          message: msg,
          buttons: [{
            icon: 'iconic iconic-x',
            cssClass: 'btn-link',
            label: 'Close',
            action: function action(dlg) {
              dlg.close();
            }
          }],
          onhide: function onhide() {
            if (force) {
              changeUserPassword(true);
            }
          }
        });
      },

      /**
       * Get/set the "Can get metrics because offline"
       * warning been shown flag.
       * @param {Boolean} warningShown Value to set state of flag.
       * @return Current state of flag if called with no arguments,
       *  otherwise this instance.
       */
      metricWarningShown: function metricWarningShown(warningShown) {
        if (!arguments.length) {
          return metricWarningWasShown;
        }

        metricWarningWasShown = warningShown;
        return instance;
      },

      /**
       * Login to a Mobility Exchange instance through Mobility Web.
       */
      webLogin: function webLogin() {
        if (_loginDialogOpen === true) {
          return;
        }

        _webLogin = true;
        _doingUserTimeout = false;
        _loginDialog = new BootstrapDialog({
          id: 'mobility-web-login-dialog',
          title: 'Login to Mobility Web',
          buttons: [{
            icon: 'iconic iconic-x',
            label: 'Cancel',
            cssClass: 'btn-link',
            action: function action(dialog) {
              dialog.close();
            }
          }, {
            icon: 'iconic iconic-account-login',
            label: 'Login',
            cssClass: 'btn-success',
            action: validateLogin
          }],
          onshown: function onshown() {
            $('#exchange-user-name').val(_loggingInUser);
          },
          message: function message() {
            var msg = $('<div></div>');
            msg.load('/mobility_clinic/html/exchange_login.html', function () {
              msg.on('keypress', function (evt) {
                if (utils.isEnterKey(evt)) {
                  validateLogin();
                }
              });
            });
            return msg;
          }
        }).open();
        _loginDialogOpen = true;
      },

      /**
       * Prompt the user for some interaction, or log them out
       * after 30 seconds.
       */
      userTimeoutLogout: function userTimeoutLogout() {
        if (_doingUserTimeout === true || _loginDialogOpen === true || !_isLoggedIn) {
          return;
        }

        _doingUserTimeout = true; // Start the 30 second timer

        var logoutTimeout = setTimeout(timedOut, 30000);
        var timeoutDialog = new BootstrapDialog({
          title: 'Logging out due to inactivity',
          message: 'You will be logged out in 30 seconds due to ' + 'inactivity. Click "Continue" if you wish to ' + 'remain logged in.',
          buttons: [{
            id: 'timeout-continue-btn',
            icon: 'iconic iconic-check',
            label: 'Continue',
            cssClass: 'btn-success',
            action: function action(dialog) {
              dialog.close();
            }
          }],
          onhide: function onhide() {
            clearTimeout(logoutTimeout); // Since dialog onhide is called on timeout,
            // only need to set flag here and not in
            // timedOut()

            _doingUserTimeout = false;
          }
        });
        timeoutDialog.open();

        function timedOut() {
          if (!_isLoggedIn) {
            return;
          }

          ServerHandler.getInstance().post({
            type: ModelProtocol.upload_trials_silent
          }, null, function () {
            // Wait for 4s before performing the logout to provide time for
            // the upload response to be handled by the client before logging
            // out
            setTimeout(function () {
              ServerHandler.getInstance().post({
                type: ModelProtocol.exchange_logout
              }, null, null, function (d) {
                ErrorHandler.showError(d.error().statusText);
              });
              timeoutDialog.close();
            }, 4000);
          }, function (d) {
            timeoutDialog.close();
            ErrorHandler.showError(d.error().statusText);
          });
        }
      },

      /**
       * Get the site name, if assigned.
       * @return {String} The site name associated with the logged
       * in user, or null if no site assigned.
       */
      getSiteName: function getSiteName() {
        // Get the site for the current study/user.
        var study = ModelDispatcher.getInstance().getCurrentStudy();

        if (study && study.getUsesSites()) {
          var members = study.getStudyMemberships().filter(function (d) {
            if (d.mobilityLabUser.id === _user.id && d.mobilityLabUser.localId === _user.localId) {
              return true;
            }

            return false;
          });

          if (members.length === 1 && members[0].site && members[0].site.name) {
            _siteName = members[0].site.name;
          } else {
            _siteName = 'N/A';
          }
        }

        return _siteName;
      },

      /**
       * Get the number of seconds of inactivity required before
       * a user should be automatically logged out of Mobility
       * Exchange.
       * @return {Number} The number of seconds of inactivity
       * before logout. 0 implies indefinite login.
       */
      getUserTimeout: function getUserTimeout() {
        return _userTimeout;
      },

      /**
       * Returns true if there has been any modification to the
       * local database.
       * @return {Boolean} True if data base has been modified.
       */
      doesLocalDataExist: function doesLocalDataExist() {
        return _localDataExists;
      },

      /**
       * Returns true if the logged in user is associated
       * with any study on the sever.
       * @return {Boolean} True if user is associated with a
       * study on the server.
       */
      associatedWithStudy: function associatedWithStudy() {
        return _associatedWithStudy;
      },

      /**
       * Refresh the exchange status for the current user.
       */
      refreshExchangeStatus: function refreshExchangeStatus() {
        getExchangeStatus();
      },

      /**
       * Get the current Mobility Lab user.
       * @return {Object} Object containing Mobility Lab user
       * fields.
       */
      getCurrentUser: function getCurrentUser() {
        return _user;
      },

      /**
       * Destroy the current the instance, and don't reinitialize.
       */
      destroy: function destroy() {
        if (_messageRoute) {
          crossroads.removeRoute(_messageRoute);
        }

        instance = null;
      }
    };
  }

  return {
    getInstance: function getInstance() {
      if (!instance) {
        instance = init();
      }

      return instance;
    }
  };
});
//# sourceMappingURL=exchange_handler.js.map
