"use strict";

/* Copyright (c) 2017-2019, APDM, Inc.
 *
 * firmware_check.js
 *
 * Module for handling standalone firmware check of hardware.
 */
define(['server_handler', 'model_protocol', 'crossroads', 'progress_dialog', 'error_handler', 'bootstrap-dialog'], function (ServerHandler, ModelProtocol, crossroads, ProgressDialog, ErrorHandler, BootstrapDialog) {
  var _server;

  var _checkSDCard;

  var _pollInterval;

  var _progDialog;

  var _currentState;

  var FIRMWARE_UPDATE_REQUIRED = 'FIRMWARE_UPDATE_REQUIRED';
  var FIRMWARE_CHECK_COMPLETED = 'FIRMWARE_CHECK_COMPLETED';
  var FIRMWARE_CHECK_COMPLETED_END = 'FIRMWARE_CHECK_COMPLETED_END';
  var FIRMWARE_AND_SD_CHECK_END = 'FIRMWARE_AND_SD_CHECK_END';
  var HW_CHECK_FAILED = 'HW_CHECK_FAILED';
  var SENSOR_SD_CLEAR_REQUIRED = 'SENSOR_SD_CLEAR_REQUIRED';
  var CANCEL_CONFIGURATION = 'CANCEL_CONFIGURATION';
  var CALIBRATION_REQUIRED = 'CALIBRATION_REQUIRED';
  /**
   * Check the firmware version, and notify the user
   * if their firmware is out of date.
   * @param {Boolean} showProgressDialog Set to true if we want to display
   * a dialog with spinner shown user we're checking the firmware.
   * @param {Boolean} checkSDCard Set to true to include a check of the
   * SD card.
   */

  function checkFirmware(showProgressDialog, checkSDCard) {
    _progDialog = null;
    _checkSDCard = checkSDCard ? checkSDCard : false;

    if (showProgressDialog === true) {
      _progDialog = new ProgressDialog({
        title: 'Checking Firmware'
      });

      _progDialog.show();
    }

    _server = ServerHandler.getInstance();
    _currentState = '';
    _pollInterval = setInterval(pollStatus, 250);

    _server.post({
      type: ModelProtocol.check_firmware,
      checkSDCard: _checkSDCard
    }, null, null, function (d) {
      if (_progDialog) {
        _progDialog.close();

        _progDialog = null;
      }

      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Poll the server for status.
   * @ param {Object} configStatus The status returned from the server.
   */


  function pollStatus() {
    _server.post({
      type: ModelProtocol.get_configuration_status
    }, 'json', function (d) {
      if (_checkSDCard) {
        checkStateWithSD(d);
      } else {
        checkState(d);
      }
    }, function (d) {
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Check the current configuration state when doing a firmware
   * only check.
   */


  function checkState(status) {
    if (_currentState === status.currentState) {
      return;
    }

    _currentState = status.currentState;

    if (_currentState === FIRMWARE_UPDATE_REQUIRED) {
      firmwareUpdateRequired();
      clearInterval(_pollInterval);
      _pollInterval = null;
    } else if (_currentState === CALIBRATION_REQUIRED) {
      clearInterval(_pollInterval);
      _pollInterval = null;
      calibrationRequired();
    } else if (_currentState === FIRMWARE_CHECK_COMPLETED_END) {
      clearInterval(_pollInterval);
      crossroads.parse('firmwareCheckCompleted/upToDate');
      _pollInterval = null;

      if (_progDialog) {
        _progDialog.close();

        _progDialog = null;
      }
    }
  }
  /**
   * Check the state of the firmware and SD card check.
   */


  function checkStateWithSD(status) {
    if (_currentState === status.currentState) {
      return;
    }

    _currentState = status.currentState;

    if (_currentState === FIRMWARE_UPDATE_REQUIRED) {
      firmwareUpdateRequired();
    } else if (_currentState === CALIBRATION_REQUIRED) {
      clearInterval(_pollInterval);
      _pollInterval = null;
      calibrationRequired();
    } else if (_currentState === FIRMWARE_CHECK_COMPLETED) {
      _progDialog.setTitle('Checking available memory on sensors');

      crossroads.parse('firmwareCheckCompleted/upToDate');
    } else if (_currentState === HW_CHECK_FAILED) {
      if (_progDialog) {
        _progDialog.close();

        _progDialog = null;
      }

      clearInterval(_pollInterval);
      sdCheckFailed(status.messages);
    } else if (_currentState === SENSOR_SD_CLEAR_REQUIRED) {
      sdClearRequired();
      clearInterval(_pollInterval);
      _pollInterval = null;
    } else if (_currentState === FIRMWARE_AND_SD_CHECK_END) {
      clearInterval(_pollInterval);
      _pollInterval = null;

      if (_progDialog) {
        _progDialog.close();

        _progDialog = null;
      }
    }
  }
  /**
   * Inform the user that the firmware needs to be updated,
   * but don't do anything else.
   */


  function firmwareUpdateRequired() {
    if (_progDialog && !_checkSDCard) {
      _progDialog.close();

      _progDialog = null;
    }

    BootstrapDialog.show({
      title: 'Firmware update is required',
      message: '<p>The hardware firmware is out of date and needs to be updated. ' + 'Please connect all hardware and select "Hardware Configuration" to update.</p>' + '<p>Firmware must be updated before any tests may be administered</p>',
      buttons: [{
        icon: 'iconic iconic-x',
        label: 'Close',
        cssClass: 'btn-link',
        action: function action(dialog) {
          dialog.close();
        }
      }],
      onshown: function onshown() {
        ServerHandler.getInstance().post({
          type: ModelProtocol.set_configuration_ui_state,
          configUIState: CANCEL_CONFIGURATION
        }, null, null, function (d) {
          ErrorHandler.showError(d.error().statusText);
        });
        crossroads.parse('firmwareCheckCompleted/needsUpdate');
      }
    });
  }
  /**
   * Display dialog informing the user that the have a bad
   * calibration file on one or more Opals.
   */


  function calibrationRequired() {
    if (_progDialog) {
      _progDialog.close();

      _progDialog = null;
    }

    BootstrapDialog.show({
      title: 'Calibration Required',
      message: '<p>One or more of your Opals has an out of date magnetometer calibration.</p>' + '<p>It is recommended that you recalibrate these Opals prior to use by ' + '<p>clicking on Calibration->"Recalibrate Magenetometers" in the menu bar.</p>' + '<p>Click on "Details" to see the list of Opals requiring recalibration.</p>',
      buttons: [{
        icon: 'iconic iconic-x',
        label: 'Close',
        cssClass: 'btn-link',
        action: function action(dialog) {
          dialog.close();
        }
      }]
    });
  }
  /**
   * Inform the user that the firmware needs to be updated,
   * but don't do anything else.
   */


  function sdClearRequired() {
    if (_progDialog) {
      _progDialog.close();

      _progDialog = null;
    }

    BootstrapDialog.show({
      title: 'Sensor SD card full',
      message: '<p>The SD cards of one or more of the docked sensors ' + 'are 90% full. Please connect all hardware and select ' + '"Hardware Configuration" and "Apply Configuration" to clear.</p>',
      buttons: [{
        icon: 'iconic iconic-x',
        label: 'Close',
        cssClass: 'btn-link',
        action: function action(dialog) {
          dialog.close();
        }
      }],
      onhide: function onhide() {
        ServerHandler.getInstance().post({
          type: ModelProtocol.set_configuration_ui_state,
          configUIState: CANCEL_CONFIGURATION
        }, null, null, function (d) {
          ErrorHandler.showError(d.error().statusText);
        });
        crossroads.parse('firmwareCheckCompleted/needsUpdate');
      }
    });
  }
  /**
   * Dialog to display if the SD card check failed.
   * @param {Array} messages List of messages returned from server.
   */


  function sdCheckFailed(messages) {
    var msg = '<p>Unable to determine Opal SD card usage.</p>';

    if (messages.length) {
      msg = '';
      messages.forEach(function (d) {
        msg = msg + '<p>' + d.type + ': ' + d.message + '</p>';
      });
    }

    BootstrapDialog.show({
      title: 'SD card check failed',
      message: msg,
      buttons: [{
        icon: 'iconic iconic-x',
        label: 'Close',
        cssClass: 'btn-link',
        action: function action(dialog) {
          dialog.close();
        }
      }]
    });
  }

  return {
    checkFirmware: checkFirmware
  };
});
//# sourceMappingURL=firmware_check.js.map
