"use strict";

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * Copyright (c) 2018-2019 APDM, Inc.
 *
 * Create a animation for a trial with joint angles.
 */
define(['jquery', 'd3', 'three', 'model_protocol', 'server_handler', 'error_handler', 'pubsub'], function ($, d3, THREE, ModelProtocol, ServerHandler, ErrorHandler, PubSub) {
  var MIN_HEIGHT = 300;
  var FRAME_STEP_TIME = 0.0078125;

  var JointAngleAnimation =
  /*#__PURE__*/
  function () {
    /**
     * Create a JointAngleStaticAnimation instance.
     * @param {d3.selection} container The container to place the
     * animation in.
     * @param {Trial} trial The trial to with joint angles to animate.
     */
    function JointAngleAnimation(container, trial) {
      _classCallCheck(this, JointAngleAnimation);

      this._trial = trial;
      this._container = container;
      this._canvas = container.select('canvas').node();
      this._renderer = null;
      this._mixer = null;
      this._clock = null;
      this._scene = null;
      this._camera = null;
      this._animationAction = null;
      this._requestAnimationId = null;
      this._timeScale = 1;
      this._paused = true;
    }
    /**
     * Create and play the animation.
     */


    _createClass(JointAngleAnimation, [{
      key: "update",
      value: function update() {
        this.clearScene();
        this.getBVHData();
      }
      /**
       * Get the BVH data from the server.
       */

    }, {
      key: "getBVHData",
      value: function getBVHData() {
        var self = this;
        ServerHandler.getInstance().post({
          type: ModelProtocol.joint_angles_to_bvh,
          analysisFile: this._trial.getAnalysisFileName()
        }, 'json', function (d) {
          self.generateAnimation(d.bvhData);
        }, function (d) {
          ErrorHandler.showError(d.error().statusText);
        });
      }
      /**
       * Generate the animation with the given BVH data.
       * @param {String} bvhData The BVH data as a string.
       */

    }, {
      key: "generateAnimation",
      value: function generateAnimation(bvhData) {
        this._clock = new THREE.Clock();
        var skeletonHelper;
        this.init();
        var loader = new THREE.BVHLoader();
        var result = loader.parse(bvhData);
        skeletonHelper = new THREE.SkeletonHelper(result.skeleton.bones[0]);
        skeletonHelper.skeleton = result.skeleton;
        var boneContainer = new THREE.Group();
        boneContainer.add(result.skeleton.bones[0]);

        this._scene.add(skeletonHelper);

        this._scene.add(boneContainer); // play animation


        this._mixer = new THREE.AnimationMixer(skeletonHelper);
        this._animationAction = this._mixer.clipAction(result.clip);

        this._animationAction.setEffectiveWeight(1.0).play();

        this._mixer.timeScale = 0; // Start paused.

        this.animate();
      }
      /**
       * Initialize the controls, scene, and renderer for the animation.
       */

    }, {
      key: "init",
      value: function init() {
        var controls; // Determine the starting height.

        var boundingRect = this._container.node().getBoundingClientRect();

        var height = 9 * boundingRect.width / 16;

        if (height < MIN_HEIGHT) {
          height = MIN_HEIGHT;
        }

        this._camera = new THREE.PerspectiveCamera(60, boundingRect.width / height, 0.01, 100);

        this._camera.position.set(3, 3, 3);

        controls = new THREE.OrbitControls(this._camera, this._canvas);
        controls.minDistance = 1;
        controls.maxDistance = 40;
        this._scene = new THREE.Scene();
        this._scene.background = new THREE.Color(0xeeeeee);

        this._scene.add(new THREE.GridHelper(5, 10));

        this._renderer = new THREE.WebGLRenderer({
          antialias: true,
          canvas: this._canvas
        });

        this._renderer.setPixelRatio(window.devicePixelRatio);

        this._renderer.setSize(boundingRect.width, height);
      }
      /**
       * Animate the scene and set the frame request function.
       */

    }, {
      key: "animate",
      value: function animate() {
        if (this._renderer === null) {
          return;
        }

        var self = this;

        var delta = this._clock.getDelta();

        if (this._requestAnimationId === null) {
          this._requestAnimationId = window.requestAnimationFrame(function () {
            self.animate();
          });
        }

        if (this._mixer) {
          this._mixer.update(delta);
        }

        this._renderer.render(this._scene, this._camera);

        if (!this._paused) {
          PubSub.publish('animation.time', this._animationAction.time * 1000);
        } // Cancel this if the canvas no longer exists


        if (!document.body.contains(this._canvas)) {
          this.clearScene();
          return;
        }

        this._requestAnimationId = null;
      }
      /**
       * Resize the canvas when window resized. Always try and
       * maintain a 16:9 aspect ratio.
       */

    }, {
      key: "resize",
      value: function resize() {
        var newRect = this._container.node().getBoundingClientRect();

        var height = 9 * newRect.width / 16;

        if (height < MIN_HEIGHT) {
          height = MIN_HEIGHT;
        }

        this._camera.aspect = newRect.width / height;

        this._camera.updateProjectionMatrix();

        this._renderer.setSize(newRect.width, height);
      }
      /**
       * Pause the animation.
       */

    }, {
      key: "pause",
      value: function pause() {
        this._paused = true;
        this._mixer.timeScale = 0;
      }
      /**
       * Play the animation, starting at the given time.
       * @param {Number} time The time to start playing at. If not
       * given, just starts playing.
       */

    }, {
      key: "play",
      value: function play(time) {
        this._paused = false;

        if (time) {
          this._animationAction.time = time / 1000;
        }

        this._mixer.timeScale = this._timeScale;
      }
      /**
       * Advance the animation by one frame. Currently assuming that
       * the frame advance is going to be 1 / 128.
       */

    }, {
      key: "frameAdvance",
      value: function frameAdvance() {
        this.pause();
        this._animationAction.time = this._animationAction.time + FRAME_STEP_TIME;
        PubSub.publish('animation.time', this._animationAction.time * 1000);
      }
      /**
       * Reverse the animation by one frame. Currently assuming that
       * the frame advance is going to be 1 / 128.
       */

    }, {
      key: "frameReverse",
      value: function frameReverse() {
        this.pause();
        this._animationAction.time = this._animationAction.time - FRAME_STEP_TIME;
        PubSub.publish('animation.time', this._animationAction.time * 1000);
      }
      /**
       * Update the current time of the animation.
       * @param {Number} time The time to update the animation to,
       * in milliseconds.
       */

    }, {
      key: "plotTime",
      value: function plotTime(time) {
        this._animationAction.time = time / 1000;
      }
      /**
       * Change the rate of the animation playback. 1 is normal
       * speed.
       * @param {Number} scaleFactor The speed at which animation plays.
       */

    }, {
      key: "playbackRate",
      value: function playbackRate(scaleFactor) {
        this._timeScale = scaleFactor;
        this._mixer.timeScale = this._timeScale;
      }
      /**
       * Clear the scene of the animation DOM element.
       */

    }, {
      key: "clearScene",
      value: function clearScene() {
        if (this._renderer) {
          if (this._requestAnimationId !== null) {
            window.cancelAnimationFrame(this._requestAnimationId);
            this._requestAnimationId = null;
          }

          this._renderer.dispose();

          this._renderer = null;
        }
      }
      /**
       * Get/set the current trial being animated.
       * @param {Trial} newTrial The new trial to plot.
       * @return Returns current trial if called with no arguments,
       * otherwise this instance.
       */

    }, {
      key: "currentTrial",
      value: function currentTrial(newTrial) {
        if (!arguments.length) {
          return this._trial;
        }

        this._trial = newTrial;
        this.update();
        return this;
      }
      /**
       * Get/set the current trial being animated.
       * @param {d3.selection} newContainer The container to put
       * animation in.
       * @return Returns current container if called with no
       * arguments, otherwise this instance.
       */

    }, {
      key: "currentContainer",
      value: function currentContainer(newContainer) {
        if (!arguments.length) {
          return this._container;
        }

        this._container = newContainer;
        this.update();
        return this;
      }
    }, {
      key: "dispose",
      value: function dispose() {
        this.clearScene();
      }
    }]);

    return JointAngleAnimation;
  }();

  return JointAngleAnimation;
});
//# sourceMappingURL=joint_angle_animation.js.map
