"use strict";

/* Copyright (c) 2018 APDM, Inc.
 *
 * local_sequence_editor.js
 *
 * An editor to set the visibility of test sequences for the current
 * local instance. If a sequence is set as visible at the study level,
 * the visibility set here will override it.
 */
define(['jquery', 'd3', 'utils', 'server_handler', 'model_protocol', 'test_definition_handler', 'crossroads', 'error_handler', 'app/test_sequence_builder', 'bootstrap-sortable', 'bootstrap-dialog'], function ($, d3, utils, ServerHandler, ModelProtocol, TestDefinitionHandler, crossroads, ErrorHandler, testSequenceBuilder, sortable, BootstrapDialog) {
  'use strict';

  var _disabled = [];
  var HIDDEN_SEQUENCES = ["CTSIB", "mBESS", "Romberg Quotient", "A. Gaze Stabilization", "B. Visual Motion Sensitivity", "C. Balance with Head Turns", "D. Walking with Head Turns"];
  crossroads.addRoute('testDefinitionsUpdated/{type}', loadSequenceVisibility);
  /**
   * Initialize the test definition handler and set the save
   * handler.
   */

  function getSequences() {
    TestDefinitionHandler.getInstance();
    $('#local-sequence-visibility-save-btn').click(saveVisibility);
  }
  /**
   * Get the sequence visibility from the servlet.
   */


  function loadSequenceVisibility() {
    ServerHandler.getInstance().post({
      type: ModelProtocol.get_local_sequence_visibility
    }, 'json', function (d) {
      _disabled = d.disabledSequences;
      buildSequenceTable();
    }, function (e) {
      ErrorHandler.showError(e.error().statusText);
    });
  }
  /**
   * Build a table (similar to the one on the options page) that
   * allows setting the visibility of the sequences.
   */


  function buildSequenceTable() {
    var testDefHandler = TestDefinitionHandler.getInstance();
    var container = d3.select('#sequence-selection-container');
    var sequences = testDefHandler.getAllSequences();
    var colHeaders = ['Name', 'Visible'];
    var rowData = Object.keys(sequences).filter(function (d) {
      return HIDDEN_SEQUENCES.indexOf(d) < 0;
    });
    var table = container.append('div').append('table').attr('class', 'tablesortable table sortable table-striped').style('width', 0);
    var theadRow = table.append('thead').attr('class', 'reports_head_row').append('tr');
    theadRow.selectAll('th').data(colHeaders).enter().append('th').text(function (d) {
      return d;
    });
    var tbodyRows = table.append('tbody').selectAll('tr').data(rowData).enter().append('tr');
    tbodyRows.selectAll('td').data(function (d) {
      return [d, createSequenceVisibilityCheckbox(d)];
    }).enter().append('td').html(function (d) {
      return d;
    }); // Add the delete, edit, and visibility handlers.

    tbodyRows.each(function (d) {
      var row = d3.select(this);
      row.select('#edit-custom-sequence-' + utils.makeHtmlId(d)).on('click', function () {
        testSequenceBuilder.open({
          pendingTests: testDefHandler.getSequenceDefinitionAsTestList(d),
          completedTests: [],
          sequence: null
        }, {
          caller: 'customSequence',
          isEdit: true,
          canEdit: false,
          sequenceName: d
        });
      });
      var checkbox = row.select('#visible-checkbox');
      checkbox.on('click', function () {
        var checked = $(this).prop('checked');

        if (checked === false) {
          _disabled.push(d);
        } else {
          var idx = _disabled.indexOf(d);

          if (idx > -1) {
            _disabled.splice(idx, 1);
          }
        }
      });
    });
    table.select('th').attr('data-defaultsort', 'asc'); // Make the table sortable.

    $.bootstrapSortable(false);
  }
  /**
   * Create the visibility checkbox for a given test sequence.
   * @param {String} name The name of the sequence.
   * @return {String} The html for the button.
   */


  function createSequenceVisibilityCheckbox(name) {
    var html = '<input id="visible-checkbox" type="checkbox">';

    if (_disabled.indexOf(name) === -1) {
      html = '<input id="visible-checkbox" type="checkbox" checked>';
    }

    return html;
  }
  /**
   * Save the current list of disabled sequences.
   */


  function saveVisibility() {
    ServerHandler.getInstance().post({
      type: ModelProtocol.set_local_sequence_visibility,
      disabledSequences: JSON.stringify({
        disabledSequences: _disabled
      })
    }, null, function () {
      BootstrapDialog.show({
        title: 'Visible Sequences Saved',
        message: 'The sequence selections have been saved.',
        buttons: [{
          label: 'OK',
          action: function action(dialog) {
            dialog.close();
          }
        }]
      });
    }, function (e) {
      ErrorHandler.showError(e.error().statusText);
    });
  }

  return {
    open: getSequences
  };
});
//# sourceMappingURL=local_sequence_editor.js.map
