"use strict";

/* Copyright (c) 2018 APDM, Inc.
 *
 * logged_recording_plot.js
 *
 * Plot for displaying and selecting logged recordings for conversion
 * to HDF.
 */
define(['jquery', 'd3', 'apdm_plot', 'utils', 'moment'], function ($, d3, ApdmPlot, utils, moment) {
  'use strict';
  /**
   * Make LoggedRecordingPlot child of ApdmPlot
   */

  LoggedRecordingPlot.prototype = new ApdmPlot();
  /**
   * Constructor for instance.
   */

  function LoggedRecordingPlot() {
    this._barHeight = 5;
    this._recordings = [];
    this._startDates = [];
    this._endDates = [];
    this._margin = {
      top: 20,
      right: 0,
      left: 100,
      bottom: 100
    };
    this._width = 800; // Need to override the defalt scale times and reset the axes.

    this._xScale = d3.scaleTime();
    this._yScale = d3.scaleOrdinal();
    this._xAxis = d3.axisBottom(this._xScale);
    this._yAxis = d3.axisLeft(this._yScale);
    this._barColorScale = null;
    this._brush = d3.svg.brush();
    this._brushing = false;
    this._dispatcher = null;
  }
  /**
   * Build or update the plot for the recording.
   * @param {d3.selection} selection The container to build the plot
   * in which has the data for the plot attached.
   */


  LoggedRecordingPlot.prototype.update = function (selection) {
    var self = this;
    this._plotData = selection.datum(); // Merge this recording and the overlapping ones

    this._plotData.overlapping.push({
      absolutePath: this._plotData.absolutePath,
      caseIdString: this._plotData.caseIdString,
      duration: this._plotData.duration,
      endDate: this._plotData.endDate,
      label: this._plotData.label,
      startDate: this._plotData.startDate
    });

    this._recordings = this._plotData.overlapping;
    this._recordings = this._recordings.map(function (d) {
      d.keyLabel = d.caseIdString + ': ' + d.label;
      return d;
    }); // Determine the height based on number of recordings.

    this._height = this._barHeight * this._recordings.length + this._margin.top + this._margin.bottom;
    this.setScaleDomains();
    this.setScaleRanges();
    var plotSvg = selection.selectAll('svg').data([self._plotData]);
    var plotG = plotSvg.enter().append('svg').append('g');
    plotG.append('defs').append('clipPath').attr('id', 'clip-box').append('rect'); // Create a container for the time range selection

    selection.selectAll('div').data([self._plotData]).enter().append('div').attr({
      'class': 'logged-recording-time-container',
      'id': function id(d) {
        return 'time-selection-div-' + d.caseIdString;
      }
    }); // Axis and label containers

    plotG.append('g').attr('id', 'bar-container');
    plotG.append('g').attr('id', 'selected-rect-container').attr('class', 'brush');
    plotG.append('g').attr('class', 'x axis');
    plotG.append('g').attr('class', 'y axis');
    plotG.append('text').attr('class', 'x vis_label');
    plotG.append('text').attr('class', 'y vis_label');
    plotG.append('g').attr('class', 'title vis_label');
    plotG = plotSvg.select('g');
    var timeSelectionDiv = selection.select('#time-selection-div-' + this._plotData.caseIdString);
    this.buildAxes(plotG);
    this.determineWidthAndMargins(plotG);
    this.setScaleRanges();
    this.buildAxes(plotG);
    this.createTimeSelection(timeSelectionDiv, plotG);
    plotSvg.attr('width', this._width).attr('height', this._height);
    plotG.attr('transform', 'translate(' + this._margin.left + ',' + this._margin.top + ')');
    plotG.select('#clip-box').select('rect').attr({
      'x': 0,
      'y': 0,
      'width': this._width - this._margin.left - this._margin.right,
      'height': this._height - this._margin.top - this._margin.bottom
    }); // Have to reset the placement of the x axis, since the
    // ApdmPlot base object doesn't handle range bands when
    // finding the bottom of the plot.

    var xAxisOffset = this._height - this._margin.top - this._margin.bottom;
    plotG.select('.x.axis').attr('transform', 'translate(0,' + xAxisOffset + ')');
    plotG.select('#bar-container').selectAll('.recording-bar').data(this._recordings).enter().append('rect').attr('class', 'recording-bar').attr({
      'x': function x(d) {
        return self._xScale(utils.timeStampToDate(d.startDate));
      },
      'y': function y(d) {
        return self._yScale(d.keyLabel);
      },
      'height': self._yScale.rangeBand(),
      'width': function width(d) {
        return self._xScale(utils.timeStampToDate(d.endDate)) - self._xScale(utils.timeStampToDate(d.startDate));
      },
      'fill': function fill(d) {
        return self._barColorScale(d.keyLabel);
      }
    });
    return this;
  };
  /**
   * Set the scale ranges. Note that this overrides the
   * ApdmPlot.setScaleRanges function in the "base" object.
   */


  LoggedRecordingPlot.prototype.setScaleRanges = function () {
    var widthNoMargin = this._width - this._margin.left - this._margin.right;

    this._xScale.range([0, widthNoMargin]);

    var heightNoMargin = this._height - this._margin.top - this._margin.bottom;

    this._yScale.rangeRoundBands([heightNoMargin, 0], 0.1, 0.1);
  };
  /**
   * Set the domains for the plot.
   */


  LoggedRecordingPlot.prototype.setScaleDomains = function () {
    this._startDates = this._recordings.map(function (d) {
      return d.startDate;
    });
    this._endDates = this._recordings.map(function (d) {
      return d.endDate;
    });
    var minMax = d3.extent(this._startDates.concat(this._endDates)).map(function (d) {
      return utils.timeStampToDate(d);
    });

    this._xScale.domain(minMax); // Use the case id and label as the y-axis scale


    var labels = this._recordings.map(function (d) {
      return d.keyLabel;
    });

    this._yScale.domain(labels);
  };
  /**
   * Create the initial conditions for the selection brush which is
   * set to the overlapping period for all the recordings. Create
   * the time selection UI elements, and attach the handlers for
   * when the range is modified.
   * @param {d3.selection} timeDiv The selection to place the time
   * selection UI elemenets in.
   * @param {d3.selection} plotG The continer of the plot.
   */


  LoggedRecordingPlot.prototype.createTimeSelection = function (timeDiv, plotG) {
    // Figure out the part were all the recordings overlap.
    // Use this as the initial selection.
    var start = utils.timeStampToDate(d3.max(this._startDates));
    var end = utils.timeStampToDate(d3.min(this._endDates));
    var self = this;

    this._brush.x(this._xScale);

    this._brush.extent([start, end]);

    plotG.select('.brush').call(self._brush).selectAll('rect').attr('height', this._height - this._margin.top - this._margin.bottom);
    var jqTimeDiv = $(timeDiv[0]);
    jqTimeDiv.load('/mobility_clinic/html/logged_recording_time_selection.html', pageLoaded);

    function pageLoaded() {
      var startMilliseconds = start.getTime();
      var endMilliseconds = end.getTime();
      jqTimeDiv.find('#recording-start-date').val(startMilliseconds);
      jqTimeDiv.find('#recording-end-date').val(endMilliseconds);
      var durationMilliseconds = endMilliseconds - startMilliseconds;
      jqTimeDiv.find('#recording-duration').text(moment.utc(durationMilliseconds).format('HH:mm:ss.SS')); // Pass back the data for the import.

      var importData = {
        recordings: self._recordings,
        startDate: startMilliseconds,
        endDate: endMilliseconds,
        duration: durationMilliseconds
      };

      self._dispatcher.importRecording(importData);
    }
  };
  /**
   * Set the colors used for the bars for recordings from a specific
   * Opal.
   * @param {d3.scaleOrdinal} d The color scale to use for the
   * opals.
   * @return The current color scale or this instance if called with
   * no arguments.
   */


  LoggedRecordingPlot.prototype.barColorScale = function (d) {
    if (!arguments.length) {
      return this._barColorScale;
    }

    this._barColorScale = d;
    return this;
  };
  /**
   * Pass in the dispatcher and an ID, so any listeners
   * won't be overwritten.
   * @param {d3.dispatch} d The dispatch object.
   * @return {d3.dispatch} The current dispatch object, or this if
   * called with no arguments.
   */


  LoggedRecordingPlot.prototype.dispatcher = function (d) {
    if (!arguments.length) {
      return this._dispatcher;
    }

    this._dispatcher = d;
    return this;
  };

  return LoggedRecordingPlot;
});
//# sourceMappingURL=logged_recording_plot.js.map
