"use strict";

/* Copyright (c) 2013-2017 APDM
 *
 * metric_selection_helper.js
 *
 * Module to aid in handling the user selection of metrics to display
 * in "Configure metrics" tab of "Options" window.
 */
define(['jquery', 'd3', 'crossroads', 'res_req_map', 'model_dispatcher', 'bootstrap-dialog', 'test_definition_handler', 'metric_definition_helper', 'error_handler', 'app/user_options_handler', 'app/metric_selection_tabs'], function ($, d3, crossroads, ResReqMap, ModelDispatcher, BootstrapDialog, TestDefinitionHandler, MetricDefinitionHelper, ErrorHandler, UserOptionsHandler, MetricSelectionTabs) {
  'use strict';

  var instance;

  function init() {
    /**
     * The default metrics to fall back on if edits are rejected.
     */
    var _defaultMetrics;
    /**
     * The updated metrics.
     */


    var _updatedMetrics = {};

    var _defHelper = MetricDefinitionHelper.getInstance();
    /**
     * Renmove selections in a group for tests that are not in the study.
     * @param {String} group The selection group to clean.
     */


    function removeUnusedTests(group) {
      var testNames = TestDefinitionHandler.getInstance().getTestNameMap().map(function (d) {
        return d.name;
      });
      var testGroups = Object.keys(_updatedMetrics[group].selections);
      testGroups.forEach(function (d) {
        if (testNames.indexOf(d) < 0) {
          delete _updatedMetrics[group].selections[d];
        }
      });
    }
    /**
     * Get the name out of the input text box and validate it.
     * @param {String} divId The id of the container that holds the
     * selection dropdown.
     * @param {MetricSelectionTabs} selectionTabs The metric selection
     * tabs instance.
     */


    function addNewMetricGroup(divId, selectionTabs) {
      var inputBox = $('#metric-new-group-name');
      var newGroup = inputBox.val(); // Validate the new name

      if (newGroup.length < 1) {
        $('#metric-group-add-input-group').addClass('has-error');
        $('#metric-new-group-name').attr('placeholder', 'Must enter name for group');
        $('#metric-group-add').removeClass('btn-default').addClass('btn-danger');
        return;
      }

      if (addGroup(newGroup, _defHelper.currentSelectionGroup()) === false) {
        $('#metric-group-add-input-group').addClass('has-error');
        $('#metric-new-group-name').attr('placeholder', '"' + newGroup + '" already exists').val('');
        $('#metric-group-add').removeClass('btn-default').addClass('btn-danger');
        return;
      } // Set the new group to current and rebuild the group dropdown


      selectionTabs.selectionGroup(newGroup);
      saveMetricSelections(selectionTabs);
      $('#metric-group-add-input-group').removeClass('has-error');
      $('#metric-new-group-name').attr('placeholder', 'New metric group name').val('');
      $('#metric-group-add').removeClass('btn-danger').addClass('btn-default');
      populateMetricGroupDropdown(d3.select(divId));
    }
    /**
     * Save the metrics selected and refresh the metric sets and definitions.
     * @param {Object} selectionTabs The MetricSelectionTabs instance.
     */


    function saveMetricSelections(selectionTabs) {
      // Update the currently displayed metric group.
      var groupName = selectionTabs.selectionGroup();
      var selected = selectionTabs.getSelected();
      Object.keys(selected).forEach(function (d) {
        updateTestGroup(d, selectionTabs.selectionGroup(), selected[d].metrics);
      });

      _defHelper.selectedMetricGroups(getUpdated());

      _defHelper.currentSelectionGroup(groupName);

      UserOptionsHandler.getInstance().setMetricSelectionGroups(getUpdated());
    }
    /**
     * Get the updated metric set. Also clean out any metric
     * selections for tests that no longer exist.
     * @param The updated metric set.
     */


    function getUpdated() {
      for (var group in _updatedMetrics) {
        removeUnusedTests(group);
      }

      return _updatedMetrics;
    }

    function updateTestGroup(testName, groupName, newMetrics) {
      _updatedMetrics[groupName].selections[testName] = newMetrics;
      return _updatedMetrics;
    }
    /**
     * Add a new metric group to the metric set.
     * @param newGroup Name of the new group.
     * @param oldGroup The group to copy selected metrics from.
     * @return True if new group is successfully created, false if
     * group name already exists.
     */


    function addGroup(newGroup, oldGroup) {
      // Check and make sure the group name does not conflict with
      // existing groups.
      for (var prop in _updatedMetrics) {
        if (_updatedMetrics[prop].displayName === newGroup) {
          return false;
        }
      } // Copy the old group metric selections into the new group


      _updatedMetrics[newGroup] = {};
      _updatedMetrics[newGroup].selections = {};

      for (var test in _updatedMetrics[oldGroup].selections) {
        _updatedMetrics[newGroup].selections[test] = _updatedMetrics[oldGroup].selections[test].slice();
      }

      _updatedMetrics[newGroup].displayName = newGroup;
      return true;
    }
    /**
     * Remove a metric group.
     * @param {String} group The group to remove.
     * @return {Object} The updated metric groups.
     */


    function removeMetricGroup(group) {
      // Use the "delete" operator to remove the object property.
      delete _updatedMetrics[group];
    }
    /**
     * Populate metric group selection dropdown.
     * @param selection D3 selection of the dropdown's parent.
     * @return The "select" element as D3 selection.
     */


    function populateMetricGroupDropdown(selection) {
      // Build the dropdown for the selectable metric groups.
      var dropDown = selection.select("#metric-group-select-dropdown");
      var allGroups = getMetricSelectionGroups();
      var listItems = dropDown.selectAll('option').data(allGroups, function (d) {
        return d.prop;
      });
      listItems.enter().append('option').attr('value', function (d) {
        return d.prop;
      }).text(function (d) {
        return d.name;
      });
      $(dropDown.node()).off('change').change(function () {
        var newGroup = $(this).val();

        _defHelper.currentSelectionGroup(newGroup);

        var key = ResReqMap.getInstance().setRequestData(newGroup);
        crossroads.parse('updateMetricSelectionTabs/' + key);
        $(dropDown.node()).selectpicker('refresh');
      });
      listItems.exit().remove();
      $(dropDown.node()).selectpicker({
        width: 'auto'
      });
      $(dropDown.node()).selectpicker('val', _defHelper.currentSelectionGroup());
      $(dropDown.node()).selectpicker('refresh');
      return dropDown;
    }
    /**
     * Get the metric group names for dropdowns and menus.
     * @return Array of objects containing the group name and display name.
     */


    function getMetricSelectionGroups() {
      var allGroups = [];

      var metricGroups = _defHelper.selectedMetricGroups();

      for (var prop in metricGroups) {
        allGroups.push({
          prop: prop,
          name: metricGroups[prop].displayName
        });
      }

      return allGroups;
    }
    /**
     * Display the info dialog.
     */


    function displayInfoDialog() {
      var dialog = new BootstrapDialog({
        id: 'metric-group-info-dialog',
        title: 'Metric Groups',
        message: $('<div></div>').load('/mobility_clinic/html/metric_group_info.html'),
        buttons: [{
          id: 'metric-group-info-dialog-close-btn',
          icon: 'iconic iconic-x',
          label: 'Close',
          cssClass: 'btn-link',
          action: function action(dialog) {
            dialog.close();
          }
        }]
      });
      dialog.open();
    }

    return {
      /**
       * Add handler to the "add" button and the metric selecting tabs for
       * each metric group.
       * @param divId The the ID of the div element that holds the tabs.
       * @param {Boolean} canEdit Can user edit metric selections
       * @param isDialog Are we building this in a dialog.
       * @param {String} testName Name of test tab to open on.
       */
      buildMetricTabs: function buildMetricTabs(divId, canEdit, isDialog, testName) {
        var self = this;
        var study = ModelDispatcher.getInstance().getCurrentStudy();
        ModelDispatcher.getInstance().checkPermissions('edit', study.getObjId()).then(function (d) {
          // Create and populate the dropdown to select the "other"
          // metric group.
          var tabDiv = d3.select(divId);
          _defaultMetrics = _defHelper.selectedMetricGroups();
          $.extend(_updatedMetrics, _defaultMetrics);
          populateMetricGroupDropdown(tabDiv);
          var selectionTabs = MetricSelectionTabs.create(self, _defHelper.currentSelectionGroup(), divId, function () {
            saveMetricSelections(selectionTabs);
          }, isDialog, testName); // Set handler on "Add" button.

          var addBtn = tabDiv.selectAll('#metric-group-add');
          addBtn.on('click', null);
          addBtn.on('click', function () {
            addNewMetricGroup(divId, selectionTabs);
          }); // Set handler on info button

          tabDiv.select('#metric-group-info-button').on('click', displayInfoDialog);

          if (d.hasPermission === false) {
            $('.has-modify-study').attr('disabled', 'disabled');
          }
        }, function (e) {
          ErrorHandler.showError(e.error().statusText);
        });
      },

      /**
       * Delete the currenly selected metric group, and save.
       * @param divId The id of the container holding the selection
       * dropdown.
       */
      deleteMetricGroup: function deleteMetricGroup(divId) {
        // Don't let user delete "default" group.
        var group = _defHelper.currentSelectionGroup();

        if (group === 'default') {
          ErrorHandler.showError('<h4>Cannot delete "Default" ' + 'metric group.</h4>');
          return;
        }

        var dialog = new BootstrapDialog({
          type: BootstrapDialog.TYPE_DANGER,
          title: 'Deleting metric group: ' + group,
          message: '<h4>Do you want to delete the metric ' + 'selection group "' + group + '"?</h4>',
          buttons: [{
            icon: 'iconic iconic-x',
            label: 'Cancel',
            cssClass: 'btn-link',
            action: function action(dialog) {
              dialog.close();
            }
          }, {
            id: 'metric-group-do-delete-btn',
            icon: 'iconic iconic-trash',
            label: 'Delete',
            cssClass: 'btn-danger',
            action: doGroupDelete
          }]
        });
        dialog.open();

        function doGroupDelete() {
          removeMetricGroup(group); // Set the current metric group to something new.

          var newMetrics = getUpdated();
          var newGroup = Object.keys(newMetrics)[0];

          _defHelper.currentSelectionGroup(newGroup);

          UserOptionsHandler.getInstance().setMetricSelectionGroups(getUpdated());

          _defHelper.selectedMetricGroups(getUpdated());

          populateMetricGroupDropdown(d3.select(divId));
          dialog.close();
        }
      },

      /**
       * Get the currently selected metrics.
       * @return {Object} The currently selected metrics.
       */
      getUpdated: function getUpdated() {
        return _updatedMetrics;
      }
    };
  }

  return {
    getInstance: function getInstance() {
      // Initialize if object has not been created.
      if (!instance) {
        instance = init();
      }

      return instance;
    }
  };
});
//# sourceMappingURL=metric_selection_helper.js.map
