"use strict";

/* Copyright (c) 2016-2018 APDM, Inc.
 *
 * mobilitylab_raw_data_plot.js
 *
 * Interface for MobilityLab to the plotRawSensors module.
 */
define(['jquery', 'd3', 'utils', 'crossroads', 'res_req_map', 'plot_raw_sensors', 'model_dispatcher', 'metric_definition_helper', 'monitor_data', 'metric_utils', 'time_based_controller'], function ($, d3, utils, crossroads, ResReqMap, plotRawSensors, ModelDispatcher, MetricDefinitionHelper, MonitorData, metricUtils, TimeBasedController) {
  'use strict';

  var dataRoute;
  var containerId = '#switchable';
  var htmlPage = 'raw_data_plot.html';
  var trial;
  var httpParams;
  var ANNOTATION_ONE_COLOR = '#ff9626';
  var ANNOTATION_TWO_COLOR = '#1f74a5';
  /**
   * Send the request to get the trial data from the parameters in
   * URL.
   */

  function initPlot() {
    httpParams = utils.parseURLParams(window.location.href); // Initialize crossroads and ModelDispatcher

    ModelDispatcher.getInstance();
    crossroads.ignoreState = true; // Setup logging for hash changes

    crossroads.routed.add(function (request) {
      utils.log('Current hash: ' + request);
    }); // Need to get the trial that raw data is being requested for.

    var trialRoute = crossroads.addRoute('trialRetrieved/{key}', function (key) {
      trial = ResReqMap.getInstance().getResponseData(key);
      crossroads.removeRoute(trialRoute);
      MetricDefinitionHelper.getInstance();
    });
    var definitionRoute = crossroads.addRoute('metricDefinitionsLoaded', function () {
      crossroads.removeRoute(definitionRoute);
      getInitialData(trial);
    });
    crossroads.parse('dispatch/getTrial/' + ResReqMap.getInstance().setRequestData({
      id: httpParams.id,
      localId: httpParams.localId
    }));
  }
  /**
   * Get the raw data for the trial, parse the events, and open up
   * the plot.
   * @param {Trial} trial The trial to plot.
   */


  function getInitialData(trial) {
    $(containerId).load(htmlPage, pageLoaded); // Clean up any thing the TimeBasedController may be
    // referencing

    TimeBasedController.getInstance().dispose();

    function pageLoaded() {
      dataRoute = crossroads.addRoute('rawDataRetrieved/{key}', function (key) {
        crossroads.removeRoute(dataRoute);
        var jsonData = ResReqMap.getInstance().getResponseData(key);
        var monitorData = new MonitorData(jsonData);
        var startTime = monitorData.getFirstSensor().getStartTime() * Math.pow(10, -3);
        setMonitorSelections(monitorData.getMonitorIds()); // Get trial metrics through its 'getMetrics' Promise.

        var events = null;
        trial.getMetrics().then(function (metrics) {
          events = parseEventData(trial, metrics, startTime);
          createEventKey(events);
          plotRawSensors.open(jsonData, dataRequestCall, events, httpParams.video, httpParams.videoMetaData);
          TimeBasedController.getInstance().registerElement(plotRawSensors);
        }, function () {
          plotRawSensors.open(jsonData, dataRequestCall, events, httpParams.video, httpParams.videoMetaData);
          TimeBasedController.getInstance().registerElement(plotRawSensors);
        });
      });
      var requestData = {
        fileName: trial.getDataFileName(),
        options: JSON.stringify({
          displayWidth: $(window).width(),
          sensors: ['Accelerometers', 'Gyroscopes', 'Magnetometers', 'Barometers', 'Orientation']
        })
      };
      crossroads.parse('dispatch/getRawData/' + ResReqMap.getInstance().setRequestData(requestData));
    }
  }
  /**
   * Parse the event data.
   * @param {Trial} trial The trial being plotted.
   * @param {Object} metrics The trial metrics to pull events from.
   * @param {Number} startTime The time to add to the events.
   */


  function parseEventData(trial, metrics, startTime) {
    var defs = MetricDefinitionHelper.getInstance().getMetricDefinitions(trial.getTestName());
    var eventKeys = defs.keys().filter(function (d) {
      return defs.get(d).metricClass === 'Events' && defs.get(d).promoteToUI === true;
    });
    var plotEvents = [];
    eventKeys.forEach(function (d) {
      var def = defs.get(d);
      plotEvents.push({
        name: def.displayName,
        color: def.eventColor,
        events: metricUtils.getEventStartEndTimes(trial.getTestName(), d, metrics, startTime)
      });
    }); // Add any annotations is they exist. Use some wack colors

    var annotations = trial.getAnnotations();
    Object.keys(annotations).forEach(function (d, i) {
      var times = annotations[d].startTimes.map(function (g) {
        return {
          start: g * 1000 + startTime,
          end: g * 1000 + startTime,
          opacity: 1.0
        };
      });
      plotEvents.push({
        name: annotations[d].annotationLabel,
        color: i === 0 ? ANNOTATION_ONE_COLOR : ANNOTATION_TWO_COLOR,
        events: times
      });
    });
    return plotEvents;
  }
  /**
   * Function passed on to plotRawSensors to handle the
   * requests to the servlet for new data.
   * @param {MonitorData} monitorData The monitor data object being plotted.
   * @param {Array} sensors The list of sensors to retrieve data for.
   * @param {Object} indexes The start and end indexes of the
   * data required.
   * @param {Number} length The number of samples we want within
   * the indexed range.
   * @param {Function} callback The function to call if the request
   * succeeds.
   */


  function dataRequestCall(monitorData, sensors, indexes, length, callback) {
    dataRoute = crossroads.addRoute('rawDataRetrieved/{key}', function (key) {
      crossroads.removeRoute(dataRoute);
      var monitorData = ResReqMap.getInstance().getResponseData(key);
      callback(monitorData);
    });
    var requestData = {
      fileName: monitorData.getDataFileName(),
      options: JSON.stringify({
        displayWidth: length,
        sensors: sensors,
        startRow: indexes.startIdx,
        endRow: indexes.endIdx
      })
    };
    crossroads.parse('dispatch/getRawData/' + ResReqMap.getInstance().setRequestData(requestData));
  }
  /**
   * Populate the "Select Monitor" drop down with the available
   * monitors.
   * @param {Object} monitorIds The id to label map.
   */


  function setMonitorSelections(monitorIds) {
    buildDropdown('#primary_monitor_select', 'Select Sensor: ', monitorIds, plotRawSensors.setPrimaryMonitor);

    if (monitorIds.length <= 1) {
      d3.select('#selection-container').attr('class', 'col-lg-3 col-md-3 col-sm-3');
      d3.select('#sensor-selection').attr('class', 'col-lg-4 col-md-5 col-sm-5');
      d3.select('#primary_monitor_select').attr('class', 'col-md-12');
      d3.select('#secondary_monitor_select').remove();
      return;
    } // Populate the secondary dropdown.


    var monitors = [];
    monitors.push({
      monitorLabel: 'None',
      caseId: 'None'
    });
    monitorIds.forEach(function (d) {
      monitors.push(d);
    });
    buildDropdown('#secondary_monitor_select', 'Compare to: ', monitors, plotRawSensors.setSecondaryMonitor);
  }
  /**
   * Build a monitor selection dropdown.
   * @param {String} id The id of dropdown container element.
   * @param {String} text Text to display next to dropdown.
   * @param {Array} monitors Array of monitor labels and case ids.
   * @param {Function} onChange Handler to attach to dropdown.
   */


  function buildDropdown(id, text, monitors, onChange) {
    var select = d3.select(id).text(text).append('select').attr('class', 'selectpicker').attr('data-width', 'auto');
    select.selectAll('option').data(monitors).enter().append('option').attr('value', function (d) {
      return d.caseId;
    }).text(function (d) {
      if (d.caseId === 'None') {
        return d.caseId;
      } else {
        return '#' + d.caseId.match(/[1-9][0-9]*$/) + ': ' + d.monitorLabel;
      }
    });
    $(select.node()).selectpicker('refresh');
    $(select.node()).change(function () {
      onChange.apply(this);
    });
  }
  /**
   * Build a key for the events.
   * @param {Array} events The events that are being plotted.
   */


  function createEventKey(events) {
    if (!events || !events.length) {
      return;
    }

    var totalWidth = 0;
    var svg = d3.select('.event_key').append('svg').attr('width', 300).attr('height', 14 * events.length);
    svg.selectAll('g').data(events).enter().append('g').each(function (d, i) {
      var g = d3.select(this);
      g.append('rect').attr('x', 0).attr('y', 0).attr('width', 30).attr('height', 10).style('fill', d.color);
      var text = g.append('text').text(d.name).attr('x', 35).attr('y', 10);
      g.attr('transform', 'translate(0, ' + i * 14 + ')');
      totalWidth = totalWidth + text.node().getBBox().width + 45;
    });
    svg.attr('width', totalWidth);
  }

  return {
    open: initPlot
  };
});
//# sourceMappingURL=mobilitylab_raw_data_plot.js.map
