"use strict";

/* Copyright (c) 2014-2017 APDM
 *
 * report_table_handler.js
 *
 * Module for handling the generation and updating of the metric
 * table.
 *
 */
define(['jquery', 'd3', 'utils', 'app/report_enums', 'app/metric_overview_vis', 'es6-promise'], function ($, d3, utils, reportEnums, MetricOverviewVis, Promise) {
  function ReportTableHandler() {
    var tableDiv;
    var currentTrial;
    var compareTrials;
    var baselineTrial = null;
    var tableType = reportEnums.reportType.table.METRICS_OVERVIEW;
    var trialsOfType;
    var metricName;
    var dispatch = null;
    /**
     * Display the overview vis for the current trial.
     */

    function buildOverviewVis() {
      // Determine base line metrics.
      if (!baselineTrial) {
        baselineTrial = trialsOfType[0];

        if (compareTrials && compareTrials !== null && compareTrials.length > 1) {
          baselineTrial = compareTrials[0];
        }
      }

      var vis = new MetricOverviewVis(tableDiv, currentTrial);
      vis.getSetDispatch(dispatch);
      vis.getSetBaselineTrial(baselineTrial);
      vis.getSetSelectedMetric(metricName);

      if (tableType === reportEnums.reportType.table.METRICS_DETAIL) {
        vis.getSetIsTable(true);
        vis.getSetShowStdDev(false);
      } else if (tableType === reportEnums.reportType.table.METRICS_OVERVIEW) {
        vis.getSetIsTable(false);
        vis.getSetShowStdDev(false);
      } else {
        vis.getSetIsTable(false);
        vis.getSetShowStdDev(true);
      }

      vis.update();
    }
    /**
     * Display just the Romberg Quotient metric.
     */


    function displayRombergQuotient() {
      var openArea;
      var closedArea;
      var quotient = '--';
      currentTrial.getAllTrials().reduce(function (sequence, trial) {
        return sequence.then(function () {
          return trial.getMetric();
        }).then(function (metrics) {
          if (trial.getConditionName() === 'Feet Apart, Eyes Open, Firm Surface') {
            if (metrics['Postural Sway/Acc/95% Ellipse Sway Area']) {
              openArea = metrics['Postural Sway/Acc/95% Ellipse Sway Area'].mean;
            }
          } else {
            if (metrics['Postural Sway/Acc/95% Ellipse Sway Area']) {
              closedArea = metrics['Postural Sway/Acc/95% Ellipse Sway Area'].mean;
            }
          }
        });
      }, Promise.resolve()).then(function () {
        if (openArea && closedArea) {
          quotient = openArea / closedArea;
        }

        tableDiv.append('h3').text('Romberg Quotient: ' + quotient.toFixed(1));
      });
    }
    /**
     * Display combined scores for the CTSIB
     */


    function displayCTSIBCombinedScores() {
      var eyesClosedFirm;
      var eyesClosedFoam;
      var eyesOpenFoam;
      var eyesOpenFirm;
      var vision = '--';
      var somatosensory = '--';
      var vestibular = '--';
      currentTrial.getAllTrials().reduce(function (sequence, trial) {
        return sequence.then(function () {
          return trial.getMetrics();
        }).then(function (metrics) {
          if (trial.getConditionName() === 'Feet Apart, Eyes Open, Firm Surface' && metrics['Postural Sway/Acc/95% Ellipse Sway Area']) {
            eyesOpenFirm = metrics['Postural Sway/Acc/95% Ellipse Sway Area'].mean;
          } else if (trial.getConditionName() === 'Feet Apart, Eyes Open, Foam Surface' && metrics['Postural Sway/Acc/95% Ellipse Sway Area']) {
            eyesOpenFoam = metrics['Postural Sway/Acc/95% Ellipse Sway Area'].mean;
          } else if (trial.getConditionName() === 'Feet Apart, Eyes Closed, Firm Surface' && metrics['Postural Sway/Acc/95% Ellipse Sway Area']) {
            eyesClosedFirm = metrics['Postural Sway/Acc/95% Ellipse Sway Area'].mean;
          } else if (trial.getConditionName() === 'Feet Apart, Eyes Closed, Foam Surface' && metrics['Postural Sway/Acc/95% Ellipse Sway Area']) {
            eyesClosedFoam = metrics['Postural Sway/Acc/95% Ellipse Sway Area'].mean;
          }
        });
      }, Promise.resolve()).then(function () {
        if (eyesOpenFirm) {
          if (eyesClosedFirm) {
            vision = (eyesClosedFirm / eyesOpenFirm).toFixed(1);
          }

          if (eyesOpenFoam) {
            somatosensory = (eyesOpenFoam / eyesOpenFirm).toFixed(1);
          }

          if (eyesClosedFoam) {
            vestibular = (eyesClosedFoam / eyesOpenFirm).toFixed(1);
          }
        }

        tableDiv.append('h4').text('Vision: ' + vision + ', Somatosensory: ' + somatosensory + ', Vestibular: ' + vestibular).style({
          'width': '90%',
          'margin': '0 auto'
        });
      });
    } // Public functions
    // Getter/setters


    this.tableDiv = function (d) {
      if (!arguments.length) {
        return tableDiv;
      }

      tableDiv = d;
      return this;
    };

    this.currentTrial = function (d) {
      if (!arguments.length) {
        return currentTrial;
      }

      currentTrial = d;
      return this;
    };

    this.compareTrials = function (d) {
      if (!arguments.length) {
        return compareTrials;
      }

      compareTrials = d;
      return this;
    };

    this.baselineTrial = function (d) {
      if (!arguments.length) {
        return baselineTrial;
      }

      baselineTrial = d;
      return this;
    };

    this.tableType = function (d) {
      if (!arguments.length) {
        return tableType;
      }

      tableType = d;
      return this;
    };

    this.metricName = function (d) {
      if (!arguments.length) {
        return metricName;
      }

      metricName = d;
      return this;
    };

    this.trialsOfType = function (d) {
      if (!arguments.length) {
        return trialsOfType;
      }

      trialsOfType = d; // Sort trials here just to make sure.

      trialsOfType.sort(function (a, b) {
        return d3.ascending(a.dateObj, b.dateObj);
      });
      return this;
    };
    /**
     * Get set the dispatch object.
     */


    this.dispatch = function (d) {
      if (!arguments.length) {
        return dispatch;
      }

      dispatch = d;
      return this;
    };

    this.update = function () {
      // Remove the existing table
      $(tableDiv.node()).empty(); // Determine the type of table to render based on test type.

      if (tableType !== reportEnums.reportType.table.NONE) {
        // Can't do the overview visualization for the multi-sway tests.
        if (utils.isMultiSwayTest(currentTrial)) {
          if (currentTrial.getTestName() === 'Romberg Quotient') {
            displayRombergQuotient();
          } else if (currentTrial.getTestName() === 'CTSIB') {
            displayCTSIBCombinedScores();
          }

          return this;
        }

        buildOverviewVis();
      }

      return this;
    };
  }

  return ReportTableHandler;
});
//# sourceMappingURL=report_table_handler.js.map
