"use strict";

/* Copyright (c) 2018 APDM, Inc.
 *
 * report_utils.js
 *
 * Utility functions used in reports.
 */
define(['d3', 'session', 'app/report_enums'], function (d3, Session, reportEnums) {
  'use strict';
  /**
   * Get the trials that match the filter type.
   * @param {Trial} trial The base trial to filter against.
   * @param {Subject} subject The current subject.
   * @param {reportEnum} filterType The filter to apply to trials.
   * @return {Array} The list of trials that match the filter.
   */

  function getFilteredTrials(trial, subject, filterType) {
    var compareTrials = [];

    switch (filterType) {
      case reportEnums.filterType.ALL:
        compareTrials = subject.getAllTrialsOfTestType(trial.getTestName());
        break;

      case reportEnums.filterType.CONDITION:
        compareTrials = subject.getAllTrialsOfTestType(trial.getTestName()).filter(function (d) {
          return trial.getConditionName() === d.getConditionName();
        });
        break;

      case reportEnums.filterType.SEQUENCE:
        if (trial instanceof Session) {
          compareTrials = trial.getAllTrials().filter(function (d) {
            return d.getTestName() === trial.getTestName();
          });
        } else {
          compareTrials = trial.getSequence().getAllTrials().filter(function (d) {
            return d.getTestName() === trial.getTestName();
          });
        }

        break;

      case reportEnums.filterType.DAY:
        var dayStr = trial.getDateString().split('-')[0];
        var trialsForDay = [];
        subject.groupTrialsAndSequencesByDay()[dayStr].trials.forEach(function (d) {
          if (d instanceof Session) {
            trialsForDay = trialsForDay.concat(d.getAllTrials());
          } else {
            trialsForDay.push(d);
          }
        });
        compareTrials = trialsForDay.filter(function (d) {
          return d.getTestName() === trial.getTestName();
        });
        break;
    }

    return compareTrials;
  }
  /**
   * Get the previous or next trial in the list.
   * @param {Subject} subject The current subject.
   * @param {Trial} trial The current trial.
   * @param {Array} trialList The list of trials.
   * @param {Boolean} isPrev If true, return trial previous to the
   * one given, otherwise the next one.
   * @return {Trial} The previous/next trial.
   */


  function getPreviousOrNextTrial(subject, trial, trialList, isPrev) {
    // If this is a Session, then get the other tests that match,
    // don't use the compare trials set (or if compare set is
    // empty).
    if (trial instanceof Session || !trialList || !trialList.length) {
      trialList = subject.getTrialsAndSequencesByTestName().get(trial.getTestName());
    } // Sort the trial list


    trialList.sort(function (a, b) {
      return d3.ascending(a.getDateObj(), b.getDateObj());
    });
    var newTrial = trial;
    var i = 0; // Find the current trial.

    for (; trialList.length > i; i++) {
      if (trialList[i].getDateString() === trial.getDateString()) {
        break;
      }
    }

    if (isPrev === true) {
      if (0 <= i - 1) {
        newTrial = trialList[i - 1];
      } else {
        newTrial = trialList[trialList.length - 1];
      }
    } else {
      if (trialList.length > i + 1) {
        newTrial = trialList[i + 1];
      } else {
        newTrial = trialList[0];
      }
    }

    return newTrial;
  }

  return {
    getFilteredTrials: getFilteredTrials,
    getPreviousOrNextTrial: getPreviousOrNextTrial
  };
});
//# sourceMappingURL=report_utils.js.map
