"use strict";

/* Copyright (c) 2014-2018 APDM, Inc.
 *
 * report_vis_handler.js
 *
 */
define(['jquery', 'd3', 'utils', 'stabilogram', 'longitudinal_plot', 'bar_plot', 'trial_event_plot', 'app/variability_plot_container', 'trial', 'server_handler', 'session', 'metric_definition_helper', 'model_protocol', 'app/report_enums', 'app/metric_vis_map', 'es6-promise'], function ($, d3, utils, Stabilogram, LongitudinalPlot, BarPlot, TrialEventPlot, variabilityPlotContainer, Trial, ServerHandler, Session, MetricDefinitionHelper, ModelProtocol, reportEnums, metricVisMap, Promise) {
  'use strict';

  function ReportVisHandler() {
    var visDiv;
    var visWidth = 720;
    var visHeight = 700;
    var visType;
    var currentTrial;
    var currentMetrics;
    var compareTrials;
    var baselineTrial = null;
    var trialsOfType;
    var metricName;
    var metricTimeArrays = {};
    var metricSet;
    var stabilogramMatrix = null;
    var dispatch = null;
    var previousType = null;
    var DEFAULT_ENTRY_COLOR = '#4ac4d4'; // Private functions

    /**
     * Update the visualization.
     */

    function doUpdate() {
      // No vis - just empty the node and remove any height/width stuff.
      if (visType === reportEnums.reportType.vis.NONE) {
        $(visDiv.node()).empty();
        visDiv.style('min-height', null).style('width', null).style('height', null);
        return;
      } // visWidth = $(visDiv.node()).width();
      // Detemine the vis type on the test type and given report vis type.


      if (currentTrial.getTestName() === 'Sway') {
        if (visType === reportEnums.reportType.vis.METRIC_BAR_COMPARISON) {
          visDiv.style('min-height', null).style('width', visWidth + 'px').style('height', null);
          buildBarPlot();
        } else if (visType === reportEnums.reportType.vis.METRIC_LONGITUDINAL) {
          visDiv.style('min-height', null).style('width', visWidth + 'px').style('height', null);
          buildMetricLongitudinalPlot();
        } else {
          buildSwayVisualization();
        }

        return;
      }

      if (visType === reportEnums.reportType.vis.SEQUENCE_VIS) {
        $(visDiv.node()).empty();
        buildSwayVisualization();
        return;
      }

      if (visType === reportEnums.reportType.vis.JOINT_ANGLE_VARIABILITY) {
        variabilityPlotContainer.open(currentTrial, visDiv, visWidth, metricName);
        return;
      } // Make sure we return to the correct height.


      visDiv.style('min-height', null).style('width', visWidth + 'px').style('height', null);

      if (visType === reportEnums.reportType.vis.METRIC_LONGITUDINAL) {
        $(visDiv.node()).empty();
        buildMetricLongitudinalPlot();
      } else if (visType === reportEnums.reportType.vis.METRIC_BAR_COMPARISON) {
        buildBarPlot();
      } else {
        determineMetricVis();
      }
    }
    /**
     * Build the correct set of stabilograms using the current test
     * to determine which to build.
     */


    function buildSwayVisualization() {
      // Need to determine if this is a single stabilogram for a
      // sway or a multi-sway trial.
      var plotWidth = 400;

      if (utils.isMultiSwayTest(currentTrial)) {
        var trials = currentTrial.getAllTrials();

        if (trials.length > 1) {
          plotWidth = visWidth / 2;
        }

        var plotContainer = visDiv.append('div').attr('class', 'container-fluid').append('div').attr('class', 'row');
        var range = getMaxFromNormalEllipses(trials);

        for (var i = 0; trials.length > i; i++) {
          var plotDiv = plotContainer.append('div').attr('id', trials[i].getDateString()).attr('class', 'col-md-6 col-sm-6'); // Build the stabilogram with a default range.

          buildEachStabilogram(trials[i], plotDiv, plotWidth, range);
        }
      } else {
        visDiv.style('height', '400px');
        buildEachStabilogram(currentTrial, visDiv, plotWidth);
      }
    }
    /**
     * Build the individual stabilogram plots inside the given div
     * with data from the given trial.
     * @param trial The trial to use to generate the plot from.
     * @param plotDiv The contianer to build the plot in.
     * @param plotWidth The width of the plot.
     * @param range The range of for the stabilogram (optional)
     */


    function buildEachStabilogram(trial, plotDiv, plotWidth, range) {
      ServerHandler.getInstance().post({
        requestType: 'mobilityclinic',
        type: ModelProtocol.get_stabilogram_data,
        analysisFile: trial.getAnalysisFileName()
      }, 'json', function (d) {
        plotDiv.selectAll('h4').remove();
        buildStabilogram(plotDiv, plotWidth, d.stabilogramMatrix, trial, range);
      }, function (d) {
        $(plotDiv.node()).empty();
        plotDiv.append('h4').attr('class', 'vis_error_msg text-danger').text(d.error().statusText);
      });
    }
    /**
     * Build the stabilogram in the given selection using the data in the
     * matrix.
     * @param selection The D3 selection to build the stabilogram in.
     * @param matrix Matrix of stabilogram data.
     * @param trial The trial being plotted.
     * @param range The range of the stabilogram (optional).
     */


    function buildStabilogram(selection, plotWidth, matrix, trial, range) {
      // Put the data into a format with [ {x1,y1}, {x2,y2}, ...]
      var points = [];

      if (matrix.length > 0) {
        for (var i = 0; i < matrix[0].length; i++) {
          points.push({
            x: matrix[0][i],
            y: matrix[1][i]
          });
        }
      }

      var plot = new Stabilogram();
      plot.margin({
        top: 0,
        bottom: 0,
        left: 0,
        right: 0
      }).width(plotWidth).height(plotWidth).showAxes(true).xAxisLabel('Coronal').yAxisLabel('Sagittal').title(trial.getConditionName()); // Get the values for the 95% confidence ellipse from
      // the metric promise and then update the plot.

      var confEllipse = {
        a: 0,
        b: 0,
        beta: 0
      };
      trial.getMetrics().then(function (metrics) {
        if (metrics['Postural Sway/Angles/95% Ellipse Axis 1 Radius']) {
          confEllipse.b = metrics['Postural Sway/Angles/95% Ellipse Axis 1 Radius'].mean;
        }

        if (metrics['Postural Sway/Angles/95% Ellipse Axis 2 Radius']) {
          confEllipse.a = metrics['Postural Sway/Angles/95% Ellipse Axis 2 Radius'].mean;
        }

        if (metrics['Postural Sway/Angles/95% Ellipse Rotation']) {
          confEllipse.beta = metrics['Postural Sway/Angles/95% Ellipse Rotation'].mean;
        }

        plot.normalEllipse(getNormalEllipse([trial])).confidenceEllipse(confEllipse).scaleDynamically(!utils.isMultiSwayTest(currentTrial));

        if (range && range !== null) {
          plot.range(range);
        }

        plot.update(selection.datum(points));
      }, function () {
        plot.update(selection.datum(points));
      });
    }
    /**
     * Get the normal ellipse for trials of a given type. If there are
     * mixed conditions in the array of trials, will return a normal ellipse
     * with a = b = 0.
     * @param trials Array of trials.
     * @return Object containing ellipse values for normal.
     */


    function getNormalEllipse(trials) {
      var ellipse = {
        a: 0,
        b: 0,
        beta: 0
      };
      var condition = trials[0].getConditionName();

      for (var i = 1; trials.length > i; i++) {
        if (trials[i].getConditionName() !== condition) {
          return ellipse;
        }
      }

      var testNorms = MetricDefinitionHelper.getInstance().getNormsForTest(trials[0].getTestName(), condition);
      var keys = Object.keys(testNorms);

      for (i = 0; keys.length > i; i++) {
        if (testNorms[keys[i]].metricFullName === 'Postural Sway/Angles/95% Ellipse Axis 1 Radius') {
          ellipse.b = testNorms[keys[i]].meanNorm;
        } else if (testNorms[keys[i]].metricFullName === 'Postural Sway/Angles/95% Ellipse Axis 2 Radius') {
          ellipse.a = testNorms[keys[i]].meanNorm;
        } else if (testNorms[keys[i]].metricFullName === 'Postural Sway/Angles/95% Ellipse Rotation') {
          ellipse.beta = testNorms[keys[i]].meanNorm;
        }
      }

      return ellipse;
    }
    /**
     * Get the max range from a list sway trials.
     * @param {Array} trials The list of sway trials.
     * @return {Array} Min and max range.
     */


    function getMaxFromNormalEllipses(trials) {
      var ellipses = trials.map(function (d) {
        return getNormalEllipse([d]);
      });
      var ranges = [];
      ellipses.map(function (d) {
        ranges.push(Math.abs(d.a * Math.sin(d.beta)));
        ranges.push(Math.abs(d.b * Math.sin(d.beta)));
      });
      var max = d3.max(ranges);
      return {
        min: -max - max * 0.1,
        max: max + max * 0.1
      };
    }
    /**
     * Build longitudinal plot to display values of a metric for a
     * single trial.
     */


    function buildMetricLongitudinalPlot() {
      var plotData = getMetricLongitudinalPlotData();
      visDiv.selectAll('.longitudinal-plot-div').data([plotData]).enter().append('div').attr('class', 'longitudinal-plot-div');
      var plot = new LongitudinalPlot().width(visWidth).height(260).x(function (d) {
        return d.time;
      }).y(function (d) {
        return d.value;
      }).xAxisLabel('Time (s)').yAxisLabel(metricSet[metricName].units).isTimeSeries(false).title(metricSet[metricName].metricGroupDisplayName + ' - ' + metricSet[metricName].displayName).trendLine(true).includeKey(true).isScalar(plotData.scalar);
      var normVals = MetricDefinitionHelper.getInstance().getMeanStdDevAndNorms(currentTrial.getTestName(), currentTrial.getConditionName(), metricSet[metricName]);

      if (normVals && normVals !== null && normVals.norms && normVals.norm !== null && normVals.norms.length > 1) {
        plot.normRange(normVals.fifthToNinteyFifth);
      }

      if (plotData.type === 'BILATERAL') {
        var primary = 'Left';
        var secondary = 'Right';
        plot.plotSecondaryData(true).primaryLabel(primary).secondaryLabel(secondary);
      } // Include turn events if this is a "Walk" trial.


      if (!plotData.scalar && currentMetrics && (currentTrial.getTestName() === 'Walk' || currentTrial.getTestName() === 'SAW' || currentTrial.getTestName() === 'MLK Walk') && metricName.search('Turns') < 0 && currentMetrics['Turns/Start'] && currentMetrics['Turns/Duration']) {
        var turnStarts = currentMetrics['Turns/Start'].values;
        var turnDurations = currentMetrics['Turns/Duration'].values;
        var events = [];

        if (turnStarts && turnDurations) {
          for (var i = 0; turnStarts.length > i; i++) {
            events.push({
              start: turnStarts[i],
              duration: turnDurations[i]
            });
          }
        }

        plot.events(events).eventClass('Turns').eventLabel('Turns');
      }

      plot.update(visDiv.select('.longitudinal-plot-div').datum(plotData.values));
    }
    /**
     * Get the data for metric longitudinal plot. If metric is
     * bilateral, then get both left and values.
     * @return Array of data to plot.
     */


    function getMetricLongitudinalPlotData() {
      var def = MetricDefinitionHelper.getInstance().getTestMetrics(currentTrial.getTestName())[metricName];
      var plotData = {
        type: def.metricType,
        baseName: metricName,
        scalar: false,
        values: []
      };

      if (!currentMetrics) {
        return plotData;
      }

      if (def.metricCardinality === 'ONE') {
        plotData.values = getScalarLongitudinalPlotData(def);
        plotData.scalar = true;
        return plotData;
      }

      var primaryTimingData = def.timingData;

      if (plotData.type === 'BILATERAL') {
        var primary = metricName + ' L';
        var secondary = metricName + ' R';
        primaryTimingData = def.timingData + ' L';
        var secondaryTimingData = def.timingData + ' R';

        if (metricName.match(/R$/)) {
          primary = secondary;
          primaryTimingData = secondaryTimingData;
          secondary = metricName + ' L';
          secondaryTimingData = def.timingData + ' L';
        }

        plotData.values = [processMetric(currentMetrics[primary], currentMetrics[primaryTimingData]), processMetric(currentMetrics[secondary], currentMetrics[secondaryTimingData])];
      } else {
        plotData.values = processMetric(currentMetrics[metricName], currentMetrics[primaryTimingData]);
      }

      return plotData;

      function processMetric(metric, timing) {
        if (!metric || !metric.values || !timing || !timing.values) {
          return null;
        }

        var values = metric.values;
        var timingVals = timing.values;
        var metricValues = [];

        for (var i = 0; values.length > i; i++) {
          metricValues.push({
            time: timingVals[i],
            value: values[i]
          });
        }

        return metricValues;
      }
    }
    /**
     * Get the data to display in the longitudinal plot for scalars.
     * @param {Object} metricDef The metric defintion.
     * @return {Array} Data to plot.
     */


    function getScalarLongitudinalPlotData(metricDef) {
      if (metricDef.metricType === 'UNILATERAL') {
        if (!currentMetrics || !currentMetrics[metricName] || !currentMetrics[metricName].mean) {
          return null;
        }

        return [{
          time: 1,
          value: currentMetrics[metricName].mean
        }];
      }

      var plotData = [];

      if (currentMetrics[metricName + ' L'].mean) {
        plotData.push([{
          time: 1,
          value: currentMetrics[metricName + ' L'].mean
        }]);
      }

      if (currentMetrics[metricName + ' R'].mean) {
        plotData.push([{
          time: 1,
          value: currentMetrics[metricName + ' R'].mean
        }]);
      }

      return plotData;
    }
    /**
     * Build an event plot for the given trial.
     */


    function buildTrialEventPlot() {
      var plotDiv = visDiv.append('div').attr('class', 'longitudinal-plot-div');
      var plot = new TrialEventPlot().width(visWidth).height(260).duration(currentMetrics.Duration.mean).startDelay(currentTrial.getTestDefinition().startDelay).endDelay(currentTrial.getTestDefinition().endDelay).xAxisLabel('Time (s)').title('Detected Events').showDelays(false);
      plot.update(plotDiv.datum(currentMetrics));
    }
    /**
     * Get unilateral data for bar plot.
     * @param {Array} plotTrials The trials being plotted.
     * @return {Array} Matrix of data to be plotted.
     */


    function getUnilateralBarPlotData(plotTrials) {
      var plotData = [];
      plotTrials.forEach(function (d) {
        if (d.metrics[metricName]) {
          var entry = {};
          entry.value = d.metrics[metricName].mean;

          if (isNaN(entry.value)) {
            return;
          }

          entry.date = d.trial.getDateObj();
          entry.color = d.trial.getTestDefinition().displayColor;

          if (!entry.color || entry.color === '') {
            entry.color = DEFAULT_ENTRY_COLOR;
          }

          entry.id = 'id' + d.trial.getDateString();
          plotData.push(entry);
          entry.trial = d.trial;
        }
      });
      return plotData;
    }
    /**
     * Get bilateral data for bar plot.
     * @param {Array} plotTrials The trials being plotted.
     * @return {Array} Matrix of data to be plotted.
     */


    function getBilateralBarPlotData(plotTrials) {
      var plotData = [];
      plotTrials.forEach(function (d) {
        if (d.metrics[metricName + ' L']) {
          var entry = {};
          entry.value = d.metrics[metricName + ' L'].mean;
          entry.secondaryValue = d.metrics[metricName + ' R'].mean;

          if (isNaN(entry.value) || isNaN(entry.secondaryValue)) {
            return;
          }

          entry.date = d.trial.getDateObj();
          entry.color = d.trial.getTestDefinition().displayColor;

          if (!entry.color || entry.color === '') {
            entry.color = DEFAULT_ENTRY_COLOR;
          }

          entry.id = 'id' + d.trial.getDateString();
          plotData.push(entry);
          entry.trial = d.trial;
        }
      });
      return plotData;
    }
    /**
     * Build the bar plot, making sure to get required data before
     * doing the drawing.
     */


    function buildBarPlot() {
      var plotTrials = trialsOfType;

      if (compareTrials && compareTrials.length) {
        plotTrials = compareTrials;
      }

      var plotData = [];
      plotTrials.reduce(function (sequence, trial) {
        return sequence.then(function () {
          return trial.getMetrics();
        }).then(function (metrics) {
          plotData.push({
            trial: trial,
            metrics: metrics
          });
        });
      }, Promise.resolve()).then(function () {
        if (metricSet[metricName].metricType === 'BILATERAL') {
          drawBarPlot(getBilateralBarPlotData(plotData));
        } else {
          drawBarPlot(getUnilateralBarPlotData(plotData));
        }
      }, function () {
        drawBarPlot([]);
      }).catch(function (e) {
        utils.error('In buildBarPlot: ' + e);
      });
    }
    /**
     * Draw the bar plot.
     * @param {Array} plotData The list of trials with their metrics.
     */


    function drawBarPlot(plotData) {
      var metricDef = metricSet[metricName];
      visDiv.selectAll('.bar-plot-div').data([plotData]).enter().append('div').attr('class', 'bar-plot-div');
      var plot = new BarPlot().width(visWidth).height(250).x(function (d) {
        return d.date;
      }).y(function (d) {
        return d.value;
      }).barColor(function (d) {
        return d.color;
      }).barId(function (d) {
        return d.id;
      }).yAxisLabel(metricDef.units).title(metricSet[metricName].metricGroupDisplayName + ' - ' + metricSet[metricName].displayName).dispatch(dispatch).showBaselineButtons(true).currentBaseline(baselineTrial).currentTrial(currentTrial);

      if (!(currentTrial instanceof Session)) {
        var normVals = MetricDefinitionHelper.getInstance().getMeanStdDevAndNorms(currentTrial.getTestName(), currentTrial.getConditionName(), metricDef);

        if (normVals && normVals !== null && normVals.norms && normVals.norm !== null && normVals.norms.length > 1) {
          plot.normRange(normVals.fifthToNinteyFifth);
        }
      }

      if (metricDef.validRange === 'POSITIVE') {
        plot.normPositive(true);
      }

      if (metricDef.metricType === 'BILATERAL') {
        plot.bilateral(true).secondaryY(function (d) {
          return d.secondaryValue;
        });
      }

      plot.update(visDiv.select('.bar-plot-div').datum(plotData));
    }
    /**
     * Build the metric visualization given the specification within
     * the visDataObj.
     * @param {Object} visDataObj Object containing the visualization
     * class to use, the file name for the SVG and options to set on
     * the visualization class.
     */


    function buildVisualization(visDataObj) {
      visDiv.style('height', '260px');
      var plot = new visDataObj.visClass(visDiv, visDataObj.baseSvg, currentTrial.getTestName(), currentTrial.getConditionName(), metricName, currentMetrics);

      if (visDataObj.options) {
        for (var option in visDataObj.options) {
          plot[option](visDataObj.options[option]);
        }
      }

      plot.update();
    }
    /**
     * Determine the visualization to display based on the base name
     * of the current metric. If there is no visualization for the
     * given metric, defaults to a longitudinal plot.
     */


    function determineMetricVis() {
      var testName = currentTrial.getTestName();
      var baseName = metricName;
      $(visDiv.node()).empty(); // Handle special visualization for 'Duration' on the
      // TUG and Sit to Stand test types.

      if (baseName === 'Duration' && (testName === 'TUG' || testName === 'Sit to Stand')) {
        buildTrialEventPlot();
        return;
      }

      if (metricVisMap[baseName]) {
        switch (metricVisMap[baseName].type) {
          case reportEnums.metricVisType.APDM_VIS:
            buildVisualization(metricVisMap[baseName]);
            break;

          case reportEnums.metricVisType.STABILOGRAM:
            buildSwayVisualization();
            break;

          case reportEnums.metricVisType.EVENT_PLOT:
            buildTrialEventPlot();
            break;

          default:
            buildMetricLongitudinalPlot();
        }
      }
    } // Public functions


    this.visDiv = function (d) {
      if (!arguments.length) {
        return visDiv;
      }

      visDiv = d;
      return this;
    };

    this.visWidth = function (d) {
      if (!arguments.length) {
        return visWidth;
      }

      visWidth = d;
      return this;
    };

    this.visHeight = function (d) {
      if (!arguments.length) {
        return visHeight;
      }

      visHeight = d;
      return this;
    };

    this.visType = function (d) {
      if (!arguments.length) {
        return visType;
      }

      previousType = visType;
      visType = d;
      return this;
    };

    this.currentTrial = function (d) {
      if (!arguments.length) {
        return currentTrial;
      }

      currentTrial = d; // Clear the data we've cached for the previous trial

      stabilogramMatrix = null;
      metricTimeArrays = {};
      return this;
    };

    this.compareTrials = function (d) {
      if (!arguments.length) {
        return compareTrials;
      }

      compareTrials = d;
      return this;
    };

    this.baselineTrial = function (d) {
      if (!arguments.length) {
        return baselineTrial;
      }

      baselineTrial = d;
      return this;
    };

    this.metricTimeArrays = function (d) {
      if (!arguments.length) {
        return metricTimeArrays;
      }

      metricTimeArrays = d;
      return this;
    };

    this.trialsOfType = function (d) {
      if (!arguments.length) {
        return trialsOfType;
      }

      trialsOfType = d;
      return this;
    };

    this.metricName = function (d) {
      if (!arguments.length) {
        return metricName;
      }

      metricName = d;
      return this;
    };

    this.metricSet = function (d) {
      if (!arguments.length) {
        return metricSet;
      }

      metricSet = d;
      return this;
    };
    /**
     * Get set the dispatch object.
     */


    this.dispatch = function (d) {
      if (!arguments.length) {
        return dispatch;
      }

      dispatch = d;
      return this;
    };
    /**
     * Update/create the visualization based on the visType value
     * and the name of the test.
     */


    this.update = function () {
      // Make sure this style is cleared if necessary
      if (visType !== previousType) {
        $(visDiv.node()).empty();

        if (previousType === reportEnums.reportType.vis.NONE) {
          visDiv.transition().duration(100).style('min-height', '260px');
          setTimeout(doUpdate, 100);
          return this;
        }
      }

      if (currentTrial instanceof Trial) {
        currentTrial.getMetrics().then(function (metrics) {
          currentMetrics = metrics;
          doUpdate();
        }, function (e) {
          utils.error(e.error().statusText);
          doUpdate();
        });
      } else {
        doUpdate();
      }

      return this;
    };
  }

  return ReportVisHandler;
});
//# sourceMappingURL=report_vis_handler.js.map
