"use strict";

/* Copyright (c) 2016-2019, APDM Inc.
 *
 * run_test.js
 *
 * Module to handle running test.
 */
define(['jquery', 'd3', 'utils', 'server_handler', 'model_dispatcher', 'model_protocol', 'res_req_map', 'crossroads', 'progress_dialog', 'error_handler', 'test_definition_handler', 'session', 'trial', 'app/ui_state', 'app/user_options_handler', 'app/dialog_in_series', 'app/hardware_placement_dialog', 'app/trial_timer', 'app/hw_error_handler', 'metric_definition_helper', 'app/delete_handler', 'app/report_container', 'app/license_handler', 'bootstrap-dialog', 'app/user_timeout', 'app/single_user_handler', 'app/placement_labels_EN'], function ($, d3, utils, ServerHandler, ModelDispatcher, ModelProtocol, ResReqMap, crossroads, ProgressDialog, ErrorHandler, TestDefinitionHandler, Session, Trial, uiState, UserOptionsHandler, DialogInSeries, hardwarePlacementDialog, Timer, hwErrorHandler, MetricDefinitionHelper, deleteHandler, reportContainer, LicenseHandler, BootstrapDialog, Timeout, SingleUserHandler, placementLabels) {
  'use strict'; // Keycodes to start and stop on the "Run Test" dialog

  var REMOTE_START_KEYCODE = 34; // page-down key

  var REMOTE_STOP_KEYCODE = 33; // page-up key

  var REMOTE_END_KEYCODE = 27; // escape key

  var REMOTE_REFRESH_KEYCODE = 116; // F5 key

  var REMOTE_PERIOD_KEYCODE = 190; // . key

  var REMOTE_B_KEYCODE = 66; // "b" key

  var _server;

  var _subject;

  var _serverStatus;

  var _serverPoll = null;

  var _polling;

  var _progDialog;

  var _keydownTriggered = false;
  var _bufferingCanceled = false;
  var _completingSequence = false;
  var _isV1 = false;
  var _sensorConfiguration = null;
  var _hasVideoSource = false; // Crossroads path for catching docked monitors

  crossroads.addRoute('check_for_docked_monitors/{key}', function (key) {
    checkForDockedMonitors(ResReqMap.getInstance().getResponseData(key));
  }); // Popover tooltip text and element.
  // TODO: Maybe move this to an external file that we load in - easier
  // for localization.

  var popovers = [{
    id: '#exit-sequence-button',
    title: 'Exit This Sequence',
    content: 'Keep all recorded trials and exit the recording session now. ' + 'You can continue an incomplete sequence from the ' + 'Subject\'s trial view.'
  }, {
    id: '#skip-test-button',
    title: 'Skip This Test',
    content: 'Skip this test and continue to the next test in the sequence.'
  }, {
    id: '#next-test-exit-button',
    title: 'Exit This Sequence',
    content: 'Keep all recorded trials and exit the recording session now. ' + 'You can continue an incomplete sequence from the ' + 'Subject\'s trial view.'
  }, {
    id: '#next-test-repeat-button',
    title: 'Redo This Test',
    content: 'Discard the trial just completed and record it again.'
  }, {
    id: '#next-test-discard-button',
    title: 'Discard This Test',
    content: 'Discard the trial just completed and continue to the ' + 'next trial in the sequence.'
  }, {
    id: '#next-test-continue-button',
    title: 'Continue',
    content: 'Keep the trial just completed and continue to the next ' + 'trial in the sequence.'
  }];
  /**
   * Handle an external sync start message from the server.
   * NOTE: Put in global namespace so accessable by SWT Browser
   * widget JavaScript call.
   */

  window.externalSyncStart = function () {
    $('#start-test-button').trigger('click');
  };
  /**
   * Handle an external sync stop message from the server.
   * NOTE: Put in global namespace so accessable by SWT Browser
   * widget JavaScript call.
   */


  window.externalSyncStop = function () {
    $('#start-test-button').trigger('click');
  };
  /**
   * Get the location label to monitor case id mapping. On success,
   * build the hardware placement dialog.
   * @param {Object} allTests Object containing all pending and
   * completed series tests.
   * @param {Subject} currSubject The subject for the tests.
   * @param {Boolean} completingSequence True is user is completing
   * an incomplete sequence.
   */


  function loadMonitorMapping(allTests, currSubject, completingSequence) {
    _subject = currSubject;
    _completingSequence = completingSequence;
    _server = ServerHandler.getInstance();
    startPolling(); // Skip this check and the hardware placement dialog if inside
    // Mobility Web

    if (_server.hasJsPostHandler()) {
      _progDialog = new ProgressDialog();

      _progDialog.show();

      loadInitHardware(allTests);
      return;
    }

    setNumberOfTests(allTests.pendingTests);
    var isSixSensor = !LicenseHandler.getInstance().isKinematics();

    _server.post({
      type: ModelProtocol.load_sensor_configuration
    }, 'json', function (d) {
      _sensorConfiguration = d.sensorConfiguration; // TODO: Uncomment once we have better support for video
      // _hasVideoSource = d.hasVideoSource;

      _isV1 = _sensorConfiguration.isV1 ? _sensorConfiguration.isV1 : false;

      if (!isSixSensor && _isV1) {
        noV1WithKinematics();
        return;
      } // Tell them to configure if no monitors in map.


      if (!_sensorConfiguration.placements || !Object.keys(_sensorConfiguration.placements).length) {
        _serverStatus.viewState = 'TEST_CONFIGURATION_ERROR';
        d3.select('.modal-backdrop').remove();
        hwErrorHandler.checkForHwError(_serverStatus.viewState, _isV1);
        return;
      }

      hardwarePlacementDialog.open(allTests, _sensorConfiguration, completingSequence, isSixSensor, _isV1);
    }, function (d) {
      d3.select('.modal-backdrop').remove();
      _sensorConfiguration = null;
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Show dialog informing user that they can't use V1 with
   * Moveo.
   */


  function noV1WithKinematics() {
    d3.select('.modal-backdrop').remove();
    BootstrapDialog.show({
      title: 'Hardware not compatable with Moveo Explorer',
      message: '<p>You currently have Version 1 hardware ' + 'configured.</p><p>Moveo Explorer is only compatable ' + 'with Version 2 hardware.</p>',
      buttons: [{
        id: 'error-dialog-close-btn',
        icon: 'iconic iconic-x',
        label: 'Close',
        cssClass: 'btn-link',
        action: function action(dialog) {
          dialog.close();
        }
      }]
    });
  }
  /**
   * Tell the server how many tests we are going to run in a series.
   * @param tests The array of tests that are going to be run.
   */


  function setNumberOfTests(tests) {
    utils.log('Setting number of tests to: ' + tests.length);

    _server.post({
      type: ModelProtocol.set_number_tests_in_series,
      numSeriesTest: tests.length
    }, null, null, null);
  }
  /**
   * Check to if any monitors are still docked.
   * @param {Object} allTests The pending and completed tests.
   */


  function checkForDockedMonitors(allTests) {
    _progDialog = new ProgressDialog();

    _progDialog.setTitle('Initializing Sensors');

    _progDialog.show(); // This little mapping needed because Object.values is not supported by some browsers


    var sensorList = Object.keys(_sensorConfiguration.placements).map(function (d) {
      return _sensorConfiguration.placements[d];
    });

    _server.post({
      type: ModelProtocol.check_for_docked_monitors,
      configured_monitors: sensorList
    }, 'json', function (d) {
      // After implementing logic to only complain if a *configured* sensor remained docked, APDM sales
      // pushed back, thinking that this would complicate things until we can also configure a subset
      // of Opals. New code is maintained here. Changing the value below to "false" will implement the
      // new behavior.
      var errorOnAnyDockedOpal = true;
      var messageText;

      if (d.monitorIsDocked === true) {
        if (!errorOnAnyDockedOpal) {
          var dockedConfiguredMonitors = [];
          var i;

          for (i = 0; i < d.dockedMonitors.length; i++) {
            var match;

            if (_isV1) {
              match = sensorList.indexOf(parseInt(d.dockedMonitors[i].substring(3)));
            } else {
              match = sensorList.indexOf(d.dockedMonitors[i]);
            }

            if (match >= 0) {
              dockedConfiguredMonitors.push(d.dockedMonitors[i]);
              var last = dockedConfiguredMonitors.length - 1;

              if (_isV1) {
                dockedConfiguredMonitors[last] = dockedConfiguredMonitors[last] + ' : ' + Object.keys(_sensorConfiguration.placements)[match];
              } else {
                dockedConfiguredMonitors[last] = dockedConfiguredMonitors[last] + ' : ' + placementLabels[Object.keys(_sensorConfiguration.placements)[match]];
              }
            }
          }

          if (dockedConfiguredMonitors.length > 0) {
            _progDialog.close();

            messageText = '<p>The following, configured sensors are still docked.</p>';
            messageText += '<p>';

            for (i = 0; i < dockedConfiguredMonitors.length; i++) {
              messageText += dockedConfiguredMonitors[i] + '<br/>';
            }

            messageText += '</p>';
            messageText += '<p>Please undock these sensors, and then press "Continue"</p>';
            new ErrorHandler({
              type: ErrorHandler.WARNING,
              title: 'Configured sensors are still docked',
              message: messageText,
              buttonText: 'Continue',
              buttonIcon: 'iconic-caret-right',
              closeCall: function closeCall() {
                checkForDockedMonitors(allTests);
              }
            }).show();
            return;
          }
        } else {
          _progDialog.close();

          messageText = '<p>Sensors are still docked.</p>';
          messageText += '<p>Please remove all sensors from the ' + 'docking stations, and then press "Continue"</p>';
          new ErrorHandler({
            type: ErrorHandler.WARNING,
            title: 'Sensors are still docked',
            message: messageText,
            buttonText: 'Continue',
            buttonIcon: 'iconic-caret-right',
            closeCall: function closeCall() {
              checkForDockedMonitors(allTests);
            }
          }).show();
          return;
        }
      }

      loadInitHardware(allTests);
    }, function (d) {
      _progDialog.close();

      d3.select('.modal-backdrop').remove();
      endPolling();
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Post the test name and condition and let the server know we are
   * waiting on hardware and display a spinner page in the meantime.
   * @param {Object} allTests Pending and completed tests.
   */


  function loadInitHardware(allTests) {
    _progDialog.show();

    _server.postUIState(uiState.WAITING_FOR_HW);

    var testDef = TestDefinitionHandler.getInstance().getTestDefinition(allTests.pendingTests[0].testName, allTests.pendingTests[0].conditionName);
    setTestSession(allTests, testDef);

    _server.post({
      type: ModelProtocol.set_test_and_condition,
      testName: testDef.testName,
      conditionName: testDef.conditionName
    }, null, null, function (d) {
      _progDialog.close();

      endPolling();
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Set or create the session that the tests being run will
   * be placed in.
   * @param allTests Object containing tests to run.
   * @param testDef The current test definition.
   */


  function setTestSession(allTests, testDef) {
    var request;
    var key;
    var sessionId;

    if (!allTests.session) {
      var sessionAddedRoute = crossroads.addRoute('sessionAdded/{key}/:id:/:localId:', function (key) {
        crossroads.removeRoute(sessionAddedRoute);
        var session = new Session(ResReqMap.getInstance().getResponseData(key));
        allTests.session = session;
        checkHardwareState(allTests, testDef);
      });

      if (allTests.sequence !== null) {
        // Only create the session for sequence if it doesn't exist.
        request = {
          name: allTests.sequence.name,
          notes: ''
        };
        key = ResReqMap.getInstance().setRequestData(request);
        crossroads.parse('dispatch/addSession/' + key);
      } else {
        var sessionSelectedRoute = crossroads.addRoute('sessionSelected', function () {
          crossroads.removeRoute(sessionSelectedRoute);
          crossroads.removeRoute(sessionAddedRoute);
          checkHardwareState(allTests, testDef);
        });

        var adHoc = _subject.getAdHocSession(); // If there is no "Ad-Hoc" session, then create it.


        if (adHoc === null) {
          request = {
            name: 'Ad-Hoc',
            notes: ''
          };
          key = ResReqMap.getInstance().setRequestData(request);
          crossroads.removeRoute(sessionSelectedRoute);
          crossroads.parse('dispatch/addSession/' + key);
          return;
        }

        sessionId = adHoc.getObjId();
        allTests.session = adHoc;
        crossroads.parse('dispatch/selectSession/' + sessionId.getId() + '/' + sessionId.getLocalId());
      }
    } else if (_server.hasJsPostHandler()) {
      // FIXME: Hacky hacky hack. For the mTBI Mobility Web
      // trials, selecting the session doesn't work because of a
      // disconnect between the id and local id.
      checkHardwareState(allTests, testDef);
    } else {
      var selectedRoute = crossroads.addRoute('sessionSelected', function () {
        crossroads.removeRoute(selectedRoute);
        checkHardwareState(allTests, testDef);
      });
      sessionId = allTests.session.getObjId();
      crossroads.parse('dispatch/selectSession/' + sessionId.getId() + '/' + sessionId.getLocalId());
    }
  }
  /**
   * Check the hardware state, close the spinner and display the
   * "run test" dialog if ready.
   * @param allTests Object containing tests to run.
   * @param testDef The current test definition.
   */


  function checkHardwareState(allTests, testDef) {
    if (_serverStatus.viewState === 'TEST_WAITING_TO_RUN') {
      statusCheck(null, _serverStatus);
    } else if (hwErrorHandler.checkForHwError(_serverStatus.viewState, _isV1)) {
      _progDialog.close();

      loadMonitorMapping(allTests, _subject, _completingSequence);
    } else {
      $(document).on('serverStateUpdate', statusCheck);
    }

    function statusCheck(_, status) {
      if (status.viewState === 'TEST_WAITING_TO_RUN') {
        _progDialog.close();

        $(document).off('serverStateUpdate');

        _progDialog.close();

        loadRunTest(testDef, allTests);
      } else if (hwErrorHandler.checkForHwError(status.viewState, _isV1)) {
        _progDialog.close();

        $(document).off('serverStateUpdate');
        loadMonitorMapping(allTests, _subject, _completingSequence);
      }
    }
  }
  /**
   * Load the page containing the timers and instructions to run the test.
   * @param testDef The test's definition.
   * @param allTests Object containing pending and completed tests.
   */


  function loadRunTest(testDef, allTests) {
    var _server = ServerHandler.getInstance();

    var seriesDialog = new DialogInSeries('dialog-test');
    var dialogSelection = $('#dialog-test'); // Only have page for displaying the subject instructions if
    // this is a single user UI instance.

    var dialogHtml = 'run_test.html';

    if (SingleUserHandler.getInstance().isSingleUserUI()) {
      dialogHtml = 'single_user_ui_run_test.html';
    } else if (_hasVideoSource) {
      dialogHtml = 'run_test_video.html';
    }

    $('#dialog-test-body').empty();
    $('#dialog-test-body').load('/mobility_clinic/html/' + dialogHtml, pageLoaded);
    var titleText = '<div>Test: <span style="font-weight:normal">' + testDef.testDisplayName + '</span></div>';
    var recordBtnText = 'Start';

    if (!_server.hasJsPostHandler()) {
      titleText = titleText + '<div>Condition: ' + '<span style="font-weight:normal">' + testDef.conditionDisplayName + '</span></div>';
      recordBtnText = 'Record';
    }

    $('#dialog-test-title-text').html(titleText);
    $('#dialog-test-footer').empty(); // Add the footer buttons

    var footer = d3.select('#dialog-test-footer');
    footer.append('button').attr('id', 'start-test-button').html('<span class="iconic iconic-chevron-right"></span>\n' + recordBtnText + '\n').attr('type', 'button').attr('class', 'btn btn-success');

    if (!SingleUserHandler.getInstance().isSingleUserUI()) {
      footer.append('button').attr('id', 'skip-test-button').html('<span class="iconic iconic-action-redo"></span>\nSkip\n').attr('type', 'button').attr('class', 'btn btn-default').on('click', function () {
        allTests.pendingTests.shift();
        setNumberOfTests(allTests.pendingTests);

        if (allTests.pendingTests.length) {
          seriesDialog.hide(false, function () {
            loadInitHardware(allTests);
          });
        } else {
          exitRunTest(allTests);
        }
      });
    }

    if (allTests.pendingTests.length > 1) {
      footer.append('button').html('<span class="iconic iconic-x">' + '</span>\nExit Sequence\n').attr('type', 'button').attr('class', 'btn btn-default').attr('id', 'exit-sequence-button').on('click', function () {
        allTests.pendingTests = [];
        setNumberOfTests(allTests.pendingTests);
        exitRunTest(allTests);
      });
    } // Add popovers to buttons


    if (!SingleUserHandler.getInstance().isSingleUserUI()) {
      addPopovers(['#exit-sequence-button', '#skip-test-button']); // Explicitly destroy the 'skip' button popover on hides

      $('#dialog-test').off('hide.bs.modal');
      $('#dialog-test').on('hide.bs.modal', function () {
        $('#skip-test-button').popover('destroy');
        $('#exit-sequence-button').popover('destroy');
      });
    }

    function exitRunTest(tests) {
      endPolling();
      endVideoPreview(); // Make sure the streaming session gets closed.

      _server.postUIState(uiState.CANCELING_TEST);

      if (tests.completedTests.length) {
        seriesDialog.hide(false, function () {
          var requestData = {
            objId: _subject.getObjId(),
            allTests: tests
          };
          crossroads.parse('dispatch/testCompleted/' + ResReqMap.getInstance().setRequestData(requestData));
        });
      } else {
        // Make sure to delete empty sessions.
        if (tests.session && !tests.session.getAllTrials().length) {
          crossroads.parse('dispatch/deleteSession/' + ResReqMap.getInstance().setRequestData(tests.session.getObjId()));
        }

        seriesDialog.hide(true);
      }
    }

    function pageLoaded() {
      if (testDef.subjectInstructions && testDef.subjectInstructions !== '') {
        d3.select('#subject-test-instructions').html(testDef.subjectInstructions);
      }

      if (testDef.adminInstructions && testDef.adminInstructions !== '') {
        d3.select('#admin-test-instructions').html(testDef.adminInstructions);
      } // Start the video preview, if there is a source.


      if (_hasVideoSource) {
        seriesDialog.onShown(function () {
          startVideoPreview(dialogSelection);
        });
      } // Set up the run trial page according to the test definition


      d3.select('.subject_instructions').html(testDef.subjectInstructions);
      var startDelay = parseInt(testDef.startDelay, 10);
      var duration = parseInt(testDef.recordDuration, 10); // Determine starting value for timer and its title

      var timerValue = d3.select('#trial-timer-value');
      var timerLabel = d3.select('#trial-timer-label');

      if (startDelay > 0) {
        timerLabel.text('Countdown: ');
        timerValue.text(startDelay);
      } else {
        timerValue.text(duration);

        if (duration > 0) {
          timerLabel.text('Remaining: ');
        }
      }

      setupAnnotations();

      var endTest = function endTest() {
        $(document).off('serverStateUpdate');

        if (hwErrorHandler.checkForHwError(_serverStatus.viewState, _isV1) === true) {
          // Bail if there was a hardware error.
          seriesDialog.hide(true, function () {
            dialogSelection.off('keydown');
          });
          return;
        }

        seriesDialog.hide(false, function () {
          endVideoPreview();
          loadBufferPage(allTests);
        });
      };

      var testTimer = new Timer();
      testTimer.startDelay(startDelay).duration(duration).timerLabelText(timerLabel).timerValueText(timerValue).testEndCb(endTest);
      var startBtn = $('#start-test-button');
      startBtn.click(testRunning); // Setup the check for hardware errors that occur during the test.

      setupHwErrorCheck(testTimer); // Remote key handler of start/stop button.

      addRemoteKeydownHandlers($(document));
      $(document).off('remoteNext');
      $(document).on('remoteNext', function () {
        startBtn.trigger('click');
      }); // Handlers for annotations

      $(document).on('annotationOne', function () {
        setAnnotation($('#annotation-one-text-input').val());
      });
      $(document).on('annotationTwo', function () {
        setAnnotation($('#annotation-two-text-input').val());
      }); // Show the dialog

      seriesDialog.show(false);

      function testRunning() {
        $('#skip-test-button').attr('disabled', 'disabled').popover('destroy');
        $('#exit-sequence-button').attr('disabled', 'disabled').popover('destroy');
        testTimer.start(); // Lock out editing on annotations and save if necessary

        $('#annotation-one-text-input').attr('disabled', 'disabled');
        $('#annotation-two-text-input').attr('disabled', 'disabled');
        $('#add-annotation-one-button').attr('disabled', null);
        $('#add-annotation-two-button').attr('disabled', null);
        checkAnnotationsForChange();
      }
    }
  }
  /**
   * Setup the check for a hardware error during the test.
   * @param {TrialTimer} timer The trial timer.
   */


  function setupHwErrorCheck(timer) {
    $(document).on('serverStateUpdate', function () {
      if (hwErrorHandler.checkForHwError(_serverStatus.viewState, _isV1)) {
        $(document).off('serverStateUpdate');
        timer.stop();
      }
    });
  }
  /**
   * Request the server start the video preview.
   * @param {jQuery.selection} container The container for the run
   * test html.
   */


  function startVideoPreview(container) {
    var offset = container.find('#video-preview-div').offset();
    utils.log('x:' + offset.left + ', y: ' + offset.top);

    _server.post({
      type: ModelProtocol.start_video_preview,
      xPosition: offset.left,
      yPosition: offset.top
    }, null, null, function (e) {
      ErrorHandler.showError(e.error().statusText);
    });
  }
  /**
   * Request the server end the video preview.
   */


  function endVideoPreview() {
    _server.post({
      type: ModelProtocol.stop_video_preview
    }, null, null, function (e) {
      ErrorHandler.showError(e.error().statusText);
    });
  }
  /**
   * Setup handlers on annotation buttons and get the starting
   * values for the annotations.
   */


  function setupAnnotations() {
    if (UserOptionsHandler.getInstance().getAnnotationsEnabled() === false) {
      $('#annotation-container').remove();
      return;
    }

    var notationBtnOne = $('#add-annotation-one-button');
    var notationBtnTwo = $('#add-annotation-two-button');
    var textInputOne = $('#annotation-one-text-input');
    var textInputTwo = $('#annotation-two-text-input');
    notationBtnOne.attr('disabled', 'disabled');
    notationBtnTwo.attr('disabled', 'disabled');
    var annotations = UserOptionsHandler.getInstance().getAnnotations();
    textInputOne.val(annotations.one);
    textInputTwo.val(annotations.two);
    notationBtnOne.click(function () {
      setAnnotation(textInputOne.val());
    });
    notationBtnTwo.click(function () {
      setAnnotation(textInputTwo.val());
    });
  }
  /**
   * Check the currently saved annotations, and if they've changed
   * then save the new values.
   */


  function checkAnnotationsForChange() {
    var optionsHandler = UserOptionsHandler.getInstance();

    if (optionsHandler.getAnnotationsEnabled() === false) {
      return;
    }

    var saved = optionsHandler.getAnnotations();
    var newAnnotations = $.extend({}, saved);
    var currTextOne = $('#annotation-one-text-input').val();
    var currTextTwo = $('#annotation-two-text-input').val();

    if (saved.one !== currTextOne) {
      newAnnotations.one = currTextOne;
    }

    if (saved.two !== currTextTwo) {
      newAnnotations.two = currTextTwo;
    }

    optionsHandler.setAnnotations(newAnnotations);
  }
  /**
   * Make the server call to set an annotation.
   * @param {String} annotation The annotation to set on recording.
   */


  function setAnnotation(annotation) {
    _server.post({
      type: ModelProtocol.add_recording_annotation,
      annotation: annotation
    });
  }
  /**
   * Load the spinner window for buffering and analyzing. Once analysis is
   * complete, get the study from the server and pass to the reports page.
   * @param {Object} allTests Object containing pending and completed tests.
   */


  function loadBufferPage(allTests) {
    if (ServerHandler.getInstance().hasJsPostHandler()) {
      getCompletedTrials(dialog, allTests);
      return;
    }

    var dialog = new DialogInSeries('dialog-spinner');
    $('#dialog-spinner-body').empty();
    $('#dialog-spinner-body').load('/static/html/buffering.html', pageLoaded);
    var spinner = null;
    _bufferingCanceled = false;

    function pageLoaded() {
      var infoText = d3.select('#dialog-spinner-body').select('h3');
      var spinnerDiv = d3.select('#buffering-canvasloader-container');
      var progressBarDiv = d3.select('#buffering-progress-bar-div');
      d3.select('#buffering-progress-bar').style('width', '0%'); // Show/hide the correct divs.

      spinnerDiv.style('display', 'none');
      progressBarDiv.style('display', 'block'); // Start update for progress bar.

      var progressUpdate = setInterval(updateBufferingProgress, 100);
      infoText.text('Buffering data from  monitors.');
      dialog.show(false); // Add a cancel button.

      d3.select('.spinner').append('button').attr('class', 'btn btn-default').attr('id', 'buffering-cancel-button').html('<span class="iconic iconic-x"></span>\nCancel\n').on('click', function () {
        clearInterval(progressUpdate);

        _server.post({
          type: ModelProtocol.cancel_buffering
        }, null, null, null);

        _bufferingCanceled = true;
        statusCheck(null, _serverStatus);
      }); // Remove handler, if needed.

      $(document).off('serverStateUpdate');
      statusCheck(null, _serverStatus);
      $(document).on('serverStateUpdate', statusCheck);

      function statusCheck(_, status) {
        if (status.viewState === 'TEST_PROCESSING') {
          _server.postUIState(uiState.WAITING_FOR_ANALYSIS);

          clearInterval(progressUpdate);
          spinnerDiv.style('display', 'block');
          progressBarDiv.style('display', 'none');
          progressBarDiv.style('display', 'none');
          spinner = getWaitSpinner('buffering-canvasloader-container');
          infoText.text('Analyzing sensor data.');
          d3.select('#buffering-cancel-button').remove();
        } else if (status.viewState === 'TEST_PROCESSED' || status.viewState === 'TEST_WAITING_FOR_HW' || hwErrorHandler.checkForHwError(status.viewState, _isV1)) {
          clearInterval(progressUpdate);

          if (spinner && spinner !== null) {
            spinner.kill();
          }

          $(document).off('serverStateUpdate');
          getCompletedTrials(dialog, allTests);
        }
      }
    }
  }
  /**
   * Get the completed trials, hide the buffering/analysis
   * and get the next test.
   * @param {DialogInSeries} dialog The buffering/analysis dialog.
   * @param {Object} allTests Object containing completed and pending tests.
   */


  function getCompletedTrials(dialog, allTests) {
    if (_server.hasJsPostHandler() && SingleUserHandler.getInstance().isSingleUserUI()) {
      getCompletedWebTrial(dialog, allTests);
      return;
    }

    _server.post({
      type: ModelProtocol.get_completed_trials
    }, 'json', function (d) {
      if (dialog && dialog !== null) {
        dialog.hide(false, function () {
          loadNextTest(allTests, d);
        });
      } else {
        loadNextTest(allTests, d);
      }
    }, function (d) {
      ErrorHandler.showError(d.error().statusText, function () {
        if (dialog) {
          dialog.hide(true);
        }
      });
    });
  }
  /**
   * Get a completed trial when running a test from within Mobility Web.
   * FIXME: Pass in the duration here, since it's the only metric we can
   * get.
   */


  function getCompletedWebTrial(dialog, allTests) {
    _server.post({
      type: ModelProtocol.get_completed_web_trial,
      testType: allTests.pendingTests[0].testName,
      testCondition: allTests.pendingTests[0].conditionName
    }, 'json', function (d) {
      if (dialog && dialog !== null) {
        dialog.hide(false, function () {
          loadNextTest(allTests, d);
        });
      } else {
        loadNextTest(allTests, d);
      }
    }, function (d) {
      ErrorHandler.showError(d.error().statusText, function () {
        if (dialog) {
          dialog.hide(true);
        }
      });
    });
  }
  /**
   * Check the buffering status and update the progress bar.
   */


  function updateBufferingProgress() {
    _server.post({
      type: ModelProtocol.get_buffering_progress
    }, 'json', function (d) {
      d3.select('#buffering-progress-bar').style('width', d.bufferingProgress + '%');
    }, null);
  }
  /**
   * Load the discard/retry/continue dialog after a test in a series has
   * been completed.
   * @param allTests Pending and completed tests.
   * @param trialData Completed trial data.
   */


  function loadNextTest(allTests, trialData) {
    // Remove test from pending and mark completed.
    var curTest = allTests.pendingTests.shift();
    var completedTrials = trialData.completedTrials.map(function (d) {
      return new Trial(d);
    });
    var noneCompleted = true;

    if (completedTrials.length) {
      noneCompleted = false; // Assign the sequence value to the Trial instance.

      if (trialData.currentSession) {
        completedTrials[0].sequence = new Session(trialData.currentSession);
      }
    }

    allTests.completedTests.push(completedTrials[0]);
    var hasError = false;
    var dialog = new DialogInSeries('dialog-next-test');
    $('#invalid-trials-text').empty();
    var title = d3.selectAll('#dialog-next-test-title-text');
    var testDef = TestDefinitionHandler.getInstance().getTestDefinition(curTest.testName, curTest.conditionName);
    title.select('#completed-test-name').text(testDef.testDisplayName);
    title.select('#completed-condition-name').text(testDef.conditionDisplayName); // Clear the notes field.

    $('#dialog-next-test').find('#trial-notes').val(''); // Set up footer buttons

    var footer = d3.select('#dialog-next-test-footer');
    footer.select('#next-test-repeat-button').on('click', null).on('click', function () {
      // If the completed trial came back as undefined or null
      // from the server, use the shifted test.
      var redoTrial = completedTrials[0] ? completedTrials[0] : curTest;
      redoWithKeepOption(dialog, allTests, noneCompleted, redoTrial);
    });

    if (UserOptionsHandler.getInstance().getDisableDeleteAfterRun() === true) {
      footer.select('#next-test-discard-button').remove();
    } else {
      footer.select('#next-test-discard-button').on('click', null).on('click', function () {
        var deleteFunction = function deleteFunction() {
          allTests.completedTests.pop(); // Clear the session if no tests completed.

          if (allTests.completedTests.length < 1) {
            allTests.session = null;
          }

          if (allTests.pendingTests.length > 0) {
            dialog.hide(false, function () {
              loadInitHardware(allTests);
            });
          } else if (allTests.completedTests) {
            dialog.hide(true, function () {
              var requestData = {
                objId: _subject.getObjId(),
                allTests: allTests
              };
              endPolling();
              removeRemoteHandlers($(document));
              crossroads.parse('dispatch/testCompleted/' + ResReqMap.getInstance().setRequestData(requestData));
            });
          } else {
            dialog.hide(true);
          }
        };

        deleteHandler.deleteTest(completedTrials[0], deleteFunction);
      }).classed('disabled', noneCompleted);
    }

    footer.select('#next-test-preview-button').on('click', null).on('click', function () {
      reportContainer.singleTrialPreview(_subject, completedTrials[0]);
    }); // Add popovers - don't add if this is a single user ui

    if (!SingleUserHandler.getInstance().isSingleUserUI()) {
      addPopovers(['#next-test-repeat-button', '#next-test-discard-button', '#next-test-continue-button']);
    } // The "Next Test" text in dialog.


    if (allTests.pendingTests.length > 0) {
      d3.select('#next-test-in-series').text(allTests.pendingTests[0].testDisplayName);
      d3.select('#next-condition-in-series').text(allTests.pendingTests[0].conditionDisplayName);
    } else {
      d3.select('#next-test-in-series').text('None');
      d3.select('#next-condition-in-series').text('None');
    } // Generate the display string for failed trials.
    // NOTE: If this is a single user UI, then we ignore any warnings
    // and just move on - always report to the user "Success!"


    var completedIcon = d3.select('#completed-test-icon');
    var invalidText = d3.select('#invalid-trials-text');

    if (noneCompleted && !SingleUserHandler.getInstance().isSingleUserUI()) {
      hasError = true;
      completedIcon.classed('iconic-check', false).classed('text-success', false).classed('iconic-warning', true).classed('text-danger', true).classed('iconic-info', false).classed('text-warning', false);
      invalidText.append('h4').text('Issues detected with test:');
      invalidText.append('p').attr('class', 'text-danger').text('Error: Recording may be incomplete.');
    }

    if (showLogMessages(completedTrials[0], invalidText) === true && !SingleUserHandler.getInstance().isSingleUserUI()) {
      hasError = true;
    }

    $(document).off('remoteNext');
    $(document).on('remoteNext', function () {
      $('#next-test-continue-button').trigger('click');
    }); // Remote keydown handler on "Continue"

    addRemoteKeydownHandlers($(document)); // Set up the contiune button.

    footer.select('#next-test-continue-button').on('click', null).on('click', function () {
      confirmContinueOnError(dialog, allTests, hasError, !noneCompleted);
    }); // Setup the exit button if there are tests remaining in the queue.

    var exitBtn = $('#next-test-exit-button');

    if (allTests.pendingTests.length) {
      exitBtn.css('display', 'inline-block').click(function () {
        confirmTestSequenceExit(dialog, allTests);
      });
    } else {
      exitBtn.css('display', 'none');
    }

    _server.postUIState(uiState.READING_RESULTS);

    dialog.show(false);
  }
  /**
   * Show the messages from the trial analysis log.
   * @param {Trial} trial The last completed trial.
   * @param {d3.selection} textNode The selection to insert text into.
   * @return {Boolean} True if there was an error in the log.
   */


  function showLogMessages(trial, textNode) {
    if (!trial) {
      return true;
    }

    var error = false;
    var logMsgs = utils.parseTrialLogString(trial.getLogStr()); // And an error message if the buffering was canceled.

    if (_bufferingCanceled === true) {
      logMsgs.push({
        type: 'Error',
        message: 'Buffering was canceled. Recording may be incomplete.'
      });
    }

    var completedIcon = d3.select('#completed-test-icon');
    completedIcon.classed('iconic-check', true).classed('text-success', true).classed('iconic-warning', false).classed('text-danger', false).classed('iconic-info', false).classed('text-warning', false); // Don't show any errors if we're in single user mode.

    if (SingleUserHandler.getInstance().isSingleUserUI()) {
      return false;
    }

    if (logMsgs.filter(function (d) {
      return d.type === 'Warning';
    }).length) {
      completedIcon.classed('iconic-check', false).classed('text-success', false).classed('iconic-warning', false).classed('text-danger', false).classed('iconic-info', true).classed('text-warning', true);
    }

    if (logMsgs.filter(function (d) {
      return d.type === 'Error';
    }).length) {
      completedIcon.classed('iconic-check', false).classed('text-success', false).classed('iconic-warning', true).classed('text-danger', true).classed('iconic-info', false).classed('text-warning', false);
      error = true;
    }

    textNode.append('h4').text('Analysis Log');
    textNode.selectAll('p').data(logMsgs).enter().append('p').each(function (d) {
      var selection = d3.select(this);
      selection.text(d.type + ': ' + d.message);

      if (d.type === 'Error') {
        selection.attr('class', 'text-danger');
      } else if (d.type === 'Warning') {
        selection.attr('class', 'text-warning');
      }
    });
    return error;
  }
  /**
   * Display dialog giving user the option to either keep or delete
   * the trial that is being redone.
   * @param {DialogInSeries} dialog The dialog containing the "Redo" button.
   * @param {Object} allTests The object map of pending and compeleted trials.
   * @param {Boolean} noneCompleted True if no trials have
   * been returned completed.
   * @param {Trial} trial The trial being redone.
   */


  function redoWithKeepOption(dialog, allTests, noneCompleted, trial) {
    var testDef = TestDefinitionHandler.getInstance().getTestDefinition(trial.getTestName(), trial.getConditionName());
    var msg = '<p>Do you want to redo this test?</p>' + '<p>The current test will be kept.</p>';
    var btns = [{
      icon: 'iconic iconic-check',
      label: 'Redo',
      cssClass: 'btn-default',
      action: function action(d) {
        keepTrial();
        d.close();
      }
    }];

    if (UserOptionsHandler.getInstance().getDisableDeleteAfterRun() === false) {
      msg = 'Would you like to keep or delete the test ' + 'you are redoing?';
      btns[0].label = 'Keep';
      btns.push({
        icon: 'iconic iconic-trash',
        label: 'Delete',
        cssClass: 'btn-danger',
        action: function action(d) {
          deleteTrial();
          d.close();
        }
      });
    }

    BootstrapDialog.show({
      title: 'Redo Test',
      message: msg,
      buttons: btns
    });

    function keepTrial() {
      allTests.pendingTests.splice(0, 0, testDef);
      setNumberOfTests(allTests.pendingTests);
      continueAfterTrial(dialog, allTests, !noneCompleted);
    }

    function deleteTrial() {
      if (!noneCompleted) {
        deleteHandler.deleteTestNoDialog(trial);
      }

      allTests.completedTests.pop(); // Clear out session if no completed tests.

      if (allTests.completedTests.length < 1) {
        allTests.session = null;
      }

      allTests.pendingTests.splice(0, 0, testDef);
      setNumberOfTests(allTests.pendingTests);
      dialog.hide(false, function () {
        loadInitHardware(allTests);
      });
    }
  }
  /**
   * If there are invalid trials, confirm that the user wants to continue
   * with a dialog.
   * @param {DialogInSeries} dialog The completed test dialog.
   * @param {Object} allTests Object containing completed and pending tests.
   * @param {Boolean} hasError True if current trial has an analysis error.
   * @param {Boolean} saveNotes True if trial notes should be saved.
   */


  function confirmContinueOnError(dialog, allTests, hasError, saveNotes) {
    if (hasError === false) {
      dialog.hide(false);
      continueAfterTrial(dialog, allTests, saveNotes);
      return;
    }

    var confirmDialog = new DialogInSeries('dialog-confirm-continue');
    $('#confirm-continue-back-button').off('click');
    $('#confirm-continue-back-button').on('click', function () {
      confirmDialog.hide(false);
    });
    $('#confirm-continue-continue-button').off('click');
    $('#confirm-continue-continue-button').on('click', function () {
      confirmDialog.hide(false);
      continueAfterTrial(dialog, allTests, saveNotes);
    });
    confirmDialog.show(false);
  }
  /**
   * Confirm that the user wants to exit from the sequence of tests.
   * @param {DialogInSeries} dialog The "Test Completed" dialog.
   * @param {Object} allTests The object containing pending and
   * completed tests.
   */


  function confirmTestSequenceExit(dialog, allTests) {
    var confirmDialog = new DialogInSeries('dialog-confirm-exit');
    $('#confirm-exit-back-button').off('click');
    $('#confirm-exit-back-button').on('click', function () {
      confirmDialog.hide(false);
    });
    $('#confirm-exit-continue-button').off('click');
    $('#confirm-exit-continue-button').on('click', function () {
      _server.postUIState(uiState.EXITING_SEQUENCE);

      confirmDialog.hide(false);
      allTests.pendingTests = [];
      continueAfterTrial(dialog, allTests);
    });
    confirmDialog.show(false);
  }
  /**
   * Handle the pressing of the "Continue" button after a trial is
   * completed.
   * @param {DialogInSeries} dialog The test completed dialog.
   * @param {Object} allTests Object containing the completed and pending tests.
   * @param {Boolean} saveNotes Flag determining if trial notes should be saved.
   */


  function continueAfterTrial(dialog, allTests, saveNotes) {
    // Remove the remote keydown handler
    $('#dialog-next-test').off('keydown');
    var notes = $('#dialog-next-test').find('#trial-notes').val();

    var objId = _subject.getObjId();

    var rrMap = ResReqMap.getInstance(); // Sanitize the complete trial list.

    allTests.completedTests = allTests.completedTests.filter(function (d) {
      if (d) {
        return d;
      }
    });

    if (allTests.pendingTests.length > 0) {
      if (saveNotes) {
        crossroads.parse('dispatch/addCurrentTrialNotes/' + rrMap.setRequestData({
          notes: notes
        }));
      }

      loadInitHardware(allTests);
    } else {
      removeRemoteHandlers($(document));
      endPolling();
      var requestData = {
        nextHash: 'dispatch/testCompleted/' + rrMap.setRequestData({
          objId: objId,
          allTests: allTests
        }),
        notes: notes
      };

      if (saveNotes) {
        var key = rrMap.setRequestData(requestData);
        crossroads.parse('dispatch/addCurrentTrialNotes/' + key);
      } else {
        crossroads.parse(requestData.nextHash);
      }
    }

    if (!allTests.completedTests.length && !allTests.pendingTests.length) {
      dialog.hide(true);
    } else if (allTests.pendingTests.length) {
      if (SingleUserHandler.getInstance().isSingleUserUI() && !allTests.pendingTests.length) {
        dialog.hide(true);
        return;
      }

      dialog.hide(false);
    } else {
      dialog.hide(true);
    }
  }
  /**
   * Add a keydown handler for the remote.
   * @param {jQuery.selection} element The element to attach handler to.
   * @param {Function} callback Function to call when REMOTE_START_KEYCODE
   * event received.
   */


  function addRemoteKeydownHandlers(element) {
    element.keydown(function (evt) {
      if (_keydownTriggered === false) {
        // Determine what event to send.
        if (evt.keyCode === REMOTE_START_KEYCODE) {
          _keydownTriggered = true;
          evt.preventDefault();
          $(document).trigger('remoteNext');
        } else if (evt.keyCode === REMOTE_END_KEYCODE || evt.keyCode === REMOTE_REFRESH_KEYCODE) {
          if (_serverStatus.viewState !== 'TEST_RUNNING') {
            return;
          }

          _keydownTriggered = true;
          evt.preventDefault();
          $(document).trigger('annotationOne');
        } else if (evt.keyCode === REMOTE_PERIOD_KEYCODE || evt.keyCode === REMOTE_B_KEYCODE) {
          if (_serverStatus.viewState !== 'TEST_RUNNING') {
            return;
          }

          _keydownTriggered = true;
          evt.preventDefault();
          $(document).trigger('annotationTwo');
        } else if (evt.keyCode === REMOTE_STOP_KEYCODE) {
          evt.preventDefault();
        }
      } // Always keep from refreshing


      if (evt.keyCode === REMOTE_REFRESH_KEYCODE) {
        evt.preventDefault();
        return false;
      }
    });
    element.keyup(function () {
      if (_keydownTriggered === true) {
        _keydownTriggered = false;
      }
    });
  }
  /**
   * Remove the key handlers that are used for the remote.
   * @param {jQuery.selection} element The element to remove
   * the handlers from.
   */


  function removeRemoteHandlers(element) {
    element.off('keyup');
    element.off('keydown');
  }
  /**
   * Add popovers to the elements with the given ids.
   * @param {Array} elementIds The ids of the elements to add popovers too.
   */


  function addPopovers(elementIds) {
    elementIds.forEach(function (id) {
      popovers.forEach(function (pop) {
        if (pop.id === id) {
          $(id).popover({
            placement: 'bottom',
            trigger: 'hover',
            title: pop.title,
            content: pop.content,
            container: 'body',
            delay: {
              'show': 500,
              'hide': 100
            }
          });
        }
      });
    });
  }
  /**
   * Start the polling of the server.
   */


  function startPolling() {
    _serverStatus = {
      viewState: 'UNDEFINED'
    };
    _polling = false;

    if (_serverPoll) {
      clearInterval(_serverPoll);
      _serverPoll = null;
    }

    _serverPoll = setInterval(pollServer, 250); // Bypass inactivity timer

    Timeout.getInstance().bypassTimeout(true);
  }
  /**
   * End the server polling.
   */


  function endPolling() {
    if (_serverPoll) {
      clearInterval(_serverPoll);
      _serverPoll = null;
      Timeout.getInstance().bypassTimeout(false);
    }
  }
  /**
   * Poll the server for its current state.
   */


  function pollServer() {
    if (_polling === false) {
      _polling = true;

      _server.post({
        type: ModelProtocol.get_status
      }, 'json', function (d) {
        _polling = false;

        if (_serverStatus.viewState !== d.viewState) {
          _serverStatus = d;
          utils.log('viewState: ' + _serverStatus.viewState);
          $(document).trigger('serverStateUpdate', [_serverStatus]);
        }
      }, null);
    }
  }
  /**
  * Create a "wait spinner" in the given div container and show it.
  * Responsibility of caller to shutdown spinner when not being used.
  * @param divId The id of div container to place spinner in.
  * @return The spinner instance.
  */


  function getWaitSpinner(divId) {
    var cl = new CanvasLoader(divId);
    cl.setColor('#009fbf');
    cl.setDiameter(64);
    cl.setDensity(103);
    cl.setRange(1);
    cl.setFPS(33);
    cl.show();
    return cl;
  }

  return {
    beginTest: loadMonitorMapping
  };
});
//# sourceMappingURL=run_test.js.map
