"use strict";

/* Copyright (c) 2018 APDM, Inc
 *
 * trial_from_logged.js
 *
 * Handle the selection of logged recordings and
 * test type for the selected recording.
 */
define(['jquery', 'd3', 'server_handler', 'model_protocol', 'error_handler', 'progress_dialog', 'crossroads', 'utils', 'app/logged_recording_plot', 'bootstrap-dialog', 'test_definition_handler', 'moment'], function ($, d3, ServerHandler, ModelProtocol, ErrorHandler, ProgressDialog, crossroads, utils, LoggedRecordingPlot, BootstrapDialog, TestDefinitionHandler, moment) {
  'use strict';

  var _trialFromLogged = {};

  var _subject;

  var _progDialog = null;

  var _loggedRecordings;

  var _plotWidth = 800;

  var _barColorScale;

  var _dispatcher;

  var _messageRoute = null; // Setup crossroads path to get messages on import status.

  _messageRoute = crossroads.addRoute('message/IMPORT_LOGGED/{key}', handleMessage);
  /**
   * Open the import page, and setup the handlers for any
   * permament buttons on the page.
   */

  function openImportLoggedPage() {
    _dispatcher = d3.dispatch('importRecording');

    _dispatcher.on('importRecording', function (d) {
      importRecording(d);
    });

    _progDialog = new ProgressDialog({
      title: 'Scanning Opals for recordings'
    }).show();
    $('#switchable').load('/mobility_clinic/html/logged_import.html', pageLoaded);

    function pageLoaded() {
      $('#return-to-subject-button').click(returnToSubject);
      getApdmFiles();
    }
  }
  /**
   * Return to the subject page from whence we came.
   */


  function returnToSubject() {
    var objId = _subject.getObjId();

    crossroads.parse('dispatch/subject?id=' + objId.getId() + '&localId=' + objId.getLocalId());
  }
  /**
   * Get the ApdmFiles that are present on the Opals that are
   * currently connected and mounted.
   */


  function getApdmFiles() {
    ServerHandler.getInstance().post({
      type: ModelProtocol.get_apdm_files
    }, 'json', function (d) {
      _loggedRecordings = d;
      buildOpalSelection();

      _progDialog.close();
    }, function (e) {
      _progDialog.close();

      ErrorHandler.showError(e.error().statusText);
    });
  }
  /**
   * Build the list of Opals that can be selected.
   */


  function buildOpalSelection() {
    if (_loggedRecordings.length) {
      $('#opals-with-recordings-list').empty();
    }

    var list = d3.select('#opals-with-recordings-list').selectAll('.sensor-entry').data(_loggedRecordings, function (d) {
      return d.caseIdString;
    });
    list.enter().append('div').attr('class', 'sensor-list-container').append('span').attr({
      'class': 'sensor-entry',
      'draggable': true,
      'id': function id(d) {
        return d.caseIdString;
      }
    }).text(function (d) {
      return d.caseIdString + ': ' + d.label;
    }).on('click', function (d) {
      displayRecordingsForOpal(d);
    });
    list.exit().remove(); // Colors for each of the senors

    var keyLabels = _loggedRecordings.map(function (d) {
      return d.caseIdString + ': ' + d.label;
    });

    _barColorScale = d3.scale.category20().domain(keyLabels);
  }
  /**
   * Display the recordings on the given Opal. Also display
   * the recordings that overlap with this one.
   * @param {Object} opal The object representing the Opal and
   * the recordings on it.
   */


  function displayRecordingsForOpal(opal) {
    d3.select('#selected-opal-recording-column').text('').selectAll('div').data(opal.files).enter().append('div').each(function (d) {
      var plot = new LoggedRecordingPlot().width(_plotWidth).barColorScale(_barColorScale);
      plot.update(d3.select(this).datum(d));
    });
  }
  /**
   * Create and send off the request to import the recording.
   * Opens a dialog allowing the setting of the test type
   * and condition.
   * @param {Object} recordingData The recording to import.
   */


  function importRecording(recordingData) {
    BootstrapDialog.show({
      title: 'Import Recording As Trial',
      message: $('<div></div>').load('/mobility_clinic/html/import_recording_to_trial.html'),
      buttons: [{
        id: 'add-audio-cue-cancel-btn',
        icon: 'iconic iconic-x',
        label: 'Cancel',
        cssClass: 'btn-link',
        action: function action(dialog) {
          dialog.close();
        }
      }, {
        id: 'add-audio-cue-add-btn',
        icon: 'iconic iconic-check',
        label: 'Import',
        cssClass: 'btn-success',
        action: function action(dialog) {
          dialog.close();
          sendRequestForImport(recordingData);
        }
      }],
      onshown: function onshown() {
        populateDialog(recordingData);
      }
    });
  }
  /**
   * Populate the import confirmation dialog.
   * @param {Object} recordingData Object containing the start/end
   * time and the list of recordings.
   */


  function populateDialog(recordingData) {
    var opals = recordingData.recordings.map(function (d) {
      return d.keyLabel;
    }).join(', ');
    d3.select('#selected-opal-list').text(opals); // Convert to a date string.

    $('#import-dialog-start-time').text(recordingData.startDate);
    $('#import-dialog-end-time').text(recordingData.endDate);
    $('#import-dialog-duration-time').text(moment.utc(recordingData.duration).format('HH:mm:ss.SS'));
    var trialSelect = $('#logged-import-test-select');
    d3.select(trialSelect.get(0)).selectAll('option').data(TestDefinitionHandler.getInstance().getTestNameMap()).enter().append('option').text(function (d) {
      return d.displayName;
    }).attr('value', function (d) {
      return d.name;
    });
    trialSelect.selectpicker('refresh');
    trialSelect.change(function () {
      updateConditionSelect();
    });
    updateConditionSelect();
  }
  /**
   * Test selection dropdown has changed, updates the condition
   * dropdown.
   */


  function updateConditionSelect() {
    var testName = $('#logged-import-test-select').val();
    var conditionSelect = $('#trial-from-recording-condition-select');
    var options = d3.select(conditionSelect.get(0)).selectAll('option').data(TestDefinitionHandler.getInstance().getTestConditions(testName), function (d) {
      return d;
    });
    options.enter().append('option').text(function (d) {
      return d;
    }).attr('value', function (d) {
      return d;
    });
    options.exit().remove();
    conditionSelect.selectpicker('refresh');
  }
  /**
   * Send off the request from the server to import the data and
   * create the trial.
   * @param {Object} recordingData Object with he list of recordings
   * that are going to be imported and the start and end times.
   */


  function sendRequestForImport(recordingData) {
    recordingData.type = ModelProtocol.import_logged_recording;
    ServerHandler.getInstance().post(recordingData, null, null, function (e) {
      ErrorHandler.showError(e.error().statusText);
    });
  }
  /**
   * Handle messages from the message dispatcher relating to the
   * status of the import.
   * @param {Number} key Key into ResReqMap for data sent.
   */


  function handleMessage() {}
  /**
   * Open the page for viewing and selecting logged recordings
   * for conversion to a trial.
   * @param {Subject} subject The current subject.
   */


  _trialFromLogged.open = function (subject) {
    _subject = subject;
    openImportLoggedPage();
  };

  return _trialFromLogged;
});
//# sourceMappingURL=trial_from_logged.js.map
