"use strict";

/* Copyright (c) 2017-2018 APDM, Inc.
 *
 * trial_notes.js
 *
 * Handle displaying the trial notes in the reports.
 */
define(['jquery', 'd3', 'model_dispatcher', 'server_handler', 'res_req_map', 'crossroads', 'session', 'error_handler'], function ($, d3, ModelDispatcher, ServerHandler, ResReqMap, crossroads, Session, ErrorHandler) {
  'use strict';

  var _trialNotes = {};

  var _trial;

  var _container;
  /**
   * Check the permissions for editing the notes then setup
   * buttons and update the notes field.
   */


  function checkNotePermissions() {
    ModelDispatcher.getInstance().checkPermissions('edit', _trial.getObjId()).then(function (d) {
      setupNotesButtons(d.hasPermission);
      updateTrialNotes(d.hasPermission);
    }, function (e) {
      ErrorHandler.showError(e.error().statusText);
    });
  }
  /**
   * Setup the handlers for the buttons on the notes panel.
   * @param {Boolean} canEdit True if user has permission to
   * edit notes.
   */


  function setupNotesButtons(canEdit) {
    // Give unique ids to the collapsible notes elements
    var baseId = _container.attr('id');

    _container.select('.report-trial-notes').select('a').attr('href', '#collapse-notes-' + baseId);

    _container.select('.report-trial-notes').select('#collapse-notes').attr('id', 'collapse-notes-' + baseId);

    var saveBtn = _container.select('#report-trial-notes-save').classed('disabled', true);

    var cancelBtn = _container.select('#report-trial-notes-cancel').classed('disabled', true);

    var notesField = _container.select('#report-trial-notes-edit');

    if (canEdit) {
      notesField.on('input', function () {
        saveBtn.classed('disabled', false);
        cancelBtn.classed('disabled', false);
      });
      saveBtn.on('click', function () {
        var newNotes = notesField.node().value;

        var objId = _trial.getObjId();

        var requestData = {
          objId: objId,
          notes: newNotes,
          isSession: _trial instanceof Session
        };
        crossroads.parse('dispatch/addTrialNotes/' + ResReqMap.getInstance().setRequestData(requestData));
        saveBtn.classed('disabled', true);
        cancelBtn.classed('disabled', true);
      });
      cancelBtn.on('click', function () {
        notesField.node().value = _trial.getNotes();
        saveBtn.classed('disabled', true);
        cancelBtn.classed('disabled', true);
      });
    }

    if (_trial.getNotes() && _trial.getNotes() !== '') {
      $('#collapse-notes-' + baseId).collapse();
    }
  }
  /**
   * Update the notes field with the current trials notes.
   * @param {Boolean} canEdit True if user can edit trial.
   */


  function updateTrialNotes(canEdit) {
    if (!_container || _container === null || !_trial || _trial === null) {
      return;
    } // Handler notes for sequences differently.


    if (_trial instanceof Session) {
      updateSequenceNotes();
      return;
    }

    _container.select('#report-single-trial-notes').style('display', 'block');

    _container.select('#report-sequence-notes').style('display', 'none');

    var notes = _trial.getNotes();

    var textArea = $(_container.node()).find('#report-trial-notes-edit');
    textArea.val('');

    if (_trial && _trial !== null) {
      textArea.val(notes); // Add text if this happens to be a print view of the report.

      $(_container.node()).find('#print-report-trial-notes').val(notes);
    } // Disable text input if user can't edit


    if (!canEdit) {
      _container.select('#report-trial-notes-edit').attr('disabled', 'disabled'); // Add text if this happens to be a print view of the report.


      _container.select('#print-report-trial-notes').attr('disabled', 'disabled');
    } // Collapse, uncollapse based on content of notes.


    var baseId = _container.attr('id');

    if (notes && notes !== '') {
      $('#collapse-notes-' + baseId).collapse('show');
    } else {
      $('#collapse-notes-' + baseId).collapse('hide');
    }
  }
  /**
   * Update the notes field for a sequence.
   */


  function updateSequenceNotes() {
    _container.select('#report-single-trial-notes').style('display', 'none');

    _container.select('#report-sequence-notes').style('display', 'block');

    var notes = _trial.getNotes();

    if (!notes.length) {
      _container.select('#report-sequence-notes').style('display', 'none');

      return;
    }

    _container.select('#report-sequence-notes-body').selectAll('div').remove();

    _container.select('#report-sequence-notes-body').selectAll('div').data(notes).enter().append('div').each(function (d) {
      var div = d3.select(this);
      div.append('h4').text(d.testName + ', ' + d.conditionName);
      div.append('div').text(d.notes);
    });
  }
  /**
   * Initialize the notes field for a trial report.
   * @param {Trial} trial The trial being displayed.
   * @param {d3.selection} container The report's DOM container.
   */


  _trialNotes.init = function (trial, container) {
    _trial = trial;
    _container = container;
    checkNotePermissions();
  };

  return _trialNotes;
});
//# sourceMappingURL=trial_notes.js.map
