"use strict";

/* Copyright (c) 2016-2018, APDM Inc.
 *
 * trial_timer.js
 *
 * Object handling trial timers.
 */
define(['jquery', 'server_handler', 'model_protocol', 'app/ui_state'], function ($, ServerHandler, ModelProtocol, uiState) {
  'use strict';
  /**
   * Constructor for timer.
   */

  function TrialTimer() {
    this._startDelay = 0;
    this._duration = 0;
    this._currTimer = null;
    this._durationDone = 0;
    this._indeterminate = true;
    this._testEndCb = null;
    this._startDelayDone = true;
    this._timerLabelText = null;
    this._timerValueText = null;
    this._timerStarted = false;
    this._timerStopped = false;
  }
  /**
   * Set or return the trial's start delay.
   * @param d The start delay in seconds.
   */


  TrialTimer.prototype.startDelay = function (d) {
    if (!arguments.length) {
      return this._startDelay;
    }

    this._startDelay = d;

    if (this._startDelay > 0) {
      this._startDelayDone = false;
    }

    return this;
  };
  /**
   * Set or return the duration of the trial. A value of 0 is determined to be
   * "_Indeterminate".
   * @param d The trial delay.
   */


  TrialTimer.prototype.duration = function (d) {
    if (!arguments.length) {
      return this._duration;
    }

    this._duration = d;
    return this;
  };
  /**
   * Set the timer's label text.
   * @param {String} d The label for the timer.
   * @return Current label if called with no arguments, otherwise
   * this.
   */


  TrialTimer.prototype.timerLabelText = function (d) {
    if (!arguments.length) {
      return this._timerLabelText;
    }

    this._timerLabelText = d;
    return this;
  };
  /**
   * Set the timer's text.
   * @param {String} d The text for the timer.
   * @return Current text if called with no arguments, otherwise
   * this.
   */


  TrialTimer.prototype.timerValueText = function (d) {
    if (!arguments.length) {
      return this._timerValueText;
    }

    this._timerValueText = d;
    return this;
  };
  /**
   * Set or return the listener.
   * @param d The function to call when test ends.
   */


  TrialTimer.prototype.testEndCb = function (d) {
    if (!arguments.length) {
      return this._testEndCb;
    }

    this._testEndCb = d;
    return this;
  };
  /**
   * Start the timer.
   * @return {TrialTimer} this.
   */


  TrialTimer.prototype.start = function () {
    if (this._timerStarted) {
      return this;
    }

    var self = this;
    this._durationDone = false;
    this._timerStarted = true;

    if (this._duration > 0) {
      this._indeterminate = false;
    }

    ServerHandler.getInstance().postUIState(uiState.RUNNING_TEST);
    this._currTimer = setInterval(function () {
      self.update(self);
    }, 1000); // Add a short delay to "debounce" the button.

    setTimeout(enableStopBtn, 100);

    function enableStopBtn() {
      var startBtn = $('#start-test-button');
      startBtn.removeClass('btn-success').addClass('btn-danger');
      startBtn.html('<span class="iconic iconic-x"></span>' + '\nStop\n</button>');
      startBtn.attr('disabled', null);
      startBtn.off('click');
      startBtn.on('click', function () {
        ServerHandler.getInstance().postUIState(uiState.STOPPING_TEST);
        self.stop();
        startBtn.attr('disabled', 'disabled');
      });
    }

    return this;
  };
  /**
   * Stop the timer.
   */


  TrialTimer.prototype.stop = function () {
    if (this._timerStopped) {
      return this;
    }

    var server = ServerHandler.getInstance();
    this._timerStopped = true;
    clearInterval(this._currTimer);
    this._durationDone = true;
    server.post({
      type: ModelProtocol.play_end_beep
    }, null, null, null);

    if (this._indeterminate) {
      server.postUIState(uiState.WAITING_FOR_ANALYSIS);
    }

    if (this._testEndCb && this._testEndCb instanceof Function) {
      this._testEndCb();
    }

    if (ServerHandler.getInstance().hasJsPostHandler()) {
      var beep = new Audio('/mobility_clinic/res/audio/beep.mp3');
      beep.play();
    }

    return this;
  };
  /**
   * Cancel the timer.
   */


  TrialTimer.prototype.cancel = function () {
    if (this.currTrialTimer.Prototype !== null) {
      clearInterval(this._currTimer);
      ServerHandler.getInstance().postUIState(uiState.WAITING_FOR_ANALYSIS);
    }

    return this;
  };
  /**
   * Update the timer, and fire off an event so listeners can update
   * as well.
   */


  TrialTimer.prototype.update = function () {
    if (this._startDelayDone === false && this._startDelay >= 0) {
      this._startDelay--;

      if (this._startDelay > 0) {
        this.updateTimerText(this._startDelay);
      } else {
        this._startDelayDone = true;
        this.updateTimerText(this._duration);

        if (this._indeterminate === true) {
          this._timerLabelText.text('Duration: ');
        } else {
          this._timerLabelText.text('Remaining: ');
        }

        if (ServerHandler.getInstance().hasJsPostHandler()) {
          var beep = new Audio('/mobility_clinic/res/audio/beep.mp3');
          beep.play();
        }
      }
    } else if (this._durationDone === false) {
      if (this._indeterminate === true) {
        this._duration++;
      } else {
        this._duration--;

        if (this._duration <= 0) {
          this._durationDone = true;
          this.stop();
        }
      }

      this.updateTimerText(this._duration);
    }

    return this;
  };
  /**
   * Update the timer text.
   * @param {String} valueText The text to display in timer.
   */


  TrialTimer.prototype.updateTimerText = function (valueText) {
    this._timerValueText.text(valueText);

    if (valueText < 0) {
      this._timerValueText.classed('text-success', false).classed('text-danger', true);
    } else {
      this._timerValueText.classed('text-success', true).classed('text-danger', false);
    }

    return this;
  };

  return TrialTimer;
});
//# sourceMappingURL=trial_timer.js.map
