"use strict";

/* Copyright (c) 2017-2018 APDM, Inbc.
 *
 * upload_handler.js
 *
 * Determine if instance is configured to upload to HTTP and/or
 * Mobility Exchange and call the correct the functions.
 *
 */
define(['server_handler', 'error_handler', 'model_protocol', 'bootstrap-dialog', 'app/exchange_handler', 'crossroads', 'res_req_map'], function (ServerHandler, ErrorHandler, ModelProtocol, BootstrapDialog, ExchangeHandler, crossroads, ResReqMap) {
  var _instance;

  var _exchange;

  var _messageRoute;

  var _doLogout;

  var DATA_UPLOAD_COMPLETE = 'DATA_UPLOAD_COMPLETE';
  var ERROR = 'ERROR';
  /**
   * Initialize the upload instance.
   */

  function init() {
    _exchange = ExchangeHandler.getInstance(); // Route for getting messages from the server regarding the upload status.

    _messageRoute = crossroads.addRoute('message/DATA_UPLOAD/{key}', handleMessage);
    return {
      /**
       * Check for files to upload, and upload them
       * after asking user for conformation.
       */
      uploadTrials: uploadTrials
    };
  }
  /**
   * Upload any trials that have not yet been uploaded.
   * @param {Boolean} showNoTrials If true, dialog will be displayed
   * if there are no trials to show.
   * @param {Boolean} doLogout Logout of MX after completing the
   * upload.
   */


  function uploadTrials(showNoTrials, doLogout) {
    _doLogout = doLogout ? doLogout : false;
    ServerHandler.getInstance().post({
      type: ModelProtocol.get_num_trials_for_upload
    }).then(function (d) {
      var numExchangeUpload = parseInt(d.numExchangeUpload, 10);
      var numHttpUpload = parseInt(d.numHttpUpload, 10);
      var numPendingTransforms = parseInt(d.numPendingTransforms, 10);
      var isOnline = d.isOnline;
      determineDialog(numExchangeUpload, numHttpUpload, numPendingTransforms, isOnline, showNoTrials, _doLogout);
    }, function (d) {
      ErrorHandler.showError(d.error().statusText);
    });
  }
  /**
   * Determine what dialog to display to the user based on the
   * values that were returned from the servlet.
   * @param {Number} numExchangeUpload Number of trials to upload to
   * MX.
   * @param {Number} numHttpUpload Number of trials to upload to the
   * HTTP backup site.
   * @param {Number} numPendingTransforms Number of domain
   * transforms that have not yet been passed to the MX server.
   * @param {Boolean} isOnline True if app is online.
   * @param {Boolean} showNoTrials If true, dialog will be shown
   * if there are not trials to upload.
   */


  function determineDialog(numExchangeUpload, numHttpUpload, numPendingTransforms, isOnline, showNoTrials) {
    if (numExchangeUpload <= 0 && numHttpUpload <= 0 && numPendingTransforms <= 0) {
      if (showNoTrials === true) {
        noTrialsToUpload(_doLogout);
        return;
      }

      if (_doLogout) {
        _exchange.logout();
      }

      return;
    }

    if (numExchangeUpload <= 0 && numHttpUpload <= 0 && numPendingTransforms > 0 && !isOnline) {
      pendingTransforms(_doLogout);
      return;
    } // There are trials that need to be uploaded


    if (!isOnline) {
      offlineWithTrials(numExchangeUpload, numHttpUpload, numPendingTransforms);
    } else if (numExchangeUpload > 0 || numHttpUpload > 0) {
      // Should we do something for unpushed transforms?
      onlineWithTrials(numExchangeUpload, numHttpUpload);
    }
  }
  /**
   * Display a dialog telling the user they have pending transforms.
   */


  function pendingTransforms() {
    var closeBtnText = 'Close';

    if (_doLogout) {
      closeBtnText = 'Cancel';
    }

    var buttons = [{
      id: 'pending-transforms-close-btn',
      icon: 'iconic iconic-x',
      label: closeBtnText,
      cssClass: 'btn-link',
      action: function action(dlg) {
        dlg.close();
      }
    }];

    if (_doLogout && _exchange.associatedWithServer()) {
      buttons.push({
        id: 'pending-transforms-continue-btn',
        icon: 'iconic iconic-data-transfer-upload',
        label: 'Continue',
        cssClass: 'btn-success',
        action: function action(dialog) {
          _exchange.logout();

          dialog.close();
        }
      });
    }

    BootstrapDialog.show({
      id: 'pending-transforms-dialog',
      title: 'Pending Mobility Exchange edits',
      message: '<p>You are currently offline and there are still ' + 'pending edits that have not been pushed to Mobility ' + 'Exchange.</p><p>These edits will be pushed when you ' + 'reconnect to the internet.</p>',
      buttons: buttons
    });
  }
  /**
   * Do dialog for with trials that need to be uploaded but
   * application is not connected to the internet.
   * @param {Number} numEx Number of trials to upload to MX.
   * @param {Number} numHttp Number of trial to upload to the Http
   * backup site.
   * @param {Number} numTransforms Number of pending transforms.
   */


  function offlineWithTrials(numEx, numHttp, numTransforms) {
    var msg = '<p><strong>You are currently working offline, and nothing will be uploaded.</strong></p>' + '<p>There are trials that have not been uploaded.</p>';

    if (numEx > 0 && _exchange.associatedWithServer()) {
      msg = msg + '<p>Number of trials to upload to Mobility Exchange: ' + numEx + '</p>';
    }

    if (numHttp > 0) {
      msg = msg + '<p>Number of trials to upload to HTTP backup site: ' + numHttp + '</p>';
    }

    if (numTransforms > 0) {
      msg = msg + '<p>There are pending edits that have not been ' + 'pushed to Mobility Exchange.</p>';
    }

    msg = msg + '<p>To upload these trials, please make sure you are ' + 'connected to the internet and try again.</p>';
    BootstrapDialog.show({
      id: 'upload-trials-offline-dialog',
      title: 'Upload trials',
      message: msg,
      buttons: [{
        id: 'upload-trials-offline-cancel-btn',
        icon: 'iconic iconic-x',
        label: 'Cancel',
        cssClass: 'btn-link',
        action: function action(dlg) {
          dlg.close();
        }
      }, {
        id: 'upload-trials-offline-continue-btn',
        icon: 'iconic iconic-data-transfer-upload',
        label: 'Continue',
        cssClass: 'btn-success',
        action: function action(dialog) {
          if (_doLogout && _exchange.associatedWithServer()) {
            _exchange.logout();
          }

          dialog.close();
        }
      }]
    });
  }
  /**
   * Display dialog for trials to upload and application online.
   * @param {Number} numEx Number of trials to upload to MX.
   * @param {Number} numHttp Number of trial to upload to the Http
   * backup site.
   */


  function onlineWithTrials(numEx, numHttp) {
    var msg = '<p>There are trials that have not been uploaded.</p>';

    if (numEx > 0 && _exchange.associatedWithServer()) {
      msg = msg + '<p>Number of trials to upload to Mobility Exchange: ' + numEx + '</p>';
    }

    if (numHttp > 0) {
      msg = msg + '<p>Number of trials to upload to HTTP backup site: ' + numHttp + '</p>';
    }

    msg = msg + '<p>Do you want to upload the trials now?</p>';
    var buttons = [{
      id: 'upload-trials-online-cancel-btn',
      icon: 'iconic iconic-x',
      label: 'Cancel',
      cssClass: 'btn-link',
      action: function action(dlg) {
        dlg.close();
      }
    }];

    if (_doLogout && _exchange.associatedWithServer()) {
      buttons.push({
        id: 'upload-trials-online-continue-btn',
        icon: 'iconic iconic-caret-right',
        label: 'Continue',
        cssClass: 'btn-success',
        action: function action(dlg) {
          if (_doLogout && _exchange.associatedWithServer()) {
            _exchange.logout();
          }

          dlg.close();
        }
      });
    }

    buttons.push({
      id: 'upload-trials-online-upload-btn',
      icon: 'iconic iconic-data-transfer-upload',
      label: 'Upload',
      cssClass: 'btn-success',
      action: function action(dialog) {
        ServerHandler.getInstance().post({
          type: ModelProtocol.upload_trials
        }, null, null, function (d) {
          ErrorHandler.showError(d.error().statusText);
        });
        dialog.close();
      }
    });
    BootstrapDialog.show({
      id: 'upload-trials-online-dialog',
      title: 'Upload trials',
      message: msg,
      buttons: buttons
    });
  }
  /**
   * Display dialog informing user there are no trials to upload.
   */


  function noTrialsToUpload() {
    BootstrapDialog.show({
      title: 'No trials to upload',
      message: 'There are currently no trials to upload.',
      buttons: [{
        icon: 'iconic iconic-x',
        label: 'Close',
        cssClass: 'btn-link',
        action: function action(dlg) {
          dlg.close();

          if (_doLogout) {
            _exchange.logout();
          }
        }
      }]
    });
  }
  /**
   * Handle the messages received from the servlet.
   * @param {String} key Key into the ReReqMap holding message
   * details.
   */


  function handleMessage(key) {
    var message = ResReqMap.getInstance().getResponseData(key);

    if (message === DATA_UPLOAD_COMPLETE) {
      // Logout if necessary.
      if (_doLogout && _exchange.associatedWithServer()) {
        _exchange.logout();
      }
    } else if (message === ERROR) {// Do nothing here, error currently being handled by SWT
      // dialogs
    }
  }

  return {
    getInstance: function getInstance() {
      if (!_instance) {
        _instance = init();
      }

      return _instance;
    }
  };
});
//# sourceMappingURL=upload_handler.js.map
