/* Copyright (c) 2017 APDM, Inc.
 *
 * add_edit_study.js
 *
 * Module to handle adding and editing a Studies.
 */

define(['jquery', 'crossroads', 'model_dispatcher', 'res_req_map',
        'app/user_options_handler', 'app/exchange_handler', 'error_handler',
        'bootstrap-dialog', 'app/delete_handler'],
       function($, crossroads, ModelDispatcher, ResReqMap, UserOptionsHandler,
                ExchangeHandler, ErrorHandler, BootstrapDialog, deleteHandler)
{
    'use strict';

    var _studyNames;
    var _isEdit = false;
    var _currentStudy = null;
    var _dialog;

    /**
     * Add a Study.
     * @param {Array} studyNames The list of Study names currently in use.
     */
    function addStudy(studyNames) {
        _isEdit = false;
        _studyNames = studyNames;
        openDialog('Add New Subject Group');
    }

    /**
     * Edit the name, or delete the currently selected study.
     * @param {Array} studyNames The list of study names currently in use.
     */
    function editStudy(studyNames) {
        _isEdit = true;
        _studyNames = studyNames;
        openDialog('Edit Subject Group');
    }

    /**
     * Open the add/edit dialog.
     * @param {String} title The title for the dialog.
     */
    function openDialog(title) {
        var btnList = generateButtons();
        _currentStudy = ModelDispatcher.getInstance().getCurrentStudy();
        _dialog = new BootstrapDialog({
            title: title,
            message: $('<div></div>')
                .load('/mobility_clinic/html/add_edit_study.html'),
            closable: false,
            buttons: btnList,
            onshown: function() {
                if (_isEdit) {
                    $('#add-edit-study-input').val(_currentStudy.getStudyName());
                }
            },
        });
        _dialog.open();
    }

    /**
     * Create the list of buttons for the dialog footer.
     * @return {Array} The list of buttons for the footer.
     */
    function generateButtons() {
        var btns = [{
            id: 'add-edit-study-close',
            icon: 'iconic iconic-x',
            label: 'Close',
            cssClass: 'btn-link',
            action: function() { closeDialog(); }
        }];

        if (_isEdit) {
            btns.push({
                id: 'add-edit-study-delete',
                icon: 'iconic iconic-trash',
                label: 'Delete',
                cssClass: 'btn-default',
                action: function(dialog) {
                    deleteHandler.deleteStudy(_currentStudy.getStudyName(),
                                              function() {
                                                  deleteStudyCallback(dialog);
                                              });
                }
            });
        }

        btns.push({
            id: 'add-edit-study-save',
            icon: 'iconic iconic-check',
            label: 'Add',
            cssClass: 'btn-success',
            action: function(dialog) {
                var inputBox = $('#add-edit-study-input');
                var studyName = inputBox.val();
                if (validateStudyName(studyName, inputBox) === false) {
                    return;
                }
                crossroads.parse('dispatch/addEditStudy/' +
                                 _isEdit + '/' + ResReqMap.getInstance()
                                 .setRequestData(studyName));
                UserOptionsHandler.getInstance().setCurrentStudyName(studyName);
                dialog.close();
            }
        });

        var exchangeHandler = ExchangeHandler.getInstance();
        if (_studyNames.length === 0 &&
            !exchangeHandler.doesLocalDataExist() &&
            !exchangeHandler.associatedWithServer() &&
            (!exchangeHandler.updateHost() ||
             exchangeHandler.updateHost().length === 0)) {
            // Only provide the connect dialog if the update host is empty.
            // This means we are in a debug environment.
            btns.push({
                id: 'add-edit-study-login',
                icon: 'iconic iconic-account-login',
                label: 'Connect',
                cssClass: 'btn-default',
                action: function() {
                    exchangeHandler.setupNewServerConnection();
                }
            });
        }
        if (_studyNames.length === 0) {
            btns.push({
                id: 'add-edit-study-import',
                icon: 'iconic iconic-data-transfer-download',
                label: 'Import',
                cssClass: 'btn-default',
                action: function() {
                    crossroads.parse('dispatch/importStudy/');
                }
            });
        }

        return btns;
    }

    /**
     * Callback to pass to handler when study is deleted. Makes
     * sure to update the ExchangeHandler status for the current
     * user.
     * @param {BootstrapDialog} dialog The dialog to close when
     * callback succeeds.
     */
    function deleteStudyCallback(dialog) {
        var exchangeRoute =
            crossroads.addRoute('exchangeStatusLoaded',function() {
                crossroads.removeRoute(exchangeRoute);
                dialog.close();
            });
        ExchangeHandler.getInstance().refreshExchangeStatus();
    }

    /**
     * Check to make sure there is at least one Study before we allow
     * the dialog to be closed.
     * @return {Boolean} True if dialog can close, otherwise false.
     */
    function closeDialog() {
        if (!ModelDispatcher.getInstance().getCurrentStudy()) {
            ErrorHandler.showError('Please create a new subject group.');
            return false;
        }
        _dialog.close();
        return true;
    }

    /**
     * Validate the given study name. Make sure it exists, and it does
     * not conflict with the name of an existing study. Also sets
     * error messages in input box.
     * @param {String} studyName The name for the study.
     * @param {jQuery.selection} inputBox The input box holding the study name.
     * @return {Boolean} True if valid, otherwise false.
     */
    function validateStudyName(studyName, inputBox) {
        if (!studyName || studyName === '') {
            inputBox.attr('placeholder', 'Subject group name is required');
            inputBox.parent().parent().addClass('has-error');
            return false;
        } else if (_isEdit === true &&
                   studyName === _currentStudy.getStudyName()) {
            return true;
        } else if (_studyNames.indexOf(studyName) > -1) {
            inputBox.val('');
            inputBox.attr('placeholder', 'Subject group name already in use');
            inputBox.parent().parent().addClass('has-error');
            return false;
        }
        return true;
    }

    /**
     * Just close the dialog, without doing any check on existing studies.
     */
    function forceCloseDialog() {
        if (_dialog) {
            _dialog.close();
        }
    }

    return {
        add: addStudy,
        edit: editStudy,
        forceCloseDialog: forceCloseDialog
    };
});
