/* Copyright (c) 2016-2018 APDM, Inc.
 *
 * add_edit_subject.js
 *
 * Module for handling adding or editing a subject.
 */

define(['jquery', 'd3', 'crossroads', 'res_req_map', 'bootstrap-dialog',
        'subject', 'model_dispatcher', 'error_handler', 'bootstrap-select',
        'app/user_options_handler', 'app/exchange_handler', 'moment'],
       function($, d3, crossroads, ResReqMap, BootstrapDialog,
                Subject, ModelDispatcher, ErrorHandler, BootstrapSelect,
                OptionsHandler, ExchangeHandler, moment)
{

    var dialog;
    var isEdit = false;
    var currentSubject = null;
    var doConstraints = false;
    var required;
    var canEdit = true;

    /**
     * HTML ids for the constraint inputs.
     */
    var CONSTRAINTS = [
        'head','shoulder','chest', 'elbow', 'wrist',
        'fingertips', 'waist', 'knee', 'ankle'
    ];

    // The default percentages to use for the contstraints given the
    // height of a subject.
    var CONSTRAINT_DEFAULTS = {
        Male: {
            head: 1.0,          // FIXME: Should we be using top of head.
            shoulder: 0.870,
            chest: 0.725,
            elbow: 0.630,
            wrist: 0.485,
            fingertips: 0.357,
            waist: 0.530,
            knee: 0.285,
            ankle: 0.043
        },

        Female: {
            head: 1.0,          // FIXME: Should we be using top of head
            shoulder: 0.875,
            chest: 0.725,       // FIXME: This value not given in documentation
            elbow: 0.632,
            wrist: 0.480,
            fingertips: 0.370,
            waist: 0.524,
            knee: 0.282,
            ankle: 0.043
        }
    };

    /**
     * Add a new subject.
     * @param {Boolean} constraints True biophysical constraints
     * should be included with subject.
     */
    function addSubject(constraints) {
        isEdit = false;
        buildDialog(constraints);
    }

    /**
     * Edit an existing subject.
     * @param {Subject} subject The subject to edit.
     * @param {Boolean} constraints True biophysical constraints
     * should be included with subject.
     */
    function editSubject(subject, constraints) {
        isEdit = true;
        currentSubject = subject;
        ModelDispatcher.getInstance()
            .checkPermissions('edit', subject.getObjId())
            .then(function(d) {
                canEdit = d.hasPermission;
                buildDialog(constraints);
            }, function(e) {
                ErrorHandler.showError(e.error().statusText);
            });
    }

    /**
     * Build the dialog.
     * @param {Boolean} constraints True biophysical constraints
     * should be included with subject.
     */
    function buildDialog(constraints) {
        var title = isEdit ? 'Edit Subject' : 'Add Subject';
        doConstraints = constraints ? constraints : false;
        var html = '/mobility_clinic/html/add_edit_subject.html';
        var size = BootstrapDialog.SIZE_NORMAL;
        if (doConstraints) {
            html = '/mobility_clinic/html/add_edit_subject_constraints.html';
            size = BootstrapDialog.SIZE_WIDE;
        }

        dialog = new BootstrapDialog({
            id: 'add-edit-subject-dialog',
            size: size,
            title: title,
            closable: false,
            message: $('<div></div>').load(html),
            cssClass: 'add-subject-dialog',
            buttons: [
                {
                    id: 'add-edit-subject-close-btn',
                    icon: 'iconic iconic-x',
                    label: 'Close',
                    cssClass: 'btn-link',
                    action: function(dialog) { dialog.close(); }
                },
                {
                    id: 'add-edit-subject-save-btn',
                    icon: 'iconic iconic-check',
                    label: 'Save',
                    cssClass: 'btn-success',
                    action: saveSubject
                }
            ],
            onshown: dialogShown
        });
        required = OptionsHandler.getInstance().getPatientRequiredFields();
        dialog.open();
    }

    /**
     * Initialize elements and handlers when dialog HTML
     * is loaded.
     */
    function dialogShown() {
        $('.selectpicker').selectpicker('refresh');
        if (isEdit) {
            populateForEdit();
            if (!canEdit) {
                $('#add-edit-subject-container').find('input')
                    .attr('disabled', 'disabled');
                $('#add-edit-subject-save-btn').attr('disabled', 'disabled');
            }
        }
        $('#populate-defaults-button').click(setConstraintDefaults);
        removeUnusedSubjectFields();
    }

    /**
     * Populate the input boxes in dialog with existing subject data
     * if this is an edit.
     */
    function populateForEdit() {
        $('#public-id').val(currentSubject.getPublicId());
        $('#first-name').val(currentSubject.getFirstName());
        $('#last-name').val(currentSubject.getLastName());
        $('#height-input').val(currentSubject.getHeight());
        $('#subject-notes').val(currentSubject.getNotes());

        // Handle the select picker for the gender.
        $('.selectpicker').selectpicker('val', currentSubject.getGender());
        $('.selectpicker').selectpicker('refresh');

        var date = currentSubject.getDateOfBirthObj();
        if (date !== null && date.isValid()) {
            $('#year-of-birth').val(date.year());
            $('#month-of-birth').val(date.month() + 1);
            $('#day-of-birth').val(date.date());
        }

        var constraintMap = currentSubject.getConstraintMap();
        CONSTRAINTS.forEach(function(d) {
            $('#' + d + '-constraint').val(constraintMap.get(d + 'Height'));
        });
    }

    /**
     * Save the subject data.
     */
    function saveSubject() {
        if (validateSubject() === false) {
            return;
        }

        var requestData = {
            publicId : $('#public-id').val(),
            firstName : $('#first-name').val(),
            lastName : $('#last-name').val(),
            height: $('#height-input').val(),
            gender: $('#gender-select').val()
        };

        var dob = getDateOfBirth();
        requestData.dob = null;
        if (required.birthYear === true) {
            requestData.dob = dob.format('YYYY-MM-DD');
        }

        requestData.notes = $('#subject-notes').val();

        // Get the constraints and add to the request data
        CONSTRAINTS.forEach(function(d) {
            if ($('#' + d + '-constraint').length) {
                requestData[d + 'Height'] = $('#' + d + '-constraint').val();
            } else {
                requestData[d + 'Height'] = null;
            }
        });

        if (isEdit === true) {
            var objId = currentSubject.getObjId();
            var query = '?id=' + objId.getId() + '&localId=' +
                objId.getLocalId();
            requestData.id = objId.getId();
            requestData.localId = objId.getLocalId();
            crossroads.parse('dispatch/editSubject/' +
                             ResReqMap.getInstance()
                             .setRequestData(requestData) + query);
        } else {
            crossroads.parse('dispatch/addSubject/' +
                             ResReqMap.getInstance()
                             .setRequestData(requestData));
        }
        dialog.close();
    }

    /**
     * Validate the fields for the subject.
     */
    function validateSubject() {
        var publicId = $('#public-id');
        var height = $('#height-input');
        var valid = true;
        var errorMsg = [];

        // Remove existing error classes
        d3.select('#add-edit-subject-container').selectAll('.has-error')
            .classed('has-error', false);

        valid = validatePublicId(publicId, errorMsg);

        if (required.height && isNaN(parseInt(height.val(), 10))) {
            valid = false;
            height.parent().parent().parent().addClass('has-error');
            height.attr('placeholder', 'Height not a valid number');
            errorMsg.push('Height must be a valid integer.');
        }

        if (doConstraints) {
            var invalidConstraint = false;
            CONSTRAINTS.forEach(function(d) {
                var inputEle = $('#' + d + '-constraint');
                if (isNaN(parseInt(inputEle.val(), 10))) {
                    valid = false;
                    invalidConstraint = true;
                    inputEle.parent().addClass('has-error');
                }
            });
            if (invalidConstraint) {
                errorMsg.push('There are invalid biophysical constraints');
            }
        }

        var dob = getDateOfBirth();
        if (required.birthYear === true && dob === null) {
            errorMsg.push('Date of birth is invalid.');
            valid = false;
        }

        if (valid === false) {
            var displayMsg = '';
            errorMsg.forEach(function(d) {
                displayMsg = displayMsg + '<p>' + d + '</p>';
            });
            ErrorHandler.showError(displayMsg);
        }
        return valid;
    }

    /**
     * Validate the public id input from user.
     * @param {jQuery.selection} idInput The input field for the id.
     * @param {Array} errorMsg Array to store any error messages
     * to display to the user.
     * @return True if id is valid, otherwise false.
     */
    function validatePublicId(idInput, errorMsg) {
        var valid = true;
        var subjectIdRegEx = OptionsHandler.getInstance().getSubjectIdRegEx();
        var enteredId = idInput.val();

        if (required.id === true && enteredId.length <= 0) {
            valid = false;
            idInput.parent().parent().addClass('has-error');
            idInput.attr('placeholder', 'Public ID is required');
            errorMsg.push('Public ID is required');
            return valid;
        } else if (required.id === false && isEdit === false) {
            // Generate id from time if none is supplied, and this is
            // a new subject.
            idInput.val((new Date()).getTime());
        }

        // Determine if the idInput is unique.
        var newId = ModelDispatcher.getInstance().getCurrentStudy()
            .getSubjectByPublicId(enteredId);
        if (isEdit === true && newId) {
            if (!currentSubject.getObjId().equals(newId.getObjId())) {
                idInput.parent().parent().addClass('has-error');
                $('#add-subject-id-error').css('display', 'block');
                valid = false;
                errorMsg.push('The subject with Public ID "' + enteredId +
                              '" already exists.');
            }
        } else if (newId) {
            idInput.parent().parent().addClass('has-error');
            valid = false;
            errorMsg.push('The subject with Public ID "' + enteredId +
                          '" already exists.');
        }

        // Validate against regex if supplied.
        if (subjectIdRegEx !== null) {
            // Add the site to the regex string if it exists.
            var site = ExchangeHandler.getInstance().getSiteName();
            if (site !== null && site !== '') {
                subjectIdRegEx.regexStr = subjectIdRegEx.regexStr
                    .replace('%SITE_ID%', site);
            } else {
                subjectIdRegEx.regexStr = subjectIdRegEx.regexStr
                    .replace('%SITE_ID%', '');
            }

            var idRegEx = new RegExp(subjectIdRegEx.regexStr);
            if (idRegEx.test(enteredId) === false) {
                idInput.parent().parent().addClass('has-error');
                valid = false;
                errorMsg.push('The Public ID "' + enteredId +
                              '" does not match required format.');
                errorMsg.push(subjectIdRegEx.regexMessage);
            }
        }

        return valid;
    }

    /**
     * Get the date of birth entered for the subject.
     * @return {moment} The parsed date. Will be null if bad date
     * value encountered.
     */
    function getDateOfBirth() {
        var dob = new moment('1900-01-01', 'YYYY-MM-DD');
        var year = $('#year-of-birth');
        var month = $('#month-of-birth');
        var day = $('#day-of-birth');

        if (!required.birthYear) {
            return dob;
        }

        if (required.birthYear) {
            if (year.val() < 1900) {
                $('#year-of-birth-row').addClass('has-error');
                return null;
            } else {
                dob.set('year', year.val());
            }
        }

        if (required.birthMonth) {
            if (month.val() > 12 || month.val() < 1) {
                $('#month-of-birth-row').addClass('has-error');
                return null;
            } else {
                dob.set('month', month.val() - 1);
            }
        }

        if (required.birthDay) {
            if (day.val() > dob.daysInMonth() || day.val() < 1) {
                $('#day-of-birth-row').addClass('has-error');
                return null;
            } else {
                dob.set('date', day.val());
            }
        }

        return dob;
    }

    /**
     * Set the constraint defaults based on the given height value.
     * @param {Number} height The height of the subject.
     */
    function setConstraintDefaults() {
        var heightInput = $('#height-input');
        var height = parseInt(heightInput.val(), 10);
        if (isNaN(height)) {
            heightInput.parent().parent().parent().addClass('has-error');
            heightInput.attr('placeholder', 'Height not a valid number');
            ErrorHandler.showError('<p>Height must be a valid integer.');
            return;
        }

        // Default to 'Male' if the gender is either 'Intersex' or
        // 'Unspecified'
        var gender = $('#gender-select').val();
        if (gender !== 'Male' && gender !== 'Female') {
            gender = 'Male';
        }

        CONSTRAINTS.forEach(function(d) {
            $('#' + d + '-constraint')
                .val(height * CONSTRAINT_DEFAULTS[gender][d]);
        });
    }

    /**
     * Remove fields from the dialog based on the required fields in the subject
     * options.
     */
    function removeUnusedSubjectFields() {
        var d3Dialog = d3.select(dialog.getModalBody().get(0));
        if (required.id === true) {
            d3Dialog.select('#subject-id-row').select('input')
                .attr('readonly', null);
        }
        if (required.name === true) {
            d3Dialog.select('#subject-first-name-row').select('input')
                .attr('readonly', null);
            d3Dialog.select('#subject-last-name-row').select('input')
                .attr('readonly', null);
        }
        if (required.height === true) {
            d3Dialog.select('#height-row').select('input')
                .attr('readonly', null);
        }
        if(required.gender === true) {
            var select = $(d3Dialog.select('#gender-row')
                           .select('#gender-select').node());
            select.prop('disabled', null);
            select.selectpicker('refresh');
        }

        // Enable individual birthday fields, but require the the "parent"
        // time before enabling children
        if (required.birthYear === true) {
            d3Dialog.select('#year-of-birth-row').select('input')
                .attr('readonly', null);
            if (required.birthMonth === true) {
                d3Dialog.select('#month-of-birth-row').select('input')
                    .attr('readonly', null);
                if (required.birthDay === true) {
                    d3Dialog.select('#day-of-birth-row').select('input')
                        .attr('readonly', null);
                }
            }
        }
    }

    /**
     * Return the interface to this module.
     */
    return {
        add: addSubject,
        edit: editSubject
    };

});
