/* Copyright (c) 2015-2018 APDM, Inc.
 *
 * add_edit_test_condition.js
 *
 * Handle the add/edit condition dialog and the attendent
 * logic.
 */

define(['jquery', 'd3', 'crossroads', 'bootstrap-dialog',
        'model_protocol', 'server_handler', 'error_handler',
        'test_definition_handler', 'res_req_map', 'app/user_options_handler',
        'metric_definition_helper', 'app/license_handler'],
       function($, d3, crossroads, BootstrapDialog, ModelProtocol,
                ServerHandler, ErrorHandler, TestDefinitionHandler,
                ResReqMap, UserOptionsHandler, MetricDefinitionHelper,
                LicenseHandler)
{

    'use strict';

    var editor = {};

    var testOptions;
    var editorDialog;
    var audioCues = [];
    var canEditAdd;
    var _disabledTestConditions = {};

    // NOTE: Need crossroad module available globally, so can be called by
    // server code.
    window.crossroads = crossroads;

    /**
     * Setup elements in the dialog after it's shown.
     */
    function setupDialog() {
        var name = testOptions.testName;
        if (!testOptions.isNewTestType) {
            name = testOptions.testDisplayName;
        }
        $('#dialog-add-edit-condition-test-name').text(name);

        $('.colorpicker-group').colorpicker();
        $('#add-test-type-start-delay').slider();
        $('#add-test-type-start-delay').on('change', function(slideEvt) {
            $('#add-test-type-start-delay-value').text(slideEvt.value.newValue);
        });

        // Are we doing an add or edit?
        if (testOptions.conditionName && testOptions.conditionName !== '') {
            initForEdit();
        } else {
            initForAdd();
        }

        // Determine if we should build the audio cue portion of the dialog.
        if (LicenseHandler.getInstance().isResearchView()) {
            buildAudioCueTable();
        } else {
            $('.condition-cue-container').css('display', 'none');
        }
    }

    /**
     * Initialize the dialog for editing a condition.
     */
    function initForEdit() {
        testOptions.type = ModelProtocol.edit_test_definition;
        var defHandler = TestDefinitionHandler.getInstance();
        var testDef = defHandler.getTestDefinition(testOptions.testName,
                                                   testOptions.conditionName);
        var defaultConditions =
            defHandler.getDefaultTestConditions(testOptions.testName);
        audioCues = testDef.audioCues;

        if (defaultConditions[testOptions.conditionName] || !canEditAdd) {
            editorDialog.setTitle('View Condition');
            $('#edit-default-warning')
                .append('<h4 class="text-warning">Note: ' +
                        'Edits to default conditions ' +
                        'will not be saved');
        } else {
            editorDialog.setTitle('Edit Condition');
            defaultNormsMessage();
        }
        $('#new-condition').val(testDef.conditionDisplayName);
        $('#new-condition').parent().removeClass('has-error');
        $('#add-edit-condition-subject-inst').val(testDef.subjectInstructions);
        $('#add-edit-condition-admin-inst').val(testDef.adminInstructions);
        $('#new-duration').val(testDef.recordDuration);

        $('#add-test-type-start-delay')
            .slider('setValue', +testDef.startDelay, true, true);

        $('#add-edit-condition-play-default')
            .prop('checked', (testDef.includeBeeps === 'true'));
        var color = testDef.displayColor;
        if (!color || color === '') {
            color = '#4ac4d4';
        }
        $('#add-edit-condition-color-group')
            .colorpicker('setValue', color);
    }

    /**
     * Initialize the dialog for adding a new condition.
     */
    function initForAdd() {
        testOptions.type = ModelProtocol.add_test_definition;
        editorDialog.setTitle('Add Condition');
        $('#add-edit-condition-play-default')
            .prop('checked', true);
        defaultNormsMessage();
    }

    /**
     * Check if this is an edit, if it is then display a warning
     * dialog if there are trials associated with the test definition.
     */
    function checkIfEdit() {
        if (testOptions.type !== ModelProtocol.edit_test_definition) {
            saveCondition();
            return;
        }
        var testDef = TestDefinitionHandler.getInstance()
            .getTestDefinition(testOptions.testName, testOptions.conditionName);
        if (testDef.numberTrials <= 0) {
            saveCondition();
            return;
        }

        // Display the warning dialog.
        BootstrapDialog.show({
            title: 'Tests with this condition exist',
            message: '<h4>There are ' + testDef.numberTrials + ' tests that ' +
                'are already using this condition.</h4> ' +
                '<h4>Are you sure you want to save your edits?</h4>',
            buttons: [
                {
                    icon: 'iconic iconic-x',
                    label: 'Cancel',
                    cssClass: 'btn-link',
                    action: function(dialog) {
                        dialog.close();
                    }
                },
                {
                    icon: 'iconic iconic-check',
                    label: 'Save',
                    cssClass: 'btn-success',
                    action: function(dialog) {
                        saveCondition();
                        dialog.close();
                    }
                }]
        });
    }

    /**
     * Display a message giving the default norms for custom conditions.
     */
    function defaultNormsMessage() {
        var norms = MetricDefinitionHelper.getInstance()
            .getDefaultNorm(testOptions.testName);
        var msg = '';
        if (norms) {
            msg = 'For custom ' + testOptions.testName + ' conditions, ' +
                'the normative data from the "' + norms +
                '" condition will be used.';
        } else {
            msg = 'No normative data will be displayed for this test type.';
        }
        d3.select('#add-edit-condition-norms')
            .attr('class', 'text-muted')
            .style('font-weight', 'bold').text(msg);
    }

    /**
     * Copy the test condition.
     */
    function saveCopy() {
        var condition = $('#new-condition').val();

        while (TestDefinitionHandler.getInstance()
               .getTestDefinition(testOptions.testName, condition)) {
            condition = condition + '-copy';
        }
        $('#new-condition').val(condition);
        testOptions.type = ModelProtocol.add_test_definition;
        saveCondition();
    }

    /**
     * Export the current test condition. Most of the work
     * will be handled by the server.
     */
    function exportTestCondition() {
        var key = ResReqMap.getInstance().setRequestData(
            { testName: testOptions.testName,
              conditionName: testOptions.conditionName });
        crossroads.parse('dispatch/exportTestDefinition/' + key);
    }

    /**
     * Save the edited condition.
     */
    function saveCondition() {
        var condition = $('#new-condition').val();
        testOptions.duration = $('#new-duration').val();
        testOptions.subjectInstructions = $('#add-edit-condition-subject-inst').val();
        testOptions.adminInstructions = $('#add-edit-condition-admin-inst').val();
        testOptions.displayColor = $('.colorpicker-group')
            .colorpicker('getValue', '#4ac4d4');
        testOptions.startDelay = $('#add-test-type-start-delay')
            .slider('getValue');

        testOptions.oldCondition = testOptions.conditionName;
        testOptions.conditionDisplayName = condition;
        if (testOptions.type === ModelProtocol.add_test_definition) {
            testOptions.conditionName = condition;
        }
        if (!isValidConditionName($('#new-condition'), testOptions)) {
            return;
        }
        testOptions.conditionDisplayName = condition;

        testOptions.includeBeeps = $('#add-edit-condition-play-default')
            .prop('checked');

        // Get the audio cue times from the table.
        var times = [];
        var paths = [];
        d3.select('#add-edit-condition-cue-table-container')
            .select('tbody').selectAll('tr')
            .each(function(d) {
                times.push(d.time);
                paths.push(d.audioCueFilePath);
            });
        testOptions.audioCueTimes = times.join();
        testOptions.audioCueFilePaths = paths.join();
        if (testOptions.isNewTestType) {
            testOptions.type = ModelProtocol.add_test_definition;
        }
        crossroads.parse('dispatch/addEditTestDefinition/' +
                         ResReqMap.getInstance().setRequestData(testOptions));
        editorDialog.close();
    }

    /**
     * Check that the new condition name is valid, and that it is not
     * currently in use.
     * @param {jQuery.selection} nameField The input field for the name.
     * @return {Boolean} True if condition name is valid, otherwise false.
     */
    function isValidConditionName(nameField) {
        var newName = nameField.val();
        if (!newName || newName === '') {
            nameField.parent().addClass('has-error');
            nameField.attr('placeholder', 'Must provide name');
            return false;
        }
        // Editing condition, but keeping the old name.
        if (newName === testOptions.oldCondition) {
            return true;
        }
        // Definition with name exists.
        if (TestDefinitionHandler.getInstance()
            .getTestDefinition(testOptions.testName, newName)) {
            nameField.parent().addClass('has-error');
            nameField.attr('placeholder', '"' + newName + '" already exists.');
            return false;
        }
        return true;
    }

    /**
     * Build the audio cue table in the editor dialog.
     */
    function buildAudioCueTable() {
        // Remove any table that may already exist.
        d3.select('#add-edit-condition-table').remove();

        var tbl = d3.select('#add-edit-condition-cue-table-container')
            .append('table')
            .attr('id', 'add-edit-condition-table')
            .attr('class', 'table');
        var headRow = tbl.append('thead').append('tr');
        headRow.append('th').text('Time (s)');
        headRow.append('th').text('Audio File');
        headRow.append('th').text('Delete');
        tbl.append('tbody');

        updateConditionCueTable(audioCues);
    }

    /**
     * Update the audio cue table.
     */
    function updateConditionCueTable() {
        // Handler on new audio cue button
        $('#add-edit-condition-add-cue').off('click')
            .click(function() {
                openNewCueDialog();
            });

        // Sort the audio cues.
        audioCues.sort(function(a, b) { return d3.ascending(+a.time, +b.time); });
        var body = d3.select('#add-edit-condition-table').select('tbody');
        var rows = body.selectAll('tr')
            .data(audioCues, function(d) {
                return d.time + d.audioCueFilePath;
            });
        rows.enter()
            .append('tr')
            .each(function(d) {
                var row = d3.select(this);
                row.append('td').text(d.time);
                row.append('td').text(d.audioCueFilePath);
                row.append('td').append('button')
                    .attr('class', 'btn btn-default delete-audio-cue-button')
                    .append('span')
                    .attr('class', 'iconic iconic-trash');
            });
        rows.exit().remove();

        // Set the handlers on the buttons.
        body.selectAll('button').on('click', null)
            .on('click', function(d) {
                deleteAudioCue(d);
            });

    }

    /**
     * Open the smaller add audio cue dialog.
     */
    function openNewCueDialog() {
        // Setup file browser handler from server.
        var fileRoute = crossroads
            .addRoute('message/FILE/{key}', function(key) {
                var response = ResReqMap.getInstance().getResponseData(key);
                $('#add-audio-cue-file').val(response.filename);
            });

        BootstrapDialog.show({
            id: 'add-audio-cue-dialog',
            title: 'Add Audio Cue',
            message: $('<div></div>')
                .load('/mobility_clinic/html/add_audio_cue.html'),
            buttons: [
                {
                    id: 'add-audio-cue-cancel-btn',
                    icon: 'iconic iconic-x',
                    label: 'Cancel',
                    cssClass: 'btn-link',
                    action: function(dialog) {
                        dialog.close();
                    }
                },
                {
                    id: 'add-audio-cue-add-btn',
                    icon: 'iconic iconic-check',
                    label: 'Add',
                    cssClass: 'btn-success',
                    action: function(dialog) {
                        var time = $('#add-audio-cue-time').val();
                        var path = $('#add-audio-cue-file').val();
                        if (!path || path === '') {
                            path = 'beep';
                        }
                        validateCue(time, path, dialog);
                    }
                }],
            onhide: function() {
                crossroads.removeRoute(fileRoute);
            },
            onshown: function() {
                $('#add-audio-cue-file-browse').off('click')
                    .click(function() {
                        ServerHandler.getInstance()
                            .post({ type: ModelProtocol.get_local_filename },
                                  null, null,
                                  function(d) {
                                      ErrorHandler
                                          .showError(d.error().statusText);
                                  });
                    });
            }
        });

    }

    /**
     * Make sure that the audio cue has a time associated with it.
     * @param {String} time The delay time for the cue
     * @param {String} path The file path to the cue.
     * @param {BootstrapDialog} dialog The "Add Cue" dialog.
     */
    function validateCue(time, path, dialog) {
        if (!time || time === "") {
            var cueTime = $('#add-audio-cue-time');
            cueTime.parent().addClass('has-error');
            cueTime.val('');
            cueTime.attr('placeholder', 'Time for cue is required.');
            return;
        }

        audioCues.push({time: time, audioCueFilePath: path});
        updateConditionCueTable();
        dialog.close();
    }


    /**
     * Remove the given audio cue from the array and update the table.  If
     * no cues remain, removes the table and turns on "no cues" text.
     * @param {Object} cue The cue to remove.
     */
    function deleteAudioCue(cue) {
        // Changing the array referenced.
        audioCues = audioCues.filter(function(d) {
            return (d.time !== cue.time ||
                    d.audioCueFilePath !== cue.audioCueFilePath);
        });
        updateConditionCueTable();
    }


    /***************************************************************************
     * Public functions.
     **************************************************************************/

    /**
     * Initialize the add/edit dialog and display it.
     * @param {Boolean} userCanEditAdd True is user can edit or add a test
     * condition.
     * @param {String} id The id of the tab the dialog was opened from.
     * @param {Object} options Object containing test type data.
     */
    editor.init = function(userCanEditAdd, id, options) {
        audioCues = [];
        testOptions = options;
        canEditAdd = userCanEditAdd;

        // Get the disabled test conditions.
        var uoh = UserOptionsHandler.getInstance();
        _disabledTestConditions = uoh.getDisabledTestConditions();

        // Condition update handler
        var conditionsUpdated =
            crossroads.addRoute('testDefinitionsUpdated/testDefinition', function() {
                var queryString = '?testName=' + testOptions.testName;
                if (testOptions.isNewTestType) {
                    // Create a the metric groups for new test.
                    MetricDefinitionHelper.getInstance()
                        .selectedMetricGroups(
                            uoh.createNewMetricGroup(testOptions.testName));
                } else {
                    queryString = queryString + "&id=" + id;
                }
                crossroads.parse('updateTestConditionTabs' + queryString);
                crossroads.removeRoute(conditionsUpdated);
            });

        // No save button if this is a default
        var defaultConditions = TestDefinitionHandler.getInstance()
            .getDefaultTestConditions(testOptions.testName);
        var buttons = [{
            id: 'test-condition-close-btn',
            icon: 'iconic iconic-x',
            label: 'Close',
            cssClass: 'btn-link',
            action: function(dialog) {
                dialog.close();
            }
        }];
        if (testOptions.conditionName && testOptions.conditionName !== '' &&
            canEditAdd) {
            buttons.push({
                id: 'test-condition-copy-btn',
                icon: 'iconic iconic-layers',
                label: 'Copy',
                cssClass: 'btn-default',
                action: function() {
                    saveCopy();
                }
            });
            buttons.push({
                id: 'test-condition-export-btn',
                icon: 'iconic iconic-share',
                label: 'Export',
                cssClass: 'btn-default',
                action: function() {
                    exportTestCondition();
                }
            });
        }
        if (!defaultConditions[testOptions.conditionName] && canEditAdd) {
            buttons.push({
                id: 'test-condition-save-btn',
                icon: 'iconic iconic-check',
                label: 'Save',
                cssClass: 'btn-success',
                action: function() {
                    checkIfEdit();
                }
            });
        }

        // Create dialog
        editorDialog = new BootstrapDialog({
            message: $('<div></div>')
                .load('/mobility_clinic/html/add_edit_test_condition.html'),
            closable: false,
            buttons: buttons,
            onshown: function() {
                setupDialog();
            }
        });

        editorDialog.open();
    };

    return editor;

});
