/* Copyright © 2014-2018 APDM, Inc.
 *
 * apdm_vis.js
 *
 */

define(['jquery', 'd3', 'utils', 'metric_definition_helper'],
       function($, d3, utils, MetricDefinitionHelper)
{

    /**
     * Constructor for ApdmVis.
     * @param {d3.selection} container The d3 container to build
     * visualization in.
     * @param {String} baseSvg File name of the SVG to use for
     * visualization.
     * @param {String} testName The name of the test.
     * @param {String} conditionName The condition of the test.
     * @param {String} metricName The name of the metric.
     * @param {String} metrics The set of metrics for the current test.
     */
    function ApdmVis(container, baseSvg, testName, conditionName,
                     metricName, metrics) {
        if (!arguments.length) {
            return;
        }
        this.container = container;
        this.baseSvg = baseSvg;
        this.testName = testName;
        this.conditionName = conditionName;
        this.metricName = metricName;
        this.metrics = metrics;

        this.baseName = this.metricName;

        var metricDefs = MetricDefinitionHelper.getInstance()
            .getTestMetrics(this.testName);
        this.units = '';
        if (metricDefs[this.metricName]) {
            this.units = metricDefs[this.metricName].units;
            this.setUnitsDisplayStr();
        }

        // The precision to display the subject values at norm percentile
        // at.
        this.subjectPrecision = 2;
        this.normPrecision = 2;
    }

    /**
     * Get the data needed for visualization of a unilateral metric.
     * @return {Object} Object containing mean, values, std, and norms
     * for the metric.
     */
    ApdmVis.prototype.getUnilateralData = function() {
        var metricSet = MetricDefinitionHelper.getInstance()
            .getTestMetrics(this.testName);
        var visData = {};

        visData.mean = null;
        visData.values = null;
        visData.std = null;
        visData.norms = null;

        // Make sure that data actually exists.
        if (this.metrics[this.baseName]) {
            visData.mean = this.metrics[this.baseName].mean;
            visData.values = this.metrics[this.baseName].values.map(Number);
            visData.std = this.metrics[this.baseName].std;
        }
        visData.norms =
            MetricDefinitionHelper.getInstance()
            .getMeanStdDevAndNorms(this.testName,
                                   this.conditionName,
                                   metricSet[this.baseName]);

        return visData;
    };

    /**
     * Get the data needed for visualization of a bilateral metric.
     * @return {Object} Object with left and right properties that
     * contain the mean, values, std, and norms for the current metric.
     */
    ApdmVis.prototype.getBilateralData = function() {
        var metricSet = MetricDefinitionHelper.getInstance()
            .getTestMetrics(this.testName);
        var left = this.baseName + ' L';
        var right = this.baseName + ' R';
        var visData = {left: { mean: null, values: null,
                               std: null, norms: null },
                       right: { mean: null, values: null,
                                std: null, norms: null }};

        if (this.metrics[left]) {
            visData.left.mean = this.metrics[left].mean;
            visData.left.values = this.metrics[left].values.map(Number);
            visData.left.std = this.metrics[left].std;
        }
        if (this.metrics[right]) {
            visData.right.mean = this.metrics[right].mean;
            visData.right.values = this.metrics[right].values.map(Number);
            visData.right.std = this.metrics[right].std;
        }

        visData.norms =
            MetricDefinitionHelper.getInstance()
            .getMeanStdDevAndNorms(this.testName, this.conditionName,
                                   metricSet[this.baseName]);

        return visData;
    };

    /**
     * Get or set the precision to display the subject values at.
     * @param {Number} precision The number of digits to show.
     * @return {ApdmVis} This object if called with value. Otherwise,
     * current precision value.
     */
    ApdmVis.prototype.subjectDigitPrecision = function(precision) {
        if (!arguments.length) {
            return this.subjectPrecision;
        }
        this.subjectPrecision = precision;
        return this;
    };

    /**
     * Get or set the precision to display the norm values at.
     * @param {Number} precision The number of digits to show.
     * @return {ApdmVis} This object if called with value. Otherwise,
     * current precision value.
     */
    ApdmVis.prototype.normDigitPrecision = function(precision) {
        if (!arguments.length) {
            return this.normPrecision;
        }
        this.normPrecision = precision;
        return this;
    };

    /**
     * Get the display value for the the normative percentile.  If value x
     * is 100 < x < 0, then default to the value set in
     * normDigitPrecision. Otherwise, simply display '100%' or '0%'.
     */
    ApdmVis.prototype.getPercentileDisplay = function(percentile) {
        return utils.getPercentileDisplay(percentile, true);
    };

    /**
     * Set the text for the values on bilateral visualizations.
     * @param {Object} visData The data being displayed.
     * subject mean.
     */
    ApdmVis.prototype.setBilateralTextValues = function(visData) {
        // Get value for asymmetry text
        var asymText = this.container.select('#bilateral-asymmetry-text');
        if (visData.left && visData.right &&
            $.isNumeric(visData.left.mean) &&
            $.isNumeric(visData.right.mean)) {
            var percentChangeArray = utils.percentDifferenceArrays(visData.right.values, visData.left.values);
            var percentChangeMean = utils.mean(percentChangeArray);
            var percentChangeStd = utils.stdDeviation(percentChangeArray);

            if (!$.isNumeric(percentChangeMean)) {
                asymText.text('--');
            } else {
                var lrText = 'R';

                if (percentChangeMean < 0) {
                    lrText = 'L';
                } else if (percentChangeMean === 0) {
                    lrText = '';
                }

                asymText.text(Math.abs(percentChangeMean.toFixed()) + '% ' + lrText);
            }
            var formattedAsymSd = percentChangeStd.toLocaleString(undefined, {
                maximumFractionDigits: 2
            });
            this.container.select('#bilateral-asymmetry-cov-text').text(formattedAsymSd);
        }

        var percentile;
        ['left', 'right'].forEach(function(d) {
            if (visData[d] && $.isNumeric(visData[d].mean)) {
                this.container.select('#' + d + '-subject-text')
                    .text(utils.getFormattedNumber(visData[d].mean,
                                                   this.subjectPrecision));
                if (visData.norms && visData.norms.norms) {
                    percentile = utils.getPercentile(visData[d].mean,
                                                     visData.norms.norms);
                }
                this.container.select('#' + d + '-norm-text')
                    .text(this.getPercentileDisplay(percentile));

                if ($.isNumeric(visData[d].std)) {
                    this.container.select('#' + d + '-cov-text')
                        .text(utils.getFormattedNumber(visData[d].std,
                                                       this.subjectPrecision));
                }
            }
        }, this);
    };

    /**
     * Align the text for a bilateral visualization.
     */
    ApdmVis.prototype.doBilateralTextAlignment = function() {
        var containerDims = this.getSvgDimentions();
        var textContainer = this.container.select('#bilateral-text-container');
        var spacingFromBar = 10;
        var unitsPad = 3;
        var leftContainer = textContainer.select('#bilateral-left-text-container');
        var rightContainer = textContainer
            .select('#bilateral-right-text-container');

        // Set the units first
        var unitsText = textContainer.select('#bilateral-subject-units');
        unitsText.text(this.units);
        var pctlUnits = textContainer.select('#bilateral-norm-units');
        var covUnits = textContainer.select('#bilateral-cov-units');
        var asymUnits = textContainer.select('#bilateral-asymmetry-units');
        var asymCovUnits =
            textContainer.select('#bilateral-asymmetry-cov-units');

        var unitsMax = d3.max([unitsText.node().getBBox().width,
                               pctlUnits.node().getBBox().width,
                               covUnits.node().getBBox().width,
                               asymUnits.node().getBBox().width,
                               asymCovUnits.node().getBBox().width]);

        var leftText = leftContainer.selectAll('text');
        var leftBoxes = [];
        leftText.each( function() {
            leftBoxes.push(d3.select(this).node().getBBox());
        });
        var leftTextMax = d3.max(leftBoxes, function(d) { return d.width; });

        var rightText = rightContainer.selectAll('text');
        var rightBoxes = [];
        rightText.each( function() {
            rightBoxes.push(d3.select(this).node().getBBox());
        });
        var rightTextMax = d3.max(rightBoxes, function(d) { return d.width; });

        var textMax = d3.max([leftTextMax, rightTextMax]);
        var topOffset = (containerDims.height * 0.35) -
            (textContainer.node().getBBox().height * 0.5);

        textContainer.attr('transform', 'translate(' +
                           (containerDims.width - ((textMax * 2) + (spacingFromBar * 2) +
                                                   unitsMax + unitsPad)) + ',' +
                           topOffset + ')');

        rightContainer.attr('transform', 'translate(' + textMax + ')');
        textContainer.select('line')
            .attr({ 'x1': textMax + spacingFromBar,
                    'x2': textMax + spacingFromBar });

        // Set the center points of the text
        leftText.attr('x', textMax / 2);
        rightText.attr('x', (spacingFromBar * 2) + (textMax / 2));
        var unitsOffset = (spacingFromBar * 2) + textMax + unitsPad;
        unitsText.attr('x', unitsOffset);
        pctlUnits.attr('x', unitsOffset);
        covUnits.attr('x', unitsOffset);
        asymUnits.attr('x', unitsOffset);
        asymCovUnits.attr('x', unitsOffset);

        // Do text for the asymmetry measure.
        var asymText = textContainer.select('#bilateral-asymmetry-container');
        asymText.attr('transform', 'translate(0)');
        asymText.select('text').attr('x', (textMax + spacingFromBar));

        var asymCovText =
            textContainer.select('#bilateral-asymmetry-cov-container');
        asymCovText.attr('transform', 'translate(0)');
        asymCovText.select('text').attr('x', (textMax + spacingFromBar));

    };

    /**
     * Set the values for the subject mean and the coefficent of
     * variability.
     * @param {Object} visData The metric data being displayed.
     */
    ApdmVis.prototype.setUnilateralTextValues = function(visData) {
        this.container.select('#unilateral-subject-text')
            .text(utils.getFormattedNumber(Math.abs(visData.mean),
                                           this.subjectPrecision));
        if (visData.std !== 0) {
            this.container.select('#unilateral-cov-text')
                .text(utils.getFormattedNumber(visData.std,
                                               this.subjectPrecision));
        }
    };

    /**
     * Align the text for a unitlateral visualization.
     */
    ApdmVis.prototype.doUnilateralTextAlignment = function() {
        var containerDims = this.getSvgDimentions();
        var textContainer = this.container.select('#unilateral-text-container');
        var unitsPad = 3;

        // Set the units first
        var unitsText = textContainer.select('#unilateral-subject-units');
        unitsText.text(this.units);
        var pctlUnits = textContainer.select('#unilateral-norm-units');
        var covUnits = textContainer.select('#unilateral-cov-units');

        var unitsMax = d3.max([unitsText.node().getBBox().width,
                               pctlUnits.node().getBBox().width,
                               covUnits.node().getBBox().width]);
        var subject = textContainer.select('#unilateral-subject-text');
        var norm = textContainer.select('#unilateral-norm-text');
        var cov = textContainer.select('#unilateral-cov-text');

        var textMax = d3.max([subject.node().getBBox().width,
                              norm.node().getBBox().width]);
        var topOffset = (containerDims.height * 0.35) -
            (textContainer.node().getBBox().height * 0.5);

        textContainer.attr('transform', 'translate(' +
                           (containerDims.width -
                            (textMax + unitsMax + unitsPad)) + ',' +
                           topOffset + ')');

        textContainer.select('line').attr('x2', textMax);

        // Set the center points of the text
        subject.attr('x', textMax / 2);
        norm.attr('x', textMax / 2);
        cov.attr('x', textMax / 2);
        var unitsOffset = textMax + unitsPad;
        unitsText.attr('x', unitsOffset);
        pctlUnits.attr('x', unitsOffset);
        covUnits.attr('x', unitsOffset);
    };

    /**
     * Set the display string to use for the metric's units.
     */
    ApdmVis.prototype.setUnitsDisplayStr = function() {
        this.units = utils.formatUnits(this.units);
    };

    /**
     * Get the width and height of the svg container.
     * @return {Object} Object with width and height properties.
     */
    ApdmVis.prototype.getSvgDimentions = function() {
        if (!this.container.node()) {
            return {width: 0, height: 0};
        }
        return {width: $(this.container.node()).width(),
                height: $(this.container.node()).height()};
    };

    return ApdmVis;
});
