/* Copyright © 2014-2017 APDM, Inc.
 *
 * bilateral_one_norm_arc_vis.js
 */

define(['jquery', 'd3', 'utils', 'app/apdm_vis'], function($, d3, utils, ApdmVis) {

    'use strict';

    /**
     * Set ApdmVis as base class.
     */
    BilateralOneNormArcVis.prototype = new ApdmVis();

    /**
     * Constructor for BilateralOneNormArcVis.
     * @param {d3.selection} container The d3 container to build
     * visualization in.
     * @param {String} baseSvg File name of the SVG to use for
     * visualization.
     * @param {String} testName The name of the test.
     * @param {String} conditionName The condition of the test.
     * @param {String} metricName The name of the metric.
     * @param {String} metrics The set of metrics for the current test.
     */
    function BilateralOneNormArcVis(container, baseSvg, testName, conditionName,
                                    metricName, metrics) {
        ApdmVis.call(this, container, baseSvg, testName, conditionName,
                     metricName, metrics);
        this.scale = d3.scaleLinear();
        this.normRadius = 149;
        this.subjectRadius = 185;

        // Size of the cirlcles.
        this.meanCircRad = 12;
        this.rangeCircRad = 4;

        // Offset angle applied to the data.
        this.offset = 0;

        // Distance between the left and right circles.
        this.lrDistance = 28;

        // Should the sign of the values be switched.
        this.changeSign = false;
    }

    /**
     * Update the plot.
     */
    BilateralOneNormArcVis.prototype.update = function() {
        var self = this;
        $(self.container.node()).load('../res/' + self.baseSvg, baseLoaded);
        var arcOffset = utils.degToRad(this.offset);
        var lineOffset = this.offset - 90;

        function baseLoaded() {
            self.container.select('svg');

            var visData = self.getBilateralData();

            var arc;
            var minMax;
            // TODO: Need to determine which norms to use.
            if (visData.norms && visData.norms.fifthToNinteyFifth &&
                visData.norms.fifthToNinteyFifth !== null) {
                minMax = visData.norms.fifthToNinteyFifth;
                if (self.changeSign) {
                    minMax = minMax.map(function(d) { return -(d); });
                }
                // D3 treats the vertical of the circle as 0, where as 0
                // is on the horizontal for initial contact.
                arc = d3.arc().innerRadius(0)
                    .outerRadius(self.normRadius)
                    .startAngle(utils.degToRad(minMax[0]) + arcOffset)
                    .endAngle(utils.degToRad(minMax[1]) + arcOffset);
                self.container.select('#norm-range')
                    .selectAll('path')
                    .data([visData.norms])
                    .enter()
                    .append('path')
                    .attr('class', 'metric-vis-norm-range')
                    .attr('d', arc);
            }

            doSide('left', visData.left, -self.lrDistance / 2, 'L');
            doSide('right', visData.right, self.lrDistance / 2, 'R');

            self.setBilateralTextValues(visData);
            self.doBilateralTextAlignment();
        }

        function doSide(side, dataValues, radiusOffset, circleLabel) {
            var subjectG = self.container.select('#' + side + '-subject-container');
            var radius = self.subjectRadius + radiusOffset;
            if (dataValues.values) {
                var minMax = d3.extent(dataValues.values);
                if (self.changeSign) {
                    minMax = minMax.map(function(d) { return -(d); });
                }
                if (!isNaN(minMax[0]) && !isNaN(minMax[1])) {
                    var arc = d3.arc().innerRadius(radius)
                        .outerRadius(radius)
                        .startAngle(utils.degToRad(minMax[0]) + arcOffset)
                        .endAngle(utils.degToRad(minMax[1]) + arcOffset);
                    subjectG.selectAll('#metric-vis-' + side + '-subject-range')
                        .data([minMax])
                        .enter()
                        .append('path')
                        .attr('id', 'metric-vis-' + side + '-subject-range')
                        .attr('class', 'metric-vis-mean-line')
                        .attr('d', arc);
                    subjectG.selectAll('#metric-vis-' + side +
                                       '-subject-range-circle')
                        .data(minMax)
                        .enter()
                        .append('circle')
                        .attr('id', 'metric-vis-' + side + '-subject-range-circle')
                        .attr('class', 'metric-vis-subject-range-circle')
                        .attr('cx', radius)
                        .attr('cy', 0)
                        .attr('r', self.rangeCircRad)
                        .attr('transform', function(d) {
                            return 'rotate(' + (d + lineOffset) + ', 0, 0)';
                        });
                }
            }

            if (!isNaN(dataValues.mean)) {
                var adjustedMean = dataValues.mean;
                if (self.changeSign) {
                    adjustedMean = -(adjustedMean);
                }
                // Set the line for the subject mean
                subjectG.selectAll('#metric-vis-' + side + '-subject')
                    .data([adjustedMean],
                          function(d) { return d; })
                    .enter()
                    .append('path')
                    .attr('id', 'metric-vis-' + side + '-subject')
                    .attr('class', 'metric-vis-mean-line')
                    .attr('d', 'M0,0 H ' + (radius))
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0)';
                    });

                // Do the circles.
                subjectG.selectAll('#metric-vis-' + side + '-subject-circle')
                    .data([adjustedMean])
                    .enter()
                    .append('circle')
                    .attr('id', 'metric-vis-' + side + '-subject-circle')
                    .attr('class', 'metric-vis-subject-range')
                    .attr('cx', radius)
                    .attr('cy', 0)
                    .attr('r', self.meanCircRad)
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0)';
                    });

                subjectG.selectAll('#metric-vis-' + side +
                                   '-subject-circle-text')
                    .data([adjustedMean])
                    .enter()
                    .append('g')
                    .attr('id', 'metric-vis-' + side + '-subject-circle-text')
                    .attr('class', 'metric-vis-circle-label')
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0) ' +
                            'translate(' + radius + ',0)';
                    })
                    .append('text')
                    .attr('class', 'metric-vis-circle-label')
                    .attr('y', 5)
                    .attr('transform', function(d) {
                        return 'rotate(' + (-(d + lineOffset)) + ', 0, 0)';
                    })
                    .text(circleLabel);
            }
        }

    };

    /**
     * Get or set the domain of the plot.
     * @param {Array} plotDomain The domain of the plot.
     * @return The domain of the plot if called with no arguments,
     * otherwise this instance.
     */
    BilateralOneNormArcVis.prototype.domain = function(plotDomain) {
        if (!arguments.length) {
            return this.scale.domain();
        }
        this.scale.domain(plotDomain);
        return this;
    };

    /**
     * Get or set the radius for the normative range arc.
     * @param {Number} normRadius The radius of the normative range arc.
     * @return The radius for the normative arc if called with no arguments,
     * otherwise the object instance.
     */
    BilateralOneNormArcVis.prototype.normArcRadius = function(radius) {
        if (!arguments.length) {
            return this.normRadius;
        }
        this.normRadius = radius;
        return this;
    };

    /**
     * Get or set the radius for the subjet range mean line.
     * @param {Number} normRadius The radius of the subject line.
     * @return The radius for the subject line if called with no arguments,
     * otherwise the object instance.
     */
    BilateralOneNormArcVis.prototype.subjectArcRadius = function(radius) {
        if (!arguments.length) {
            return this.subjectRadius;
        }
        this.subjectRadius = radius;
        return this;
    };

    /**
     * Get or set the offset for the data being displayed.
     * @param {Number} offset The offset in degrees to apply to the data.
     * @return {BilateralOneNormArcVis} Object instance if called with
     * argument, otherwise current offset.
     */
    BilateralOneNormArcVis.prototype.offsetAngle = function(offset) {
        if (!arguments.length) {
            return this.offset;
        }
        this.offset = offset;
        return this;
    };

    /**
     * Flag that determines if the sign of the values should be switched.
     * @param {Boolean} switch True if sign should be switched.
     * @return {BilateralOneNormArcVis} Object instance if called with
     * argument, otherwise current flag value.
     */
    BilateralOneNormArcVis.prototype.switchSign = function(changeSign) {
        if (!arguments.length) {
            return this.changeSign;
        }
        this.changeSign = changeSign;
        return this;
    };

    return BilateralOneNormArcVis;
});
