/* Copyright © 2014-2019 APDM, Inc.
 *
 * bilateral_two_norm_arc_vis.js
 */

define(['jquery', 'd3', 'utils', 'app/apdm_vis'], function($, d3, utils, ApdmVis) {

    /**
     * Set ApdmVis as base class.
     */
    BilateralTwoNormArcVis.prototype = new ApdmVis();

    /**
     * Constructor for BilateralTwoNormArcVis.
     * @param {d3.selection} container The d3 container to build
     * visualization in.
     * @param {String} baseSvg File name of the SVG to use for
     * visualization.
     * @param {String} testName The name of the test.
     * @param {String} conditionName The condition of the test.
     * @param {String} metricName The name of the metric.
     * @param {String} metrics The set of metrics for the current test.
     */
    function BilateralTwoNormArcVis(container, baseSvg, testName, conditionName,
                                    metricName, metrics) {
        ApdmVis.call(this, container, baseSvg, testName, conditionName,
                     metricName, metrics);
        this.scale = d3.scaleLinear();
        this.normRadius = 98;
        this.subjectRadius = 116;

        // Size of the cirlcles.
        this.meanCircRad = 12;
        this.rangeCircRad = 4;

        // Offset angle applied to the data.
        this.offset = 0;
    }

    /**
     * Update the plot.
     */
    BilateralTwoNormArcVis.prototype.update = function() {
        var self = this;
        $(self.container.node()).load('../res/' + self.baseSvg, baseLoaded);
        var arcOffset = utils.degToRad(this.offset);
        var lineOffset = this.offset - 90;

        function baseLoaded() {
            self.container.select('svg');
            var visData = self.getBilateralData();
            doSide('left', visData.left, visData.norms, 'L');
            doSide('right', visData.right, visData.norms, 'R');

            self.setBilateralTextValues(visData);
            self.doBilateralTextAlignment();
        }

        function doSide(side, dataValues, norms, circleLabel) {
            var arc;
            var minMax;
            var min;
            var max;

            if (norms && norms !== null && norms.fifthToNinteyFifth &&
                norms.fifthToNinteyFifth !== null) {
                minMax = norms.fifthToNinteyFifth;
                min = minMax[0];
                max = minMax[1];
                if (side === 'left') {
                    min = -minMax[1];
                    max = -minMax[0];
                }

                // D3 treats the vertical of the circle as 0, where as 0
                // is on the horizontal for initial contact.
                arc = d3.arc().innerRadius(0)
                    .outerRadius(self.normRadius)
                    .startAngle(utils.degToRad(min) + arcOffset)
                    .endAngle(utils.degToRad(max) + arcOffset);
                self.container
                    .select('#bilateral-arc-vis-' + side + '-norm-range')
                    .selectAll('path')
                    .data([norms])
                    .enter()
                    .append('path')
                    .attr('class', 'metric-vis-norm-range')
                    .attr('d', arc);
            }

            if (dataValues.values && dataValues.values !== null) {
                minMax = d3.extent(dataValues.values);
                min = minMax[0];
                max = minMax[1];
                var mean = dataValues.mean;
                if (side === 'left') {
                    min = -minMax[1];
                    max = -minMax[0];
                    mean = -dataValues.mean;
                }
                var radius = self.subjectRadius;

                var subjectG = self.container.select('#bilateral-arc-vis-' +
                                                     side + '-subject-container');
                arc = d3.arc()
                    .innerRadius(radius)
                    .outerRadius(radius)
                    .startAngle(utils.degToRad(min) + arcOffset)
                    .endAngle(utils.degToRad(max) + arcOffset);
                subjectG.selectAll('#bilateral-arc-vis-' + side +
                                   '-subject-range')
                    .data([min, max])
                    .enter()
                    .append('path')
                    .attr('id', 'bilateral-arc-vis-' + side + '-subject-range')
                    .attr('class', 'metric-vis-mean-line')
                    .attr('d', arc);

                // Set the line for the subject mean
                subjectG.selectAll('#bilateral-arc-vis-' + side + '-subject')
                    .data([mean],
                          function(d) { return d; })
                    .enter()
                    .append('path')
                    .attr('id', 'bilateral-arc-vis-' + side + '-subject')
                    .attr('class', 'metric-vis-mean-line')
                    .attr('d', 'M0,0 H ' + (radius))
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0)';
                    });

                // Do the circles.
                subjectG.selectAll('#bilateral-arc-vis-' + side +
                                   '-subject-range-circle')
                    .data([min, max])
                    .enter()
                    .append('circle')
                    .attr('id', 'bilateral-arc-vis-' + side +
                          '-subject-range-circle')
                    .attr('class', 'metric-vis-subject-range-circle')
                    .attr('cx', radius)
                    .attr('cy', 0)
                    .attr('r', self.rangeCircRad)
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0)';
                    });

                subjectG.selectAll('#bilateral-arc-vis-' + side +
                                   '-subject-circle')
                    .data([mean])
                    .enter()
                    .append('circle')
                    .attr('id', 'bilateral-arc-vis-' + side + '-subject-circle')
                    .attr('class', 'metric-vis-subject-range')
                    .attr('cx', radius)
                    .attr('cy', 0)
                    .attr('r', self.meanCircRad)
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0)';
                    });

                subjectG.selectAll('#bilateral-arc-vis-' + side +
                                   '-subject-circle-text')
                    .data([mean])
                    .enter()
                    .append('g')
                    .attr('id', 'bilateral-arc-vis-' + side + '-subject-circle-text')
                    .attr('class', 'metric-vis-circle-label')
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0) ' +
                            'translate(' + radius + ',0)';
                    })
                    .append('text')
                    .attr('class', 'metric-vis-circle-label')
                    .attr('y', 5)
                    .attr('transform', function(d) {
                        return 'rotate(' + (-(d + lineOffset)) + ', 0, 0)';
                    })
                    .text(circleLabel);
            }
        }

    };

    /**
     * Get or set the domain of the plot.
     * @param {Array} plotDomain The domain of the plot.
     * @return The domain of the plot if called with no arguments,
     * otherwise this instance.
     */
    BilateralTwoNormArcVis.prototype.domain = function(plotDomain) {
        if (!arguments.length) {
            return this.scale.domain();
        }
        this.scale.domain(plotDomain);
        return this;
    };

    /**
     * Get or set the radius for the normative range arc.
     * @param {Number} normRadius The radius of the normative range arc.
     * @return The radius for the normative arc if called with no arguments,
     * otherwise the object instance.
     */
    BilateralTwoNormArcVis.prototype.normArcRadius = function(radius) {
        if (!arguments.length) {
            return this.normRadius;
        }
        this.normRadius = radius;
        return this;
    };

    /**
     * Get or set the radius for the subjet range mean line.
     * @param {Number} normRadius The radius of the subject line.
     * @return The radius for the subject line if called with no arguments,
     * otherwise the object instance.
     */
    BilateralTwoNormArcVis.prototype.subjectArcRadius = function(radius) {
        if (!arguments.length) {
            return this.subjectRadius;
        }
        this.subjectRadius = radius;
        return this;
    };

    /**
     * Get or set the offset for the data being displayed.
     * @param {Number} offset The offset in degrees to apply to the data.
     * @return {BilateralTwoNormArcVis} Object instance if called with
     * argument, otherwise current offset.
     */
    BilateralTwoNormArcVis.prototype.offsetAngle = function(offset) {
        if (!arguments.length) {
            return this.offset;
        }
        this.offset = offset;
        return this;
    };

    return BilateralTwoNormArcVis;
});
