/* Copyright (c) 2016-2017 APDM, Inc.
 *
 * delete.js
 *
 * Module to handle confirmation dialog for deleting objects.
 */

define(['jquery', 'd3', 'utils', 'bootstrap-dialog', 'crossroads',
        'res_req_map', 'trial', 'session'],
       function($, d3, utils, BootstrapDialog, crossroads, ResReqMap, Trial,
                Session)
{
    var _dialog;
    var _cb;
    var _deleteRoute = null;

    /**
     * Delete the given study.
     * @param {String} studyName The name of the study to delete.
     * @param {Function} nextCall Function to call after delete
     * successful.
     */
    function deleteStudy(studyName, nextCall) {
        _cb = nextCall;
        buildDialog('Deleting subject group: ' + studyName,
                    'Do you want to delete "' + studyName +
                    '" from the database?',
                    function() {
                        doStudyDelete(studyName);
                    });
    }

    /**
     * Send the request to delete the study and do callback.
     * @param {String} name The name of the study to delete.
     */
    function doStudyDelete(name) {
        crossroads.parse('dispatch/deleteStudy/' + ResReqMap.getInstance()
                         .setRequestData(name));
        if (_cb instanceof Function) {
            _cb();
        }
    }

    /**
     * Delete the given subject.
     * @param {Subject} subject The subject to delete.
     * @param {String} displayName The display name of the subject.
     */
    function deleteSubject(subject, displayName, nextCall) {
        _cb = nextCall;
        buildDialog('Deleting subject: ' + displayName,
                    'Do you want to delete "' + displayName +
                    '" from the database? ' +
                    'This will also delete all trials.',
                    function() {
                        doSubjectDelete(subject);
                    });
    }

    /**
     * Send the request to delete the subject.
     * @param {Subject} subject The subject to delete.
     */
    function doSubjectDelete(subject) {
        var objId = subject.getObjId();
        var key = ResReqMap.getInstance().setRequestData(
            { id: objId.getId(), localId: objId.getLocalId() }
        );
        crossroads.parse('dispatch/deleteSubject/' + key);
    }

    /**
     * Delete the given test.
     * @param {Trial or Session} trial The trial to delete.
     * @param {Function} nextCall Function to call after delete successful.
     */
    function deleteTest(trial, nextCall) {
        _cb = nextCall;
        var testTitle = trial.getTestName();
        if (trial instanceof Trial) {
            testTitle = testTitle + ', ' + trial.getConditionName();
        }
        testTitle = testTitle + ' - ' +
            utils.getShortStringFromDate(trial.getDateObj());
        buildDialog('Deleting test: ' + testTitle,
                    'Do you want to delete "' + testTitle +
                    '" from the database?',
                    function() {
                        doTrialDelete(trial);
                    });
    }


    /**
     * Send the request to delete the trial, then do the callback if
     * supplied.
     * @param {Trial or Session} trial The trial to delete.
     */
    function doTrialDelete(trial) {
        if (trial instanceof Session) {
            crossroads.parse('dispatch/deleteSession/' + ResReqMap.getInstance()
                             .setRequestData(trial.getObjId()));
        } else if(trial.getSequence() &&
                  trial.getSequence().getAllTrials().length <= 1) {
            crossroads.parse('dispatch/deleteSession/' + ResReqMap.getInstance()
                             .setRequestData(trial.getSequence().getObjId()));
        } else {
            crossroads.parse('dispatch/deleteTrial/' + ResReqMap.getInstance()
                             .setRequestData(trial.getObjId()));
        }

        if (_cb instanceof Function) {
            _deleteRoute = crossroads.addRoute('testDeleted', function() {
                crossroads.removeRoute(_deleteRoute);
                _deleteRoute = null;
                _cb();
            });
        }
    }

    /**
     * Delete the given test definition.
     * @param {Object} testDef The test definition to delete.
     * @param {Function} nextCall The function to call on successuful
     * delete.
     */
    function deleteTestDefinition(testDef, nextCall) {
        _cb = nextCall;
        buildDialog('Deleting condition "' + testDef.conditionDisplayName +
                    '" for test "' + testDef.testDisplayName,
                    'Do you want to delete the "' + testDef.conditionDisplayName +
                    '" condition for "' + testDef.testDisplayName +
                    '" from the database?',
                    function() {
                        doTestDefinitionDelete(testDef);
                    });
    }

    /**
     * Send the request to do the delete and do callback if successful.
     * @param {Object} testDef The test definition to delete.
     */
    function doTestDefinitionDelete(testDef) {
        if (_cb instanceof Function) {
            _deleteRoute = crossroads
                .addRoute('testDefinitionsUpdated/testDefinition', function() {
                    crossroads.removeRoute(_deleteRoute);
                    _deleteRoute = null;
                    _cb();
                });
        }

        var key = ResReqMap.getInstance().setRequestData(
            { testName: testDef.testName,
              conditionName: testDef.conditionName }
        );
        crossroads.parse('dispatch/deleteTestDefinition/' + key);
    }

    /**
     * Delete the given test sequence.
     * @param {String} name The name of the sequence to delete.
     * @param {Function} nextCall The function to call with successful
     * delete.
     */
    function deleteTestSequence(name, nextCall) {
        _cb = nextCall;
        buildDialog('Deleting ' + name,
                    'Do you want to delete the "' + name  +
                    '" custom test sequence?',
                    function() {
                        doTestSequenceDelete(name);
                    });
    }

    /**
     * Send the request to do the delete and do callback if successful.
     * @param {String} name The name of test sequence to delete.
     */
    function doTestSequenceDelete(name) {
        if (_cb instanceof Function) {
            _deleteRoute = crossroads
                .addRoute('testSequenceDeleted', function() {
                    crossroads.removeRoute(_deleteRoute);
                    _deleteRoute = null;
                    _cb();
                });
        }
        var key = ResReqMap.getInstance().setRequestData({sequenceName: name});
        crossroads.parse('dispatch/deleteTestSequence/' + key);
    }

    /**
     * Build and display the delete conformation dialog.
     * @param {String} title The title of the dialog.
     * @param {String} msg Message for the dialog.
     * @param {Function} doDelete Delete function to call when deletion is
     * coinfirmed.
     */
    function buildDialog(title, msg, doDelete) {
        _dialog = new BootstrapDialog({
            type: BootstrapDialog.TYPE_DANGER,
            title: title,
            message: '<h4>' + msg + '</h4>',
            closable: false,
            buttons: [
                {
                    id: 'delete-dialog-cancel-btn',
                    icon: 'iconic iconic-x',
                    label: 'Cancel',
                    cssClass: 'btn-link',
                    action: function(d) {
                        if (_deleteRoute) {
                            crossroads.removeRoute(_deleteRoute);
                            _deleteRoute = null;
                        }
                        d.close();
                    }
                },
                {
                    id: 'delete-dialog-delete-btn',
                    icon: 'iconic iconic-trash',
                    label: 'Delete',
                    cssClass: 'btn-danger',
                    action: function(d) {
                        doDelete();
                        d.close();
                    }
                }
            ]
        });
        _dialog.open();
    }

    return {
        deleteStudy: deleteStudy,
        deleteSubject: deleteSubject,
        deleteTest: deleteTest,
        deleteTestNoDialog: doTrialDelete,
        deleteTestDefinition: deleteTestDefinition,
        deleteTestSequence: deleteTestSequence
    };
});
