/* Copyright (c) 2018 APDM, Inc.
 *
 * export_handler.js
 *
 * Create the export dialog and handle the progress
 * bar.
 */

define(['jquery', 'd3', 'crossroads', 'bootstrap-dialog', 'error_handler',
        'res_req_map', 'test_definition_handler','app/license_handler'],
       function($, d3, crossroads, BootstrapDialog, ErrorHandler, ResReqMap,
                TestDefinitionHandler, LicenseHandler)
{
    'use strict';

    var _dialog;
    var _baseObjId;
    var _flavor;

    crossroads.addRoute('dispatch/exportToCsv/{flavor}/{key}',
                        openExportOptions);

    /**
     * Open the export dialog to get the options from user.
     * @param {String} flavor The scope for the export.
     * @param {Number} key The key to the base object id.
     */
    function openExportOptions(flavor, key) {
        _flavor = flavor;
        _baseObjId = ResReqMap.getInstance().getRequestData(key);

        _dialog = new BootstrapDialog({
            id: 'export-options-dialog',
            title: 'Export Data',
            message: $('<div></div>')
                .load('/mobility_clinic/html/export_options.html'),
                buttons: [
                    {
                        id: 'export-close-btn',
                        icon: 'iconic iconic-x',
                        label: 'Cancel',
                        cssClass: 'btn-link',
                        action: function(dlg) { dlg.close(); }
                    },
                    {
                        id: 'export-continue-btn',
                        icon: 'iconic iconic-share',
                        label: 'Export',
                        cssClass: 'btn-success',
                        action: parseOptions
                    }
                ],
            onshown: setupDropdowns
        });
        _dialog.open();
    }

    /**
     * Setup the dropdown selectors and the date/time pickers.
     */
    function setupDropdowns() {
        if (!LicenseHandler.getInstance().isKinematics()) {
            d3.selectAll('.mobility-kinematics').style('display', 'none');
        }

        var testHandler = TestDefinitionHandler.getInstance();
        d3.select('#sequence-type')
            .selectAll('option')
            .data(['None'].concat(testHandler.getSequenceNames()))
            .enter()
            .append('option')
            .attr('value', function(g) { return g; })
            .text(function(g) { return g; });

        var testNameMap = testHandler.getTestNameMap();
        d3.select('#test-type')
            .selectAll('option')
            .data([{name: 'None', displayName: 'All'}].concat(testNameMap))
            .enter()
            .append('option')
            .attr('value', function(g) { return g.name; })
            .text(function(g) { return g.displayName; });

        updateConditionDropdown('None');

        $('.selectpicker').selectpicker();
        $('#test-type').on('changed.bs.select', function() {
            updateConditionDropdown($(this).val());
        });

        var datePickerOpts = {
            format: 'YYYY-MM-DD HH:mm',
            icons: {
                time: 'iconic iconic-clock',
                date: 'iconic iconic-calendar',
                up: 'iconic iconic-chevron-up',
                down: 'iconic iconic-chevron-down',
                previous: 'iconic iconic-chevron-left',
                next: 'iconic iconic-chevron-right',
                today: 'iconic iconic-screenshot',
                clear: 'iconic iconic-trash',
                close: 'iconic iconic-x'
            },
            showClear: true,
            showClose: true,
            widgetParent: $('#export-options-dialog')
        };
        $('#exclude-before-date').datetimepicker(datePickerOpts);
        $('#exclude-after-date').datetimepicker(datePickerOpts);

    }

    /**
     * Update the test condition dropdown based on test type.
     * @param {String} testType The test type selected in the test
     * type dropdown.
     */
    function updateConditionDropdown(testType) {
        var conditions = [];
        if (testType !== 'None') {
            conditions = TestDefinitionHandler.getInstance()
                .getTestConditionDefinitions(testType);
        }
        conditions.push({conditionName: 'None', conditionDisplayName: 'All'});

        var conditionDropdown = d3.select('#test-condition')
            .selectAll('option')
            .data(conditions, function(d) { return d.conditionName; });

        conditionDropdown.enter()
            .append('option')
            .attr('value', function(g) {
                return g.conditionName; })
            .text(function(g) {
                return g.conditionDisplayName; });

        conditionDropdown.exit()
            .remove();

        $('#test-condition').selectpicker('refresh');
    }

    /**
     * Parse the selected options and build the JSON blob
     */
    function parseOptions() {
        var exportOptions = {
            includeCompactMetricsCsv: $('#summary-analysis').is(':checked'),
            includeVerboseMetricsCsv: $('#detailed-analysis').is(':checked'),
            includeAsymmetryMetrics: $('#export-asymmetry').is(':checked'),
            includeMetadataXml: $('#export-metadata').is(':checked'),
            includeRecordingData: $('#export-raw-data').is(':checked'),
            includeJointAnglesCsv: $('#export-joint-data-csv').is(':checked'),
            includeJointAnglesHdf: $('#export-joint-data-hdf').is(':checked'),
            includeVerboseDataTransfer:  $('#export-data-transfer').is(':checked'),
            deIdentify: $('#de-identify').is(':checked'),
            compressData: $('#compress-export').is(':checked'),
            exportParentFolder: '/tmp/apdm-export'
        };

        if (exportOptions.includeVerboseDataTransfer == true) {
            exportOptions.includeDataTransferSpecification = true;
        }

        var excludeBefore = $('#exclude-before-date')
            .data("DateTimePicker").date();
        var excludeAfter = $('#exclude-after-date')
            .data("DateTimePicker").date();

        var trialFilters = {
            testTypeSelected: selectionNotNone('#test-type'),
            conditionTypeSelected: selectionNotNone('#test-condition'),
            sessionTypeSelected: selectionNotNone('#sequence-type'),
            startDateSelected: (excludeBefore !== null),
            endDateSelected: (excludeAfter !== null),

            testType: $('#test-type').selectpicker('val'),
            conditionType: $('#test-condition').selectpicker('val'),
            sessionType: $('#sequence-type').selectpicker('val'),
            startDate: (excludeBefore !== null) ? excludeBefore.valueOf() : null,
            endDate: (excludeAfter !== null) ? excludeAfter.valueOf() : null
        };

        var exportRequest = {
            flavor: _flavor,
            id: _baseObjId.getId(),
            localId: _baseObjId.getLocalId(),
            exportContentDefinition: JSON.stringify({
                trialFilters: trialFilters,
                exportOptions: exportOptions
            })
        };

        crossroads.parse('dispatch/exportData/' +
                         ResReqMap.getInstance().setRequestData(exportRequest));

        _dialog.close();
    }

    /**
     * Return true if the dropdown element has "None" selected.
     * @param {String} dropdownId jQuery selection string.
     * @return {Boolean} True if selection does not equal "None".
     */
    function selectionNotNone(dropdownId) {
        return $(dropdownId).selectpicker('val') !== 'None';
    }

    return {
        open: openExportOptions
    };
});
