/* Copyright (c) 2016-2018 APDM, Inc.
 *
 * hardware_placement_dialog.js
 *
 * Dialog to display locations of configured sensors, the positions
 * they should be using on the body, and the metrics that are expected
 * to be computed.
 */

define(['jquery', 'd3', 'utils', 'crossroads', 'bootstrap-dialog',
        'metric_definition_helper', 'res_req_map',
        'app/dialog_in_series', 'app/test_sequence_builder',
        'app/placement_labels_EN', 'app/license_handler',
        'app/single_user_handler','app/joint_definitions'],
       function($, d3, utils, crossroads, BootstrapDialog,
                MetricDefinitionHelper, ResReqMap, DialogInSeries,
                testSequenceBuilder, placementLabels, LicenseHandler,
                SingleUserHandler,jointDefinitions)
{
    'use strict';

    var _dialogBody;
    var _isSixSensor = true;
    var _allTests;
    var _monitorMap;
    var _completingSequence = false;
    var _isV1;
    var _dialogInSeries;

    /**
     * Open the hardware placement dialog.
     * @param {Object} allTests The object containing pending and completed
     * tests.
     * @param {Object} monitorMap Object mapping monitor locations to IDs.
     * @param {Boolean} completingSequence True is user is completing an
     * incomplete sequence.
     * @param {Boolean} isSixSensor True if the configuration is limited to
     * six sensor locations.
     * @param {Boolean} isV1 True if V1 hardware is being used.
     */
    function openDialog(allTests, monitorMap, completingSequence, isSixSensor,
                        isV1) {
        _isV1 = isV1 ? isV1 : false;
        _allTests = allTests;
        _monitorMap = monitorMap.placements;
        _completingSequence = completingSequence;
        _isSixSensor = isSixSensor;
        _dialogInSeries = new DialogInSeries('dialog-test');

        var dialog = $('#dialog-test');
        dialog.find('#dialog-test-body').empty();
        $('#dialog-test-title-text').text('Hardware Placement');
        $('#dialog-test-footer').empty();
        dialog.find('.modal-dialog').removeClass('modal-sm').addClass('modal-lg');

        $('#dialog-test-body')
            .load('/mobility_clinic/html/undock.html', buildUndockDialog);
    }

    /**
     * Build the dialog that tells users to undock their monitors and
     * gives the placement locations for them.
     */
    function buildUndockDialog() {
        _dialogInSeries = new DialogInSeries('dialog-test');
        _dialogBody =  d3.select('#dialog-test-body');
        var footer = d3.select('#dialog-test-footer');

        footer.append('button')
            .html('<span class="iconic iconic-x"></span>\nBack\n')
            .attr('type', 'button')
            .attr('class', 'btn btn-link')
            .attr('id', 'hardware-placement-back-btn')
            .on('click', function() {
                _dialogInSeries.hide(false, function() {
                    testSequenceBuilder
                        .open(_allTests,
                              {isIncompleteSequence: _completingSequence});
                });
            });
        footer.append('button')
            .html('<span class="iconic iconic-caret-right"></span>\nNext\n')
            .attr('type', 'button')
            .attr('id', 'hardware-placement-next-btn')
            .attr('class', 'btn btn-success')
            .on('click', checkCalibrationInstructions);

        var pendingTests = _allTests.pendingTests.map(function(d) {
            return d.testName;
        });
        if (!LicenseHandler.getInstance().isKinematics() ||
            ((pendingTests.indexOf("MLK Walk") < 0) && (pendingTests.indexOf("Free Form") < 0))) {
            d3.selectAll('.mobility-kinematics-only').style('display', 'none');
        } else {
            _dialogBody.select('#show-supported-joint-angles')
            .on('click', showSupportedJointAngles);
        }

        // Body image and placement of sensor icons
        var sensorMan = 'sensor_placement_man_6.svg';
        if (_isSixSensor === false) {
            sensorMan = 'sensor_placement_man_17.svg';
        }
        if (SingleUserHandler.getInstance().isSingleUserUI()) {
            // FIXME: At some point, probably need to rethink how this
            // is triggered if there are other single user UI
            // applications.
            sensorMan = 'sensor_placement_man_mtbi.svg';
        }
        $(_dialogBody.select('#sensor-man-image').node())
            .load('../../mobility_clinic/res/' + sensorMan, sensorManLoaded);

        _dialogInSeries.show(false);

        _dialogBody.select('#show-supported-metric-groups')
            .on('click', showSupportedMetricGroups);

        _dialogBody.select('#show-sensor-placement')
            .on('click', showSensorPlacement);

        // Remove items not necessary for single user UI
        if (SingleUserHandler.getInstance().isSingleUserUI()) {
            _dialogBody.selectAll('.single-user-hide')
                .style('display', 'none');
        }
    }

    /**
     * Set the size of the sensor man svg and highlight
     * configured sensors.
     */
    function sensorManLoaded() {
        var svg = _dialogBody.select('svg');

        var body = _dialogBody.select('#monitor-map-table').select('tbody');
        body.selectAll('tr')
            .data(Object.keys(_monitorMap))
            .enter()
            .append('tr')
            .each(function(d) {
                var row = d3.select(this);
                row.append('td').text(function(d) {
                	var displayLabel = placementLabels[d];
                	if (!displayLabel) {
                		// This handles custom labels
                		return d;
                	}
                    return displayLabel;
                });
                row.append('td').text('#' + _monitorMap[d]);
                svg.select('#' + d)
                    .style('fill', '#000');
                fadeInOut(d);
            });
        // FIXME: Remove this check when we have time to implement placement
        // info for all locations in Moveo.
        if (_isSixSensor) {
            createPlacementPopovers(svg);
        }
    }

    /**
     * Set the animations for the configured placements
     * @param {String} label The placement label for monitor.
     */
    function fadeInOut(label) {
        _dialogBody.select('svg')
            .select('#' + label + '-circles')
            .style('animation', 'sensor-circles 3s ease infinite');
    }

    /**
     * Create popovers with information about sensor placement.
     * @param {d3.selection} svg The sensor man svg element.
     */
    function createPlacementPopovers(svg) {
        var popoverData = [
            {id: 'STERNUM',
             title: 'Sternum',
             content: 'Just below the top of the chest bone. ' +
             'Connector pointed straight down towards floor.'},
            {id: 'LUMBAR',
             title: 'Lumbar',
             content: 'Middle of low back, just below the belt line. ' +
             'Connector pointed straight down towards floor.'},
            {id: 'LEFT_WRIST',
             title: 'Left Wrist',
             content: 'On the left lower arm, worn like a watch. ' +
             'Connector pointed towards middle finger.'},
            {id: 'RIGHT_WRIST',
             title: 'Right Wrist',
             content: 'On the right lower arm, worn like a watch. ' +
             'Connector pointed towards middle finger.'},
            {id: 'LEFT_FOOT',
             title: 'Left Foot',
             content: 'Centered on the top of the foot (not including toes). '+
             'Connector pointed towards middle toe.'},
            {id: 'RIGHT_FOOT',
             title: 'Right Foot',
             content: 'Top of right foot. '+
             'Connector pointed towards middle toe.'}
        ];

        var jqSvg = $(svg.node());

        popoverData.forEach(function(d) {
            jqSvg.find('#' + d.id + '-circles').popover({
                title: d.title,
                content: d.content,
                trigger: 'hover',
                container: 'body'
            });
        });
    }

    /**
     * Determine the metric groups that are supported
     * for the current test and sensor configuration
     * and display in dialog.
     */
    function showSupportedMetricGroups() {
        var groups = MetricDefinitionHelper.getInstance().getMetricGroups();
        var testNames = [];
        var groupsByTest = [];
        _allTests.pendingTests.forEach(function(d) {
            if (testNames.indexOf(d.testName) < 0) {
                testNames.push(d.testName);
                groupsByTest.push({test: d, supported: [], unsupported: []});
            }
        });
        groupsByTest.forEach(function(d) {
            groups.forEach(function(e) {
                if (e.supportedTests.indexOf(d.test.testName) >= 0) {
                    var supportedSets = e.requiredSensors.map(
                        function(s) {
                            return utils.containsAll(s, Object.keys(_monitorMap));
                        }
                    );
                    if (supportedSets.indexOf(true) > -1) {
                        d.supported.push(e.displayName);
                    } else {
                        d.unsupported.push(e.displayName);
                    }
                }
            });
            d.supported = $.unique(d.supported)
                .filter(function(d) { return d !== ''; });
            d.unsupported = $.unique(d.unsupported)
                .filter(function(d) { return d !== ''; });
        });

        BootstrapDialog.show({
            id: 'supported-metrics-dialog',
            title: 'Metrics supported by sensor configuration',
            buttons: [
                {
                    id: 'supported-metrics-close-btn',
                    icon: 'iconic iconic-x',
                    label: 'Close',
                    cssClass: 'btn-link',
                    action: function(dialog) { dialog.close(); }
                }
            ],
            message: $('<div></div>')
                .load('/mobility_clinic/html/supported_metrics_dialog.html'),
            onshown: function() {
                buildSupportedMetricsDialog(groupsByTest);
            }
        });
    }

    /**
     * Determine the joint angles that are supported
     * for the current sensor configuration
     * and display in dialog.
     */
    function showSupportedJointAngles() {
        var supported = [];
        var unsupported = [];
        var jointNames = Object.keys(jointDefinitions);
        jointNames.forEach(function(d) {
            var supportedJoints = jointDefinitions[d].required_sensors.map(
                    function(s) {
                        return Object.keys(_monitorMap).indexOf(s) > -1;
                    }
                );
            if (supportedJoints.indexOf(false) > -1) {
                unsupported.push(d);
            } else {
                supported.push(d);
            }
        });

        BootstrapDialog.show({
            id: 'supported-joint-angless-dialog',
            title: 'Joint angles supported by sensor configuration',
            buttons: [
                {
                    id: 'supported-joint-angles-close-btn',
                    icon: 'iconic iconic-x',
                    label: 'Close',
                    cssClass: 'btn-link',
                    action: function(dialog) { dialog.close(); }
                }
            ],
            message: $('<div></div>')
                .load('/mobility_clinic/html/supported_joint_angles_dialog.html'),
            onshown: function() {
                buildSupportedJointAnglesDialog(supported, unsupported);
            }
        });
    }

    /**
     * Show the dialog that gives additional information on how the sensors
     * should be placed on the body.
     */
    function showSensorPlacement() {
        var dialogHtml = 'sensor_placement_dialog.html';
        if (!_isSixSensor) {
            dialogHtml = 'sensor_placement_dialog_17.html';
        }
        BootstrapDialog.show({
            title: 'Sensor Placement',
            size: BootstrapDialog.SIZE_WIDE,
            buttons: [
                {
                    icon: 'iconic iconic-x',
                    label: 'Close',
                    cssClass: 'btn-link',
                    action: function(dialog) { dialog.close(); }
                }
            ],
            message: $('<div></div>')
                .load('/mobility_clinic/html/' + dialogHtml),
        });
    }

    /**
     * Build the dialog to display the supported and unsupported
     * metric groups for the test being run.
     * @param {Array} groupsByTest List of tests and the supported and
     * unsupported metrics.
     */
    function buildSupportedMetricsDialog(groupsByTest) {
        var sensors = Object.keys(_monitorMap).map(function(d) {
        	var displayLabel = placementLabels[d];
        	if (!displayLabel) {
        		// This handles the case for custom locations
        		return d;
        	}
            return displayLabel;
        });
        d3.select('#configured-sensors')
            .text(sensors.join(', '));

        var div = d3.select('#supported-metrics');
        var testDivs = div.selectAll('.supported-metric-div')
            .data(groupsByTest)
            .enter()
            .append('div')
            .attr('class', 'row supported-metric-div');
        testDivs.append('h4')
            .text(function(d) { return d.test.testDisplayName; });

        var supported = testDivs.append('div').attr('class', 'col-sm-6');
        supported.append('h5').text('Supported');
        supported.selectAll('.supported-metric-group')
            .data(function(d) { return d.supported; })
            .enter()
            .append('p')
            .attr('class', 'supported-metric-group')
            .text(function(d) { return d; } );

        var unsupported = testDivs.append('div').attr('class', 'col-sm-6');
        unsupported.append('h5').text('Unsupported');
        unsupported.selectAll('.unsupported-metric-group')
            .data(function(d) { return d.unsupported; })
            .enter()
            .append('p')
            .attr('class', 'unsupported-metric-group')
            .text(function(d) { return d; });
    }

    /**
     * Build the dialog to display the supported and unsupported
     * joint angles for the test being run.
     * @param {Array} groupsByTest List of tests and the supported and
     * unsupported metrics.
     */
    function buildSupportedJointAnglesDialog(supported, unsupported) {
        var sensors = Object.keys(_monitorMap).map(function(d) {
            return placementLabels[d];
        });
        d3.select('#configured-sensors')
            .text(sensors.join(', '));

        var supportedDiv = d3.select('#supported-joint-angles').append('div').attr('class', 'col-sm-6');
        supportedDiv.append('h5').text('Supported Joints');
        supportedDiv.selectAll('.supported-metric-group')
            .data(supported)
            .enter()
            .append('p')
            .attr('class', 'supported-metric-group')
            .text(function(d) { return d; } );

        var unsupportedDiv = d3.select('#supported-joint-angles').append('div').attr('class', 'col-sm-6');
        unsupportedDiv.append('h5').text('Unsupported Joints');
        unsupportedDiv.selectAll('.unsupported-metric-group')
            .data(unsupported)
            .enter()
            .append('p')
            .attr('class', 'unsupported-metric-group')
            .text(function(d) { return d; });
    }

    /**
     * Determine if we need to display the dialog showing
     * calibration instructions before continuing.
     */
    function checkCalibrationInstructions() {
        var hasJointAngleTest = _allTests.pendingTests
            .reduce(function(a, b) {
                if (a) {
                    return a;
                } else {
                    if (b.testName === 'Free Form' ||
                        b.testName === 'MLK Walk') {
                        return true;
                    } else {
                        return false;
                    }
                }
            }, false);
        if (LicenseHandler.getInstance().isKinematics() &&
            hasJointAngleTest) {
            _dialogInSeries.hide(false, function() {
                $('#dialog-test-body')
                    .load('/mobility_kinematics/html/calibration_pose_instructions.html',
                          displayCalibrationInstructions);

            });
            return;
        }

        // Not doing a joint angle test, so continue
        _dialogInSeries.hide(false, function() {
            var key = ResReqMap.getInstance().setResponseData(_allTests);
            crossroads.parse('check_for_docked_monitors/' + key);
        });

    }

    /**
     * Display the joint angle calibration instructions.
     */
    function displayCalibrationInstructions() {
        var dialog = new DialogInSeries('dialog-test');
        _dialogBody =  d3.select('#dialog-test-body');
        var footer = d3.select('#dialog-test-footer');

        $('#dialog-test-footer').empty();
         $('#dialog-test-title-text').text('Joint Angle Calibration');
        footer.append('button')
            .html('<span class="iconic iconic-caret-right"></span>\nNext\n')
            .attr('type', 'button')
            .attr('class', 'btn btn-success')
            .on('click', function() {
                dialog.hide(false, function() {
                    var key = ResReqMap.getInstance().setResponseData(_allTests);
                    crossroads.parse('check_for_docked_monitors/' + key);
                });
            });
        dialog.show(false);
    }

    return {
        open: openDialog
    };
});
