/**
 * Copyright (c) 2018-2019 APDM, Inc.
 *
 * Create a animation for a trial with joint angles.
 */

define(['jquery', 'd3', 'three', 'model_protocol', 'server_handler',
        'error_handler', 'pubsub'],
       function($, d3, THREE, ModelProtocol, ServerHandler, ErrorHandler,
                PubSub)
{
    const MIN_HEIGHT = 300;
    const FRAME_STEP_TIME = 0.0078125;

    class JointAngleAnimation {

        /**
         * Create a JointAngleStaticAnimation instance.
         * @param {d3.selection} container The container to place the
         * animation in.
         * @param {Trial} trial The trial to with joint angles to animate.
         */
        constructor(container, trial) {
            this._trial = trial;
            this._container = container;
            this._canvas = container.select('canvas').node();
            this._renderer = null;
            this._mixer = null;
            this._clock = null;
            this._scene = null;
            this._camera = null;
            this._animationAction = null;
            this._requestAnimationId =  null;
            this._timeScale = 1;
            this._paused = true;
        }

        /**
         * Create and play the animation.
         */
        update() {
            this.clearScene();
            this.getBVHData();
        }

        /**
         * Get the BVH data from the server.
         */
        getBVHData() {
            let self = this;
            ServerHandler.getInstance().post(
                {
                    type: ModelProtocol.joint_angles_to_bvh,
                    analysisFile: this._trial.getAnalysisFileName()
                },
                'json',
                function(d) {
                    self.generateAnimation(d.bvhData);
                },
                function(d) {
                    ErrorHandler.showError(d.error().statusText);
                }
            );
        }

        /**
         * Generate the animation with the given BVH data.
         * @param {String} bvhData The BVH data as a string.
         */
        generateAnimation(bvhData) {

            this._clock = new THREE.Clock();
            let skeletonHelper;

            this.init();

            let loader = new THREE.BVHLoader();
            let result = loader.parse(bvhData);
            skeletonHelper = new THREE.SkeletonHelper(result.skeleton.bones[0]);
            skeletonHelper.skeleton = result.skeleton;

            let boneContainer = new THREE.Group();
            boneContainer.add(result.skeleton.bones[0]);
            this._scene.add(skeletonHelper);
            this._scene.add(boneContainer);
            // play animation
            this._mixer = new THREE.AnimationMixer(skeletonHelper);
            this._animationAction = this._mixer.clipAction(result.clip);
            this._animationAction.setEffectiveWeight(1.0)
                .play();
            this._mixer.timeScale = 0; // Start paused.
            this.animate();
        }

        /**
         * Initialize the controls, scene, and renderer for the animation.
         */
        init() {
            let controls;

            // Determine the starting height.
            let boundingRect =  this._container.node().getBoundingClientRect();
            let height = (9 * boundingRect.width) / 16;
            if (height < MIN_HEIGHT) {
                height = MIN_HEIGHT;
            }

            this._camera = new THREE.PerspectiveCamera(60,
                                                       boundingRect.width / height,
                                                       0.01, 100);
            this._camera.position.set(3, 3, 3);
            controls = new THREE.OrbitControls(this._camera, this._canvas);
            controls.minDistance = 1;
            controls.maxDistance = 40;
            this._scene = new THREE.Scene();
            this._scene.background = new THREE.Color(0xeeeeee);
            this._scene.add(new THREE.GridHelper(5, 10));

            this._renderer = new THREE.WebGLRenderer({ antialias: true,
                                                       canvas: this._canvas });
            this._renderer.setPixelRatio(window.devicePixelRatio);

            this._renderer.setSize(boundingRect.width, height);
        }

        /**
         * Animate the scene and set the frame request function.
         */
        animate() {
            if (this._renderer === null) {
                return;
            }
            let self = this;
            let  delta = this._clock.getDelta();

            if (this._requestAnimationId === null) {
                this._requestAnimationId = window.requestAnimationFrame(() => {self.animate();});
            }
            if (this._mixer) {
                this._mixer.update(delta);
            }
            this._renderer.render(this._scene, this._camera);
            if (!this._paused) {
                PubSub.publish('animation.time',
                               (this._animationAction.time * 1000));
            }
            // Cancel this if the canvas no longer exists
            if (!document.body.contains(this._canvas)) {
                this.clearScene();
                return;
            }
            this._requestAnimationId = null;
        }

        /**
         * Resize the canvas when window resized. Always try and
         * maintain a 16:9 aspect ratio.
         */
        resize() {
            let newRect = this._container.node()
                .getBoundingClientRect();
            let height = (9 * newRect.width) / 16;
            if (height < MIN_HEIGHT) {
                height = MIN_HEIGHT;
            }
            this._camera.aspect = newRect.width / height;
            this._camera.updateProjectionMatrix();
            this._renderer.setSize(newRect.width, height);
        }

        /**
         * Pause the animation.
         */
        pause() {
            this._paused = true;
            this._mixer.timeScale = 0;
        }

        /**
         * Play the animation, starting at the given time.
         * @param {Number} time The time to start playing at. If not
         * given, just starts playing.
         */
        play(time) {
            this._paused = false;
            if (time) {
                this._animationAction.time = time / 1000;
            }
            this._mixer.timeScale = this._timeScale;
        }

        /**
         * Advance the animation by one frame. Currently assuming that
         * the frame advance is going to be 1 / 128.
         */
        frameAdvance() {
            this.pause();
            this._animationAction.time =
                this._animationAction.time + FRAME_STEP_TIME;
                PubSub.publish('animation.time',
                               (this._animationAction.time * 1000));
        }

        /**
         * Reverse the animation by one frame. Currently assuming that
         * the frame advance is going to be 1 / 128.
         */
        frameReverse() {
            this.pause();
            this._animationAction.time =
                this._animationAction.time - FRAME_STEP_TIME;
                PubSub.publish('animation.time',
                               (this._animationAction.time * 1000));
        }

        /**
         * Update the current time of the animation.
         * @param {Number} time The time to update the animation to,
         * in milliseconds.
         */
        plotTime(time) {
            this._animationAction.time = time / 1000;
        }

        /**
         * Change the rate of the animation playback. 1 is normal
         * speed.
         * @param {Number} scaleFactor The speed at which animation plays.
         */
        playbackRate(scaleFactor) {
            this._timeScale = scaleFactor;
            this._mixer.timeScale = this._timeScale;
        }

        /**
         * Clear the scene of the animation DOM element.
         */
        clearScene() {
            if (this._renderer) {
                if (this._requestAnimationId !== null) {
                    window.cancelAnimationFrame(this._requestAnimationId);
                    this._requestAnimationId = null;
                }
                this._renderer.dispose();
                this._renderer = null;
            }
        }

        /**
         * Get/set the current trial being animated.
         * @param {Trial} newTrial The new trial to plot.
         * @return Returns current trial if called with no arguments,
         * otherwise this instance.
         */
        currentTrial(newTrial) {
            if (!arguments.length) {
                return this._trial;
            }
            this._trial = newTrial;
            this.update();
            return this;
        }

        /**
         * Get/set the current trial being animated.
         * @param {d3.selection} newContainer The container to put
         * animation in.
         * @return Returns current container if called with no
         * arguments, otherwise this instance.
         */
        currentContainer(newContainer) {
            if (!arguments.length) {
                return this._container;
            }
            this._container = newContainer;
            this.update();
            return this;
        }

        dispose() {
            this.clearScene();
        }
    }

    return JointAngleAnimation;
});
