/* Copyright (c) 2016-2019 APDM, Inc.
 *
 * Strip charts for continous joint angle data.
 */

define(['jquery', 'd3', 'utils', 'progress_dialog', 'bootstrap-dialog',
        'server_handler', 'model_protocol', 'app/joint_angle_data',
        'single_sensor_plot', 'tinysort', 'metric_definition_helper',
        'metric_utils', 'test_definition_handler', 'app/joint_definitions',
        'app/user_options_handler', 'error_handler', 'pubsub',
        'time_series_controls', 'time_based_controller'],
       function($, d3, utils, ProgressDialog, BootstrapDialog,
                ServerHandler, ModelProtocol, JointAngleData,
                SingleSensorPlot, tinysort, MetricDefinitionHelper,
                metricUtils, TestDefinitionHandler, jointDefinitions,
                UserOptionsHandler, ErrorHandler, PubSub, TimeSeriesControls,
                TimeBasedController)
{
    const LEFT = 'Left';
    const RIGHT = 'Right';
    const MIDDLE_JOINTS = ['Back', 'Trunk', 'Neck'];
    const LINES_COLORS = {
        'X': {color: 'red', line: 'x'},
        'Y': {color: 'green', line: 'y'},
        'Z': {color: 'blue', line: 'z'}
    };
    const ANNOTATION_ONE_COLOR = '#ff9626';
    const ANNOTATION_TWO_COLOR = '#1f74a5';

    class JointAngleStripPlots {

        constructor() {
            this._height = 300;
            this._width = 950;
            this._margin = {top: 10, bottom: 40, left: 50, right: 20};
            this._progDialog = null;
            this._container = null;
            this._trial = null;
            this._jointList = null;
            this._jointData = null;
            this._events = [];
            this._plottedJoints = [];
            this._allPlots = [];

            // Base data is initial data retrieved from server
            this._baseData = null;
            // Current data is base data plus additional that may
            // have been spliced in to give more detail when zoomed.
            this._currentData = null;

            // Options to control what range and what detail
            // of data to get.
            this._exportOptions = {
                displayWidth: this._width
            };

            this.dispatcher = d3.dispatch('yDomainChange',
                                          'dataUpdate',
                                          'renderedLinesChange');

            // The current extent of the displayed data.
            this._extents = [];

            this._timeSeriesControls = null;
            this._timeSeriesControlsId = null;

            // Update when resize event
            this._windowResizeSub = PubSub.subscribe('window.resized', () => {
                let plotContainer = this._container
                    .select('.report_vis_div').node();
                if (!plotContainer ||
                    !document.body.contains(plotContainer)) {
                    PubSub.unsubscribe(this._windowResizeSub);
                    this._windowResizeSub = null;
                    return;
                }
                let boundingRect = plotContainer.getBoundingClientRect();
                this._width = boundingRect.width;

                this._allPlots.map((d) => {
                    d.plot.width(this._width);
                    d.plot.plot(d.container);
                });
            });
        }

        /**
         * Open the dialog that will contain the plots.
         */
        initDialog() {
            let self = this;
            let dialog = new BootstrapDialog({
                size: BootstrapDialog.SIZE_WIDE,
                closable: false,
                title: 'Joint Angle Data',
                cssClass: 'joint-angle-dialog',
                message: $('<div></div>').load('joint_angle_strip_plot.html'),
                buttons: [{
                    label: 'Close',
                    action: function(dialog) {
                        dialog.close();
                    }
                }],
                onshown: function() {
                    self.getTrialEvents();
                },
                onhide: self.savePlottedJoints
            });

            dialog.open();
        }

        /**
         * Get the metrics for the trial, then parse the event
         * data and pass that on to the plot generation.
         */
        getTrialEvents() {
            this._extents = [new Date(0), new Date(this._jointData.Duration * 1000)];
            let self = this;
            this._events = [];
            this._trial.getMetrics()
                .then(function(metrics) {
                    self.parseEventData(metrics, 0);
                    self.createPlotBrush();
                    self.buildInitialPlots();
                    self._progDialog.close();
                }, function() {
                    self.createPlotBrush();
                    self.createEventKey();
                    self.buildInitialPlots();
                    self._progDialog.close();
                });
        }

        /**
         * Parse the event data.
         * @param {Object} metrics The trial metrics to pull events from.
         * @param {Number} startTime The time to add to the events.
         */
        parseEventData(metrics, startTime) {
            let defHandler = MetricDefinitionHelper.getInstance();
            let defs = defHandler.getMetricDefinitions(this._trial.getTestName());
            let eventKeys = defHandler.getEventNames(this._trial.getTestName());
            this._events = [];
            eventKeys.forEach(function(d) {
                let def = defs.get(d);
                this._events.push({
                    name: def.displayName,
                    color: def.eventColor,
                    events: metricUtils
                        .getEventStartEndTimes(this._trial.getTestName(), d,
                                               metrics, startTime)
                });
            }, this);
            // Add any annotations is they exist. Use some wack colors
            let annotations = this._trial.getAnnotations();
            Object.keys(annotations).forEach(function(d, i) {
                let times = annotations[d].startTimes.map(function(g) {
                    return {start: (g * 1000) + startTime,
                            end: (g * 1000) + startTime,
                            opacity: 1.0};
                });
                this._events.push({
                    name: annotations[d].annotationLabel,
                    color: (i === 0 ? ANNOTATION_ONE_COLOR : ANNOTATION_TWO_COLOR),
                    events: times
                });
            }, this);
            this.createEventKey();
        }

        /**
         * Build plots for the joints that were passed in on opening.
         * Also get the list of available joints.
         */
        buildInitialPlots() {
            this._jointList = [];
            let keys = Object.keys(this._jointData.jointAngleGroup);
            let self = this;
            keys.forEach(function(d) {
                let groupKeys = Object.keys(self._jointData.jointAngleGroup[d]);
                if (groupKeys.indexOf(LEFT) > -1 || groupKeys.indexOf(RIGHT) > -1) {
                    groupKeys.forEach(function(g) {
                        self._jointList.push({path: d + '/' + g,
                                              display: g + ' ' + d});
                    });
                } else {
                    self._jointList.push({path: d, display: d});
                }
            });

            // Filter out joints that we want to plot, but are not in the
            // joint list.
            let availableJoints = this._jointList.map(function(d) { return d.path; });
            this._plottedJoints = this._plottedJoints.filter(function(d) {
                return availableJoints.indexOf(d) > -1;
            });
            if (!this._plottedJoints.length) {
                this._plottedJoints.push(availableJoints[0]);
            }

            this._plottedJoints.forEach(function(d) {
                this.createSelectAndPlot(d);
            }, this);
        }

        /**
         * Create the brush that will handle the zoom and pan on the
         * sensor plots.
         */
        createPlotBrush() {
            this._timeSeriesControls =
                new TimeSeriesControls(d3.select('.sensor-plot-controls'),
                                       false)
                .startAndEndTimes(this._extents)
                .width(this._width)
                .isRelative(true)
                .update();
            this._timeSeriesControlsId = TimeBasedController.getInstance()
                .registerElement(this._timeSeriesControls);
        }

        /**
         * Create the dropdown selection and the plot for the given joint.
         * @param {String} jointPath The joint.
         */
        createSelectAndPlot(jointPath) {
            let joint = this._jointList.filter(function(d) {
                return d.path === jointPath;
            })[0];
            let container = d3.select('#joint-angle-container')
                .select('.report_vis_div')
                .append('div');

            if (!joint) {
                container.append('h3')
                    .style('width', '100%')
                    .style('text-align', 'center')
                    .attr('class', 'text-danger')
                    .text('No joint angle data available.');
                this._progDialog.close();
                return;
            }

            // Top and plot containers.
            let topContainer = container.append('div')
                .attr('class', 'margin-10-left');
            let plotContainer = container.append('div')
                .style('margin', '0 auto');

            this.populateSelection(jointPath, container,
                                   topContainer, plotContainer);
            this.generateJointPlot(topContainer, plotContainer, joint.path);
        }

        /**
         * Populate all the selection dropdowns after getting the list of
         * availble joints. Also add a "-" button to remove the plot.
         * @param {String} joint The joint to default selection to.
         * @param {d3.selection} container The overall container for plot and
         * select.
         * @param {d3.selection} topContainer The container above the plot.
         * @param {d3.selection} plotContainer The container to build plot in.
         */
        populateSelection(joint, container, topContainer,
                          plotContainer) {
            let selectEle = topContainer.append('select')
                .attr('class', 'test-btn', 'joint-angle-select');
            selectEle.selectAll('option')
                .data(this._jointList)
                .enter()
                .append('option')
                .attr('value', function(d) { return d.path; })
                .text(function(d) { return d.display; });

            let selectNode = $(selectEle.node());

            tinysort(selectNode.find('option'));
            selectNode.selectpicker({ width: 'auto' });
            selectNode.selectpicker('val', joint);

            // Have to keep track of the previous value.
            let self = this;
            selectNode.change(function() {
                self.generateJointPlot(topContainer, plotContainer, $(this).val());
            });

            topContainer.append('button')
                .attr('id', 'joint-angle-remove-plot-btn')
                .attr('class', 'btn btn-default test-btn')
                .on('click', function() {
                    container.remove();
                })
                .append('span').attr('class', 'iconic iconic-minus');
        }

        /**
         * Generate a plot for a given joint.
         * @param {d3.selection} topContainer The container holding selection
         * drop-down and key.
         * @param {d3.selection} plotContainer The container to build plot in.
         * @param {String} jointPath The joint to plot.
         */
        generateJointPlot(topContainer, plotContainer, jointPath) {
            // Handle non-bilateral joints.
            if (MIDDLE_JOINTS.indexOf(jointPath) > -1) {
                jointPath = jointPath + '/Middle';
            }
            let primaryData = this.getDataToPlot(jointPath);
            // Unique id for container
            let id = 'id-' + Date.now();
            this.drawPlot(id, plotContainer, primaryData, jointPath);

            this.buildKey(id, jointPath, topContainer);
        }

        /**
         * Find the data that is going to be plotted.
         * @param {String} path The path for the joint data.
         * @return {JointAngleData} The data to plot.
         */
        getDataToPlot(path) {
            if (path === 'None') {
                return null;
            }

            let splitPath = path.split('/');
            let angleData =
                new JointAngleData(path,
                                   this._jointData.jointAngleGroup[splitPath[0]][splitPath[1]]);
            return angleData;
        }

        /**
         * Draw the joint angle plot.
         * @param {String} id ID for the plot dispatcher.
         * @param {d3.selection} container Node to build plot in.
         * @param {JointAngleData} primaryData The primary data to plot.
         */
        drawPlot(id, container, primaryData, jointPath) {
            let plot = new SingleSensorPlot()
                .width(this._width)
                .height(this._height)
                .margin(this._margin)
                .vertLabel('Angle (°)')
                .horzLabel('Time')
                .plotSecondary(false)
                .relativeTime(true)
                .doProgress(false);
            this.setDataAccessors(plot, jointDefinitions[jointPath]);
            container.datum({
                primaryData: primaryData
            });
            if (this._events.length) {
                plot.plotEvents(true);
                plot.eventData(this._events);
            }
            container.attr('id', id);
            plot.dispatcher(this.dispatcher, id);
            plot.plot(container);

            this._allPlots.push({plot:plot, container: container,
                                 jointData: primaryData});
        }

        /**
         * Set the data accessors for the plot.
         * @param {SingleSensorPlot} plot The plot to set accessors on.
         * @param {Object} jointDef The joint defintion defining valid axes.
         */
        setDataAccessors(plot, jointDef) {
            let axes = Object.keys(jointDef.axes);
            let renderedLines = {x: true,
                                 y: true,
                                 z: true,
                                 scalar: false};
            if (hasAxis('X')) {
                plot.x1(function(d) { return d.X; });
            } else {
                renderedLines.x = false;
            }
            if (hasAxis('Y')) {
                plot.y1(function(d) { return d.Y; });
            } else {
                renderedLines.y = false;
            }
            if (hasAxis('Z')) {
                plot.z1(function(d) { return d.Z; });
            } else {
                renderedLines.z = false;
            }

            plot.time1(function(d) { return new Date(d.time); })
                .scalar1(null);

            if (hasAxis('X')) {
                plot.x2(function(d) { return d.X; });
            }
            if (hasAxis('Y')) {
                plot.y2(function(d) { return d.Y; });
            }
            if (hasAxis('Z')) {
                plot.z2(function(d) { return d.Z; });
            }
            plot.time2(function(d) { return d.time; });

            plot.renderedLines(renderedLines);
            function hasAxis(axisName) {
                return axes.indexOf(axisName) > -1;
            }
        }

        /**
         * Build the key for the plot, along with checkboxes to disable lines
         * on the plot.
         * @param {String} id The ID of the plot's dispatcher.
         * @param {String} joint The joint being plotted.
         * @param {d3.selection} container The container to build the key in.
         */
        buildKey(id, joint, container) {
            let axes = jointDefinitions[joint].axes;
            let values = [];

            Object.keys(axes).forEach(function(d) {
                let line = LINES_COLORS[d];
                line.name = axes[d].name;
                values.push(line);
            });

            container.selectAll('.joint-plot-key')
                .data([joint])
                .enter()
                .append('div')
                .attr('class', 'joint-plot-key');
            let keyContainer = container.select('.joint-plot-key');

            let self = this;
            let selectionSet = keyContainer.selectAll('.key-axis-span')
                .data(values, function(d) { return d.name; })
                .each(function(d) {
                    let span = d3.select(this);
                    let input = span.select('input')
                        .attr('type', 'checkbox')
                        .attr('id', d.line)
                        .attr('value', d.line)
                        .style('margin-right', '5px')
                        .on('click', null)
                        .on('click', function(d) {
                            let changed = {};
                            changed[d.line] = this.checked;
                            self.dispatcher.call('renderedLinesChange', self, changed, id);
                        });
                    span.select('svg')
                        .attr('width', 30)
                        .attr('height', 10)
                        .select('rect')
                        .attr('x', 0)
                        .attr('y', 0)
                        .attr('width', 30)
                        .attr('height', 10)
                        .style('fill', d.color);
                    span.select('span').style('margin', '0 5px')
                        .text(d.name);
                    // Annoying jQuery hack to make sure box checked correctly
                    $(input.node()).prop('checked', true);
                });

            selectionSet.exit().remove();

            selectionSet.enter()
                .append('span')
                .attr('class', 'key-axis-span')
                .style('margin-right', '10px')
                .each(function(d) {
                    let span = d3.select(this);
                    span.append('input')
                        .attr('type', 'checkbox')
                        .attr('id', d.line)
                        .attr('value', d.line)
                        .attr('checked', true)
                        .style('margin-right', '5px')
                        .on('click', function(d) {
                            let changed = {};
                            changed[d.line] = this.checked;
                            self.dispatcher.call('renderedLinesChange', self, changed, id);
                        });
                    span.append('svg')
                        .attr('width', 30)
                        .attr('height', 10)
                        .append('rect')
                        .attr('x', 0)
                        .attr('y', 0)
                        .attr('width', 30)
                        .attr('height', 10)
                        .style('fill', d.color);
                    span.append('span').style('margin', '0 5px')
                        .text(d.name);
                });

        }

        /**
         * Build a key for the events, and an "+" button to add
         * additional plots and a "reset" button for the zoom.
         */
        createEventKey() {
            if (this._events && this._events.length) {
                let totalWidth = 0;
                let svg = d3.select('.event-key').append('svg')
                    .attr('width', 300)
                    .attr('height', 14);
                svg.selectAll('g')
                    .data(this._events)
                    .enter()
                    .append('g')
                    .each(function(d) {
                        let g = d3.select(this);
                        g.append('rect')
                            .attr('x', 0)
                            .attr('y', 0)
                            .attr('width', 30)
                            .attr('height', 10)
                            .style('fill', d.color);
                        let text = g.append('text')
                            .text(d.name)
                            .attr('x', 35)
                            .attr('y', 10);
                        g.attr('transform', 'translate(' + totalWidth + ',0)');
                        totalWidth = totalWidth + text.node().getBBox().width + 45;

                    });
                svg.attr('width', totalWidth);
            }

            // Create the "add plot" button.
            let div = d3.selectAll('.event-key');
            let self = this;
            div.append('button')
                .attr('id', 'reset-joint-angle-zoom-btn')
                .attr('class', 'btn btn-default pull-right test-btn')
                .on('click', function() {
                    // FIXME: Need to do something here.
                })
                .text('Reset Zoom');
            div.append('button')
                .attr('id', 'add-joint-angle-plot-btn')
                .attr('class', 'btn btn-default pull-right test-btn')
                .on('click', function() {
                    self.createSelectAndPlot(self._jointList[0].path);
                })
                .append('span')
                .attr('class', 'iconic iconic-plus');
        }


        /**
         * Handle a change in the currently plotted domain.
         * @param {Array} newExtents The new domain being plotted.
         */
        domainChange(newExtents) {
            if (this._extents[0].getTime() === newExtents[0].getTime() &&
                this._extents[1].getTime() === newExtents[1].getTime()) {
                return;
            }
            this._extents = newExtents;
            this._allPlots.forEach((d) => {
                d.plot.domainChange(newExtents);
            });
            let indexes = this._findRange();
            this._sendDataRequest(indexes);

        }

        /**
         * Handle a change in the time from an animation for the plot
         * progress bars.
         * @param {Number} newTime The new time for the progress bar
         * in milliseconds.
         */
        animationTime(newTime) {
            this._allPlots.map((d) => {
                d.plot.updateProgress(newTime);
            });
        }

        /**
         * Send a request for new data for the plots.
         * @param {Object} indexes Object containing start and end indexes.
         */
        _sendDataRequest(indexes) {
            let self = this;
            ServerHandler.getInstance().post(
                {
                    type: ModelProtocol.get_joint_angle_data,
                    analysisFile: this._trial.getAnalysisFileName(),
                    options: JSON.stringify({
                        displayWidth: this._width,
                        startRow: indexes.startIdx,
                        endRow: indexes.endIdx
                    })
                },
                'json',
                function(d) {
                    self.updatePlotData(d);
                },
                function(e) {
                    ErrorHandler.showError(e.error().statusText);
                });
        }

        /**
         * Get the start index and end index into the array where the
         * new data should be inserted.
         * @return {Object} Map of start and end index into data array.
         */
        _findRange() {
            let sampleRate = 128;   // FIXME: Get this from some attribute in HDF?
            let startIdx = (this._extents[0] * 1000) *
                sampleRate * Math.pow(10, -6);
            let endIdx = ((this._extents[1] - this._extents[0]) * 1000) *
                sampleRate * Math.pow(10, -6) + startIdx;
            if (startIdx < 0) {
                startIdx = 0;
            }
            return {startIdx: Math.floor(startIdx), endIdx: Math.ceil(endIdx)};
        }

        /**
         * Update plots after getting new data.
         * @param {Object} newJointData Then new data from the server
         * to splice in.
         */
        updatePlotData(newJointData) {
            let splitPath = this._plottedJoints[0].split('/');
            let newData = new JointAngleData(
                this._plottedJoints[0], newJointData
                    .jointAngleGroup[splitPath[0]][splitPath[1]]).getData();
            let oldData = new JointAngleData(
                this._plottedJoints[0], this._jointData
                    .jointAngleGroup[splitPath[0]][splitPath[1]]).getData();

            let startTime = newData[0].time;
            let endTime = newData[newData.length - 1].time;
            let startIdx = 0;
            while (oldData[startIdx] && oldData[startIdx].time < startTime) {
                startIdx++;
            }
            let endIdx = startIdx;
            while(oldData[endIdx] && oldData[endIdx].time < endTime) {
                endIdx++;
            }

            // Cut and splice the data.
            this._allPlots.forEach(function(d) {
                let split = d.jointData.getName().split('/');
                let spliceInto = new JointAngleData(
                    d.jointData.getName(), this._jointData
                        .jointAngleGroup[split[0]][split[1]]);
                let spliceFrom = new JointAngleData(
                    d.jointData.getName(), newJointData
                        .jointAngleGroup[split[0]][split[1]]);

                utils.spliceIntoArray(spliceFrom.getData(), spliceInto.getData(),
                                      startIdx, endIdx);
                d.jointData = spliceInto;
                d.container.datum().primaryData.setData(spliceInto.getData());
            }, this);
            this.dispatcher.call('dataUpdate');
        }

        /**
         * Open a joint angle strip chart, using the angle
         * data for the given trial.
         * @param {Trial} trial the trial to plot angles for.
         * @param {Array} jointsToPlot List of joints to plot
         * @param {d3.selection} container The container to build the plots
         * in. If undefined, then a separate dialog will be opened.
         * on start-up.
         */
        open(trial, jointsToPlot, container) {
            this._trial = trial;
            this._plottedJoints = jointsToPlot ? jointsToPlot : [];
            this._container = container;

            // Open a dialog with a waiting spinner in it
            this._progDialog = new ProgressDialog({
                title: 'Loading joint angle data'
            }).show();

            let self = this;
            ServerHandler.getInstance().post(
                {
                    type: ModelProtocol.get_joint_angle_data,
                    analysisFile: trial.getAnalysisFileName(),
                    options: JSON.stringify(this._exportOptions)
                },
                'json',
                function(d) {
                    self._jointData = d;
                    // Add delays to duration.
                    let testDef = TestDefinitionHandler.getInstance()
                        .getTestDefinition(self._trial.getTestName(),
                                           self._trial.getConditionName());
                    self._jointData.Duration =
                        parseFloat(self._jointData.Duration, 10) +
                        parseInt(testDef.startDelay, 10);

                    if (self._container) {
                        self.getTrialEvents();
                    } else {
                        self.initDialog();
                    }
                },
                function(e) {
                    self._progDialog.close();
                    ErrorHandler.showError(e.error().statusText);
                });
            return this;
        }


        /**
         * Save the plotted joints to the user options on the study.
         * @return This instance.
         */
        _savePlottedJoints() {
            let joints = d3.selectAll('.joint-plot-key').data();
            UserOptionsHandler.getInstance().setPlottedJointAngles(joints);
            return this;
        }

        /**
         * Set the height for the individual plots.
         * @param {Number} plotHeight The height, in pixels, for the plot.
         * @return Current height if called with no arguments, otherwise this.
         */
        height(plotHeight) {
            if (!arguments.length) {
                return this._height;
            }
            this._height = plotHeight;
            return this;
        }

        /**
         * Set the margins for the individual plots.
         * @param {Object} margins The margins around the plot.
         * @return Current margins if called with no arguments, otherwise this.
         */
        margins(margins) {
            if (!arguments.length) {
                return this._margins;
            }
            this._margins = margins;
            return this;
        }

        /**
         * Clean up this instance.
         */
        dispose() {
            PubSub.unsubscribe(this._windowResizeSub);
            if (this._timeSeriesControls) {
                this._timeSeriesControls.dispose();
                this._timeSeriesControls = null;
            }
            TimeBasedController.getInstance()
                .unregisterElement(this._timeSeriesControlsId);
        }
    }

    return JointAngleStripPlots;
});
