/* Copyright (c) 2016-2017 APDM, Inc.
 *
 * joint_gait_variability_plots.js
 *
 * Joint angle variability plotted over normalized gait cycles.
 */

define(['jquery', 'd3', 'trial', 'server_handler', 'model_protocol',
        'progress_dialog', 'error_handler', 'tinysort', 'bootstrap-dialog',
        'utils', 'app/variability_plot', 'app/joint_definitions', 'svg_export'],
       function($, d3, Trial, ServerHandler, ModelProtocol, ProgressDialog,
                ErrorHandler, tinysort, BootstrapDialog, utils, VariabilityPlot,
                jointDefs, svgExport)
{
    'use strict';

    var LEFT = 'Left';
    var RIGHT = 'Right';

    var _progDialog;
    var _dialog;
    var _trial;
    var _jointList;
    var _jointData;
    var _jointName;
    var _plottedAxes;

    // Plot layout
    var width = 920;
    var _minHeight = 200;
    var _fullHeight = 630;
    var margin = {top: 10, bottom: 40, left: 50, right: 10};

    /**
     * Open the joint angle variability plots using the
     * given trial.
     * @param {Trial} plotTrial The trial to plot.
     */
    function openPlots(plotTrial) {
        _trial = plotTrial;
        _dialog = new BootstrapDialog({
            size: BootstrapDialog.SIZE_WIDE,
            title: 'Joint Angle Variability',
            cssClass: 'joint-angle-dialog',
            message: $('<div></div>').load('joint_angle_variability_plot.html'),
            buttons: [{
                label: 'Close',
                action: function(dialog) {
                    dialog.close();
                }
            }],
            onshown: function() {
                // Open a dialog with a waiting spinner in it
                _progDialog = new ProgressDialog({
                    title: 'Loading joint angle data'
                }).show();
                // Get the joint angle data to get a list of joints.
                ServerHandler.getInstance().post(
                    {
                        type: ModelProtocol.get_joint_angle_data,
                        analysisFile: _trial.getAnalysisFileName(),
                    },
                    'json',
                    function(d) {
                        getJointList(d.jointAngleGroup);
                        populateDropdown();
                        exportHandlers();
                        getVariabilityData(_jointList[0].path);
                    },
                    function(d) {
                        _progDialog.close();
                        ErrorHandler.showError(d.error().statusText);
                    });
            }
        });

        _dialog.open();
    }

    /**
     * Get the list of joints in the HDF analysis file.
     * @param {Object} rawJointGroups Object map of joints out of HDF file.
     */
    function getJointList(rawJointGroups) {
        _jointList = [];
        var keys = Object.keys(rawJointGroups);
        keys.forEach(function(d) {
            var groupKeys = Object.keys(rawJointGroups[d]);
            if (groupKeys.indexOf(LEFT) > -1 || groupKeys.indexOf(RIGHT) > -1) {
                groupKeys.forEach(function(g) {
                    _jointList.push({path: d + '/' + g,
                                    display: g + ' ' + d});
                });
            } else {
                _jointList.push({path: d + '/Middle', display: d});
            }
        });
    }

    /**
     * Populate the dropdown of joints and attach the change
     * handler.
     */
    function populateDropdown() {
        var select = d3.select('#joint-angle-variability-select');
        var selectNode = $(select.node());

        select.selectAll('option').data(_jointList)
            .enter()
            .append('option')
            .attr('value', function(d) { return d.path; })
            .text(function(d) { return d.display; });
        tinysort(selectNode.find('option'));

        selectNode.selectpicker({ width: 'auto' });
        selectNode.selectpicker('val', _jointList[0].path);
        selectNode.selectpicker('refresh');

        selectNode.change(function() {
            _progDialog = new ProgressDialog({
                title: 'Loading joint angle data'
            }).show();
            getVariabilityData($(this).val());
        });
    }

    /**
     * Get the variability data for the given joint, then open the plots
     * @param {String} joint The joint to get data for.
     */
    function getVariabilityData(joint) {
        _jointName = joint;
        ServerHandler.getInstance().post(
            {
                type: ModelProtocol.get_joint_angle_variability_plot_data,
                analysisFile: _trial.getAnalysisFileName(),
                joint: joint,
                returnCycles: true
            },
            'json',
            function(d) {
                _jointData = d;
                _plottedAxes =
                    $.extend({}, jointDefs[joint].axes);
                buildPlots();
                createCheckboxes();
            },
            function(d) {
                _progDialog.close();
                ErrorHandler.showError(d.error().statusText);
            });
    }

    /**
     * Create the checkboxes for selecting the joint axes to plot.
     */
    function createCheckboxes() {
        var axes = jointDefs[_jointName].axes;
        var container = d3.select('#joint-angle-container')
            .select('#axes-checkboxes');
        container.selectAll('.variability-checkbox').remove();

        var checkboxes = container.selectAll('.variability-checkbox')
            .data(Object.keys(axes).map(function(d) { return d; }));

        checkboxes.enter()
            .append('span')
            .attr('class', 'variability-checkbox')
            .style('margin-left', '10px')
            .text(function(d) {
                return axes[d].name; })
            .append('input')
            .attr('type', 'checkbox')
            .attr('class', 'checkbox-inline')
            .attr('checked', true)
            .attr('value', function(d) { return d; })
            .style('margin-top', '0')
            .style('margin-left', '2px')
            .on('click', function() {
                if ($(this).prop('checked')) {
                    _plottedAxes[$(this).val()] = axes[$(this).val()];
                } else {
                    delete _plottedAxes[$(this).val()];
                }
                buildPlots();
            });

        checkboxes.exit().remove();
    }

    /**
     * Build a plot for each of the axes in the variability plot and
     * remove plots not needed for the joint.
     */
    function buildPlots() {
        var axes = jointDefs[_jointName].axes;
        var numPlots = Object.keys(_plottedAxes).length;
        var plotHeight = _minHeight;
        if (numPlots) {
            plotHeight = _fullHeight / numPlots;
        }

        var plotContainers = d3.select('#joint-angle-plot-container')
            .attr('height', numPlots * plotHeight)
            .selectAll('g')
            .data(Object.keys(_plottedAxes), function(d) { return d; });
        plotContainers.enter()
            .append('g')
            .attr('id', function(d) { return utils.makeHtmlId(d); })
            .attr('transform', function(d, i) {
                return 'translate(0,' + (i * plotHeight) + ')';
            })
            .each(function(d, i) {
                var plot = new VariabilityPlot()
                    .height(plotHeight)
                    .width(width)
                    .margin(margin)
                    .title(axes[d].name)
                    .yAxisLabel('Angle (°)')
                    .drawCycles(true)
                    .drawMean(true)
                    .showYGrid(true)
                    .yGridValues([0])
                    .drawPercentilesArea(false);
                if ((i + 1) >= Object.keys(axes).length) {
                    plot.xAxisLabel('Percent of Gait Cycle (%)');
                }

                plot.update(d3.select(this).datum(_jointData[d]));
            });
        plotContainers.exit()
            .remove();

        _progDialog.close();
    }

    /**
     * Setup the handlers on the plot export buttons.
     */
    function exportHandlers() {
        var container = d3.select('#joint-angle-container');
        container.select('#export-to-svg-button')
            .on('click', function() {
		svgExport.exportSvgToSvg(container.select('svg'));
            });
        container.select('#export-to-png-button')
            .on('click', function() {
		svgExport.exportSvgToPng(container.select('svg'));
            });
    }

    /**
     * Build a plot for each of the axes in the joint stat.
     */
    return {
        open: openPlots
    };

});
