/* Copyright (c) 2017 APDM, Inc.
 *
 * login_handler.js
 */

define(['jquery', 'model_protocol', 'crossroads', 'res_req_map',
        'bootstrap-dialog', 'error_handler', 'app/user_options_handler'],
       function($, ModelProtocol, crossroads, ResReqMap,
                BootstrapDialog, ErrorHandler, UserOptionsHandler)
{
    'use strict';

    var _instance;
    var _dialog;
    var _currentStudy;
    var _studyUsers;
    var _currentUser;

    var ANONYMOUS_USER = 'anonymous';

    function init() {
        // Setup path handler for successful login.
        crossroads.addRoute('lightweightLoginCompleted/{key}', function(key) {
            lightweightLoginDone(ResReqMap.getInstance().getResponseData(key));
        });

        return {
            /**
             * Login a lightweight user.
             */
            login: getStudyUsers,

            /**
             * Logout the current user.
             */
            logout: logoutLightweightUser
        };
    }

    /**
     * Make a request to get the current list of study members
     * for the given study, then when that returns open the
     * login dialog.
     * @param {Study} study The current study.
     */
    function getStudyUsers(study) {
        _currentStudy = study;
        _studyUsers = [];
        var studyUsersRoute = crossroads
            .addRoute('studyUsersRetrieved/{key}', function(key) {
                crossroads.removeRoute(studyUsersRoute);
                _studyUsers = ResReqMap.getInstance()
                    .getResponseData(key).studyUsers;
                openLoginDialog();
            });
        var key = ResReqMap.getInstance().setRequestData(_currentStudy.getObjId());
        crossroads.parse('dispatch/getStudyUsers/' + key);
    }

    /**
     * Open the dialog to get the user name.
     */
    function openLoginDialog() {
        _dialog = new BootstrapDialog({
            title: 'User Login',
            message: $('<div></div>')
                .load('/mobility_clinic/html/lightweight_login.html'),
            closable: false,
            buttons: [
                {
                    icon: 'iconic iconic-account-login',
                    label: 'Login',
                    cssClass: 'btn-success',
                    action: validateLogin,
                }],
            onshown: populateTypeahead,
        });
        _dialog.open();
    }

    /**
     * Add the user names to the input typeahead and the custom login
     * message, if defined.
     */
    function populateTypeahead() {
        $('#login-name-input').typeahead({
            source: _studyUsers.map(function(d) {
                return d.mobilityLabUser.userName; })
        });

        var loginMessage = UserOptionsHandler.getInstance()
            .getLoginMessage();
        if (loginMessage && loginMessage !== '') {
            $('#custom-login-message').html(loginMessage);
        }
    }

    /**
     * Validate the login, then make call to server to login
     * given user.
     * NOTE: Currently, we're only making sure that a name exists.
     */
    function validateLogin() {
        var name = $('#login-name-input').val();
        if (!name || name === '') {
            ErrorHandler.showError('Login name is required.');
            return;
        }

        loginLightweightUser(name);
    }

    /**
     * Login a user by name only.
     */
    function loginLightweightUser(name) {
        var key = ResReqMap.getInstance()
            .setRequestData({userName: name});
        crossroads.parse('dispatch/lightweightLogin/' + key);
    }

    /**
     * Handler called when server informs us the lightweight login was
     * successful. Refresh the study when we do it.
     * @param {Object} login The login object from the server.
     */
    function lightweightLoginDone(login) {
        _currentUser = login;
        if (!login.userName || login.userName === '' ||
            login.userName === ANONYMOUS_USER) {
            return;
        }
        crossroads.parse('dispatch/refreshStudy');
        _dialog.close();
    }

    /**
     * Logout a lightweight user. Reopen the login dialog once done.
     */
    function logoutLightweightUser() {
        loginLightweightUser(ANONYMOUS_USER);
        getStudyUsers(_currentStudy);
    }

    return {
        getInstance : function() {
            // Initialize if the object hasn't been created before.
            if (!_instance) {
                _instance = init();
            }
            return _instance;
        }
    };
});
