/* Copyright (c) 2014-2018 APDM, Inc.
 *
 * mc_report_handler.js
 */

define(['jquery', 'd3', 'utils', 'session', 'trial', 'server_handler',
        'error_handler', 'model_protocol', 'res_req_map', 'model_dispatcher',
        'crossroads', 'hasher', 'metric_definition_helper',
        'app/report_enums', 'app/report_vis_handler',
        'app/report_table_handler', 'svg_export', 'app/user_options_handler',
        'app/trial_notes', 'app/report_utils', 'bootstrap-dialog', 'videojs',
        'pubsub'],
       function($, d3, utils, Session, Trial, ServerHandler,
                ErrorHandler, ModelProtocol, ResReqMap, ModelDispatcher,
                crossroads, hasher, MetricDefinitionHelper, reportEnums,
                VisHandler, TableHandler, svgExport, UserOptionsHandler,
                trialNotes, reportUtils, BootstrapDialog, videojs, PubSub)
{

    function McReport(trial) {

        var currSubject = null;
        var currTrial = trial;
        var compareTrials = [];
        var comparisonFilter = reportEnums.filterType.ALL;
        var reportContainer = null;
        var visWidth = 720;
        var visHeight = 400;
        var visType = reportEnums.reportType.vis.METRIC_VIS;
        var metricName = null;
        var testId = utils.makeHtmlId(currTrial.getTestName());

        // Get the table to show from the user options.
        var tableType = UserOptionsHandler.getInstance().getDefaultReportTable();
        if (!tableType) {
            tableType = reportEnums.reportType.table.METRICS_OVERVIEW_SD;
        }

        var visHandler = new VisHandler();
        var tableHandler = new TableHandler();

        // FIXME: BootstrapDialog is out of scope when update is called
        // by another module.
        var bsDialog = BootstrapDialog;

        // Object to hold array of single metric values.
        var metricTimeArrays = {};

        // The dispatch object to pass onto handler that allows events to
        // be passed between the visualization and table.
        var dispatch = d3.dispatch('barMouseIn',
                                   'barMouseOut',
                                   'prevTrialSelected',
                                   'nextTrialSelected',
                                   'barSelected',
                                   'metricSelected',
                                   'trialSelected',
                                   'baselineSelected',
                                   'metricGroupChange');

        // Setup the handlers we want here for dispatch events.
        // Handle bar selected dispatch. Argument is expected to be a test.
        dispatch.on('barSelected.' + testId, function(d) {
            testSelected(null, d);
        });

        dispatch.on('prevTrialSelected.' + testId, function() {
            selectPrevNextTrial(true);
        });
        dispatch.on('nextTrialSelected.' + testId, function() {
            selectPrevNextTrial(false);
        });

        // Metric in table selected.
        dispatch.on('metricSelected.' + testId, function(metricName) {
            // If previous vis was NONE or a JOINT_ANGLE_VARIABILITY,
            // then switch to the METRIC_VIS
            if (visType === reportEnums.reportType.vis.NONE ||
                visType === reportEnums.reportType.vis.JOINT_ANGLE_VARIABILITY) {
                visType = reportEnums.reportType.vis.METRIC_VIS;
                reportContainer.select('#metric-vis-report-button')
                    .classed('active', true);
                reportContainer.select('#metric-variability-report-button')
                    .classed('active', false);
            }
            updateTrialMetric(metricName);
        });

        // New baseline selected in bar plot.
        dispatch.on('baselineSelected.' + testId, baselineSelected);

        // Metric group has changed.
        var self = this;
        dispatch.on('metricGroupChange.' + testId, function() {
            self.update(true);
        });

        // Handler for change in size of the div
        var windowResizeSub = PubSub.subscribe('window.resized', () => {
            if (!reportContainer || !document.body.contains(reportContainer.node())) {
                PubSub.unsubscribe(windowResizeSub);
                return;
            }
            update();
        });

        // Private functions

        /**
         * Setup the handlers on the report type buttons.
         */
        function setupReportTypeButtons() {
            var visButton = reportContainer.select('#metric-vis-report-button');
            var metricData = reportContainer.select('#metric-data-report-button');
            var compareButton = reportContainer.select('#compare-report-button');
            var variabilityBtn =
                reportContainer.select('#metric-variability-report-button');

            visButton.on('click', function() {
                if (!isActive(this)) {
                    visType = reportEnums.reportType.vis.METRIC_VIS;
                    variabilityBtn.classed('active', false);
                }
                update();
            });

            metricData.on('click', function() {
                if (!isActive(this)) {
                    visType = reportEnums.reportType.vis.METRIC_LONGITUDINAL;
                    variabilityBtn.classed('active', false);
                }
                update();
            });

            compareButton.on('click', function() {
                if (!isActive(this)) {
                    visType = reportEnums.reportType.vis.METRIC_BAR_COMPARISON;
                    variabilityBtn.classed('active', false);
                }
                // Get some metric data if this is the first time through.
                if (metricName === null) {
                    updateMetricSelections();
                    updateTrialMetric(metricName);
                    return;
                }
                update();
            });

            variabilityBtn.on('click', function() {
                if (!isActive(this)) {
                    visType = reportEnums.reportType.vis.JOINT_ANGLE_VARIABILITY;
                    // Make sure none of the other buttons are active
                    reportContainer.select('#report-selection-button-group')
                        .selectAll('label').classed('active', false);
                }
                update();
            });

            function isActive(btn) {
                var active = d3.select(btn).classed('active');
                if (active) {
                    d3.select(btn).classed('active', false);
                    d3.event.stopPropagation();
                    d3.event.preventDefault();
                    visType = reportEnums.reportType.vis.NONE;
                }
                return active;
            }

            // If one of these buttons is active, set the visualization to that,
            // then update
            if(visButton.classed('active')) {
                visType = reportEnums.reportType.vis.METRIC_VIS;
                update();
            } else if(metricData.classed('active')) {
                visType = reportEnums.reportType.vis.METRIC_LONGITUDINAL;
                update();
            } else if(compareButton.classed('active')) {
                visType = reportEnums.reportType.vis.METRIC_BAR_COMPARISON;
                update();
            }
        }

        /**
         * Setup handler and behavior for the sequence report button.
         */
        function setupSequenceReportButton() {
            // Special button for handling sequence report.
            reportContainer.select('#report-sequence-report')
                .on('click', function() {
                    if (visType === reportEnums.reportType.vis.SEQUENCE_VIS) {
                        visType = reportEnums.reportType.vis.METRIC_VIS;
                        tableType = reportEnums.reportType.table.METRICS_OVERVIEW_SD;
                        reportContainer.select('#report-selection-button-group')
                            .style('display', 'inline-block');
                        updateTrial(currTrial.getAllTrials()[0]);
                        tableHandler.currentTrial(currTrial);
                        visHandler.currentTrial(currTrial);
                        setupTableTypeButtons();
                        setupReportTypeButtons();
                        setupMetricInfoButton();
                        updateJointAngleBtn();
                        filterTrials();
                        reportContainer.selectAll('.csv-export-option')
                            .style('display', 'list-item');
                    } else {
                        visType = reportEnums.reportType.vis.SEQUENCE_VIS;
                        tableType = reportEnums.reportType.table.NONE;
                        reportContainer.select('.hide-for-ctsib')
                            .style('display', 'none');
                        reportContainer.select('#report-selection-button-group')
                            .style('display', 'none');
                        reportContainer.select('#report-metric-info-button')
                            .style('display', 'none');
                        updateTrial(currTrial.sequence);
                        setupTableTypeButtons();
                        reportContainer.selectAll('.csv-export-option')
                            .style('display', 'none');
                        reportContainer.select('.report_vis_div')
                            .style('height', null);
                    }
                    update();
                });
            if (utils.isMultiSwayTest(currTrial) ) {
                reportContainer.select('#report-sequence-report')
                    .style('display', 'inline-block');
                reportContainer.select('#report-selection-button-group')
                    .style('display', 'none');
                visType = reportEnums.reportType.vis.SEQUENCE_VIS;
                reportContainer.selectAll('.csv-export-option')
                    .style('display', 'none');
            } else {
                reportContainer.select('#report-sequence-report')
                    .style('display', 'none');
            }

            // Open a stabilogram at start for sway trials.
            if (currTrial.getTestName() === 'Sway') {
                visType = reportEnums.reportType.vis.METRIC_VIS;
                reportContainer.select('#metric-vis-report-button')
                    .classed('active', true);
            }
        }

        /**
         * Set the handlers on the overview/table buttons.
         */
        function setupTableTypeButtons() {

            reportContainer.selectAll('#report-table-view-button')
                .on('click', null)
                .on('click', function() {
                    tableType = reportEnums.reportType.table.METRICS_DETAIL;
                    tableHandler.tableType(tableType);
                    tableHandler.update();
                });

            reportContainer.selectAll('#report-overview-view-button')
                .on('click', null)
                .on('click', function() {
                    tableType = reportEnums.reportType.table.METRICS_OVERVIEW;
                    tableHandler.tableType(tableType);
                    tableHandler.update();
                });

            reportContainer.selectAll('#report-overview-sd-view-button')
                .on('click', null)
                .classed('active', true)
                .on('click', function() {
                    tableType = reportEnums.reportType.table.METRICS_OVERVIEW_SD;
                    tableHandler.tableType(tableType);
                    tableHandler.update();
                });

            reportContainer.select('#report-table-button-group')
                .attr('id', 'report-table-button-group-' + testId);
            reportContainer.select('#upper-table-button-container')
                .attr('id', 'upper-table-button-container-' + testId);

            // Remove the buttons if this is a CTSIB like test.
            if (utils.isMultiSwayTest(currTrial)) {
                reportContainer.selectAll('.table-button-container')
                    .style('display', 'none');
                return;
            }
            reportContainer.selectAll('.table-button-container')
                .style('display', 'inline-block');

            if (currTrial.getTestName() === 'MLK Walk') {
                tableType = reportEnums.reportType.table.METRICS_DETAIL;
                reportContainer.selectAll('#report-overview-sd-view-button')
                    .classed('active', false);
                reportContainer.selectAll('#report-table-view-button')
                    .classed('active', true);
            }
        }

        function addSelectionPopovers() {
            // Determine the text for the popover.
            var popovers = [
                {
                    id: '#report-table-view-button',
                    title: 'Metric Values',
                    content: 'The subject\'s value and the percentage change ' +
                        'compared to the baseline selected within the ' +
                        'Longitudinal view. The normative range represents ' +
                        'the 5th-95th percentile of the values for this ' +
                        'metric in our normative database (if normative data is available).'
                },
                {
                    id: '#report-overview-sd-view-button',
                    title: 'Z-Score',
                    content: 'The subject\'s value displayed as the number ' +
                        'of standard deviations from the mean normative value (if normative data is available).' +
                        ' The normative range represents the 5th-95th percentile ' +
                        'of the values for this metric in our normative database.'
                },
                {
                    id: '#report-overview-view-button',
                    title: 'Percentiles',
                    content: 'The subject\'s value displayed as a percentile ' +
                        'relative to the values for this metric in our ' +
                        'normative database (if normative data is available).'
                },
                {
                    id: '#metric-vis-report-button',
                    title: 'Detailed Visualization',
                    content: 'A visual overview of the metric, providing ' +
                        'the metric value, percent of norm (if normative data is available), variability ' +
                        'within the trial, and asymmetry (if bilateral).'
                },
                {
                    id: '#metric-data-report-button',
                    title: 'Drill-Down',
                    content: 'A graph of how the metric progressed over ' +
                        'time within the selected trial.'
                },
                {
                    id: '#compare-report-button',
                    title: 'Longitudinal',
                    content: 'A comparison of the metric across multiple ' +
                        'trials of the same trial test type.'
                },
                {
                    id: '#metric-variability-report-button',
                    title: 'Joint Angle Variability',
                    content: 'Joint angle through each detected gait ' +
                        'cycle. Lines represent the mean and 5th to 95th ' +
                        'percentiles of these values.'
                }
            ];

            popovers.forEach(function(d) {
                $(reportContainer.select(d.id).node()).popover({
                    placement: 'bottom',
                    trigger: 'hover',
                    title: d.title,
                    content: d.content,
                    container: 'body',
                    delay: {'show': 500, 'hide': 100}
                });
            });
        }

        /**
         * Create the handler on the Next/Prev arrows on report.
         */
        function setNavigationHandlers() {
            var prevBtn = reportContainer.select('#report-prev-trial-button');
            prevBtn.on('click', null);
            prevBtn.on('click', function() {
                dispatch.call('prevTrialSelected', this);
            });

            var nextBtn = reportContainer.select('#report-next-trial-button');
            nextBtn.on('click', null);
            nextBtn.on('click', function() {
                dispatch.call('nextTrialSelected', this);
            });

            var backBtn = d3.select('#report-back-to-subject-button');
            backBtn.on('click', null);
            backBtn.on('click', function() {
                var objId = currSubject.getObjId();
                crossroads.parse('dispatch/subject?id=' + objId.getId() +
                                 '&localId=' + objId.getLocalId());
            });
        }

        /**
         * Set which visualization buttons are displayed based on the
         * current selection
         */
        function setVisualizationOptions() {
            if (!metricName) {
                // If no metrics are selected for display, do not enable
                // any report style buttons.baselineSelected
                reportContainer.select('#report-selection-button-group')
                    .style('display', 'none');
                return;
            }
            reportContainer.select('#report-selection-button-group')
                .style('display', 'inline-block');

            updateJointAngleBtn();
        }

        /**
         * Set the visibility options on the joint angle variability plot
         * button.
         */
        function updateJointAngleBtn() {
            if (currTrial.getTestName() !== 'Free Form' &&
                currTrial.getTestName() !== 'MLK Walk') {
                return;
            }
            var variabilityBtn =
            reportContainer.select('#metric-variability-report-button');
            ServerHandler.getInstance().post(
                {
                    type: ModelProtocol.get_joint_angle_list,
                    analysisFile: currTrial.getAnalysisFileName()
                },
                'json',
                function(d) {
                    if (d.jointList.length) {
                        variabilityBtn.style('display','inline-block');
                    } else {
                        variabilityBtn.style('display','none');
                    }
                },
                function() {
                    variabilityBtn.style('display','none');
                }
            );
        }

        /**
         * Hander for test selection.
         * @param _ The event - ignored.
         * @param newTest The test to display report for.
         */
        function testSelected(_, newTest) {
            updateTrial(newTest);
            if (newTest instanceof Trial) {
                updateTrialMetric(metricName);
            } else {
                update();
            }
        }

        /**
         * Update the table and visualization with the new trial single
         * metric data.
         * @param metric The new metric name.
         */
        function updateTrialMetric(metric) {
            metricName = metric;
            // Set which visualization buttons are active.
            setVisualizationOptions();

            update();
            if (currTrial instanceof Session) {
                return;
            }
        }

        /**
         * Set a new baseline trial.
         * @param {Trial} trial Trial to set as new baseline.
         */
        function baselineSelected(trial) {
            tableHandler.baselineTrial(trial);
            visHandler.baselineTrial(trial);
            tableHandler.update();
        }
        /**
         * Setup handlers on the info button for metrics.
         */
        function setupMetricInfoButton() {
            if (utils.isMultiSwayTest(currTrial)) {
                reportContainer.select('.hide-for-ctsib')
                    .style('display', 'none');
                return;
            } else {
                reportContainer.select('.hide-for-ctsib')
                    .style('display', 'inline');
            }
            if (!metricName) {
                reportContainer.select('#report-metric-info-button')
                    .style('display', 'none');
            } else {
                reportContainer.select('#report-metric-info-button')
                    .style('display', 'inline');
            }
            var selector = 'report-metric-info-button-' + testId;
            var testName = currTrial.getTestName();
            reportContainer.select('#report-metric-info-button')
                .attr('id', selector);
            $(reportContainer.node()).popover('destroy');
            $(reportContainer.node()).popover({
                selector: '#' + selector,
                placement: 'bottom',
                trigger: 'hover',
                html: true,
                title: function() {
                    return getMetricInfoTitle(testName, visType);
                },
                content: function() {
                    var metricDefs = MetricDefinitionHelper.getInstance()
                        .getMetricDefinitions(testName);
                    var content = metricDefs.get(metricName).definition;

                    var metricObj = [];
                    metricObj.metricFullName = metricName;
                    var metricStdAndNorms = MetricDefinitionHelper.getInstance()
                        .getMeanStdDevAndNorms(testName, "", metricObj);
                    var normsExistForMetric = !!metricStdAndNorms;

                    if (visType === reportEnums.reportType.vis.NONE) {
                        return content;
                    }

                    var key = '../../mobility_clinic/res/';

                    var normString = 'The normative range represents the ' +
                        '5th-95th percentile of the values for this metric ' +
                        'in our normative database.';

                    switch (visType) {
                    case reportEnums.reportType.vis.METRIC_VIS:
                        if (testName === 'Sway') {
                            key = key + 'stabilogram_key.svg';
                        } else {
                            if (normsExistForMetric) {
                                key = key + 'vis_key.svg';
                                content += '</br></br>' + normString;
                            } else {
                                key = key + 'vis_key_no_norms.svg';
                            }
                            if (metricDefs.get(metricName).metricCardinality ===  'MANY') {
                                content += '</br></br>The subject range represents ' +
                                    'the full range of the individual values within ' +
                                    'this trial.';
                            }
                        }
                        break;
                    case reportEnums.reportType.vis.METRIC_LONGITUDINAL:
                        content += '</br></br>Individual values indicate how the metric ' +
                            'progressed over the duration of the trial.';
                        if (normsExistForMetric) {
                            key = key + 'longitudinal_key.svg';
                            content += '</br></br>' + normString;
                        } else {
                            key = key + 'longitudinal_key_no_norms.svg';
                        }
                        break;
                    case reportEnums.reportType.vis.METRIC_BAR_COMPARISON:
                        content += '</br></br>The color of each bar is dependent on the  ' +
                            'the color specified for the test condition within the Test and Condition editor.';
                        if (normsExistForMetric) {
                            key = key + 'bar_key.svg';
                            content += '</br></br>' + normString;
                        } else {
                            key = "";
                        }
                        break;
                    case reportEnums.reportType.vis.JOINT_ANGLE_VARIABILITY:
                        key = key + 'variability_key.svg';
                        content = 'These plots represent the variability of ' +
                            'the joint during detected gait cycles. The mean ' +
                            'value over the gait cycles is represented for ' +
                            'both the left and right hand side, along with ' +
                            'the 5th-95th percentile range. Each gait cycle ' +
                            'is normalized, with a range of 0 (heel strike) ' +
                            'to 100% (next heel strike)';
                    }
                    if (key === "") {
                        return content;
                    } else {
                        return content + '<div class="metric-info-popover">'+
                            '<img src="' + key + '"></div>';
                    }
                },
                container: 'body'
            });
        }

        /**
         * Get the title for the metric info popover.
         * @param {String} testName The name of the current test type.
         * @param {reportEnums.reportType.vis} reportType The currently selected plot type.
         * @return {String} The title for the popover.
         */
        function getMetricInfoTitle(testName, reportType) {
            if (reportType === reportEnums.reportType.vis.JOINT_ANGLE_VARIABILITY) {
                return 'Gait Joint Angle Variability';
            }
            return MetricDefinitionHelper.getInstance()
                .getTestMetrics(testName)[metricName].displayName;
        }

        /**
         * Update the report with the prev/next trial in the compare set,
         * or the next trial by type if the compare set is empty.
         * @param isPrev True if moving to previous trial, otherwise
         * move to next.
         */
        function selectPrevNextTrial(isPrev) {
            var newTrial = reportUtils
                .getPreviousOrNextTrial(currSubject, currTrial,
                                        compareTrials, isPrev);
            // Let listeners know that a new trial has been selected.
            dispatch.call('trialSelected', this, newTrial);
            updateTrial(newTrial);

            // Only update this if trial is not a session.
            if (newTrial instanceof Trial) {
                updateTrialMetric(metricName);
                // If trial is part of session, the activate the report
                // sequence button, otherwise hide it.
                if (currTrial.getSequence() &&
                    utils.isMultiSwayTest(currTrial.getSequence())) {
                    reportContainer.select('#report-sequence-report')
                        .style('display', 'inline-block');
                } else {
                    reportContainer.select('#report-sequence-report')
                        .style('display', 'none');
                }
            } else {
                update();
            }
        }

        /**
         * Update the current trial.
         * @param newTrial The new trial.
         */
        function updateTrial(newTrial) {
            var oldTestName;
            if (currTrial && currTrial !== null) {
                oldTestName = currTrial.getTestName();
            }
            currTrial = newTrial;
            if (metricName === null || metricName === '' ||
                currTrial.getTestName() !== oldTestName) {
                updateMetricSelections();
            }

            metricTimeArrays = {};
            visHandler.currentTrial(newTrial);
            tableHandler.currentTrial(newTrial);
            trialNotes.init(currTrial, reportContainer);
            if (!ServerHandler.getInstance().hasJsPostHandler()) {
                getVideoURL();
            }
        }

        /**
         * Filter the comparison trials and set the comparison group.
         */
        function filterTrials() {
            compareTrials = reportUtils
                .getFilteredTrials(currTrial, currSubject, comparisonFilter);
            tableHandler.compareTrials(compareTrials);
            visHandler.compareTrials(compareTrials);
            if (visHandler.visType() ===
                reportEnums.reportType.vis.METRIC_BAR_COMPARISON) {
                visHandler.update();
            }
        }

        /**
         * Update the metric name with a change in metric selections from
         * the metric helper.
         */
        function updateMetricSelections() {
            if (!utils.isMultiSwayTest(currTrial)) {
                var metricSet = MetricDefinitionHelper.getInstance()
                    .getTestMetrics(currTrial.getTestName());
                // Set the metric name to the first entry in the new metric set.
                metricName = Object.keys(metricSet)[0];
                tableHandler.metricName(metricName);
            }
        }

        /**
         * Generate the proper title for the report.
         */
        function generateTitle() {
            var title = d3.select('#report-title');
            var testName = currTrial.getTestDisplayName();
            var text = '';

            text = testName;
            if (!utils.isMultiSwayTest(currTrial)) {
                text = text + ', ' + currTrial.getConditionDisplayName();
            }
            text = text + ' - ' +
                utils.getShortStringFromDate(currTrial.getDateObj());

            // Get the error messages and put them in the dialog.
            var logMsgs = getCurrTrialLogString();
            if (logMsgs.length > 0) {
                var msgLink = '<a id="trial-log-errors-btn" ' +
                    'class="trial-log-button text-muted" >' +
                    '<span class="iconic iconic-info"></span></a>';
                var types = logMsgs.map(function(d) { return d.type; });
                if (types.indexOf('Error') > -1) {
                    msgLink = '<a id="trial-log-errors-btn" ' +
                        'class="trial-log-button text-danger" >' +
                        '<span class="iconic iconic-warning"></span></a>';
                } else if (types.indexOf('Warning') > -1) {
                    msgLink = '<a id="trial-log-errors-btn" ' +
                        'class="trial-log-button text-warning" >' +
                        '<span class="iconic iconic-info"></span></a>';
                }
                text = text + msgLink;
            }
            title.html(text);

            d3.select('#trial-log-errors-btn')
                .on('click', function() {
                    showTrialAnalysisLog(logMsgs);
                });
        }

        /**
         * Show the trial analysis log in a dialog.
         * @param {Array} logMsgs Array of log messages to display in the dialog.
         */
        function showTrialAnalysisLog(logMsgs) {
            bsDialog.show({
                id: 'trial-analysis-log-dialog',
                title: 'Trial Analysis Log',
                message: '<div id="trial-log-container"></div>',
                buttons: [
                    {
                        id: 'trial-analysis-log-dialog-close-btn',
                        icon: 'iconic iconic-x',
                        label: 'Close',
                        cssClass: 'btn-link',
                        action: function(dialog) { dialog.close(); }
                    }
                ],
                onshown: function() {
                    d3.select('#trial-analysis-log-dialog')
                        .select('#trial-log-container')
                        .selectAll('p')
                        .data(logMsgs)
                        .enter()
                        .append('p')
                        .text(function(d) { return d.message; })
                        .each(function(d) {
                            var classType = 'lead';
                            if (d.type === 'Error') {
                                classType = classType + ' text-danger';
                            } else if (d.type === 'Warning') {
                                classType = classType + ' text-warning';
                            }
                            d3.select(this).attr('class', classType);
                        });
                }
            });
        }

        /**
         * Get the log strings for the current trial or session.
         * @return The log messages for the current trial/session.
         */
        function getCurrTrialLogString() {
            var messages = [];
            // If there is no "getLogStr", must be a Session.
            if (!currTrial.getLogStr) {
                var sessionTrials = currTrial.getAllTrials();
                sessionTrials.forEach(function(d){
                    var trialLogs = [];
                    if (d.getLogStr() !== '') {
                        trialLogs.push(d.getConditionName());
                        trialLogs = trialLogs
                            .concat(utils.parseTrialLogString(d.getLogStr()));
                    }
                    messages = messages.concat(trialLogs);
                });
                return messages;
            }
            return utils.parseTrialLogString(currTrial.getLogStr());
        }

        /**
         * Update the report.
         * TODO: Decide if we want to do all the setting of the vis and
         * table handler stuff here, or piecemeal as we get it.
         */
        function update() {
            var metricSet = MetricDefinitionHelper.getInstance()
                .getTestMetrics(currTrial.getTestName());
            // Make sure we have metrics selected to display
            if (metricSet && !Object.keys(metricSet).length) {
                metricSet.Duration = MetricDefinitionHelper.getInstance()
                    .getMetricDefinitions(currTrial.getTestName()).get('Duration');
            }

            var trialsOfType = currSubject
                .getTrialsAndSequencesByTestName()
                .get(currTrial.getTestName());
            // Handle the preview case where there may not be any trials
            // for the subject.
            // trialsOfType = trialsOfType ? trialsOfType : [currTrial];

            generateTitle();
            currSubject.getTrialsAndSequencesByTestName();
            visHandler
                .metricSet(metricSet)
                .metricName(metricName)
                .trialsOfType(trialsOfType)
                .visWidth(visWidth)
                .visHeight(visHeight)
                .visType(visType)
                .dispatch(dispatch);
            tableHandler
                .trialsOfType(trialsOfType)
                .metricName(metricName)
                .tableType(tableType)
                .dispatch(dispatch);

            visHandler.update();
            tableHandler.update();
        }


        /***************************************************************************
         * Video playback
         **************************************************************************/

        /**
         * Retrieve the URL of the video corresponding to the current trial.
         * @param dive The trial to retrive the video URL from
         */
        function getVideoURL() {
            if (!currTrial || currTrial === null || currTrial instanceof Session) {
                return;
            }

            ServerHandler.getInstance().post(
                {
                    type : ModelProtocol.get_video_metadata,
                    fileName : currTrial.getDataFileName()
                },
                'json',
                function(d) {
                    var videoURL = null;
                    if (d.available === true) {
                        videoURL = '/file/' + d.localFilePathName;
                    }
                    updateVideo(videoURL);
                },
                function(d) {
                    ErrorHandler.showError(d.error().statusText);
                });
        }

        /**
         * Called to update the display of the video component, depending on
         * specified selections and the availability of the video(s).
         */
        function updateVideo(videoURL) {
            reportContainer.select('#trial-video').remove();
            var menuOption = reportContainer.select('#play-video-button');
            menuOption.on('click', null);
            if (videoURL !== null) {
                menuOption.on('click', function() {
                    buildVideoDialog(videoURL);
                });
            } else {
                reportContainer.select('#play-video-button').style('display', 'none');
            }
        }

        /**
         * Create the dialog to display the video.
         * @param {String} videoURL The location of the video file.
         */
        function buildVideoDialog(videoURL) {
            bsDialog.show({
                id: 'report-video-dialog',
                title: 'Trial video',
                message: '<div id="dialog-report-video-container"></div>',
                buttons: [
                    {
                        id: 'report-video-dialog-close-btn',
                        icon: 'iconic iconic-x',
                        label: 'Close',
                        cssClass: 'btn-link',
                        action: function(dialog) { dialog.close(); }
                    }
                ],
                onshown: function() {
                    initVideoPlayer(videoURL);
                }
            });
        }

        /**
         * Initialize the settings for the video player.
         * @param {String} videoURL The URL of the video to play.
         */
        function initVideoPlayer(videoURL) {
            // Get rid of any existing player.
            // FIXME: Doesn't seem to be a way to tell if the player
            // has beem properly initialized here.
            try {
                videojs('trial-video').dispose();
            } catch(e) {
                // Don't need to do anything, since the player doesn't exist.
            }
            d3.select('#dialog-report-video-container')
                .append('video').attr('id', 'trial-video')
                .attr('class', 'video-js vjs-default-skin');

            videojs('trial-video',
                    {'preload': 'auto', 'controls': true, 'loop': true,
                     'width': 560, 'height': 560})
                .src(videoURL);
        }

        /***************************************************************************
         * Public functions
         **************************************************************************/

        /**
         * Get or set the current subject.
         */
        this.currentSubject = function(d) {
            if (!arguments.length) {
                return currSubject;
            }
            currSubject = d;
            return this;
        };

        /**
         * Get or set the current trial.
         */
        this.currentTrial = function(d) {
            if (!arguments.length) {
                return currTrial;
            }
            updateTrial(d);
            return this;
        };

        /**
         * Get or set the current comparison set.
         */
        this.compareTrials = function(d) {
            if (!arguments.length) {
                return compareTrials;
            }
            compareTrials = d;
            visHandler.compareTrials(compareTrials);
            tableHandler.compareTrials(compareTrials);
            return this;
        };

        /**
         * Set the filter type for trials to display in comparisons.
         * @param {reportEnums.reportType.filter} d Enum of filter to use.
         * @return If called with no arguments, current filter, otherwise this.
         */
        this.comparisonFilter = function(d) {
            if (!arguments.length) {
                return comparisonFilter;
            }
            comparisonFilter = d;
            if (currTrial) {
                filterTrials();
            }
            return this;
        };

        /**
         * Set the container for the report, and assign the handlers we need.
         */
        this.reportContainer = function(d) {
            if (!arguments.length) {
                return reportContainer;
            }
            reportContainer = d;
            setNavigationHandlers();

            tableHandler.tableDiv(d.select('.report_table_div'));
            visHandler.visDiv(d.select('.report_vis_div'));

            setupTableTypeButtons();
            setupReportTypeButtons();
            setupMetricInfoButton();
            setupSequenceReportButton();
            addSelectionPopovers();
            trialNotes.init(currTrial, reportContainer);

            return this;
        };

        /**
         * Get or set the maximum width of the visualization.
         */
        this.visWidth = function(d) {
            if (!arguments.length) {
                return visWidth;
            }
            visWidth = d;
            visHandler.visWidth(visWidth);
            return this;
        };

        /**
         * Get or set the maximum height of the visualization.
         */
        this.visHeight = function(d) {
            if (!arguments.length) {
                return visHeight;
            }
            visHeight = d;
            visHandler.visHeight(visHeight);
            return this;
        };

        /**
         * Export the current visualization as a png.
         */
        this.exportVisAsPng = function() {
            svgExport.exportSvgToPng(visHandler.visDiv().select('svg'));
            return this;
        };

        /**
         * Export the current visualization as an SVG.
         */
        this.exportVisAsSvg = function() {
            svgExport.exportSvgToSvg(visHandler.visDiv().select('svg'));
            return this;
        };

        /**
         * Update the trial string in the report header.
         */
        this.updateTrialTitle = function() {
            generateTitle();
        };

        /**
         * Get the dispatcher for the report.
         * @return {d3.dispatch} The dispatch object for the report.
         */
        this.getDispatcher = function() {
            return dispatch;
        };

        /**
         * Get/set the table to display with the report.
         * @param {reportEnum.reportType.table} d The table type.
         * @return {reportEnum.reportType.table} The table type.
         */
        this.tableType = function(d) {
            if (!arguments.length) {
                return tableType;
            }
            tableType = d;
            return this;
        };

        /**
         * Update the report. Updates both the visualization and the data
         * table.
         * @param groupChange True if update is being called because of a
         * change in the selected metric group.
         */
        this.update = function(groupChange) {
            // Handle the update through updateTrialMetric. This will
            // make sure we have vaild time data for the selected metric.
            if (groupChange === true || !metricName ||
                metricName === null) {
                updateMetricSelections();
            }
            updateTrialMetric(metricName);
            return this;
        };

    }

    return McReport;
});
