/* Copyright (c) 2014-2019 APDM, Inc.
 *
 * metric_overview_vis.js
 *
 * Class for creating the metric overview table visualization.
 */

define(['jquery', 'd3', 'metric_definition_helper', 'utils', 'error_handler',
        'app/exchange_handler'],
       function($, d3, MetricDefinitionHelper, utils, ErrorHandler,
                ExchangeHandler)
{

    /**
     * Constructor for instance of MetricOverviewVis object.
     * @param {d3.selection} container The HTML node to build overview in.
     * @param {Trial} trial The current trial to display.
     * @param {Trial} baseline The trial to use as a baseline.
     */
    function MetricOverviewVis(container, currTrial, baseline) {
        this.container = container;
        this.currTrial = currTrial;
        this.baseline = baseline;
        this.currMetrics = null;

        this.allGroups = [];
        this.metricList = [];

        this.metricName = null;

        this.dispatch = null;

        this.tableSvg = null;
        this.tableG = null;

        // Layout sizes
        this.width = 100;
        this.headerHeight = 30;
        this.subheaderHeight = 62;
        this.entryHeight = 26;

        // Defaults for x range
        this.xRange = [5, 755];

        // Scales
        this.xScale = d3.scaleLinear().domain([0,100]);
        this.xScaleNeg = d3.scaleLog();
        this.verticalTicks = [25, 50, 75];

        // Circles
        this.endCircleR = 3;
        this.solidCircleR = 10;
        this.emptyCircleR = 10;

        // Offsets for value text
        this.leftValueOffset = this.xRange[1] + 43;
        this.rightValueOffset = this.xRange[1] + 95;

        // TODO: These are the "Subtitles" that map to metric groups.
        // This does not seem like a good place to define them.
        this.subtitleMap = {
            'Gait/Lumbar': 'Range of Motion',
            'Gait/Trunk': 'Range of Motion',
            'Gait/Head': 'Range of Motion'
        };

        // Show standard deviation instead of % of norm.
        this.showStdDev = false;
        this.stdDevMax = 8;

        // Table.
        this.isTable = true;
        this.metricNameWidth = 170;
        this.cellWidth = 217;
        this.baseMetrics = null;
        this.minCellWidth = 120;

        this.leftCircleSvg = 'm 5.9482807,15.748417 c -0.0072,-0.01895 -0.00992,-2.620115 -0.00605,-5.7803661 l 0.00705,-5.745909 1.1546805,0 1.1546804,0 0.00695,4.694199 0.00695,4.6942001 2.8934944,0.007 2.893495,0.007 0,1.072203 0,1.072204 -4.049082,0.007 c -3.224805,0.0055 -4.0517472,-5.5e-5 -4.0621761,-0.02749 l 0,0 z';

        this.rightCircleSvg = 'M12.7,4.6c0.4,0.2,0.8,0.4,1,0.8C14,5.7,14.2,6,14.4,6.4c0.1,0.3,0.2,0.7,0.2,1.2c0,0.5-0.1,1.1-0.4,1.6 c-0.3,0.5-0.7,0.9-1.3,1.1c0.5,0.2,0.9,0.5,1.1,0.9c0.2,0.4,0.3,1,0.3,1.8v0.8c0,0.5,0,0.9,0.1,1c0.1,0.3,0.2,0.5,0.4,0.6v0.3h-2.6 c-0.1-0.2-0.1-0.4-0.2-0.6c-0.1-0.3-0.1-0.6-0.1-1l0-1c0-0.7-0.1-1.2-0.4-1.4c-0.2-0.2-0.7-0.4-1.3-0.4H7.9v4.4H5.6V4.4H11 C11.7,4.4,12.3,4.5,12.7,4.6z M7.9,6.3v3h2.5c0.5,0,0.9-0.1,1.1-0.2c0.4-0.2,0.7-0.6,0.7-1.3c0-0.7-0.2-1.1-0.6-1.4 c-0.2-0.1-0.6-0.2-1.1-0.2H7.9z';

    }

    /**
     * Update the plot, but after the Promise to get the needed
     * metrics is complete.
     */
    MetricOverviewVis.prototype.update = function() {
        var self = this;
        var exchangeHandler = ExchangeHandler.getInstance();

        this.currTrial.getMetrics().then(function(metrics) {
            self.currMetrics = metrics;
            return self.baseline.getMetrics();
        }).then(function (metrics) {
            self.baseMetrics = metrics;
            self.buildOverview();
        }, function(e) {
            // Handle disconnected MX differently.
            if (e.status !== 503) {
                ErrorHandler.showError(e.error().statusText);
            } else if (!exchangeHandler.metricWarningShown()){
                ErrorHandler.showError(e.error().statusText);
                exchangeHandler.metricWarningShown(true);
            }
            self.buildOverview();
        });
    };

    MetricOverviewVis.prototype.buildOverview = function() {
        var self = this;
        this.getMetricGroupMap(MetricDefinitionHelper.getInstance()
                               .getTestMetrics(this.currTrial.getTestName()));

        var containerWidth = $(this.container.node()).width();
        if (containerWidth > this.width) {
            this.width = containerWidth;
            this.xRange = [5, containerWidth - 45];
            this.leftValueOffset = this.xRange[1] + 43;
            this.rightValueOffset = this.xRange[1] + 95;
        }
        this.determineTextWidth();

        var spacing = this.determineGroupSpacing(this.allGroups);

        // Header
        var subtitle = this.container.selectAll('svg')
            .data([self.currTrial], function(d) {
                return d.getObjId().getIdStr();
            })
            .enter()
            .append('svg')
            .attr('width', this.width)
            .attr('height', this.headerHeight)
            .append('g')
            .attr('class', 'overview-header-g')
            .attr('transform', 'translate(0,' + (this.headerHeight) + ')')
            .append('text').text('Overview ').append('tspan')
            .attr('id', 'overview-header-subtitle')
            .text('Percentiles');

        // Create SVG elements for every metric group.
        var groupContainers = this.container.selectAll('.metric-group-svg')
            .data(this.allGroups, function(d) { return d.htmlId; })
            .enter()
            .append('div')
            .append('svg')
            .attr('class', 'metric-group-svg')
            .attr('width', this.width)
            .attr('height', function(d) { return spacing[d.htmlId]; });
        var groupGs = groupContainers
            .append('g')
            .attr('class', 'metric-group-g')
            .attr('id', function(d) { return (d.htmlId); });

        groupGs.append('g').attr('class', 'x axis overview');
        groupGs.append('g').attr('class', 'y axis overview');
        groupGs.append('g').attr('class', 'overview-vertical-grid');

        groupContainers.exit().remove();

        groupContainers = this.container.selectAll('.metric-group-g')
            .attr('transform', 'translate(0,' + this.subheaderHeight + ')');
        groupContainers.append('text').text(function(d) {return d.name;})
            .attr('class', 'overview-subheader').attr('dy', '-1.75em');

        // Add the subtitle, and if it equals 'Bilateral', add the bilateral key.
        var columnHeaders = [
            'NORMATIVE',
            utils.getShortFormattedDate(this.currTrial.getDateString()),
            utils.getShortFormattedDate(this.baseline.getDateString()),
            'CHANGE FROM'
        ];

        var circleOffset = this.solidCircleR * 2;
        groupContainers
            .each(function(d) {
                var container = d3.select(this);
                container.append('text').text(function(d) {return d.subgroup;})
                    .attr('class', 'overview-subheader-subtitle')
                    .attr('dy', '-0.5em');
                if (d.subgroup === 'BILATERAL' && !self.isTable) {
                    var circleG = container.append('g')
                        .attr('transform',
                              'translate(' +
                              (self.leftValueOffset - circleOffset)  + ',' +
                              (-circleOffset) + ')')
                        .attr('class', 'overview-bilateral-circle');

                    circleG.append('circle')
                        .attr('cx', 10)
                        .attr('cy', 10)
                        .attr('r', 10)
                        .attr('fill', '#000000');
                    circleG.append('path')
                        .attr('fill', '#ffffff')
                        .attr('d', self.leftCircleSvg);

                    circleG = container.append('g')
                        .attr('transform',
                              'translate(' +
                              (self.rightValueOffset - circleOffset) + ',' +
                              (-circleOffset) + ')')
                        .attr('class', 'overview-bilateral-circle');

                    circleG.append('circle')
                        .attr('cx', 10).attr('cy', 10)
                        .attr('r', 10).attr('fill', '#000000');
                    circleG.append('path')
                        .attr('fill', '#ffffff')
                        .attr('d', self.rightCircleSvg);
                }

                // Add table headers, if we need them
                if (self.isTable) {
                    container.selectAll('.overview-table-column-header')
                        .data(columnHeaders)
                        .enter()
                        .append('text').attr('class', 'overview-table-column-header')
                        .each(function(d, i) {
                            var header = d3.select(this);
                            var textElm = header.text(d)
                                .attr('dx', self.metricNameWidth +
                                       (i * self.cellWidth) +
                                      (self.cellWidth * 0.5))
                                .attr('dy', '-0.25em');
                            if (d === 'CHANGE FROM') {
                                textElm.attr('dy', '-1.5em')
                                    .attr('dx', self.metricNameWidth +
                                          ((i - 1) * self.cellWidth) +
                                          (self.cellWidth * 0.5));
                            }
                        });
                }
            });

        // Adjust xScale if we're doing std dev.
        if (this.showStdDev) {
            subtitle.text('Z-Score');
            var midPoint = this.xRange[0] + ((this.xRange[1] - this.xRange[0]) * 0.5);
            this.xScale = d3.scaleLog().domain([1, this.stdDevMax + 1])
                .range([midPoint, this.xRange[1]]);
            this.xScaleNeg.domain([-this.stdDevMax - 1, -1])
                .range([this.xRange[0], midPoint]);
            this.verticalTicks = [-8, -6, -4, -2, 1, 2, 4, 6, 8];
        } else {
            this.xScale.range(this.xRange);
        }

        // If the subtitle is 'Bilateral', add the bilateral key.
        groupContainers.each(function() {
            if (self.isTable) {
                self.buildSubgroupTable(d3.select(this));
                return;
            }
            self.buildSubgroupPlot(d3.select(this));
        });

        // Update the vertical grid
        if (!this.isTable) {
            var vertGrid = this.container.selectAll('.overview-vertical-grid');
            vertGrid.selectAll('line')
                .data(this.verticalTicks).enter()
                .append('line')
                .attr('y1', 0)
                .attr('y2', spacing.totalHeight - this.headerHeight -
                      this.subheaderHeight)
                .attr('x1', function(d) { return getScale(d); })
                .attr('x2', function(d) { return getScale(d); });

            vertGrid.selectAll('text').data(this.verticalTicks)
                .enter()
                .append('text').text(function(d) {
                    if (self.showStdDev) {
                        if (d > 0) {
                            return d - 1;
                        }
                        return d + 1;
                    }
                    return d + '%';
                })
                .attr('x',function(d) { return getScale(d); })
                .attr('y', -2);
        } else {
            subtitle.text('Metric Values');
        }

        // Setup the key handers for navigation.
        this.setupKeyHandlers();

        function getScale(num) {
            if (self.showStdDev && num < 0) {
                return self.xScaleNeg(num);
            }
            return self.xScale(num);
        }
    };

    /**
     * Setup key handlers that switch between different metrics and trials.
     */
    MetricOverviewVis.prototype.setupKeyHandlers = function() {
        var self = this;
        var id = utils.makeHtmlId(this.currTrial.getTestName());
        d3.select(window).on('keydown.' + id, null);
        d3.select(window).on('keydown.' + id, function() {
            if (!($(self.container.node()).is(':visible'))) {
                return;
            }
            var keycode = d3.event.keyCode;
            // Handle changes to metric selection with up and down arrow keys.
            if (keycode === 38 || keycode === 40) {
                d3.event.preventDefault();
                var idx = self.metricList
                    .map(function(d) { return d.metricFullName; })
                    .indexOf(self.metricName);

                if (d3.event.keyCode === 38) {
                    idx = idx - 1;
                    if (idx < 0) {
                        idx = self.metricList.length - 1;
                    }
                } else if (d3.event.keyCode === 40) {
                    idx = idx + 1;
                    if (idx >= self.metricList.length) {
                        idx = 0;
                    }
                }
                self.dispatch.call('metricSelected', null,
                                   self.metricList[idx].metricFullName);
            } else if (keycode === 37) {
                d3.event.preventDefault();
                self.dispatch.call('prevTrialSelected', null);
            } else if (keycode === 39) {
                d3.event.preventDefault();
                self.dispatch.call('nextTrialSelected', null);
            }
        });
    };

    /**
     * Build the subgroup entries.
     * @param {d3.selection} groupG The container for the subgroup plot.
     */
    MetricOverviewVis.prototype.buildSubgroupPlot = function(groupG) {
        var self = this;
        var groupEntry = groupG.datum();
        var yAxisG = groupG.select('.y.axis.overview');
        var yScale = this.getSubgroupYScale(groupEntry, yAxisG);

        // Add the line and the end dots.
        var entries =  groupG.selectAll('.overview-entry')
            .data(groupEntry.metrics, function(d) { return d.displayName; })
            .enter()
            .insert('g', '.y.axis.overview')
            .attr('class', 'overview-entry link')
            .attr('metric-name', function(d) { return d.metricName; })
            .attr('transform', function(d) {
                var yOffset = yScale(d.displayName) - (self.entryHeight * 0.5);
                return 'translate(0, '  + yOffset + ')';
            })
            .on('click', function(d) {
                if (self.dispatch !== null) {
                    self.dispatch.call('metricSelected', null, d.metricFullName);
                }
            });

        this.createHighlightRects(entries);
        if (this.showStdDev) {
            this.createNormRangeRects(entries);
        }
        entries.append('line')
            .attr('x1', this.xRange[0])
            .attr('y1', this.entryHeight * 0.5)
            .attr('x2', this.xRange[1])
            .attr('y2', this.entryHeight * 0.5);

        entries.append('circle')
            .attr('cx', this.xRange[0])
            .attr('cy', this.entryHeight * 0.5)
            .attr('r', this.endCircleR);
        entries.append('circle')
            .attr('cx', this.xRange[1])
            .attr('cy', this.entryHeight * 0.5)
            .attr('r', this.endCircleR);

        // Shift the axis over a "reasonable" amount
        yAxisG.attr('transform', 'translate(' + (this.xRange[0] - 5) + ',0)');

        // Add the circles for the values.
        entries.each( function(d) {
            self.createEntry(d3.select(this), d);
        });

    };

    /**
     * Create the individual plot entry.
     * @param {d3.selection} selection The selection to put entry in.
     * @param {Object} metric The metric being displayed.
     */
    MetricOverviewVis.prototype.createEntry = function(selection, metric) {
        var metricNorms =
            MetricDefinitionHelper.getInstance()
            .getMeanStdDevAndNorms(this.currTrial.getTestName(),
                                   this.currTrial.getConditionName(),
                                   metric);

        if (metric.metricType === 'UNILATERAL') {
            this.createUnilateralEntry(selection, metric, metricNorms);
        } else {
            this.createBilateralEntry(selection, metric, metricNorms);
        }
    };

    /**
     * Create the plot entry for a unilateral metric.
     * @param {d3.selection} selection The selection to put entry in.
     * @param {Object} metric The metric for the entry.
     * @param {Object} metricNorms The norms for the metrics.
     */
    MetricOverviewVis.prototype.createUnilateralEntry = function(selection, metric,
                                                                 metricNorms) {
        // Bail if metric does not exist for trial
        if (!this.currMetrics || !this.currMetrics[metric.metricFullName]) {
            this.createValueEntry(selection, NaN, false);
            return;
        }

        var metricValue;
        var value = NaN;
        if (this.currMetrics && this.currMetrics[metric.metricFullName]) {
            metricValue = this.currMetrics[metric.metricFullName].mean;
        }
        if (metricNorms) {
            if (this.showStdDev) {
                value = utils.getValueInStandardUnits(metricValue,
                                                      metricNorms.mean,
                                                      metricNorms.sd);
            } else {
                value = utils.getPercentile(metricValue, metricNorms.norms);
            }
        }
        if (!isNaN(value)) {
            var adjustedVal = value;
            if (this.showStdDev) {
                adjustedVal = this.valueToLog(adjustedVal);
            }
            this.createCircleEntry(selection, adjustedVal, metric, true);
        }
        this.createValueEntry(selection, value, false);
    };

    /**
     * Create the overview percent of norm entry for a bilateral metric.
     * @param {d3.selection} selection The selection to build entry within.
     * @param {Object} metric The metric defintion for the metric to display.
     * @param {Object} metricNorms Object containing the norms for metric.
     */
    MetricOverviewVis.prototype.createBilateralEntry = function(selection, metric,
                                                                metricNorms) {
        var baseName = metric.metricFullName;
        var metricValue;
        var value;
        var adjusted;

        // Bail if metric does not exist for trial
        if (this.currMetrics && this.currMetrics[baseName + ' L']) {
            metricValue = this.currMetrics[baseName + ' L'].mean;
            value = NaN;
            if (metricNorms) {
                if (this.showStdDev) {
                    value = utils.getValueInStandardUnits(metricValue,
                                                          metricNorms.mean,
                                                          metricNorms.sd);
                } else {
                    value = utils.getPercentile(metricValue, metricNorms.norms);
                }
                if (!isNaN(value)) {
                    adjusted = value;
                    if (this.showStdDev) {
                        adjusted = this.valueToLog(adjusted);
                    }
                    this.createCircleEntry(selection, adjusted, metric, false);
                }
            }
            this.createValueEntry(selection, value, false);
        }

        if (this.currMetrics && this.currMetrics[baseName + ' R']) {
            metricValue = this.currMetrics[baseName + ' R'].mean;
            value = NaN;
            if (metricNorms) {
                if (this.showStdDev) {
                    value = utils.getValueInStandardUnits(metricValue,
                                                          metricNorms.mean,
                                                          metricNorms.sd);
                } else {
                    value = utils.getPercentile(metricValue, metricNorms.norms);
                }
                if (!isNaN(value)) {
                    adjusted = value;
                    if (this.showStdDev) {
                        adjusted = this.valueToLog(adjusted);
                    }
                    this.createCircleEntry(selection, adjusted, metric, true);
                }
            }
            this.createValueEntry(selection, value, true);
        }
    };

    /**
     * Create the circle for the entry.
     * @param {d3.selection} selection The selection to put circle in.
     * @param {Number} value The value to plot
     * @param {Object} metric The metric being plotted.
     * @param {Boolean} isSolid True if circle should be solid.
     */
    MetricOverviewVis.prototype.createCircleEntry = function(selection, value,
                                                             metric, isSolid) {
        // Standard dev values over max go to edges
        if (this.showStdDev)  {
            if (value > this.stdDevMax) {
                value = this.stdDevMax + 1;
            } else if (value < -this.stdDevMax) {
                value = -this.stdDevMax - 1;
            }
        }
        var radius = this.solidCircleR;
        var scale = this.xScale;
        if (this.showStdDev && value < 0) {
            scale = this.xScaleNeg;
        }

        if (metric.metricType === 'BILATERAL') {
            var circleG = selection.append('g')
                .attr('transform', 'translate(' + scale(value) + ',3)')
                .attr('class', 'overview-bilateral-circle');
            circleG.append('circle')
                .attr('cx', 10).attr('cy', 10)
                .attr('r', 10)
                .attr('fill', metric.metricGroupDisplayColor);
            if (isSolid) {
                circleG.append('path')
                    .attr('fill', '#ffffff')
                    .attr('d', this.rightCircleSvg);
            } else {
                circleG.append('path')
                    .attr('fill', '#ffffff')
                    .attr('d', this.leftCircleSvg);
            }
        } else {
            selection.append('circle')
                .attr('class', 'overview-bilateral-circle')
                .attr('r', radius)
                .attr('cx', scale(value))
                .attr('cy', this.entryHeight * 0.5)
                .style('fill', metric.metricGroupDisplayColor)
                .style('stroke', metric.metricGroupDisplayColor);
        }
    };

    /**
     * Create the value entry displayed on the right.
     * @param {d3.selection} selection The selection to add text to.
     * @param {Number} value The value to display.
     * @param {Boolean} isRight True if this is right value of bilateral metric.
     */
    MetricOverviewVis.prototype.createValueEntry = function(selection, value,
                                                            isRight) {
        var offset = this.leftValueOffset;
        if (isRight) {
            offset = this.rightValueOffset;
        }
        var displayValue;
        if (this.showStdDev) {
            if (value > -0.1 && value < 0.1) {
                displayValue = '0.0';
            } else if (value > -1 && value < 1) {
                displayValue = value.toPrecision(1);
            } else if (isNaN(value)) {
                displayValue = '--';
            } else {
                displayValue = value.toPrecision(2);
            }
        } else {
            displayValue = utils.getPercentileDisplay(value, false);
        }
        selection.append('text').text(displayValue)
            .attr('class', 'overview-value-text')
            .attr('x', offset)
            .attr('y', this.entryHeight * 0.5)
            .attr('dy', '0.32em')
            .classed('svg-text-right', true);
    };

    /**
     * Create the normal range rects for the standard deviation
     * overview.
     * @param {d3.selection} entries The entries to add rects to.
     */
    MetricOverviewVis.prototype.createNormRangeRects = function(entries) {
        var self = this;
        entries.append('rect').attr('class', 'overview-norm-range-rect')
            .attr('rx', 10)
            .attr('ry', 10)
            .attr('y', 1)
            .attr('height', self.entryHeight - 2)
            .each(function (d) {
                var rect = d3.select(this);
                var normsObj =
                    MetricDefinitionHelper.getInstance()
                    .getMeanStdDevAndNorms(self.currTrial.getTestName(),
                                           self.currTrial.getConditionName(),
                                           d);
                if (!normsObj || !normsObj.mean || !normsObj.sd) {
                    rect.remove();
                    return;
                }
                var min = utils.getValueInStandardUnits(normsObj.fifthToNinteyFifth[0],
                                                        normsObj.mean, normsObj.sd);
                var max = utils.getValueInStandardUnits(normsObj.fifthToNinteyFifth[1],
                                                        normsObj.mean, normsObj.sd);
                var leftScale = self.xScale;
                var rightScale = self.xScale;
                if (min < 0) {
                    leftScale = self.xScaleNeg;
                }
                min = self.valueToLog(min);
                if (max < 0) {
                    rightScale = self.xScaleNeg;
                }
                max = self.valueToLog(max);

                rect.attr('x', leftScale(min))
                    .attr('width', rightScale(max) - leftScale(min));
            });
    };

    /**
     * Build the subgroups with metric table format.
     * @param {d3.selection} groupG The selection for the metric subgroup.
     */
    MetricOverviewVis.prototype.buildSubgroupTable = function(groupG) {
        var self = this;
        var groupEntry = groupG.datum();
        var yAxisG = groupG.select('.y.axis.overview');
        var yScale = this.getSubgroupYScale(groupEntry, yAxisG);

        // Add the line and the end dots.
        var entries =  groupG.selectAll('.overview-entry')
            .data(groupEntry.metrics, function(d) { return d.displayName; })
            .enter()
            .insert('g', '.y.axis.overview')
            .attr('metric-name', function(d) { return d.metricName; })
            .attr('class', 'overview-entry link')
            .attr('transform',function(d) {
                var yOffset =
                    yScale(d.displayName +
                           '(' + utils.formatUnits(d.units) + ')') -
                           (self.entryHeight * 0.5);
                       return 'translate(0, '  + yOffset + ')';
            })
            .on('click', function(d) {
                if (self.dispatch !== null) {
                    self.dispatch.call('metricSelected', null, d.metricFullName);
                }
            });

        this.createHighlightRects(entries);

        entries.append('line')
            .attr('x1', 0)
            .attr('y1', 0)
            .attr('x2', this.xRange[1] + 75)
            .attr('y2', 0);

        // Shift the axis over a "reasonable" amount
        // TODO: Perhaps make this dynamic.
        yAxisG.attr('transform', 'translate(' + (this.xRange[0] + 30) + ',0)');

        // Add the circles for the values.
        entries.each( function(d) {
            self.createTableRow(d3.select(this), d);
        });

    };

    /**
     * Create the table row for the given metric.
     * @param {d3.selection} selection The selection to add text content to.
     * @param {Object} metric The metric definition for this row's metric.
     */
    MetricOverviewVis.prototype.createTableRow = function(selection, metric) {
        var self = this;
        var columnValues = this.getValuesForTableEntry(metric);
        selection.selectAll('.overview-table-cell')
            .data(columnValues, function(d, i) { return d + i; })
            .enter()
            .append('text').attr('class', 'overview-table-cell')
            .each(function(d, i) {
                var cell = d3.select(this);
                cell.text(function(d) { return d; });
                cell.attr('dx', self.metricNameWidth + (i * self.cellWidth) +
                          (self.cellWidth * 0.5));
                cell.attr('dy', '1.25em');
            });
    };
    /**
     * Get the values to place in the table for the given metric.
     * @param {Object} metric The metric definition for metric to get
     * values for.
     * @return {Array} The normative range, metric value, and change from
     * baseline.
     */
    MetricOverviewVis.prototype.getValuesForTableEntry = function(metric) {
        var values = [];
        var baseName = metric.metricFullName;

        var normEntry = '--';
        var meanAndNorms = MetricDefinitionHelper.getInstance()
            .getMeanStdDevAndNorms(this.currTrial.getTestName(),
                                   this.currTrial.getConditionName(),
                                   metric);
        if (meanAndNorms) {
            if (meanAndNorms.fifthToNinteyFifth) {
                // Note that the dash is an em-dash, not ASCII dash.
                normEntry = meanAndNorms.fifthToNinteyFifth[0].toPrecision(3) +
                    ' — ' + meanAndNorms.fifthToNinteyFifth[1].toPrecision(3);
            }
        }
        values.push(normEntry);

        var valueEntry = '--';
        if (metric.metricType === 'BILATERAL') {
            if (this.currMetrics && this.currMetrics[baseName + ' L'] &&
                this.currMetrics[baseName + ' R']) {
                var left = '--';
                var right = '--';
                if (!isNaN(this.currMetrics[baseName + ' L'].mean)) {
                    left = this.currMetrics[baseName + ' L']
                        .mean.toPrecision(3);
                }
                if (!isNaN(this.currMetrics[baseName + ' R'].mean)) {
                    right = this.currMetrics[baseName + ' R'].mean.toPrecision(3);
                }
                valueEntry = left + ' │ ' + right;
            }
        } else {
            if (this.currMetrics && this.currMetrics[baseName]) {
                if (this.currMetrics[baseName].mean !== undefined &&
                    this.currMetrics[baseName].mean !== null &&
                    !isNaN(this.currMetrics[baseName].mean)) {
                    valueEntry = this.currMetrics[baseName].mean.toPrecision(3);
                }
            }
        }
        values.push(valueEntry);

        var percentChange = '--';
        if (this.baseMetrics && this.currMetrics) {
            if (metric.metricType === 'BILATERAL') {
                percentChange =
                    utils.getSinglePercentChange(baseName + ' L', this.currMetrics,
                                                 this.baseMetrics) + ' │ ' +
                    utils.getSinglePercentChange(baseName + ' R', this.currMetrics,
                                                 this.baseMetrics, true);
            } else {
                percentChange =
                    utils.getSinglePercentChange(baseName, this.currMetrics,
                                                 this.baseMetrics);
            }
        }
        values.push(percentChange);

        return values;
    };

    /**
     * Process the selected metrics and create the list of metric groups
     * to display as well as a flattened version of all metrics.
     * NOTE: Subgroups should probably be defined in MetricDefinitions.java,
     * so we don't have to special case stuff here.
     * @param {Object} metricSet The object containing the selected metrics.
     */
    MetricOverviewVis.prototype.getMetricGroupMap = function(metricSet) {
        var groupMap = {};
        var group;
        var ordering = MetricDefinitionHelper.getInstance()
            .getTestMetricsOrder(this.currTrial.getTestName());
        var i;

        for (i = 0; ordering.metrics.length > i; i++) {
            var name = ordering.metrics[i];
            if (metricSet[name] && metricSet[name].promoteToUI === true) {
                group = metricSet[name].metricGroup;
                if (!groupMap[group]) {
                    groupMap[group] =
                        { name: metricSet[name].metricGroupDisplayName,
                          subgroups: {} };
                    if (this.subtitleMap[group]) {
                        groupMap[group].subgroups[this.subtitleMap[group]] =
                            [metricSet[name]];
                    } else if (metricSet[name].metricType === 'BILATERAL') {
                        groupMap[group].subgroups.Bilateral = [metricSet[name]];
                    } else {
                        groupMap[group].subgroups.Unilateral = [metricSet[name]];
                    }
                } else {
                    if (this.subtitleMap[group]) {
                        groupMap[group].subgroups[this.subtitleMap[group]]
                            .push(metricSet[name]);
                    } else if (metricSet[name].metricType === 'BILATERAL') {
                        if (!groupMap[group].subgroups.Bilateral) {
                            groupMap[group].subgroups.Bilateral = [metricSet[name]];
                        } else {
                            groupMap[group].subgroups.Bilateral.push(metricSet[name]);
                        }
                    } else {
                        if (!groupMap[group].subgroups.Unilateral) {
                            groupMap[group].subgroups.Unilateral = [metricSet[name]];
                        } else {
                            groupMap[group].subgroups.Unilateral.push(metricSet[name]);
                        }
                    }
                }
            }
        }

        for (i = 0; ordering.groups.length > i; i++) {
            group = ordering.groups[i];
            if (groupMap[group]) {
                for (var subgroup in groupMap[group].subgroups) {
                    this.allGroups.push({
                        name: groupMap[group].name,
                        subgroup: subgroup.toUpperCase(),
                        metrics: groupMap[group].subgroups[subgroup],
                        htmlId: utils.makeHtmlId(groupMap[group].name +
                                                 '-' + subgroup)
                    });
                    this.metricList =
                        this.metricList.concat(groupMap[group].subgroups[subgroup]);
                }
            }
        }
    };

    /**
     * Determine the spacing between the metric groups.
     * @param {Object} allGroups Map of the metric groups and the metrics
     * they contain.
     * @return {Object} spacing Map with spacing keyed by group name.
     */
    MetricOverviewVis.prototype.determineGroupSpacing = function(allGroups) {
        var spacing = {totalHeight: this.headerHeight};

        allGroups.forEach(function(d) {
            spacing[d.htmlId] = this.subheaderHeight + this.entryHeight +
                (d.metrics.length * this.entryHeight);
            spacing.totalHeight = spacing.totalHeight + spacing[d.htmlId];
        }, this);

        return spacing;
    };

    /**
     * Figure out what the widest display name is for the displayed
     * metrics and adjust the x axis range. Also determine the spacing
     * for the column tables.
     */
    MetricOverviewVis.prototype.determineTextWidth = function() {
        var widths = [];
        var self = this;
        var svg = this.container.append('svg');
        this.allGroups.forEach(function(d) {
            d.metrics.forEach(function(m) {
                var name = m.displayName;
                if (self.isTable) {
                    name = name + '(' + utils.formatUnits(m.units) + ')';
                }
                var text = svg.append('text').text(name);
                widths.push(text.node().getBBox().width);
                text.remove();
            });
        });
        svg.remove();
        this.xRange[0] = d3.max(widths) + 20;
        this.metricNameWidth = this.xRange[0];

        this.cellWidth = (this.width - this.metricNameWidth) / 3;
        if (this.cellWidth < this.minCellWidth) {
            this.cellWidth = this.minCellWidth;
        }
    };

    /**
     * Get the y scale for the subgroup and build the y axis.
     * @param {Object} groupEntry The object containing the data for the subgroup.
     * @param {d3.selection} axisG The container for the y axis.
     * @return {d3.scale} The y scale for the subgroup.
     */
    MetricOverviewVis.prototype.getSubgroupYScale = function (groupEntry, axisG) {
        var self = this;
        var domainValues = groupEntry.metrics
            .map(function(d) {
                var name = d.displayName;
                if (self.isTable) {
                    name = name + '(' + utils.formatUnits(d.units) + ')';
                }
                return name;
            });
        var yScale = d3.scalePoint().domain(domainValues)
            .range([0, (domainValues.length * this.entryHeight) + this.entryHeight])
            .padding(1.0);
        axisG.call(d3.axisLeft(yScale));
        return yScale;
    };

    /**
     * Create the rects that highlight the selected metric and
     * on hover.
     * @param {d3.selection} entries The entries in the table.
     * @param {d3.scale} yScale The y scale for the table.
     */
    MetricOverviewVis.prototype.createHighlightRects = function(entries) {
        var self = this;
        var entryRects = entries.append('rect')
            .attr('class', 'overview-entry-select-rect')
            .attr('rx', 10)
            .attr('ry', 10)
            .attr('x', 0)
            .attr('y', 0)
            .attr('width', this.xRange[1] + 45)
            .attr('height', this.entryHeight);
        entryRects.each(function(d) {
            if (d.metricFullName === self.metricName) {
                d3.select(this).classed('overview-entry-select-rect', false)
                    .classed('overview-entry-select-rect-selected', true);
            }
        });

        entryRects.on('click', function() {
            d3.selectAll('.overview-entry-select-rect-selected')
                .attr('class', 'overview-entry-select-rect');
            d3.select(this).attr('class', 'overview-entry-select-rect-selected');
        });
    };

    /**
     * Adjust the value to get it to work with the log scale.
     * @param {Number} metric The metric value to adjust.
     * @return {Number} metric + 1 if metric > 0, else metric - 1
     */
    MetricOverviewVis.prototype.valueToLog = function(metric) {
        if (metric >= 0) {
            return metric + 1;
        }
        return metric - 1;
    };

    /**
     * Get or set the d3.dispatch object to broadcast metric selection
     * events through.
     * @param {d3.dispatch} dispatch The dispatch object.
     * @return The d3.dispatch object, if call with no arguments,
     * otherwise "this".
     */
    MetricOverviewVis.prototype.getSetDispatch = function(dispatch) {
        if (!arguments.length) {
            return this.dispatch;
        }
        this.dispatch = dispatch;
        return this;
    };

    /**
     * Get or set the flag that determines if table or percent of norm
     * displayed.
     * @param {Boolean} isTable Set to true to display table.
     * @return The current flag value if called with no arguments,
     * otherwise "this".
     */
    MetricOverviewVis.prototype.getSetIsTable = function(isTable) {
        if (!arguments.length) {
            return this.isTable;
        }
        this.isTable = isTable;
        return this;
    };

    /**
     * Get or set the flag that determines percent of norm or
     * standard deviation is displayed.
     * @param {Boolean} showStdDev Set to true to display standard deviation.
     * @return The current flag value if called with no arguments,
     * otherwise "this".
     */
    MetricOverviewVis.prototype.getSetShowStdDev = function(showStdDev) {
        if (!arguments.length) {
            return this.showStdDev;
        }
        this.showStdDev = showStdDev;
        return this;
    };

    MetricOverviewVis.prototype.getSetBaselineTrial = function(baseline) {
        if (!arguments.length) {
            return this.baseline;
        }
        this.baseline = baseline;
        return this;
    };

    /**
     * Get or set the currently selected metric.
     * @param {String} metricName The name of the metric to highlight
     * in the overview.
     * @return The current metric name if called with no arguments,
     * otherwise "this".
     */
    MetricOverviewVis.prototype.getSetSelectedMetric = function(metricName) {
        if (!arguments.length) {
            return this.metricName;
        }
        this.metricName = metricName;
        return this;
    };

    return MetricOverviewVis;
});
