/* Copyright (c) 2016-2017 APDM, Inc.
 *
 * reassign_trial.js
 *
 * Handle reassigning a trial to a different subject.
 */

define(['jquery', 'd3', 'crossroads', 'bootstrap-dialog', 'utils',
        'model_dispatcher', 'res_req_map', 'subject', 'session',
        'trial', 'app/user_options_handler'],
       function ($, d3, crossroads, BootstrapDialog, utils, ModelDispatcher,
                 ResReqMap, Subject, Session, Trial, UserOptionsHandler)
{

    var subjects;
    var dialog;
    var target;
    var currentSubject;

    // Create route for successful reassignment.
    crossroads.addRoute('changedTrialStudySubject', function() {
        dialog.close();
        // Let it be known that the study has refreshed.
        crossroads.parse('study_refreshed');
    });
    crossroads.addRoute('changedSessionStudySubject', function() {
        dialog.close();
        // Let it be known that the study has refreshed.
        crossroads.parse('study_refreshed');
    });

    /**
     * Open the dialog for switching the given trial to
     * a different subject.
     * @param {Trial} trial The current trial to switch.
     * @param {Subject} subject The current subject.
     */
    function openDialog(trial, subject) {
        subjects = ModelDispatcher.getInstance().getCurrentStudy().getAllSubjects();
        target = trial;
        currentSubject = subject;
        // Filter out the current subject.
        subjects = subjects.filter(function(d) {
            return d.getPublicId() !== subject.getPublicId();
        });

        buildDialog();
    }

    /**
     * Build the dialog to display.
     */
    function buildDialog() {
        dialog = new BootstrapDialog({
            title: 'Assign to different subject.',
            message: $('<div></div>')
                .load('/mobility_clinic/html/reassign_trial.html',
                      function() { dialog.open(); }),
            buttons: [
                {
                    id: 'reassign-trial-cancel-btn',
                    icon: 'iconic iconic-x',
                    label: 'Cancel',
                    cssClass: 'btn-link',
                    action: function(dialog) {
                        dialog.close();
                    }
                },
                {
                    id: 'reassign-trial-assign-btn',
                    icon: 'iconic iconic-transfer',
                    label: 'Assign',
                    cssClass: 'btn-success',
                    action: function() {
                        reassign();
                        // Close the report
                        $('.dialog-report-close').click();
                    }
                }],
            onshown: populateSelect
        });
    }

    /**
     * Populate the selection dropdown with study subject.
     */
    function populateSelect() {
        var testTitle = target.getTestName();
        if (target instanceof Trial) {
            testTitle = testTitle + ', ' + target.getConditionName();
        }
        testTitle = testTitle + ' - ' +
            utils.getShortStringFromDate(target.getDateObj());
        d3.select('#reassign-trial-name')
            .text(testTitle);

        var dropdown = d3.select('#reassign-trial-subject-select');
        dropdown.selectAll('option').data(subjects)
            .enter()
            .append('option')
            .attr('value', function(d) { return d.getPublicId(); })
            .text(function(d) {
                return UserOptionsHandler.getInstance().getSubjectName(d);
            });

        $(dropdown.node())
            .selectpicker('val', currentSubject.getPublicId());
        $(dropdown.node()).selectpicker('refresh');
    }

    /**
     * Make call to reassign the subject to the currently selected
     * subject.
     */
    function reassign() {
        var route = 'dispatch/changeTrialStudySubject/';
        if (target instanceof Session) {
            route = 'dispatch/changeSessionStudySubject/';
        }
        var newPublicId = $('#reassign-trial-subject-select').val();
        // Check for no change.
        if (newPublicId === currentSubject.getPublicId()) {
            return;
        }
        var subject = ModelDispatcher.getInstance().getCurrentStudy()
            .getSubjectByPublicId(newPublicId);
        var key = ResReqMap.getInstance().setRequestData({
            subject: subject,
            target: target
        });
        crossroads.parse(route + key);
    }

    return {
        open: openDialog
    };
});
