/* Copyright (c) 2018 APDM, Inc.
 *
 * tbi_longitudinal_report.js
 *
 * Create the logitudinal reports for TBI trials.
 *
 */

define(['jquery', 'd3', 'moment', 'crossroads', 'hasher',
        'metric_definition_helper', 'es6-promise', 'error_handler',
        'bar_longitudinal_plot', 'utils', 'metric_utils', 'session',
        'app/user_options_handler'],
       function($, d3, moment, crossroads, hasher, MetricDefinitionHelper,
                Promise, ErrorHandler, BarLongitudinalPlot, utils,
                metricUtils, Session, UserOptionsHandler)
{
    'use strict';

    var _subject;
    var _startDate;
    var _endDate;
    var _plotWidth = 600;
    var _plotMargins = { right: 80, top: 50, left: 70, bottom: 60 };

    // We have TBI base test types and then how the head is moving.
    var _tbiBaseTestTypes = [
        'Head and Eyes Separate', 'Head and Eyes Together',
        'Balance with Head Turns', 'Walking with Head Turns'
    ];

    var HEAD_SIDE_SIDE = 'Side-Side';
    var HEAD_UP_DOWN = 'Up-Down';

    // Mapping of test type and the plots to generate. For example,
    // Head and Eyes Separate:Up-Down would show plots for comparing
    // "Sternum" and "Neck" ROM
    var _testPlots = {
        'Side-Side': [
            {
                title: 'Transverse ROM',
                plottedMetrics: ['mTBI/Forehead Transverse ROM',
                                 'mTBI/Sternum Transverse ROM'],
                domain: [0, 180],
                units: 'degrees'
            },
            {
                title: 'Transverse Maximum Velocity',
                plottedMetrics: ['mTBI/Forehead Transverse Maximum Velocity',
                                 'mTBI/Sternum Transverse Maximum Velocity'],
                domain: [0, 250],
                units: 'degrees/s'

            }
        ],
        'Up-Down': [
            {
                title: 'Sagittal ROM',
                plottedMetrics: ['mTBI/Forehead Sagittal ROM',
                                 'mTBI/Sternum Sagittal ROM'],
                domain: [0, 180],
                units: 'degrees'
            },
            {
                title: 'Sagittal Maximum Velocity',
                plottedMetrics: ['mTBI/Forehead Sagittal Maximum Velocity',
                                 'mTBI/Sternum Sagittal Maximum Velocity'],
                domain: [0, 250],
                units: 'degrees/s'
            }
        ]
    };

    /**
     * Open the initial report, using the last week as the starting
     * date range. Initialize the date range picker and setup button
     * and date change handlers. Get the trials that we're interested
     * in.
     * @param {Subject} subject The subject to generate report for.
     */
    function openInitialReport(subject) {
        _subject = subject;

        // Default to the previous week.
        // FIXME: Currently set to range that works for sample data.
        _startDate = moment(moment().subtract(7, 'days').format('YYYY-MM-DD'));
        _endDate = moment(moment().format('YYYY-MM-DD'));

        $('#switchable')
            .load('/mobility_clinic/html/tbi_report.html', pageLoaded);

        function pageLoaded() {
            $('#subject-header-title').text(function() {
                return UserOptionsHandler.getInstance()
                    .getSubjectName(_subject);
            });

            $('#tbi-report-back-button').click(function() {
                var objId = _subject.getObjId();
                hasher.setHash('');
                crossroads.parse('dispatch/subject?id=' + objId.getId() +
                                     '&localId=' + objId.getLocalId());

            });
            var datePickerOpts = {
                format: 'YYYY-MM-DD HH:mm',
                icons: {
                    time: 'iconic iconic-clock',
                    date: 'iconic iconic-calendar',
                    up: 'iconic iconic-chevron-up',
                    down: 'iconic iconic-chevron-down',
                    previous: 'iconic iconic-chevron-left',
                    next: 'iconic iconic-chevron-right',
                    today: 'iconic iconic-screenshot',
                    clear: 'iconic iconic-trash',
                    close: 'iconic iconic-x'
                },
                showClose: true,
                defaultDate: _startDate
            };
            $('#report-start-date').datetimepicker(datePickerOpts);
            datePickerOpts.defaultDate = _endDate;
            datePickerOpts.useCurrent = false;
            $('#report-end-date').datetimepicker(datePickerOpts);
            $('#report-start-date').on('dp.change', function (d) {
                $('#report-end-date').data("DateTimePicker").minDate(d.date);
                _startDate = d.date;
                update();
            });
            $('#report-end-date').on('dp.change', function (d) {
                $('#report-start-date').data('DateTimePicker').maxDate(d.date);
                _endDate = d.date;
                update();
            });
            update();
        }
    }

    /**
     * Update the report.
     */
    function update() {
        _plotWidth = ($(window).width() - 30) / 2;

        // Remove any existing plots on update
        _tbiBaseTestTypes.forEach(function(d) {
            [HEAD_SIDE_SIDE, HEAD_UP_DOWN].forEach(function(g) {
                d3.select('#' + utils.makeHtmlId(d))
                    .select('.' + g)
                    .selectAll('.single-tbi-plot')
                    .remove();
            });
        });

        _tbiBaseTestTypes.forEach(function(d) {
            [HEAD_SIDE_SIDE, HEAD_UP_DOWN].forEach(function(g) {
                d3.select('#' + utils.makeHtmlId(d))
                    .select('.' + g)
                    .selectAll('.single-tbi-plot')
                    .data(_testPlots[g])
                    .enter()
                    .append('div')
                    .attr('class', 'single-tbi-plot')
                    .each(function(h) {
                        plotForTestType(d, g, h, d3.select(this));
                    });
            });
        });
    }

    /**
     * Build a plot for the given test type.
     * @param {String} baseType The test base type we're creating the
     * plot for.
     * @param {String} subType The sub type of this test.
     * @param {Object} metricsToPlot Object with list of metrics,
     * title, and units for plot
     * @param {d3.selection} container The container for the plot
     */
    function plotForTestType(baseType, subType, metricsToPlot, container) {
        var plotData = [];
        var testType = baseType + ':' + subType;
        var trialsByDay = _subject.groupTrialsAndSequencesByDay();

        var keys = Object.keys(trialsByDay);
        var flattenedByDay = {};
        keys.forEach(function(d) {
            var trialDay = moment(d, 'YYYYMMDD');
            // Only trials within the date range set.
            if (trialDay.isBefore(_startDate) || trialDay.isAfter(_endDate)) {
                return;
            }
            flattenedByDay[d] = {};
            flattenedByDay[d].day = trialsByDay[d].day;
            flattenedByDay[d].trials = [];

        });
        var flattenedKeys = Object.keys(flattenedByDay);
        flattenedKeys.forEach(function(d) {
            trialsByDay[d].trials.forEach(function(g) {
                    if (g instanceof Session) {
                        flattenedByDay[d].trials = flattenedByDay[d].trials.concat(g.getAllTrials());
                    } else {
                        flattenedByDay[d].trials.push(g);
                    }
                });
        });

        flattenedKeys.forEach(function(d) {
            flattenedByDay[d].trials = flattenedByDay[d].trials
                .filter(function(h) {
                    return h.testName === testType;
                });
        });

        // Clear any existing plots
        container.selectAll('svg').remove();
        var title = metricsToPlot.title;

        // Get the data for the plot
        flattenedKeys.reduce(function(promise, key) {
            return promise.then(function() {
                if (!flattenedByDay[key].trials.length) {
                    return Promise.resolve();
                }
                var meansObj = {date: flattenedByDay[key].day,
                                conditionName: flattenedByDay[key].trials[0].getConditionDisplayName(),
                                metricValues: []};
                return combinedMeans(meansObj, metricsToPlot.plottedMetrics,
                                     flattenedByDay[key].trials)
                    .then(function() {
                        if (meansObj.metricValues &&
                            meansObj.metricValues.length) {
                            plotData.push(meansObj);
                        }
                    })
                    .catch(function(error) {
                        utils.error(error);
                    });
            })
                .catch(function(error) {
                    utils.error(error);
                });
        }, Promise.resolve())
            .then(function() {
                buildSinglePlot(baseType, plotData, title,
                                metricsToPlot.units, metricsToPlot.domain,
                                container);
            })
            .catch(function(ex) {
                console.log(ex);
            });

    }

    /**
     * Get the combined means for all the metrics given from with the
     * list of supplied trials.
     * @param {Object} meansObj Object to pack the returned values into.
     * @param {Array} metricNames List of metric names to generate the
     * combined means for.
     * @param {Array} trials list of trials to combine.
     * @return {Promise} Promise that resolves when all combined means
     * have been calculated.
     */
    function combinedMeans(meansObj, metricNames, trials) {
        var metricDefs = MetricDefinitionHelper.getInstance()
            .getMetricDefinitions(trials[0].getTestName());
        return metricNames.reduce(function(promise, name) {
            return promise.then(function() {
                return metricUtils.getMultipleTrialMean(name, trials)
                    .then(function(result) {
                        if ($.isNumeric(result.mean)) {
                            var displayName = metricDefs.get(name).displayName;
                            meansObj.metricValues
                                .push({name: meansObj.conditionName +
                                       ': ' + displayName,
                                       means: [result.mean],
                                       stds: [result.stdDev]});
                        }
                    })
                    .catch(function(error) {
                        utils.error(error);
                    });
            })
                .catch(function(error) {
                    utils.error(error);
                });
        }, Promise.resolve());
    }

    /**
     * Build a plot for a single metric.
     * @param {String} baseType The test base type for this plot
     * @param {Object} plotData The data to plot.
     * @param {String} title The title for the plot.
     * @param {String} units Units for the y-axis.
     * @param {Array} yDomain The y domain range.
     * @param {d3.selection} container The container for the plot.
     */
    function buildSinglePlot(baseType, plotData, title, units, yDomain, container) {
        var plot = new BarLongitudinalPlot()
            .width(_plotWidth)
            .height(280)
            .margin(_plotMargins)
            .x(function(d) { return d.date; })
            .y(function(d) { return d.metricValues; })
            .xAxisLabel('Date')
            .yAxisLabel(units)
            .xDomain([_startDate, _endDate])
            .yDomain(yDomain)
            .title(title);

        // Draw the plot if the tab if active
        if ($('a[href="#' + utils.makeHtmlId(baseType) + '"]')
            .parent().hasClass('active')) {
            plot.update(container.datum(plotData));
        }

        if (!plotData.length) {
            container
                .append('div')
                .append('h4')
                .text('No data to plot for ' + title);
        }

        // Need to do this to make sure that the plot is rendered correctly
        // since hidden elements don't report their sizes correctly.
        $('a[href="#' + utils.makeHtmlId(baseType) + '"]')
            .on('shown.bs.tab', function() {
                plot.update(container.datum(plotData));
            });
    }

    return {
        open: openInitialReport
    };
});
