/* Copyright © 2014-2018 APDM, Inc.
 *
 * test_sequence_builder.js
 *
 * Create the dialog for handling creation of a test sequence.
 */

define(['jquery', 'd3', 'utils', 'crossroads', 'test_definition_handler',
        'res_req_map', 'app/dialog_in_series', 'app/user_options_handler',
        'app/single_user_handler', 'server_handler', 'model_protocol'],
       function($, d3, utils, crossroads, TestDefinitionHandler,
                ResReqMap, DialogInSeries, UserOptionsHandler,
                SingleUserHandler, ServerHandler, ModelProtocol)
{
    'use strict';

    var builder = {};

    var selectedTests = {};
    var returnCaller = '';
    var editing;
    var canEdit;
    var sequenceName = '';
    var isIncompleteSequence = false;
    var oldSequenceName = null;
    var isDefault = false;
    var _disabledSequences = [];

    var dialog;

    var rootPath = '/mobility_clinic/html/';

    /**
     * Open the test sequence builder dialog.
     */
    function openDialog() {
        _disabledSequences = [];
        dialog = new DialogInSeries('dialog-test');
        $('#dialog-test-body').empty();

        if (isIncompleteSequence) {
            $('#dialog-test').find('.modal-dialog').removeClass('modal-lg');
            buildIncompleteSequenceDialog();
        } else {
            $('#dialog-test').find('.modal-dialog').addClass('modal-lg');
            buildTestSelectionDialog();
        }
    }

    /**
     * Build the dialog for displaying remaining tests in a sequence.
     */
    function buildIncompleteSequenceDialog() {
        $('#dialog-test-body')
            .load(rootPath + 'complete_sequence.html', pageLoaded);
        $('#dialog-test-title-text').html('Complete Test Sequence');
        buildSaveAndCloseButtons();

        function pageLoaded() {
            buildTestLists('#completed-test-list', selectedTests.completedTests);
            buildTestLists('#remaining-test-list', selectedTests.pendingTests);
            updateButtons();
            dialog.show(true);
        }

        function buildTestLists(listId, testList) {
            d3.select(listId).selectAll('div')
                .data(testList)
                .enter()
                .append('div')
                .attr('class', 'test-completion-label')
                .text(function(d) {
                    var def = TestDefinitionHandler.getInstance()
                        .getTestDefinition(d.testName, d.conditionName);
                    return def.testDisplayName + ', ' +
                        def.conditionDisplayName;
                });
        }
    }

    /**
     * Build the test selection dialog used for selecting tests for
     * running and sequence building.
     */
    function buildTestSelectionDialog() {
        $('#dialog-test-body').load(rootPath + 'test_select.html', pageLoaded);
        $('#dialog-test-title-text').html('Test Selection');
        // Set the size of the dialog
        $('#dialog-test').find('.modal-dialog').addClass('modal-lg');

        function pageLoaded() {
            buildSaveAndCloseButtons();
            getDisabledTestSequences(d3.select('#select-test-content'));
            dialog.show(true);

            // Set the path to show the dialog has opened.
            crossroads.parse('test_sequence_builder/open');
        }
    }

    /**
     * Create the "Save", "Close", "Copy" and "Export" buttons for the
     * dialog.
     */
    function buildSaveAndCloseButtons() {
        var saveButton = '<span class="iconic iconic-caret-right"></span>\n' +
            'Next\n';
        if (returnCaller === 'customSequence') {
            saveButton = '<span class="iconic iconic-check"></span>\n' +
                'Save\n';
        }
        var buttonHtml = '<button type="button" class="btn btn-link" ' +
            'id="test-sequence-close-button">' +
            '<span class="iconic iconic-x"></span>\n' +
            'Close\n</button>';
        if (isDefault) {
            // Add a warning message
            $('#edit-default-sequence-warning')
                .append('<h4>Note: Edits to default sequences cannot ' +
                        'be saved</h4>');
            if (canEdit) {
                buttonHtml = buttonHtml + '<button id="copy-test-series-button" ' +
                    'type="button" class="btn btn-default disabled="disabled"">' +
                    '<span class="iconic iconic-layers"></span>' +
                    'Copy</button>';
            }
            buttonHtml = buttonHtml + '<button id="export-test-series-button" ' +
                'type="button" class="btn btn-default disabled="disabled"">' +
                '<span class="iconic iconic-share"></span>' +
                'Export</button>';
        } else {
            if (editing) {
                if (canEdit) {
                    buttonHtml = buttonHtml + '<button id="copy-test-series-button" ' +
                        'type="button" class="btn btn-default" disabled="disabled">' +
                        'Copy</button>';
                }
                buttonHtml = buttonHtml + '<button id="export-test-series-button" ' +
                    'type="button" class="btn btn-default disabled="disabled"">' +
                    '<span class="iconic iconic-share"></span>' +
                    'Export</button>';
            }
            buttonHtml = buttonHtml + '<button id="run-test-series-button" ' +
                'type="button" class="btn btn-success" disabled="disabled">' +
                saveButton + '</button>';
        }

        // Add buttons to footer
        $('#dialog-test-footer').empty();
        $('#dialog-test-footer').append(buttonHtml);

        // Setup "Close" handler
        $('#test-sequence-close-button').click(function() {
            dialog.hide(true);
        });
    }


    /**
     * Get the disabled test sequences. If this is a single user UI
     * instance, the get the "local visibility" settings as well.
     * @param {d3.selection} element The element selection table is
     * being built.
     */
    function getDisabledTestSequences(element) {
        _disabledSequences = UserOptionsHandler.getInstance()
            .getDisabledTestSequences();

        if (!SingleUserHandler.getInstance().isSingleUserUI()) {
            buildSelectTestTable(element);
            return;
        }

        if (ServerHandler.getInstance().hasJsPostHandler()) {
            $.getJSON('/mobility_clinic/res/local_sequence_visibility.json',
                      function(d) {
                          _disabledSequences = _disabledSequences
                              .concat(d.disabledSequences);
                          d.enabledSequences.forEach(function(d) {
                              var idx = _disabledSequences.indexOf(d);
                              if (idx > -1) {
                                  _disabledSequences.splice(idx, 1);
                              }
                          });
                      })
                .always(function() { buildSelectTestTable(element); });
        } else {
            ServerHandler.getInstance()
                .post({type: ModelProtocol.get_local_sequence_visibility},
                      'json',
                      function(d) {
                          _disabledSequences = _disabledSequences
                              .concat(d.disabledSequences);
                          buildSelectTestTable(element);
                      },
                      function (e) {
                          // Not actually going to return an error
                          // here, just return the disabled that we
                          // know about.
                          buildSelectTestTable(element);
                          utils.error('Failed to get local sequence ' +
                                      'visibility: ' + e.error().statusText);
                      });
        }
    }

    /**
     * Build the table of tests and conditions for the given list.
     * @param element The element to build the table in.
     */
    function buildSelectTestTable(element) {
        var defHandler = TestDefinitionHandler.getInstance();
        var testList = defHandler.getTestNameMap();
        var sequences = defHandler.getSequenceNames();

        // Filter out tests with no visible conditions
        testList = testList.filter(function(d) {
            return getVisibleConditionDefs(d.name).length;
        });

        sequences = sequences.filter(function(d) {
            return _disabledSequences.indexOf(d) === -1;
        }).sort(d3.decending);

        // Handle there being no tests
        if (!testList.length && !sequences) {
            element.append('p').text('No tests currently visible.');
            return;
        }

        var formDiv = element.append('div').attr('class', 'form-horizontal')
            .attr('id', 'add-single-tests');
        // Add a field to specify the sequence name if caller is customSequence.
        if (returnCaller === 'customSequence') {
            buildNameFormEntry(formDiv, testList);
        }

        if (testList.length) {
            populateSingleTestsList(formDiv, testList);
        }

        // Populate selections for test sequences, if desired
        if (returnCaller === 'admin') {
            addSequencesToForm(element);
        }

        // Initialize the selection dropdowns.
        $('.selectpicker').selectpicker();

        addPreselectedTests();
    }

    /**
     * Populate the list of single tests available to user.
     * @param {d3.selection} formDiv Container where list is being added.
     * @param {Array} testList The list of available tests.
     */
    function populateSingleTestsList(formDiv, testList) {
        var topHeader = formDiv.append('h4').text('Single tests:');
        if (returnCaller === 'admin') {
            topHeader.attr('class', 'test-selection-admin-top-header');
        } else {
            d3.select('#test-selection-selected-header')
                .attr('class', 'test-selection-custom-top-header');
        }
        // Populate selections for single tests
        var fg = formDiv.selectAll('div').filter('.test-entry')
            .data(testList, function(d) { return d.name; });
        var fgEnter = fg.enter().append('div')
            .attr('class', 'form-group test-entry');

        fgEnter.each(function(d) {
            var div = d3.select(this);
            var id = 'add-test-to-series-' + utils.makeHtmlId(d.name);
            div.append('div').attr('class', 'col-sm-5')
                .append('span')
                .attr('class', 'control-label test-selection-label')
                .text(d.displayName);
            var dropdown = div.append('div')
                .attr('class', 'col-sm-5')
                .append('select')
                .attr('class', 'form-control selectpicker');
            dropdown.selectAll('option')
                .data(getVisibleConditionDefs(d.name))
                .enter()
                .append('option')
                .attr('value', function(g) { return g.conditionName; })
                .text(function(g) { return g.conditionDisplayName; });
            div.append('div').attr('class', 'col-sm-2').call(createTestBtns, id);
            div.select('#' + id)
                .on('click', function(d) {
                    return addTestToSeries(
                        d.name, div.select('select').node().value, null);
                });
        });
    }

    /**
     * Get the visible test conditions for the given test name.
     * @param {String} testName The name of the test to check.
     * @return {Array} The visible test conditions for the given test
     * type.
     */
    function getVisibleConditionDefs(testName) {
        var disabled = UserOptionsHandler.getInstance()
            .getDisabledTestConditions();
        var visible = TestDefinitionHandler.getInstance()
            .getTestConditionDefinitions(testName)
            .filter(function(d) {
                if (disabled[testName]) {
                    return disabled[testName].indexOf(d.conditionName) === -1;
                }
                return true;
            });
        return visible;
    }

    /**
     * Create the sequence name input box.
     * @param {d3.selection} formDiv The element to place the name entry in.
     */
    function buildNameFormEntry(formDiv) {
        formDiv.append('div')
            .attr('class', 'form-group row test-selection-header-container')
            .html('<div class="col-sm-5"><h4 id="sequence-name-label" ' +
                  'class="test-selection-sequence-header">Sequence:</h4>' +
                  '</div><div class="col-sm-7">' +
                  '<input type="text" name="sequenceName" ' +
                  'id="sequence-name" ' +
                  'class="form-control" placeholder="Sequence Name" />' +
                  '</div>'
                 );
        if (editing === true) {
            oldSequenceName = sequenceName;
            $('#sequence-name').val(sequenceName);
        }
    }

    /**
     * Add preselected tests to the list of selected tests.
     */
    function addPreselectedTests() {
        // Populate the list below with preselected tests.
        if (selectedTests.pendingTests.length > 0) {
            if (selectedTests.sequence && !selectedTests.session) {
                var name = selectedTests.sequence.name;
                removeSequenceFromSelected(); // Clear existing
                addSequenceToSeries(name);
            } else if (selectedTests.session &&
                       selectedTests.session.getTestName() !== 'Ad-Hoc') {
                addSequenceToSeries(selectedTests.session.name, true);
            } else {
                selectedTests.pendingTests.forEach( function(d) {
                    addTestToSeries(d.testName, d.conditionName, d.sequence);
                });
            }
        }
    }

    /**
     * Add the pre-built and user defined test sequences to the dialog form.
     * @param {d3.selection} element The element to build sequence content in.
     */
    function addSequencesToForm(element) {
        var sequences = TestDefinitionHandler.getInstance()
            .getSequenceNames();

        // Filter out sequences set not visible
        sequences = sequences.filter(function(d) {
            return _disabledSequences.indexOf(d) === -1;
        }).sort(d3.decending);

        var seqFormDiv = element.append('div')
            .attr('class', 'form-horizontal')
            .attr('id', 'add-test-sequence');
        seqFormDiv.append('h4').attr('class', 'test-selection-admin-sub-header')
            .text('Test sequences:');
        var seqG = seqFormDiv.selectAll('div').filter('.form-group')
            .data(sequences);
        seqG.enter().append('div').attr('class', 'form-group test-entry')
            .each(function(d) {
                var div = d3.select(this);
                var id = 'add-sequence-to-series-' + utils.makeHtmlId(d);
                div.append('div').attr('class', 'col-sm-10')
                    .append('span')
                    .attr('class', 'control-label test-selection-label')
                    .text(d);
                div.append('div').attr('class', 'col-sm-2')
                    .call(createTestBtns, id);
                div.select('#' + id)
                    .on('click', function(d) {
                        return addSequenceToSeries(d);
                    });
            });
    }

    /**
     * Create jQueryUI buttons to add test to series or to run singly.
     * @param selection The parent element to place buttons in.
     */
    function createTestBtns(selection, id) {
        var btn = selection.append('button')
            .attr('id', id)
            .attr('class', 'btn btn-default');
        btn.append('span').attr('class', 'iconic iconic-plus');
    }

    /**
     * Add a test to the series of selected tests.
     * @param test The name of the test to add.
     * @param condition The condition of the test to add.
     * @param sequence The sequence the test belongs to, if any.
     */
    function addTestToSeries(test, condition, sequence) {
        var def = TestDefinitionHandler.getInstance()
            .getTestDefinition(test, condition);
        var listItem = d3.select('#test-list').append('div')
            .attr('class', 'test-series-list-item')
            .data([ {testName: test,
                     conditionName: condition,
                     sequence: sequence} ]);
        var btn = listItem.append('button')
            .attr('class', 'btn btn-default test-btn')
            .on('click', function() {
                listItem.remove();
                updatePending();
                updateButtons();
            });
        btn.append('span').attr('class', 'iconic iconic-x');
        var textSpan = listItem.append('span')
            .attr('class', 'test-selection-label');
        textSpan.text(def.testDisplayName + ', ' + def.conditionDisplayName);
        // Disable the "Add" buttons for just the sequences if a
        // single test has been selected.
        d3.select('#add-test-sequence').selectAll('button')
            .attr('disabled', 'disabled');
        updatePending();
        updateButtons();

        function updatePending() {
            var pending = [];
            d3.selectAll('.test-series-list-item')
                .each(function(d) {
                    pending.push(TestDefinitionHandler.getInstance()
                                 .getTestDefinition(d.testName, d.conditionName));
                });
            selectedTests.pendingTests = pending;
        }
    }

    /**
     * Add the selected sequence to the "Selected Tests" to the dialog.
     * @param {String} sequenceName The name of the sequence to add.
     * @param {Boolean} isIncomplete Is this an incomplete sequence that
     * user is trying to complete.
     */
    function addSequenceToSeries(sequenceName, isIncomplete) {
        if (isIncomplete !== true) {
            addSequenceToSelected(sequenceName);
        }
        var listItem = d3.select('#test-list').append('div')
            .attr('class', 'test-series-list-item');
        var btn = listItem.append('button')
            .attr('class', 'btn btn-default test-btn')
            .on('click', function() {
                listItem.remove();
                removeSequenceFromSelected();
                updateButtons();
            });
        btn.append('span').attr('class', 'iconic iconic-x');
        listItem.append('span').attr('class', 'test-selection-label')
            .text(sequenceName);
        addSequenceTrialsToSeries(listItem);

        // Disable all the "Add" buttons if a sequence has been selected.
        d3.select('#add-single-tests').selectAll('button')
            .attr('disabled', 'disabled');
        d3.select('#add-test-sequence').selectAll('button')
            .attr('disabled', 'disabled');
        d3.select('#add-user-test-sequence').selectAll('button')
            .attr('disabled', 'disabled');
        updateButtons();
    }

    /**
     * Add a list of the trials in the selected sequence.
     * @param {d3.selection} element The element to add the list to.
     */
    function addSequenceTrialsToSeries(element) {
        selectedTests.pendingTests.forEach(function(d) {
            var def = TestDefinitionHandler.getInstance()
                .getTestDefinition(d.testName, d.conditionName);
            element.append('span').attr('class', 'test-series-sequence-trial')
                .text(def.testDisplayName + ', ' + def.conditionDisplayName);
        });
    }

    /**
     * Add a sequence of tests to the selected tests.
     * @param sequenceName The name of the sequence to add to the series.
     */
    function addSequenceToSelected(sequenceName) {
        var sequence = TestDefinitionHandler.getInstance()
            .getSequenceDefinition(sequenceName);
        var testNames = sequence.testNames.split('::');
        var testConditions = sequence.testConditions.split('::');
        var length = Math.min.apply(null,
                                    [testNames.length, testConditions.length]);
        // Need to make sure that the ids for sequence are zeroed.
        sequence.id = 0;
        sequence.localId = 0;

        for (var i = 0; length > i; i++) {
            var def = TestDefinitionHandler.getInstance()
                .getTestDefinition(testNames[i], testConditions[i]);
            def.sequence = sequence;
            selectedTests.pendingTests.push(def);
        }

        // Add the sequence to selectedTests object.
        selectedTests.sequence = sequence;
    }

    function removeSequenceFromSelected() {
        selectedTests = { sequence: null,
                          pendingTests: [],
                          completedTests: [] };
    }

    /**
     * Update the list of currently selected tests in the series.  If the
     * selected test is a test sequence, add all of its tests to the list
     * of pending. This assumes that the user can either build their own
     * test sequence, or choose a previously created one. They cannot mix
     * and match the two.
    */
    function updateButtons() {
        var btn = d3.select('#run-test-series-button');
        var copyBtn = d3.select('#copy-test-series-button');
        var exportBtn = d3.select('#export-test-series-button');
        if (selectedTests.pendingTests.length > 0) {
            d3.select('#test-series-default-text').style('display', 'none');
            btn.attr('disabled', null);
            copyBtn.attr('disabled', null);
            exportBtn.attr('disabled', null);
            // Make sure we don't have multiple handlers on this.
            btn.on('click', null);
            btn.on('click', function() {
                dispatch();
            });
            copyBtn.on('click', null);
            copyBtn.on('click', function() {
                dispatchCopy();
            });
            exportBtn.on('click', null);
            exportBtn.on('click', function() {
                dispatchExport();
            });

        } else {
            d3.select('#test-series-default-text').style('display', null);
            btn.attr('disabled', 'disabled');
            copyBtn.attr('disabled', 'disabled');
            // Enable all the "add" buttons.
            d3.select('#add-single-tests').selectAll('button')
                .attr('disabled', null);
            d3.select('#add-test-sequence').selectAll('button')
                .attr('disabled', null);
            d3.select('#add-user-test-sequence').selectAll('button')
                .attr('disabled', null);
        }
    }

    /**
     * Send a dispatch with the tests in the URL. If this is a new sequence,
     * and name is not valid, do not dispatch.
     */
    function dispatch() {
        if (returnCaller === 'customSequence') {
            selectedTests.isEdit = editing;
            if (editing === true) {
                selectedTests.oldSequenceName = oldSequenceName;
            }
            if (validateSequenceName() === false) {
                    return;
            }
            selectedTests.sequenceName = $('#sequence-name').val();
        }

        var key;
        if (returnCaller === 'admin') {
            if (isIncompleteSequence) {
                dialog.hide(false, function() {
                    key = ResReqMap.getInstance().setResponseData(selectedTests);
                    crossroads.parse('completing_sequence/' + key);
                });
            } else {
                dialog.hide(false, function() {
                    key = ResReqMap.getInstance().setResponseData(selectedTests);
                    crossroads.parse('tests_selected/' + key);
                });
            }
        } else {
            dialog.hide(true, function() {
                key = ResReqMap.getInstance().setRequestData(selectedTests);
                crossroads.parse('dispatch/addEditTestSequence/' + key);
            });
        }
    }

    /**
     * Dispatch a copy of the current sequence.
     */
    function dispatchCopy() {
        selectedTests.sequenceName = $('#sequence-name').val();
        var sequences = TestDefinitionHandler.getInstance().getAllSequences();
        while (sequences[selectedTests.sequenceName]) {
            selectedTests.sequenceName = selectedTests.sequenceName + '-copy';
        }
        dialog.hide(true, function() {
            var key = ResReqMap.getInstance().setRequestData(selectedTests);
            crossroads.parse('dispatch/addEditTestSequence/' + key);
        });
    }

    /**
     * Dispatch a request to export th current sequence.
     */
    function dispatchExport() {
        var sequenceName = $('#sequence-name').val();
        var sequences = TestDefinitionHandler.getInstance().getAllSequences();
        if (!sequences[sequenceName]) {
            return;
        }
        var key = ResReqMap.getInstance()
            .setRequestData({ sequenceName: sequenceName });
        crossroads.parse('dispatch/exportTestSequence/' + key);
    }

    /**
     * Validate the name of the sequence. Make sure there is a name
     * for the sequence. If this is a new sequence, make sure the name
     * is not the same as an existing sequence.
     * @return {Boolean} True if name is valid, otherwise false.
     */
    function validateSequenceName() {
        var nameField = $('#sequence-name');
        var newName = nameField.val();
        var allSequences = TestDefinitionHandler.getInstance()
            .getAllSequences();

        if (!newName || newName === null || newName === '') {
            nameField.parent().addClass('has-error');
            nameField
                .attr('placeholder', 'Must provide a name');
            return false;
        } else if (editing) {
            if (newName === oldSequenceName) {
                return true;
            } else if (allSequences[newName]) {
                nameField.val('');
                nameField.parent().addClass('has-error');
                nameField
                    .attr('placeholder', 'Sequence with name "' + newName +
                          '" already exists.');
                return false;
            }
        } else if (allSequences[newName]) {
            nameField.val('');
            nameField.parent().addClass('has-error');
            nameField
                .attr('placeholder', 'Sequence with name "' + newName +
                      '" already exists.');
            return false;
        }

        // Require that sequence names be unique from test names.
        // If there are no conditions, then no tests of this name exist.
        if (TestDefinitionHandler.getInstance().getTestConditions(newName).length) {
            nameField.val('');
            nameField.parent().addClass('has-error');
            nameField
                .attr('placeholder', 'Test with name "' + newName +
                      '" exists.');
            return false;
        }

        return true;
    }

    /***************************************************************************
     * Public functions.
     **************************************************************************/

    /**
     * Open the dialog with the given preselected tests.
     * @param {Object} preselectedTests Object containing the tests to
     * prepopulate selection with.
     * @param {String} caller Name to append to URL so proper
     * dispatching can be done when the dialog is closed.
     * @param {Boolean} isEdit Is this an edit operation on an existing sequence.
     * @param {String} name The name of the custom sequence, if this is an edit.
     */
    builder.open = function(preselectedTests, options) {
        if (preselectedTests && preselectedTests !== null) {
            selectedTests = preselectedTests;
        } else {
            selectedTests = { pendingTests: [],
                              completedTests: [],
                              sequence: null };
        }

        canEdit = options && options.canEdit ? options.canEdit : false;
        editing = options && options.isEdit ? options.isEdit : false;
        sequenceName = options && options.sequenceName ? options.sequenceName : null;
        returnCaller = options && options.caller ? options.caller : 'admin';
        isIncompleteSequence = options && options.isIncompleteSequence ?
            options.isIncompleteSequence : false;

        // Determine if we are view a default sequence.
        var defaultSequences = TestDefinitionHandler.getInstance()
            .getDefaultSequences();
        isDefault = ((defaultSequences[sequenceName] && editing) ||
                     !canEdit && editing);

        openDialog();
    };

    return builder;
});
