/* Copyright © 2014-2018 APDM, Inc.
 *
 * unilateral_arc_vis.js
 */

define(['jquery', 'd3', 'utils', 'app/apdm_vis'], function($, d3, utils, ApdmVis) {

    'use strict';

    /**
     * Set ApdmVis as base class.
     */
    UnilateralArcVis.prototype = new ApdmVis();

    /**
     * Constructor for UnilateralArcVis.
     * @param {d3.selection} container The d3 container to build
     * visualization in.
     * @param {String} baseSvg File name of the SVG to use for
     * visualization.
     * @param {String} testName The name of the test.
     * @param {String} conditionName The condition of the test.
     * @param {String} metricName The name of the metric.
     * @param {String} metrics The set of metrics for the current test.
     */
    function UnilateralArcVis(container, baseSvg, testName, conditionName,
                              metricName, metrics) {
        ApdmVis.call(this, container, baseSvg, testName, conditionName,
                     metricName, metrics);
        this.scale = d3.scaleLinear();
        this.normRadius = 149;
        this.subjectRadius = 165;

        // Size of the cirlcles.
        this.meanCircRad = 10;
        this.rangeCircRad = 4;

        // Offset angle applied to the data.
        this.offset = 0;
    }

    /**
     * Update the plot.
     */
    UnilateralArcVis.prototype.update = function() {
        var self = this;
        $(self.container.node()).load('../res/' + self.baseSvg, baseLoaded);
        var arcOffset = utils.degToRad(this.offset);
        var lineOffset = this.offset - 90;

        function baseLoaded() {
            self.container.select('svg');
            var visData = self.getUnilateralData();

            var arc;
            var minMax;
            // TODO: Need to determine which norms to use.
            if (visData.norms && visData.norms !== null &&
                visData.norms.fifthToNinteyFifth &&
                visData.norms.fifthToNinteyFifth !== null) {
                minMax = visData.norms.fifthToNinteyFifth;
                // D3 treats the vertical of the circle as 0, where as 0
                // is on the horizontal for initial contact.
                arc = d3.arc().innerRadius(0)
                    .outerRadius(self.normRadius)
                    .startAngle(utils.degToRad(minMax[0]) + arcOffset)
                    .endAngle(utils.degToRad(minMax[1]) + arcOffset);
                self.container.select('#rom-norm-range')
                    .selectAll('path')
                    .data([visData.norms])
                    .enter()
                    .append('path')
                    .attr('class', 'metric-vis-norm-range')
                    .attr('d', arc);

                var percentile = utils.getPercentile(visData.mean,
                                                     visData.norms.norms);
                if (!isNaN(percentile)) {
                    self.container.select('#unilateral-norm-text')
                        .text(self.getPercentileDisplay(percentile));
                }
            }

            if (visData.values !== null && visData.mean !== null) {
                minMax = d3.extent(visData.values);
                self.setUnilateralTextValues(visData);
                var subjectG = self.container
                    .select('#rom-subject-container');
                arc = d3.arc().innerRadius(self.subjectRadius)
                    .outerRadius(self.subjectRadius)
                    .startAngle(utils.degToRad(minMax[0]) + arcOffset)
                    .endAngle(utils.degToRad(minMax[1]) + arcOffset);
                subjectG.selectAll('#rom-subject-range')
                    .data([minMax])
                    .enter()
                    .append('path')
                    .attr('id', 'rom-subject-range')
                    .attr('class', 'metric-vis-mean-line')
                    .attr('d', arc);

                // Set the line for the subject mean
                subjectG.selectAll('#rom-subject-mean')
                    .data([visData.mean],
                          function(d) { return d; })
                    .enter()
                    .append('path')
                    .attr('id', 'rom-subject-mean')
                    .attr('class', 'metric-vis-mean-line')
                    .attr('d', 'M0,0 H ' + (self.subjectRadius))
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0)';
                    });

                // Do the circles.
                subjectG.selectAll('#rom-subject-range-circle')
                    .data(minMax)
                    .enter()
                    .append('circle')
                    .attr('id', 'rom-subject-range-circle')
                    .attr('class', 'metric-vis-subject-range-circle')
                    .attr('cx', self.subjectRadius)
                    .attr('cy', 0)
                    .attr('r', self.rangeCircRad)
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0)';
                    });

                subjectG.selectAll('#rom-subject-circle')
                    .data([visData.mean])
                    .enter()
                    .append('circle')
                    .attr('id', 'rom-subject-circle')
                    .attr('class', 'metric-vis-subject-range')
                    .attr('cx', self.subjectRadius)
                    .attr('cy', 0)
                    .attr('r', self.meanCircRad)
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0)';
                    });

                subjectG.selectAll('#rom-subject-circle-text')
                    .data([visData.mean])
                    .enter()
                    .append('g')
                    .attr('id', 'rom-subject-circle-text')
                    .attr('class', 'metric-vis-circle-label')
                    .attr('transform', function(d) {
                        return 'rotate(' + (d + lineOffset) + ', 0, 0) ' +
                            'translate(' + self.subjectRadius + ',0)';
                    })
                    .append('text')
                    .attr('class', 'metric-vis-circle-label')
                    .attr('y', 5)
                    .attr('transform', function(d) {
                        return 'rotate(' + (-(d + lineOffset)) + ', 0, 0)';
                    });

            }
            self.doUnilateralTextAlignment();
        }

    };

    /**
     * Get or set the domain of the plot.
     * @param {Array} plotDomain The domain of the plot.
     * @return The domain of the plot if called with no arguments,
     * otherwise this instance.
     */
    UnilateralArcVis.prototype.domain = function(plotDomain) {
        if (!arguments.length) {
            return this.scale.domain();
        }
        this.scale.domain(plotDomain);
        return this;
    };

    /**
     * Get or set the self.subjectRadius for the normative range arc.
     * @param {Number} normRadius The radius of the normative range arc.
     * @return The self.subjectRadius for the normative arc if called with no arguments,
     * otherwise the object instance.
     */
    UnilateralArcVis.prototype.normArcRadius = function(radius) {
        if (!arguments.length) {
            return this.normRadius;
        }
        this.normRadius = radius;
        return this;
    };

    /**
     * Get or set the self.subjectRadius for the subjet range mean line.
     * @param {Number} radius The radius of the subject line.
     * @return The self.subjectRadius for the subject line if called with no arguments,
     * otherwise the object instance.
     */
    UnilateralArcVis.prototype.subjectArcRadius = function(radius) {
        if (!arguments.length) {
            return this.subjectRadius;
        }
        this.subjectRadius = radius;
        return this;
    };

    /**
     * Get or set the offset for the data being displayed.
     * @param {Number} offset The offset in degrees to apply to the data.
     * @return {UnilateralArcVis} Object instance if called with
     * argument, otherwise current offset.
     */
    UnilateralArcVis.prototype.offsetAngle = function(offset) {
        if (!arguments.length) {
            return this.offset;
        }
        this.offset = offset;
        return this;
    };

    return UnilateralArcVis;
});
