/* Copyright © 2014-2017 APDM, Inc.
 *
 * unilateral_bar_vis.js
 */

define(['jquery', 'd3', 'utils', 'app/apdm_vis'], function($, d3, utils, ApdmVis) {

    'use strict';

    /**
     * Set ApdmVis as base class.
     */
    UnilateralBarVis.prototype = new ApdmVis();

    /**
     * Constructor for UnilateralBarVis.
     * @param {d3.selection} container The d3 container to build
     * visualization in.
     * @param {String} baseSvg File name of the SVG to use for
     * visualization.
     * @param {String} testName The name of the test.
     * @param {String} conditionName The condition of the test.
     * @param {String} metricName The name of the metric.
     * @param {String} metrics The set of metrics for the current test.
     */
    function UnilateralBarVis(container, baseSvg, testName, conditionName,
                              metricName, metrics) {
        ApdmVis.call(this, container, baseSvg, testName, conditionName,
                     metricName, metrics);
        this.scale = d3.scaleLinear();

        // Is the layout of the plot horizontal or vertical?
        // Default is vertical.
        this.isHorizontal = false;
    }

    /**
     * Update the plot.
     */
    UnilateralBarVis.prototype.update = function() {
        var self = this;
        $(self.container.node()).load('../res/' + self.baseSvg, baseLoaded);

        // Determine which direction to move the rectangles.
        var rectThickness = 'height';
        var posAxis = 'y';
        if (this.isHorizontal) {
            rectThickness = 'width';
            posAxis = 'x';
        }

        function baseLoaded() {
            self.container.select('svg').attr({'width': 700, 'height': 260});
            var visData = self.getUnilateralData();

            // Set the max range text.
            self.container.select('#metric-vis-max-range').text(self.scale.domain()[1]);

            // TODO: Determine if we should left or right norm.
            var height;
            var minMax;
            var rectPos;
            var norms = visData.norms;
            if (norms && norms !==null && norms.fifthToNinteyFifth &&
                norms.fifthToNinteyFifth !== null) {
                minMax = norms.fifthToNinteyFifth;
                height = Math.abs(self.scale(minMax[0]) -
                                  self.scale(minMax[1]));
                self.container.select('#unilateral-bar-norm-range')
                    .attr({ 'rx': 4, 'ry': 4 });

                rectPos = self.scale(minMax[1]);
                if (self.isHorizontal) {
                    rectPos = rectPos - height;
                }
                self.container.select('#unilateral-bar-norm-range')
                    .attr(rectThickness, height)
                    .attr(posAxis, rectPos);

                if (visData.mean) {
                    var percentile =
                        utils.getPercentile(visData.mean, norms.norms);
                    self.container.select('#unilateral-norm-text')
                        .text(self.getPercentileDisplay(percentile));
                }
            } else {
                self.container.select('#unilateral-bar-norm-range').remove();
            }

            if (visData.mean !== null) {
                rectPos = self.scale(visData.mean);
                if (self.isHorizontal) {
                    rectPos = rectPos - height;
                }
                // A little added to the height, to make sure it is below
                // the line.
                height =  Math.abs(self.scale(0) -
                                   self.scale(visData.mean)) + 3;

                self.container.select('#unilateral-bar-subject')
                    .attr(posAxis, rectPos)
                    .attr(rectThickness, height)
                    .attr({ 'rx': 4, 'ry': 4 });
                // Do the text.
                self.setUnilateralTextValues(visData);
                // If the data value is above the domain, then put in the up arrow.
                if (visData.mean > self.scale.domain()[1]) {
                    self.addOverrangeArrow('/mobility_clinic/res/overrange_arrow.svg', 'overrange-arrow');
                }
            } else {
                self.container.select('#unilateral-bar-subject')
                    .remove();
            }

            if (visData.values !== null && visData.values.length > 1) {
                minMax = d3.extent(visData.values);
                self.container
                    .select('#unilateral-bar-subject-range-top')
                    .attr('c' + posAxis, self.scale(minMax[1]));
                self.container
                    .select('#unilateral-bar-subject-range-bottom')
                    .attr('c' + posAxis, self.scale(minMax[0]));

                self.container
                    .select('#unilateral-bar-subject-range-line')
                    .attr(posAxis + '1', self.scale(minMax[1]))
                    .attr(posAxis + '2', self.scale(minMax[0]));
                if (minMax[1] > self.scale.domain()[1]) {
                    self.addOverrangeArrow('/mobility_clinic/res/overrange_arrow_sm.svg', 'overrange-arrow-sm');
                }
            } else {
                self.container
                    .select('#unilateral-bar-subject-range-top')
                    .remove();
                self.container
                    .select('#unilateral-bar-subject-range-bottom')
                    .remove();
                self.container
                    .select('#unilateral-bar-subject-range-line')
                    .remove();
            }

            self.doUnilateralTextAlignment();
        }
    };

    /**
     * Add an overrange arrow to center of the bar.
     * @param {String} foot The foot being handled.
     * @param {String} arrowFile Path to the arrow SVG file.
     * @param {String} arrowId The id of the arrow's g element.
     */
    UnilateralBarVis.prototype.addOverrangeArrow = function(arrowFile, arrowId) {
        var self = this;
        d3.xml(arrowFile, function(err, arrow) {
            if (err) {
                utils.error(err);
                return;
            }
            var outerSvg = self.container.select('svg');
            var arrowSvg = arrow.getElementsByTagName('svg')[0];
            outerSvg.node().appendChild(arrowSvg);

            // Get the position of the rectangle
            var x = outerSvg.select('#unilateral-bar-subject')
                .node().getBBox().x;

            // Get the transform on the bar container.
            var containerTrans =
                d3.transform(self.container
                             .select('#unilateral-bar-graph-container')
                             .attr('transform'));
            x = x + containerTrans.translate[0];

            var arrowG = outerSvg.select('svg').select('#' + arrowId);
            x = x + (arrowG.node().getBBox().width * 0.5) + 2.5;
            arrowG.attr('transform', 'translate(' + x +',0)');
        });
    };

    /**
     * Get or set the range of the plot.
     * @param {Array} plotRange The range of the plot as two element array.
     * @return The range of the plot if called with no arguments,
     * otherwise this instance.
     */
    UnilateralBarVis.prototype.range = function(plotRange) {
        if (!arguments.length) {
            return this.scale.range();
        }
        this.scale.range(plotRange);
        return this;
    };

    /**
     * Get or set the domain of the plot.
     * @param {Array} plotDomain The domain of the plot.
     * @return The domain of the plot if called with no arguments,
     * otherwise this instance.
     */
    UnilateralBarVis.prototype.domain = function(plotDomain) {
        if (!arguments.length) {
            return this.scale.domain();
        }
        this.scale.domain(plotDomain);
        return this;
    };

    /**
     * Get or set the flag that determines if the layout of the bars are
     * horizontal instead of vertical.
     * @param {Boolean} isHorizontal Flag that determines if layout of bars
     * are horizontal.
     * @return {UnilateralBarVis} Current instance if called with value,
     * otherwise the current flag value.
     */
    UnilateralBarVis.prototype.horizontal = function(isHorizontal) {
        if (!arguments.length) {
            return this.isHorizontal;
        }
        this.isHorizontal = isHorizontal;
        return this;
    };

    return UnilateralBarVis;
});
