/* Copyright © 2014-2019 APDM, Inc.
 *
 * unilateral_speedo_vis.js
 */

define(['jquery', 'd3', 'utils', 'app/apdm_vis'], function($, d3, utils, ApdmVis) {

    'use strict';

    /**
     * Set ApdmVis as base class.
     */
    UnilateralSpeedoVis.prototype = new ApdmVis();

    /**
     * Constructor for UnilateralSpeedoVis.
     * @param {d3.selection} container The d3 container to build
     * visualization in.
     * @param {String} baseSvg File name of the SVG to use for
     * visualization.
     * @param {String} testName The name of the test.
     * @param {String} conditionName The condition of the test.
     * @param {String} metricName The name of the metric.
     * @param {String} metrics The set of metrics for the current test.
     */
    function UnilateralSpeedoVis(container, baseSvg, testName, conditionName,
                                 metricName, metrics) {
        ApdmVis.call(this, container, baseSvg, testName, conditionName,
                     metricName, metrics);
        this.scale = d3.scaleLinear();
        this.scale.range([0, Math.PI]);
        this.normRadius = 101;
        this.subjectRadius = 147;

        // Size of the cirlcles.
        this.meanCircRad = 10;
        this.rangeCircRad = 3;

        // Offset so that '0' will always be horizontal left.
        this.offset = -90;
    }

    /**
     * Update the plot.
     */
    UnilateralSpeedoVis.prototype.update = function() {
        var self = this;
        $(self.container.node()).load('../res/' + self.baseSvg, baseLoaded);
        var arcOffset = utils.degToRad(this.offset);
        var lineOffset = this.offset - 90;

        function baseLoaded() {
            self.container.select('svg');
            var visData = self.getUnilateralData();

            // Set the ranges
            self.container.select('#unilateral-speedo-range-min')
                .text(self.scale.domain()[0]);
            self.container.select('#unilateral-speedo-range-max')
                .text(self.scale.domain()[1]);

            var arc;
            var minMax;
            // TODO: Need to determine which norms to use.
            if (visData.norms.fifthToNinteyFifth &&
                visData.norms.fifthToNinteyFifth !== null) {
                minMax = visData.norms.fifthToNinteyFifth;
                // D3 treats the vertical of the circle as 0, where as 0
                // is on the horizontal for initial contact.
                arc = d3.arc().innerRadius(0)
                    .outerRadius(self.normRadius)
                    .startAngle(self.scale(minMax[0]) + arcOffset)
                    .endAngle(self.scale(minMax[1]) + arcOffset);
                self.container.select('#unilateral-speedo-norm-range')
                    .selectAll('path')
                    .data([visData.norms])
                    .enter()
                    .append('path')
                    .attr('class', 'metric-vis-norm-range')
                    .attr('d', arc);
            }

            if (visData.values) {
                minMax = d3.extent(visData.values);
                var subjectG = self.container.select('#unilateral-speedo' +
                                                     '-subject-container');
                arc = d3.arc().innerRadius(self.subjectRadius)
                    .outerRadius(self.subjectRadius)
                    .startAngle(self.scale(minMax[0]) + arcOffset)
                    .endAngle(self.scale(minMax[1]) + arcOffset);
                subjectG.selectAll('#unilateral-speedo-subject-range')
                    .data([minMax])
                    .enter()
                    .append('path')
                    .attr('id', 'unilateral-speedo-subject-range')
                    .attr('class', 'metric-vis-mean-line')
                    .attr('d', arc);

                // Set the line for the subject mean
                subjectG.selectAll('#unilateral-speedo-subject')
                    .data([visData.mean],
                          function(d) { return d; })
                    .enter()
                    .append('path')
                    .attr('id', 'unilateral-speedo-subject')
                    .attr('class', 'metric-vis-mean-line')
                    .attr('d', 'M0,0 H ' + (self.subjectRadius))
                    .attr('transform', function(d) {
                        return 'rotate(' + (utils.radToDeg(self.scale(d)) + lineOffset) +
                            ', 0, 0)';
                    });

                // Do the circles.
                subjectG.selectAll('#unilateral-speedo-subject-range-circle')
                    .data(minMax)
                    .enter()
                    .append('circle')
                    .attr('id', 'unilateral-speedo-subject-range-circle')
                    .attr('class', 'metric-vis-subject-range-circle')
                    .attr('cx', self.subjectRadius)
                    .attr('cy', 0)
                    .attr('r', self.rangeCircRad)
                    .attr('transform', function(d) {
                        return 'rotate(' + (utils.radToDeg(self.scale(d)) + lineOffset) +
                            ', 0, 0)';
                    });

                subjectG.selectAll('#unilateral-speedo-subject-circle')
                    .data([visData.mean])
                    .enter()
                    .append('circle')
                    .attr('id', 'unilateral-speedo-subject-circle')
                    .attr('class', 'metric-vis-subject-range')
                    .attr('cx', self.subjectRadius)
                    .attr('cy', 0)
                    .attr('r', self.meanCircRad)
                    .attr('transform', function(d) {
                        return 'rotate(' + (utils.radToDeg(self.scale(d)) + lineOffset) +
                            ', 0, 0)';
                    });
            }

            if (visData.mean) {
                // Set the text
                self.setUnilateralTextValues(visData);
                var percentile = utils.getPercentile(visData.mean,
                                                     visData.norms.norms);
                self.container.select('#unilateral-norm-text')
                    .text(self.getPercentileDisplay(percentile));
            }
            self.doUnilateralTextAlignment();
        }
    };

    /**
     * Get or set the domain of the plot.
     * @param {Array} plotDomain The domain of the plot.
     * @return The domain of the plot if called with no arguments,
     * otherwise this instance.
     */
    UnilateralSpeedoVis.prototype.domain = function(plotDomain) {
        if (!arguments.length) {
            return this.scale.domain();
        }
        this.scale.domain(plotDomain);
        return this;
    };

    /**
     * Get or set the radius for the normative range arc.
     * @param {Number} normRadius The radius of the normative range arc.
     * @return The radius for the normative arc if called with no arguments,
     * otherwise the object instance.
     */
    UnilateralSpeedoVis.prototype.normArcRadius = function(radius) {
        if (!arguments.length) {
            return this.normRadius;
        }
        this.normRadius = radius;
        return this;
    };

    /**
     * Get or set the radius for the subjet range mean line.
     * @param {Number} normRadius The radius of the subject line.
     * @return The radius for the subject line if called with no arguments,
     * otherwise the object instance.
     */
    UnilateralSpeedoVis.prototype.subjectArcRadius = function(radius) {
        if (!arguments.length) {
            return this.subjectRadius;
        }
        this.subjectRadius = radius;
        return this;
    };

    return UnilateralSpeedoVis;
});
