/* Copyright (c) 2014-2018 APDM, Inc.
 *
 * user_options_handler.js
 *
 */

/* global defaultPatientOptions, defaultMetricSelections */

/**
 * Singleton class to handle user options.
 */
define(['jquery', 'server_handler', 'error_handler', 'model_protocol',
        'crossroads', 'res_req_map', 'app/report_enums'],
       function ($, ServerHandler, ErrorHandler, ModelProtocol, crossroads,
                 ResReqMap, reportEnums)
{
    var instance;

    // Constants to use for the the properties we set on the study.
    var PATIENT_OPTIONS = 'patientOptions';
    var METRIC_SELECTION_GROUPS = 'metricSelectionGroups';
    var ANNOTATIONS_ENABLED = 'enableAnnotations';
    var ANNOTATIONS_TEXT = 'annotations';
    var JOINT_ANGLE_PLOTS = 'jointAnglePlots';
    var HTTP_BACKUP_SETTINGS = 'HttpBackupSettings';
    var UPLOAD_AFTER_SEQUENCE = 'uploadAfterSequence';
    var DEFAULT_REPORT_TABLE = 'defaultReportTable';
    var DISABLE_DELETE_AFTER_RUN = 'disableDeleteAfterRun';
    var SUBJECT_ID_REGEX = 'subjectIdRegEx';
    var DISABLED_TEST_CONDITIONS = 'disabledTestConditions';
    var DISABLED_TEST_SEQUENCES = 'disabledTestSequences';
    var ALLOW_LOGGING_CONFIGURATION = 'allowLoggingConfiguration';

    function init() {
        var studyProperties = {};
        var currentStudy = null;
        var researchView = false;
        var requireLightweightLogin = false;
        var loginMessage;

        /**
         * Save a Study property.
         * @param {String} name The name of the property to save.
         * @param {Object} value The value of the property to save.
         */
        function saveProperty(name, value) {
            ServerHandler.getInstance().post(
                {
                    type: ModelProtocol.set_study_property,
                    name: name,
                    value: JSON.stringify(value)
                },
                null, null,
                function(d) {
                    ErrorHandler.showError(d.error().statusText);
                });
        }

        /**
         * Get the subject options for the current study.
         * @return {Object} The subject options.
         */
        function getPatientOptions() {
            if (!studyProperties[PATIENT_OPTIONS]) {
                // Need to clone the default options so they don't change.
                var clonedOptions =
                    $.extend({}, defaultPatientOptions);
                studyProperties[PATIENT_OPTIONS] = clonedOptions;
                saveProperty(PATIENT_OPTIONS, clonedOptions);
            }
            // User the default for missing values
            Object.keys(defaultPatientOptions.requiredFields).forEach(
                function(d) {
                    if (studyProperties[PATIENT_OPTIONS]
                        .requiredFields[d] === null ||
                        studyProperties[PATIENT_OPTIONS]
                        .requiredFields[d] === undefined) {
                        studyProperties[PATIENT_OPTIONS].requiredFields[d] =
                            defaultPatientOptions.requiredFields[d];
                    }
                });
            return studyProperties[PATIENT_OPTIONS];
        }

        return {
            /**
             * Get the last requested study name.
             * @return {String} Name of the last requested study.
             */
            getCurrentStudyName: function() {
                return currentStudy;
            },

            /**
             * Set the study name.
             * @param {String} The name of the study.
             */
            setCurrentStudyName: function(studyName) {
                currentStudy = studyName;
                instance.setMobilityLabProperty('last_viewed_study_name', studyName);
            },

            /**
             * Register the study properties for the current study.
             * @param {Array} properties The study's current set
             * of study properties.
             */
            registerStudyProperties: function(properties) {
                studyProperties = {};
                // Turn the array of study properties into Object to
                // use as map.
                var propertyName = '';
                var propertyValue = '';
                try {
                    properties.forEach(function(d) {
                    	propertyName = d.name;
                    	propertyValue = d.value;
                        studyProperties[propertyName] = JSON.parse(propertyValue);
                    });
                } catch (ex) {
                    ErrorHandler.showError(
                        'Failed to load one or more study properties. ' +
                            'Please contact your system administrator or ' +
                            'APDM support.\n' +
                            'Property Name: ' + propertyName + '\n' +
                            'Property Value: ' + propertyValue);
                }
            },

            /**
             * Get a all the Mobility Lab properties from the
             * MobilityLabPropertyManager that we care about.  When
             * propertly values returned, calls
             * crossroads.parse('mobilitylabPropertiesLoaded').
             */
            getRequiredMobilityLabProperties: function() {
                // The list of properties to get for the user options.
                var properties = '["last_viewed_study_name", ' +
                    '"require_lightweight_login", "login_message"]';
                ServerHandler.getInstance().post(
                    { type: ModelProtocol.get_properties,
                      properties: properties },
                    'json',
                    function(d) {
                        currentStudy = d.last_viewed_study_name;
                        requireLightweightLogin = d.require_lightweight_login;
                        loginMessage = d.login_message;
                        crossroads.parse('mobilitylabPropertiesLoaded');
                    },
                    function(d) {
                        ErrorHandler.showError(d.error().statusText);
                    }
                );
            },

            /**
             * Get a single Mobility Lab property from the
             * MobilityLabPropertyManager.  When property value
             * returned, calls crossroads.parse() with property name
             * and value in hash.
             * @param {String} property The name of the property to
             * retrieve.
             */
            getMobilityLabProperty: function(property) {
                ServerHandler.getInstance().post(
                    { type: ModelProtocol.get_property,
                      property: property },
                    'json',
                    function(d) {
                        crossroads.parse('singlePropertyLoaded/' +
                                         ResReqMap.getInstance().setResponseData(d));
                    },
                    function(d) {
                        ErrorHandler.showError(d.error().statusText);
                    });
            },

            /**
             * Set a single Mobility Lab property with the MobilityLabPropertyManager.
             * @param {String} name The name of the property to set.
             * @param {String} value The value to set property to.
             */
            setMobilityLabProperty: function(name, value) {
                ServerHandler.getInstance().post(
                    { type: ModelProtocol.set_property,
                      property: name,
                      value: value },
                    null, null,
                    function(d) {
                        ErrorHandler.showError(d.error().statusText);
                    });
            },

            /**
             * Return the value of the research view flag.
             * @return {Boolean} True if this is a research instance of
             * Mobility Lab.
             */
            isResearchView: function() {
                return researchView;
            },

            /**
             * Get required fields for creating a subject.
             * @return {Object} Object containing the required fields.
             */
            getPatientRequiredFields: function() {
                return getPatientOptions().requiredFields;
            },

            /**
             * Set the required fields for creating a subject. Triggers
             * a saving of the user options.
             * @param {Object} required Object containing the required fields.
             */
            setPatientRequiredFields: function(required) {
                getPatientOptions().requiredFields = required;
                saveProperty(PATIENT_OPTIONS, getPatientOptions());
            },

            /**
             * Get the metric selection groups.
             * @return {Object} The metric selection groups for the current study
             */
            getMetricSelectionGroups: function() {
                if(!studyProperties[METRIC_SELECTION_GROUPS]) {
                    studyProperties[METRIC_SELECTION_GROUPS] =
                        $.extend({}, defaultMetricSelections);
                    saveProperty(METRIC_SELECTION_GROUPS,
                                 defaultMetricSelections);
                    return studyProperties.metricSelectionGroups;
                }

                // Check for missing tests in groups. If a selection
                // group is missing a test, assign the default values.
                var defaults = defaultMetricSelections['default'].selections;
                for (var group in studyProperties[METRIC_SELECTION_GROUPS]) {
                    for (var test in defaults) {
                        if (!studyProperties[METRIC_SELECTION_GROUPS][group]
                            .selections[test]) {
                            studyProperties[METRIC_SELECTION_GROUPS][group]
                                .selections[test] = defaults[test].slice(0);
                        }
                    }
                }

                return studyProperties[METRIC_SELECTION_GROUPS];
            },

            /**
             * Set the metric selection groups for the current study.
             * @param {Object} selectionGroups Object containing
             * metric selection groups.
             */
            setMetricSelectionGroups: function(selectionGroups) {
                studyProperties[METRIC_SELECTION_GROUPS] = selectionGroups;
                saveProperty(METRIC_SELECTION_GROUPS, selectionGroups);
            },

            /**
             * Get the default set of metric selection groups.
             * @return {Object} The default metric selection groups.
             */
            getDefaultMetricSelectionGroups: function() {
                return defaultMetricSelections;
            },

            /**
             * Create a new metric selection group for the
             * given test name using the "Custom Test" as the
             * base default.
             * @param {String} testName Name of test to create metric
             * group for.
             */
            createNewMetricGroup: function(testName) {
                var defaults = defaultMetricSelections['default'].selections;
                for (var group in studyProperties[METRIC_SELECTION_GROUPS]) {
                    studyProperties[METRIC_SELECTION_GROUPS][group]
                        .selections[testName] = defaults['Custom Test'];
                }
                saveProperty(METRIC_SELECTION_GROUPS,
                             studyProperties[METRIC_SELECTION_GROUPS]);
                return studyProperties[METRIC_SELECTION_GROUPS];
            },

            /**
             * Get flag for enabling annotations in recordings.
             * @return {Boolean} True if annotations should be enabled.
             */
            getAnnotationsEnabled: function() {
                if (!studyProperties[ANNOTATIONS_ENABLED]) {
                    return false;
                }
                return studyProperties[ANNOTATIONS_ENABLED];
            },

            /**
             * Set and save flag for enabling annotations.
             * @param {Boolean} enabled
             */
            setAnnotationsEnabled: function(enabled) {
                studyProperties[ANNOTATIONS_ENABLED] = enabled;
                saveProperty(ANNOTATIONS_ENABLED, enabled);
            },

            /**
             * Get the current annotations.
             * @return {Object} Object containing annotations "one"
             * and "two"
             */
            getAnnotations: function() {
                if (!studyProperties[ANNOTATIONS_TEXT]) {
                    return { one: '', two: ''};
                }
                return studyProperties[ANNOTATIONS_TEXT];
            },

            /**
             * Set and save annotations.
             * @param {Object} annotations Object with annotations
             * "one" and "two".
             */
            setAnnotations: function(annotations) {
                studyProperties[ANNOTATIONS_TEXT] = annotations;
                saveProperty(ANNOTATIONS_TEXT, annotations);
            },

            /**
             * Get the last plotted joint angles.
             * @return {Array} The joints that were last plotted.
             */
            getPlottedJointAngles: function() {
                if (!studyProperties[JOINT_ANGLE_PLOTS]) {
                    return [];
                }
                return studyProperties[JOINT_ANGLE_PLOTS];
            },

            /**
             * Set the last plotted joint angles.
             * @param {Array} joints The list of joints plotted.
             */
            setPlottedJointAngles: function(joints) {
                studyProperties[JOINT_ANGLE_PLOTS] = joints;
                saveProperty(JOINT_ANGLE_PLOTS, joints);
            },

            /**
             * Get the string to display for the subject based on
             * the options.
             * @param {Subject} subject The subject to genereate name for.
             * @return {String} The display name.
             */
            getSubjectName: function(subject) {
                var fields = getPatientOptions().requiredFields;
                var displayName = '';
                if (fields.id === true) {
                    displayName = subject.getPublicId();
                }
                if (fields.name === true) {
                    if (fields.id === true) {
                        displayName = displayName + ' - ';
                    }
                    displayName = displayName + subject.getFullName();
                }
                return displayName;
            },

            /**
             * Determine if lightweight login is required.
             * @return {Boolean} True if lightweight login should be done.
             */
            getRequireLightweightLogin: function() {
                return (requireLightweightLogin === 'true');
            },

            /**
             * Get the message to display in login dialog.
             * @return {String} The message to display.
             */
            getLoginMessage: function() {
                return loginMessage;
            },

            /**
             * Determine if the study is set to support Http backup for trials.
             * @return True if study has HttpBackSettings set, otherwise false.
             */
            hasHttpBackup: function() {
                if (studyProperties[HTTP_BACKUP_SETTINGS]) {
                    return true;
                }
                return false;
            },

            /**
             * Get the HTTP backup parameters in the study properties.
             * @return {Object} The HTTP backup parameters defined by
             * the study.
             */
            getHttpBackupParams: function() {
                return studyProperties[HTTP_BACKUP_SETTINGS];
            },

            /**
             * Get the status of the flag determining if trials
             * should be uploaded after a sequence is completed.
             */
            getUploadAfterSequence: function() {
                if (studyProperties[UPLOAD_AFTER_SEQUENCE]) {
                    return studyProperties[UPLOAD_AFTER_SEQUENCE] === true;
                }
                return false;
            },

            /**
             * Set the
             * @param {String} table The type of table to display first in
             * report.
             */
            setDefaultReportTable: function(table) {
                studyProperties[DEFAULT_REPORT_TABLE] = table;
                saveProperty(DEFAULT_REPORT_TABLE, table);
            },

            /**
             * Get the default table to display for reports.
             * @return {Number} The table to display.
             */
            getDefaultReportTable: function() {
                if (studyProperties[DEFAULT_REPORT_TABLE]) {
                    return parseInt(studyProperties[DEFAULT_REPORT_TABLE], 10);
                }
                return reportEnums.reportType.table.METRICS_DETAIL;
            },

            /**
             * Set property determining if deleting a test immediatly
             * after running it should be allowed.
             * @param {Boolean} disableDelete Should delete be allowed.
             */
            setDisableDeleteAfterRun: function(disableDelete) {
                studyProperties[DISABLE_DELETE_AFTER_RUN] = disableDelete;
                saveProperty(DISABLE_DELETE_AFTER_RUN, disableDelete);
            },

            /**
             * Get flag to determine if deletes should be allowed after
             * a test is run. If unset, defaults to "false".
             * @return {Boolean} Deletes disabled if true.
             */
            getDisableDeleteAfterRun: function() {
                if (studyProperties[DISABLE_DELETE_AFTER_RUN]) {
                    return studyProperties[DISABLE_DELETE_AFTER_RUN];
                }
                return false;
            },

            /**
             * Get the regular expression to use for validating the
             * subject id entered when user created. Replaces %SITE_ID%
             * string in the regex if a site id exists.
             * @return {Object} Object map with regex as a string, and
             * a message string describing the regex format.
             */
            getSubjectIdRegEx: function() {
                if (studyProperties[SUBJECT_ID_REGEX]) {
                    return  studyProperties[SUBJECT_ID_REGEX];
                }
                return null;
            },

            /**
             * Set the map of disabled test condition to the one
             * provided in the form of {'Test Name': [list of disabled
             * test conditions]}.
             * @param {Object} disabledTestConditions Object map of
             * test names and a list of the disabled conditions.
             */
            setDisabledTestConditions: function(disabledTestConditions) {
                studyProperties[DISABLED_TEST_CONDITIONS] =
                    disabledTestConditions;
                saveProperty(DISABLED_TEST_CONDITIONS, disabledTestConditions);
            },

            /**
             * Get the object map defining the disabled test
             * conditions for test types.
             * @return {Object} Object map of test names and a list of
             * the disabled conditions.
             */
            getDisabledTestConditions: function() {
                if (studyProperties[DISABLED_TEST_CONDITIONS]) {
                    return  studyProperties[DISABLED_TEST_CONDITIONS];
                }
                return {};
            },

            /**
             * Set the list of disabled test sequences.
             * @param {Array} disabledTestSequences The list of disabled
             * test sequences.
             */
            setDisabledTestSequences: function(disabledTestSequences) {
                studyProperties[DISABLED_TEST_SEQUENCES] =
                    disabledTestSequences;
                saveProperty(DISABLED_TEST_SEQUENCES, disabledTestSequences);
            },

            /**
             * Get the list of disabled test sequences.
             * @return {Array} The list of disabled test sequences.
             */
            getDisabledTestSequences: function() {
                if (studyProperties[DISABLED_TEST_SEQUENCES]) {
                    return studyProperties[DISABLED_TEST_SEQUENCES];
                }
                return [];
            },

            /**
             * Set the list of disabled test sequences.
             * @param {Boolean} allowLogging Whether or not users will be allowed to
             * configure hardware for "Logging" mode.
             */
            setAllowLoggingConfiguration: function(allowLogging) {
                studyProperties[ALLOW_LOGGING_CONFIGURATION] =
                    allowLogging;
                saveProperty(ALLOW_LOGGING_CONFIGURATION, allowLogging);
            },

            /**
             * Get the study property that allows selection of
             * "Logging" or "Streaming" on the configuration page.
             * @return {Boolean} True if users are allowed to
             * configure for "Logging".
             */
            getAllowLoggingConfiguration: function() {
                if (studyProperties[ALLOW_LOGGING_CONFIGURATION]) {
                    return studyProperties[ALLOW_LOGGING_CONFIGURATION];
                }
                return false;
            },

            /**
             * Destroy the current the instance, and don't reinitialize.
             */
            destroy: function() {
                instance = null;
            }
        };
    }

    return {
        getInstance: function() {
            // Initialize if the object hasn't been created before.
            if (!instance) {
                instance = init();
            }
            return instance;
        }
    };
});
