/* Copyright (c) 2015-2017 APDM, Inc.
 *
 * Heatmap for variability data.
 */

define(['d3', 'apdm_plot'], function(d3, ApdmPlot) {

    VariabilityHeatmapPlot.prototype = new ApdmPlot();

    /**
     * Class for constructing variability plot with bilateral means
     */
    function VariabilityHeatmapPlot() {
        'use strict';

        this._height = 260;
        this._width = 700;

        this._yStep = 0.5;
        this._xStep = 1;

        this._colorScale = d3.scaleLinear().range(['green', 'red']);
    }

    /**
     * Update the plot.
     * @param {d3.selection} selection The container for the plot
     * with the data to plot associated with it.
     */
    VariabilityHeatmapPlot.prototype.update = function(selection) {
        var self = this;
        this._plotData = selection.datum();

        // Get our y bounds
        var yMax = 0;
        var yMin = 0;
        this._plotData.cycles.forEach(function(d) {
            d.gyroValues.map(function(e) {
                e = +e;
                if (e > yMax) {
                    yMax = e;
                } else if (e < yMin) {
                    yMin = e;
                }
                return e;
            });
        });

        var buckets = this.generateMatrix();

        var xMax = this._plotData.cycles[0].gyroValues.length;
        var widthNoMargins = this._width - this._margin.left - this._margin.right;
        var heightNoMargins = this._height - this._margin.top - this._margin.bottom;
        this._xScale = d3.scaleLinear().range([0, widthNoMargins])
            .domain([0, xMax]);
        this._yScale = d3.scaleLinear()
            .range([heightNoMargins, 0])
            .domain(d3.extent(Object.keys(buckets), function(d) {
                return buckets[d].box;
            }));
        this._colorScale.domain(d3.extent(Object.keys(buckets), function(d) {
            return buckets[d].count;
        }));

        this._xAxis = d3.axisBottom(this._xScale)
            .tickFormat(function(d) { return d / 2; });
        this._yAxis = d3.axisLeft(this._yScale)
            .tickFormat(function(d) { return d * self._yStep; });

        var plotSvg = selection.selectAll('svg').data([this._plotData]);
        var plotG = plotSvg.enter().append('svg').append('g');

        plotSvg.attr({'width': this._width, 'height': this._height});

        // Axis and label containers
        plotG.append('g').attr('class', 'x axis');
        plotG.append('g').attr('class', 'y axis');
        plotG.append('text').attr('class', 'x vis_label');
        plotG.append('text').attr('class', 'y vis_label');
        plotG.append('g').attr('class', 'title vis_label');

        plotG = plotSvg.select('g');
        this.buildAxes(plotG);
        this.determineWidthAndMargins(plotG);
        this.setScaleRanges();
        this.buildAxes(plotG);

        plotG.attr('transform', 'translate(' + this._margin.left +
                   ',' + this._margin.top + ')');

        var rectContainer = plotG.selectAll('.rect-container')
            .data([this._plotData.leftMean])
            .enter()
            .append('g')
            .attr({
                'width': this._width - this._margin.right - this._margin.left,
                'height': this._width - this._margin.top - this._margin.bottom,
                'class': 'rect-container'
            });

        var height = this._yScale(0) - this._yScale(1);
        var width =  this._xScale(1) - this._xScale(0);
        rectContainer.selectAll('rect').data(Object.keys(buckets))
            .enter()
            .append('rect')
            .attr({'x': function(d) { return self._xScale(buckets[d].x); },
                   'y': function(d) {return self._yScale(buckets[d].box); },
                   'width': width,
                   'height': height})
            .style('fill',  function(d) {
                return self._colorScale(buckets[d].count); });
    };

    VariabilityHeatmapPlot.prototype.generateMatrix = function() {
        var buckets = {};
        var idx = 0;
        {}
        for (idx = 0; idx < this._plotData.cycles[0].gyroValues.length; idx++) {
            this.mappingForDataIdx(buckets, idx, this._plotData.cycles);
        }

        return buckets;
    };

    VariabilityHeatmapPlot.prototype.mappingForDataIdx = function(buckets, idx, cycles) {
        var self = this;
        cycles.forEach(function(d) {
            var box = Math.floor(d.gyroValues[idx] / self._yStep);
            if (buckets[idx + ',' + box]) {
                buckets[idx + ',' + box].count = buckets[idx + ',' + box].count + 1;
            } else {
                buckets[idx + ',' + box] = {
                    count: 1,
                    box: box,
                    x: idx
                };
            }
        });
    };

    return VariabilityHeatmapPlot;
});
