/* Copyright (c) 2015-2018 APDM, Inc.
 *
 * Line plot to show variability.
 */

define(['jquery', 'd3', 'apdm_plot'], function($, d3, ApdmPlot) {

    VariabilityPlot.prototype = new ApdmPlot();

    /**
     * Class for constructing variability plot based on a normalized
     * set of data points.
     */
    function VariabilityPlot() {
        this._height = 260;
        this._width = 700;

        // Percentage of the range to add as padding at the top and
        // bottom.
        this._percentPad = 10;

        this._drawMean = true;
        this._drawCycles = true;
        this._drawPercentiles = true;
        this._drawPercentilesArea = false;

        this._drawSecondary = false;
        this._secondaryData = null;

        this._lineClasses = {
            cyclesPrimary: 'variability-cycles-primary',
            cyclesSecondary: 'variability-cycles-secondary',
            meanPrimary: 'variability-mean-primary',
            meanSecondary: 'variability-mean-secondary',
            fifthPrimary: 'variability-5-percentile-primary',
            fifthSecondary: 'variability-5-percentile-secondary',
            ninteyfifthPrimary: 'variability-95-percentile-primary',
            ninteyfifthSecondary: 'variability-95-percentile-secondary',
            percentileAreaPrimary: 'variability-area-fill-primary',
            percentileAreaSecondary: 'variability-area-fill-secondary'
        };

        this._yDomain = null;
    }


    /**
     * Update the plot.
     * @param {d3.selection} selection The container for the plot
     * with the data to plot associated with it.
     */
    VariabilityPlot.prototype.update = function(selection) {
        var self = this;
        this._selection = selection;
        if (this._drawSecondary) {
            this._plotData = selection.datum().primary;
            this._secondaryData = selection.datum().secondary;
        } else {
            this._plotData = selection.datum();
        }

        var widthNoMargins = this._width - this._margin.left - this._margin.right;
        var heightNoMargins = this._height - this._margin.top - this._margin.bottom;

        function getYBounds(d) {
            d.y.map(function(e) {
                e = +e;
                if (e > yMax) {
                    yMax = e;
                } else if (e < yMin) {
                    yMin = e;
                }
                return e;
            });
        }

        if (!this._yDomain) {
            // Get our y bounds
            var yMax = 0;
            var yMin = 0;

            this._plotData.cycles.forEach(getYBounds);
            if (this._drawSecondary) {
                this._secondaryData.cycles.forEach(getYBounds);
            }
            // Add some padding
            var range = 0;
            if ((yMin <= 0 && yMax > 0)) {
                range = Math.abs(yMin) +  yMax;
            } else if (yMin <= 0 && yMax <= 0) {
                range = Math.abs(yMin - yMax);
            } else {
                range = yMax - yMin;
            }

            var pad = range * (this._percentPad / 100);
            yMin = yMin <= 0 ? yMin - pad : yMin + pad;
            yMax = yMax <= 0 ? yMax - pad : yMax + pad;
            this._yDomain = [yMin, yMax];
        }

        var xMax = this._plotData.cycles[0].y.length;
        this._xScale = d3.scaleLinear().range([0, widthNoMargins])
            .domain([0, xMax]);

        this._yScale = d3.scaleLinear()
            .range([heightNoMargins, 0])
            .domain(this._yDomain);

        this._xAxis = d3.axisBottom(this._xScale)
            .tickFormat(function(d) { return d / 2; });
        this._yAxis = d3.axisLeft(this._yScale);

        var line = d3.line()
            .x(function(d, i) {
                return self._xScale(i); })
            .y(function(d) {
                return self._yScale(d); });

        var plotSvg = selection.selectAll('svg')
            .data([this._plotData])
            .enter()
            .append('svg');
        var plotG = plotSvg.append('g');

        plotSvg.attr('width', this._width)
            .attr('height', this._height);

        // Axis and label containers
        plotG.append('g').attr('class', 'x axis');
        plotG.append('g').attr('class', 'y axis');
        plotG.append('g').attr('class', 'y grid');
        plotG.append('g').attr('class', 'x grid');
        plotG.append('text').attr('class', 'x vis_label');
        plotG.append('text').attr('class', 'y vis_label');
        plotG.append('g').attr('class', 'title vis_label');

        plotSvg = selection.select('svg');
        plotG = plotSvg.select('g');
        this.buildAxes(plotG);
        this.determineWidthAndMargins(plotG);
        this.setScaleRanges();
        this.buildAxes(plotG);
        this.buildExportMenu();

        plotG.attr('transform', 'translate(' + this._margin.left +
                   ',' + this._margin.top + ')');

        // Render the lines for every gait cycle.
        function renderCycles(cycles, lineClass) {
            var timeKey = (new Date()).getTime();
            plotG.selectAll('.variability-line-' + lineClass)
                .data(cycles, function(d, i) { return i + timeKey; })
                .enter()
                .append('g')
                .attr('width', this._width - this._margin.right - this._margin.left)
                .attr('height', this._width - this._margin.top - this._margin.bottom)
                .attr('class', 'variability-line-' + lineClass)
                .append('path');

            plotG.selectAll('.variability-line-' + lineClass)
                .select('path')
                .attr('class', lineClass)
                .attr('d', function(d) {
                    return line(d.y);
                });
        }

        if (this._drawCycles) {
            renderCycles.call(this, this._plotData.cycles,
                              this._lineClasses.cyclesPrimary);
            if (this._drawSecondary) {
                renderCycles.call(this, this._secondaryData.cycles,
                                  this._lineClasses.cyclesSecondary);
            }
        }

        // Render the mean line
        function renderMean(mean, lineClass) {
            plotG.selectAll('.variability-mean-line-' + lineClass)
                .data([mean])
                .enter()
                .append('g')
                .attr('width', this._width - this._margin.right - this._margin.left)
                .attr('height', this._width - this._margin.top - this._margin.bottom)
                .attr('class', 'variability-mean-line-' + lineClass)
                .append('path');

            plotG.select('.variability-mean-line-' + lineClass)
                .select('path')
                .attr('class', lineClass)
                .attr('d', line(mean));
        }

        if (this._drawMean) {
            renderMean.call(this, this._plotData.meanCycle,
                            this._lineClasses.meanPrimary);
            if (this._drawSecondary) {
                renderMean.call(this, this._secondaryData.meanCycle,
                           this._lineClasses.meanSecondary);
            }
        }

        // Draw the percentile lines
        function renderPercentiles(percentile, type, lineClass) {
            plotG.selectAll('.' + type)
                .data([percentile])
                .enter()
                .append('g')
                .attr('width', this._width - this._margin.right - this._margin.left)
                .attr('height', this._width - this._margin.top - this._margin.bottom)
                .attr('class', type)
                .append('path');

            plotG.select('.' + type)
                .select('path')
                .attr('class', lineClass)
                .attr('d', line(percentile));
        }

        if (this._drawPercentiles){
            renderPercentiles.call(this, this._plotData.percentile95,
                                   'variability-95th',
                                   this._lineClasses.ninteyfifthPrimary);
            renderPercentiles.call(this, this._plotData.percentile5,
                                   'variability-5th',
                                   this._lineClasses.fifthPrimary);
            if (this._drawSecondary) {
                renderPercentiles.call(this, this._secondaryData.percentile95,
                                       'variability-95th-secondary',
                                       this._lineClasses.ninteyfifthSecondary);
                renderPercentiles.call(this, this._secondaryData.percentile5,
                                       'variability-5th-secondary',
                                       this._lineClasses.fifthSecondary);
            }
        }
        // Draw the percentile lines
        function renderPercentilesArea(percentileY1, percentileY2, type, areaClass) {
            var area = d3.area()
                .x(function(d, i) {
                    return self._xScale(i); })
                .y0(function(d) {
                    return self._yScale(d);
                })
                .y1(function(d, i) {
                    return self._yScale(percentileY2[i]);
                });

            plotG.selectAll('.' + type)
                .data([percentileY1])
                .enter()
                .append('g')
                .attr('width', this._width - this._margin.right - this._margin.left)
                .attr('height', this._width - this._margin.top - this._margin.bottom)
                .attr('class', type)
                .append('path');

            plotG.select('.' + type)
                .select('path')
                .attr('class', areaClass)
                .attr('d', area(percentileY1));
        }

        if (this._drawPercentilesArea){
            renderPercentilesArea.call(this, this._plotData.percentile95,
                                        this._plotData.percentile5,
                                       'variability-area',
                                       this._lineClasses.percentileAreaPrimary);
            if (this._drawSecondary) {
                renderPercentilesArea.call(this, this._secondaryData.percentile95,
                                           this._secondaryData.percentile5,
                                           'variability-area-secondary',
                                           this._lineClasses.percentileAreaSecondary);
            }
        }
    };

    /**
     * Toggle plotting of the mean line.
     * @param {Boolean} isPlotted True if mean should be plotted.
     * @return Current value if called with no arguments, otherwise this.
     */
    VariabilityPlot.prototype.drawMean = function(isPlotted) {
        if (!arguments.length) {
            return this._drawMean;
        }
        this._drawMean = isPlotted;
        return this;
    };

    /**
     * Toggle the plotting of all the cycles.
     * @param {Boolean} isPlotted True if all cycles should be plotted.
     * @return Current value if called with no arguments, otherwise this.
     */
    VariabilityPlot.prototype.drawCycles = function(isPlotted) {
        if (!arguments.length) {
            return this._drawCycles;
        }
        this._drawCycles = isPlotted;
        return this;
    };

    /**
     * Draw the 5th and 95th percentiles.
     * @param {Boolean} isPlotted True if percentiles should be plotted.
     * @return Current value if called with no arguments, otherwise this.
     */
    VariabilityPlot.prototype.drawPercentiles = function(isPlotted) {
        if (!arguments.length) {
            return this._drawPercentiles;
        }
        this._drawPercentiles = isPlotted;
        return this;
    };

    /**
     * Draw a fill area between the 5th and 95th percentile lines.
     * @param {Boolean} doFill True if area percentiles should be area.
     * @return Current value if called with no arguments, otherwise this.
     */
    VariabilityPlot.prototype.drawPercentilesArea = function(doFill) {
        if (!arguments.length) {
            return this._drawPercentilesArea;
        }
        this._drawPercentilesArea = doFill;
        return this;
    };

    /**
     * Plot a secondary set of values.
     * @param {Boolean} plot True if there is a second set of data that
     * is going to be plotted.
     * @return Current flag  value if called with no arguments, otherwise this.
     */
    VariabilityPlot.prototype.plotSecondary = function(doPlot) {
        if (!arguments.length) {
             return this._drawSecondary;
        }
        this._drawSecondary = doPlot;
        return this;
    };

    /**
     * Set the classes for the lines in the plot.
     * @param {Object} classes Object/map of "line type": "class".
     * @return Current classes if called with no argumenets, otherwise
     * this.
     */
    VariabilityPlot.prototype.lineClasses =function(classes) {
        if (!arguments.length) {
             return this._lineClasses;
        }
        $.extend(this._lineClasses, classes);
        return this;
    };

    /**
     * Set the y-axis domain to use for the plot. If left
     * unspecified, them if will default to using the range of
     * the given data.
     * @param {Array} domainRange The domain range to use.
     * @return Current domain if called with no arguments,
     * otherwise this.
     */
    VariabilityPlot.prototype.yDomain = function(domainRange) {
        if (!arguments.length) {
            return this._yDomain;
        }
        this._yDomain = domainRange;
        return this;
    };

    return VariabilityPlot;
});
