/* Copyright (c) 2018 APDM, Inc.
 *
 * varibility_plot_container.js
 *
 * Container for the joint variability plots displayed in-line in the
 * report.
 */

define(['jquery', 'd3', 'server_handler', 'model_protocol', 'es6-promise',
        'metric_definition_helper', 'app/variability_plot', 'app/joint_definitions',
        'error_handler', 'tinysort'],
       function($, d3, ServerHandler, ModelProtocol, Promise, MetricDefinitionHelper,
                VariabilityPlot, jointDefinitions, ErrorHandler, tinysort)
{
    'use strict';

    var _trial;
    var _plotDiv;
    var _visWidth;
    var _jointList;
    var _unilateralJoints = ['Neck', 'Trunk', 'Back', 'Lumbar'];

    var _variabilityClasses = {
        meanPrimary: 'trend_line',
        meanSecondary: 'secondary-trend-line',
        fifthPrimary: 'variability-percentile-left',
        fifthSecondary: 'variability-percentile-right',
        ninteyfifthPrimary: 'variability-percentile-left',
        ninteyfifthSecondary: 'variability-percentile-right'
    };

    function openPlot(trial, plotDiv, visWidth, metricName) {
        _trial = trial;
        _plotDiv = plotDiv;
        _visWidth = visWidth;

        ServerHandler.getInstance().post(
            {
                type: ModelProtocol.get_joint_angle_list,
                analysisFile: _trial.getAnalysisFileName()
            },
            'json',
            function(d) {
                _jointList = d.jointList;
                populateDropdown(metricName);
            },
            function(d) {
                ErrorHandler.showError(d.error().statusText);
            }
        );
    }

    /**
     * Populate the joint selection dropdown and pass on to
     * @param {String} metricName The name of the currently selected metric.
     */
    function populateDropdown(metricName) {
        $(_plotDiv.node()).load('report_variability_plots.html', pageLoaded);

        function pageLoaded() {
            var select = d3.select('#joint-angle-variability-select');
            var selectNode = $(select.node());
            var startingJoint = _jointList[0];

            var metricDef = MetricDefinitionHelper.getInstance()
                .getTestMetrics(_trial.getTestName())[metricName];

            var curJoint = metricDef.metricGroup.split('/')[2];
            if (_jointList.indexOf(curJoint) > -1) {
                startingJoint = curJoint;
            }

            select.selectAll('option').data(_jointList)
                .enter()
                .append('option')
                .attr('value', function(d) { return d; })
                .text(function(d) { return d; });
            tinysort(selectNode.find('option'));

            selectNode.selectpicker({ width: 'auto' });
            selectNode.selectpicker('val', startingJoint);
            selectNode.selectpicker('refresh');

            selectNode.change(function() {
                buildVaribilityPlot($(this).val());
            });

            buildVaribilityPlot(startingJoint);
        }
    }

    /**
     * Build the varibility plot after determining the "laterality".
     * @param {String} joint The name of joint to plot.
     */
    function buildVaribilityPlot(joint){
        if (_unilateralJoints.indexOf(joint) > -1) {
            buildUnilateralVariabilityPlot(joint);
        } else {
            buildBilateralVariabilityPlot(joint);
        }
    }

    /**
     * Do the variability plot for a bilateral joint.
     * @param {String} jointName The joint's name.
     */
    function buildBilateralVariabilityPlot(jointName) {
        var server = ServerHandler.getInstance();
        var plotData = {};

        _plotDiv.selectAll('.variability-plot-div')
            .data([plotData])
            .enter()
            .append('div')
            .attr('class', 'variability-plot-div');
        var div = _plotDiv.select('.variability-plot-div');
        div.selectAll('.vis_error_msg').remove();

        server.post({
            type: ModelProtocol.get_joint_angle_variability_plot_data,
            analysisFile: _trial.getAnalysisFileName(),
            returnCycles: true,
            joint: jointName + '/Left'
        }).then(function(jointData) {
            plotData.primary = jointData;
            return server.post({
                type: ModelProtocol.get_joint_angle_variability_plot_data,
                analysisFile: _trial.getAnalysisFileName(),
                returnCycles: true,
                joint: jointName + '/Right'
            });
        }).then(function(jointData) {
            plotData.secondary = jointData;
            doVariabilityPlots(div, jointName + '/Left', plotData, true);

        }, function(e) {
            $(div.node()).empty();
            div.append('h4').attr('class', 'vis_error_msg text-danger')
                .text(e.error().statusText);
        });
    }


    /**
     * Do the variability plot for a unilateral joint.
     * @param {String} jointName The joint's name.
     */
    function buildUnilateralVariabilityPlot(jointName) {
        var plotData = {};
        _plotDiv.selectAll('.variability-plot-div')
            .data([plotData])
            .enter()
            .append('div')
            .attr('class', 'variability-plot-div');
        var div = _plotDiv.select('.variability-plot-div');
        div.selectAll('.vis_error_msg').remove();

        ServerHandler.getInstance().post({
            type: ModelProtocol.get_joint_angle_variability_plot_data,
            analysisFile: _trial.getAnalysisFileName(),
            returnCycles: true,
            joint: jointName + '/Middle'
        }).then(function(jointData) {
            plotData = jointData;
            doVariabilityPlots(div, jointName + '/Middle', jointData, false);
        }, function(e) {
            $(div.node()).empty();
            div.append('h4').attr('class', 'vis_error_msg text-danger')
                .text(e.error().statusText);
        });
    }

    /**
     * Build a variability plot for each joint axis.
     * @param {d3.selection} div The element to build plots in.
     * @param {String} jointName The name of the joint being plotted.
     * @param {Object} plotData Object containing the data to plot.
     * @param {Boolean} doSecondary Set to true if there is primary and
     * secondary data to plot.
     */
    function doVariabilityPlots(div, jointName, plotData, doSecondary) {
        var axes = Object.keys(jointDefinitions[jointName].axes);
        var height = 260 * axes.length;
        _plotDiv.style('width', '700px')
            .style('height', height + 'px');

        var plotDivs = div.selectAll('div')
            .data(axes, function(d) { return d; });
        plotDivs.enter()
            .append('div')
            .each(function(d, i) {
                var title = jointDefinitions[jointName]
                    .axes[d].name + ' Variability';
                var plot= new VariabilityPlot()
                    .height(250).width(_visWidth)
                    .title(title)
                    .yAxisLabel('Angle (°)')
                    .drawCycles(false)
                    .drawMean(true)
                    .showYGrid(true)
                    .yGridValues([0])
                    .plotSecondary(doSecondary)
                    .includeExport(false)
                    .drawPercentilesArea(true)
                    .lineClasses(_variabilityClasses);
                if ((i + 1) >= Object.keys(axes).length) {
                    plot.xAxisLabel('Percent of Gait Cycle (%)');
                }

                var data = {};
                if (doSecondary) {
                    data.primary = plotData.primary[d];
                    data.secondary = plotData.secondary[d];
                } else {
                    data = plotData[d];
                }
                plot.update(d3.select(this).datum(data));
            });
        plotDivs.exit().remove();
    }

    return {
        open: openPlot
    };
});
